<?php

App::uses('Cache', 'Cache');

App::uses('Controller', 'Controller');
App::uses('AppController', 'Controller');
App::uses('ComponentCollection', 'Controller');

App::uses('AuthComponent', 'Controller/Component');
App::uses('AclComponent', 'Controller/Component');

App::uses('Model', 'AppModel');
App::uses('DbAcl', 'Model');
App::uses('AclNode', 'Model');
App::uses('Aco', 'Model');
App::uses('Aro', 'Model');

App::uses('ConnectionManager', 'Model');

/**
 *
 * AdminTools shell class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 *
 * @package		cake.app
 * @subpackage	cake.app.shell
 */
class AdminToolsShell extends AppShell {

	/**
	 *
	 * @var type
	 */
	private $User;

	/**
	 *
	 * @var type
	 */
	private $Collectivite;

	/**
	 *
	 * @var type
	 */
	private $Group;

	/**
	 *
	 * @var type
	 */
	private $Desktop;

	/**
	 *
	 * @var type
	 */
	private $_showHelp;

	/**
	 *
	 * @var type
	 */
	private $_bkpdir;

	/**
	 *
	 * @var type
	 */
	private $_bkpdate;

	/**
	 *
	 * @var type
	 */
	private $_fsOwner = "www-data";

	/**
	 *
	 * @var type
	 */
	private $_fsGroup = "www-data";

	/**
	 *
	 * @var type
	 */
	private $_conn;
	
	const SUCCESS = 0;

	/**
	 *
	 */
	public function startup() {
		parent::startup();

		$this->_conn = 'default';
		if (!empty($this->params['connection'])) {
			Configure::write('conn', $this->params['connection']);
			$this->_conn = $this->params['connection'];
		}

		ClassRegistry::init(('User'));
		$this->User = new User();
		$this->User->setDataSource($this->_conn);

		if ($this->_conn != 'default') {
			ClassRegistry::init(('Desktop'));
			$this->Desktop = new Desktop();
			$this->Desktop->setDataSource($this->_conn);

			ClassRegistry::init(('Group'));
			$this->Group = new Group();
			$this->Group->setDataSource($this->_conn);

			$collection = new ComponentCollection();
			$this->Acl = new AclComponent($collection);
			$this->Acl->startup(new AppController());

			$this->Aco = $this->Acl->Aco;
			$this->Aco->setDataSource($this->_conn);

			$this->Aro = $this->Acl->Aro;
			$this->Aro->setDataSource($this->_conn);
		} else {
			ClassRegistry::init(('Collectivite'));
			$this->Collectivite = new Collectivite();
			$this->Collectivite->setDataSource($this->_conn);
		}
	}

	/**
	 *
	 */
	public function main() {
		$this->_showHelp = false;

		list($usec, $sec) = explode(' ', microtime());
		$script_start = (float) $sec + (float) $usec;

		if (!empty($this->args[0])) {
			if ($this->args[0] == "clear_cache") {
				$this->_clear_cache();
			} else if ($this->args[0] == "hash_pass") {
				$this->_hash_pass($this->args[1]);
			} else if ($this->args[0] == "rights") {
				$this->_rights();
			} else if ($this->args[0] == "clear_logs") {
				$this->_clear_logs();
			} else if ($this->args[0] == "clean_tree_tables") {
				if (!empty($this->args[1]) && $this->args[1] == "with_acl") {
					$this->_clean_tree_tables(true);
				} else {
					$this->_clean_tree_tables();
				}
			} else if ($this->args[0] == "clean") {
				$this->_clean();
			} else if ($this->args[0] == "reset") {
				$this->_reset();
			} else if ($this->args[0] == "install") {
				$this->_install();
			} else if ($this->args[0] == "add") {
				$this->_add();
			} else if ($this->args[0] == "fix_fs_rights") {
				$this->_fixFsRigths();
			} else if ($this->args[0] == "backup") {
				$this->_backup();
			} else if ($this->args[0] == "list_collectivites") {
				$this->_getListColl();
			} else if ($this->args[0] == "repair_addressbook_slugs") {
				$this->_repairAdressBookSlugs();
			} else if ($this->args[0] == "delegation") {
				$this->_delegation();
			} else if ($this->args[0] == "set_light_rights") {
				$this->_setLightRights();
			} else {
				$this->_showHelp = true;
			}
		} else {
			$this->_showHelp = true;
		}

		if ($this->_showHelp) {
			$this->_displayHelp('');
		} else {
			list($usec, $sec) = explode(' ', microtime());
			$script_end = (float) $sec + (float) $usec;

			$elapsed_time = round($script_end - $script_start, 5);
			$this->out('Elapsed time : ' . $elapsed_time . 's');
			$this->out("<warning>Don't forget to check right access on APP/tmp.</warning>");
			$this->hr();
		}
	}

	private function _getListColl() {

		$qd = array(
			'fields' => array(
				'Collectivite.name',
				'Collectivite.id',
				'Collectivite.conn',
				'Collectivite.active'
			),
			'recursive' => -1,
			'order' => array(
				'Collectivite.active',
				'Collectivite.name'
			)
		);
		$colls = $this->Collectivite->find('all', $qd);

		$this->out('Liste des collectivités paramétrées :');
		$this->out('identifiant - statut - nom - connexion');
		foreach ($colls as $coll) {
			$this->out($coll['Collectivite']['id'] . ' - ' . ($coll['Collectivite']['active'] ? 'activée' : 'désactivée') . ' - ' . $coll['Collectivite']['name'] . ' - ' . $coll['Collectivite']['conn']);
		}
	}

	/**
	 *
	 */
	private function _fixFsRigths() {
		if (!empty($this->params['owner'])) {
			$this->_fsOwner = $this->params['owner'];
		}
		if (!empty($this->params['group'])) {
			$this->_fsGroup = $this->params['group'];
		}
		$cmdCakeChown = "sudo chown " . $this->_fsOwner . ":" . $this->_fsGroup . " " . CAKE . " -Rvf";
		$cmdCakeChmod = "sudo chmod u+rw,g+rw,o-w " . CAKE . " -Rvf";
		$cmdAppChown = "sudo chown " . $this->_fsOwner . ":" . $this->_fsGroup . " " . ROOT . " -Rvf";
		$cmdAppChmod = "sudo chmod u+rw,g+rw,o-w " . ROOT . " -Rvf";
		passthru(escapeshellcmd($cmdCakeChown));
		passthru(escapeshellcmd($cmdCakeChmod));
		passthru(escapeshellcmd($cmdAppChown));
		passthru(escapeshellcmd($cmdAppChmod));
	}

	/**
	 *
	 * @return string
	 */
	private function _backupDb($connName = "") {
		$conns = ConnectionManager::enumConnectionObjects();
		if ($connName == "") {
			$connName = $this->User->useDbConfig;
		}
		$conn = $conns[$connName];
		$filename = '';
		if ($conn) {

			$tmpBkpSchema = ROOT . "/webgfc_bd_" . $connName . "_schema_" . $this->_bkpdate . ".sql";
			$tmpBkpData = ROOT . "/webgfc_bd_" . $connName . "_data_" . $this->_bkpdate . ".sql";

			$cmdSchema = "pg_dump --host " . $conn['host'] . " --username " . $conn['login'] . (!empty($conn['port']) ? " --port " . $conn['port'] : "") . " -s " . $conn['database'] . " -f " . $tmpBkpSchema;
			$cmdData = "pg_dump --host " . $conn['host'] . " --username " . $conn['login'] . (!empty($conn['port']) ? " --port " . $conn['port'] : "") . " --insert --data-only " . $conn['database'] . " -f " . $tmpBkpData;

			passthru(escapeshellcmd($cmdSchema));
			passthru(escapeshellcmd($cmdData));

			$filename = $this->_bkpdir . "/webgfc_bd_" . $connName . "_" . $this->_bkpdate . ".tar.bz2";
			$cmd = "tar --create --bzip --remove-files --file " . $filename . " " . $tmpBkpSchema . " " . $tmpBkpData;
			passthru(escapeshellcmd($cmd));
		}
		return $filename;
	}

	/**
	 *
	 * @return type
	 */
	private function _backupApp() {
		$filename = $this->_bkpdir . "/webgfc_app_" . $this->_bkpdate . ".tar.bz2";
		$cmd = "tar --create --verbose --bzip --file " . $filename . " --exclude=.svn --exclude=*.tar.bz2 " . ROOT;
		passthru(escapeshellcmd($cmd));
		return($filename);
	}

	/**
	 *
	 */
	private function _backup() {
		$this->_bkpdir = ROOT;
		if (!empty($this->params['backup_direcory']) && is_dir($this->params['backup_directory'])) {
			$this->_bkpdir = $this->params['backup_direcory'];
		}
		$this->_bkpdate = date('Ymd_His');
		if ($this->params['backup_mode'] == 'full') {
			$filenames = array($this->_backupApp());
			$conns = ConnectionManager::enumConnectionObjects();
			foreach ($conns as $kConn => $conn) {
				$filenames[] = $this->_backupDb($kConn);
			}
			$filename = $this->_bkpdir . "/webgfc_full_" . $this->_bkpdate . ".tar.bz2";
			$cmd = "tar --create --bzip --remove-files --file " . $filename;
			foreach ($filenames as $file) {
				$cmd .= " " . $file;
			}
			passthru(escapeshellcmd($cmd));
			$this->hr(1);
			$this->out("fichier généré : " . $filename);
			$this->hr(1);
		} else if ($this->params['backup_mode'] == 'db') {
			$filename = $this->_backupDb();
			$this->hr(1);
			$this->out("fichier généré : " . $filename);
			$this->hr(1);
		} else if ($this->params['backup_mode'] == 'all_db') {
			$conns = ConnectionManager::enumConnectionObjects();
			foreach ($conns as $kConn => $conn) {
				$filename = $this->_backupDb($kConn);
				$this->hr(1);
				$this->out("fichier généré : " . $filename);
				$this->hr(1);
			}
		} else if ($this->params['backup_mode'] == 'app') {
			$filename = $this->_backupApp();
			$this->hr(1);
			$this->out("fichier généré : " . $filename);
			$this->hr(1);
		}
	}

	/**
	 *
	 */
	private function _install() {
		$this->hr(1);
		$this->out('Install starts. Please wait ...');
		$this->hr(1);

		$conns = ConnectionManager::enumConnectionObjects();
		$filename = APP . 'Config/sql/001_admin_schema.sql';
		
		putenv("PGPASSWORD=webgfc");
		
		$cmd = 'psql -U ' . $conns[$this->User->useDbConfig]['login'] . ' -p ' . $conns[$this->User->useDbConfig]['port'] . ' -h ' . $conns[$this->User->useDbConfig]['host'] . ' -f ' . $filename . ' ' . $conns[$this->User->useDbConfig]['database'] ;


		$this->hr(1);
		$this->out('<warning>Résultat de la commande</warning>');
		passthru(escapeshellcmd($cmd),$success);
		$this->hr(1);
		if( $success == self::SUCCESS ) {
			$this->out( 'Base de données superadmin créée' );
			$createUser = false;
			$this->out('Information du compte superadmin :');
	// 		$username = $this->in(__d('user', 'User.username'));
	// 		$password = $this->in(__d('user', 'User.password'));
	// 		$nom = $this->in(__d('user', 'User.nom'));
	// 		$prenom = $this->in(__d('user', 'User.prenom'));
	// 		$mail = $this->in(__d('user', 'User.mail'));
			$username = 'admin';
			$password = 'admin';
			$nom = 'Super';
			$prenom = 'Administrateur';
			$mail = 'admin@localhost.fr';

			$user = array(
				'User' => array(
					'username' => $username,
					'password' => $password,
					'nom' => $nom,
					'prenom' => $prenom,
					'mail' => $mail
				)
			);
			$this->User->create($user);
			if ($this->User->save()) {
				$createUser = true;
			}

			$this->hr(1);
			$this->out('Install done.');
			if ($createUser) {
				$this->out("<info>L'utilisateur a été créé.</info>");
			} else {
				$this->out("<error>L'utilisateur n'a pas été créé.</error>");
			}
		} else {
			$this->out( 'Aucune base créée' );
		}
		$this->hr(1);
	}

	/**
	 * Normalize simplified Rights table (fields 'model' and 'foreign_key')
	 */
	private function _normalizeRights() {
		$return = true;
		$Right = ClassRegistry::init('Right');
		$Right->useDbConfig($this->_conn);

		$rights = $Right->find('all');
		foreach ($rights as $right) {
			$aro = $this->Aro->find('first', array('conditions' => array('Aro.id' => $right['Right']['aro_id'])));
			$str = 'Normalize ' . $aro['Aro']['model'] . ' ' . $aro['Aro']['foreign_key'];

			$alreadyNormalized = true;
			if (empty($right['Right']['model'])) {
				$right['Right']['model'] = $aro['Aro']['model'];
				$alreadyNormalized = false;
			}
			if (empty($right['Right']['foreign_key'])) {
				$right['Right']['foreign_key'] = $aro['Aro']['foreign_key'];
				$alreadyNormalized = false;
			}

			if ($alreadyNormalized) {
				$str .= ': <info>Already normalized</info>';
			} else {
				$Right->create();
				$saved = $Right->save($right);
				if (!empty($saved)) {
					$str .= ': <success>Normalized</success>';
				} else {
					$str .= ': <error>Error</error>';
					$return = false;
				}
			}
			$this->out($str);
		}
		return $return;
	}

	/**
	 *
	 */
	private function _repairAdressBookSlugs() {

		/**
		 * Contact slugs
		 */
		$Contact = ClassRegistry::init('Contact');
		$Contact->useDbConfig($this->_conn);


		$this->hr();
		$this->out('Repairing Contact slugs');
		$this->hr();

		$Contact->begin();
		$validContact = array();

		$contacts = $Contact->find('all', array('fields' => array('Contact.id', 'Contact.name'), 'recursive' => -1));
		foreach ($contacts as $contact) {
			$Contact->id = $contact['Contact']['id'];
			$slug = strtolower(Inflector::slug($contact['Contact']['name']));
			$ok = $Contact->saveField('slug', $slug);
			$this->out($contact['Contact']['name'] . ' : ' . $slug . ' - ' . ($ok ? '<success>ok</success>' : '<error>nok</error>'));
			$validContact[] = $ok;
		}

		if (!empty($validContact) && !in_array(false, $validContact, true)) {
			$Contact->commit();
			$this->out('<success>Contact repair slug done</success>');
		} else {
			$Contact->rollback();
			$this->out('<warning>Contact repair slug ended with error (rollback)</warning>');
		}



		/**
		 * Contactinfo slugs
		 */
		$Contactinfo = ClassRegistry::init('Contactinfo');
		$Contactinfo->useDbConfig($this->_conn);


		$this->hr();
		$this->out('Repairing Contactinfo slugs');
		$this->hr();

		$Contactinfo->begin();
		$validContactinfo = array();

		$contactinfos = $Contactinfo->find('all', array('fields' => array('Contactinfo.id', 'Contactinfo.nom', 'Contactinfo.prenom'), 'recursive' => -1));
		foreach ($contactinfos as $contactinfo) {
			$Contactinfo->id = $contactinfo['Contactinfo']['id'];
			$slug = strtolower(Inflector::slug($contactinfo['Contactinfo']['nom'] . ' ' . $contactinfo['Contactinfo']['prenom']));
			$ok = $Contactinfo->saveField('slug', $slug);
			$this->out($contactinfo['Contactinfo']['nom'] . ' ' . $contactinfo['Contactinfo']['prenom'] . ' : ' . $slug . ' - ' . ($ok ? '<success>ok</success>' : '<error>nok</error>'));
			$validContactinfo[] = $ok;
		}

		if (!empty($validContactinfo) && !in_array(false, $validContactinfo, true)) {
			$Contactinfo->commit();
			$this->out('<success>Contactinfo repair slug done</success>');
		} else {
			$Contactinfo->rollback();
			$this->out('<warning>Contactinfo repair slug ended with error (rollback)</warning>');
		}
	}

	/**
	 *
	 */
	private function _reindexRights() {
		$return = true;
		$Right = ClassRegistry::init('Right');
		$Right->useDbConfig($this->_conn);

		$rights = $Right->find('all');
		foreach ($rights as $right) {
			$aro = $this->Aro->find('first', array('conditions' => array('Aro.model' => $right['Right']['model'], 'Aro.foreign_key' => $right['Right']['foreign_key'])));
			$str = 'Reindexing ' . $aro['Aro']['model'] . ' ' . $aro['Aro']['foreign_key'];
			$right['Right']['aro_id'] = $aro['Aro']['id'];

			$valid = array();
			$Right->begin();
			$valid[] = $Right->delete($right['Right']['id']);

			$Right->create();
			$saved = $Right->save($right);
			$valid[] = !empty($saved);

			if (!in_array(false, $valid, true)) {
				$Right->commit();
				$str .= ': <success>Ok</success>';
				$return = true;
			} else {
				$Right->rollback();
				$str .= ': <error>Error</error>';
			}
			$this->out($str);
		}
		return $return;
	}

	/**
	 * Call of DbToolsShell delete_db method ( Delete ACL Table's content (MySQL: autoincrement reseted, PostgreSQL: serial not reseted) )
	 *
	 * @return boolean
	 */
	private function _delete_db() {
		$ret = false;
		$this->out(__('**** ACL tables delete starts. Please wait ... ****'));
		$this->Aro->query("TRUNCATE table acos CASCADE;");
		$this->Aro->query("ALTER SEQUENCE acos_id_seq RESTART WITH 1;");
		$this->Aro->query("TRUNCATE table aros CASCADE;");
		$this->Aro->query("ALTER SEQUENCE aros_id_seq RESTART WITH 1;");
		$this->Aro->query("TRUNCATE table aros_acos CASCADE;");
		$this->Aro->query("ALTER SEQUENCE aros_acos_id_seq RESTART WITH 1;");
		$this->out(__('==== ACL tables delete done. ===='));
		$ret = true;
		return $ret;
	}

	/**
	 * Call of DbToolsShell aro_sync method ( Synchronisation between Aro table and User/Group tables )
	 *
	 * @return boolean
	 */
	private function _aro_sync($admin = false) {
		$ret = false;
		$this->out(__('**** Aro Sync starts. Please wait ... ****'));
		$this->_aro_sync_group();
		$this->_aro_sync_desktop();
		$this->out(__('==== Aro Sync done. ===='));
		$ret = true;
		return $ret;
	}

	/**
	 * Call of AclExtrasShell aco_sync method ( Sync the ACO table )
	 *
	 * @return boolean
	 * @link http://mark-story.com
	 */
	private function _aco_sync() {
		$ret = false;
		$this->out(__('**** Aco Sync starts. Please wait ... ****'));
		$this->dispatchShell('AclExtras.AclExtras aco_sync');
		$this->out(__('==== Aco Sync done. ===='));
		$ret = true;
		return $ret;
	}

	/**
	 * Full rigths init
	 *
	 * @return void
	 */
	private function _rights() {
		$this->out(__('**** Full rights init starts. Please wait ... ****'));
		if ($this->_normalizeRights()) {
			if ($this->_delete_db()) {
				if ($this->_aro_sync()) {
					if ($this->_aco_sync()) {
						if ($this->_reindexRights()) {
							if ($this->_setLightRights()) {
								$this->out(__('==== Full rights init done. ===='));
							} else {
								$this->out('<error>Error while setting simplified rights table.</error>');
							}
						} else {
							$this->out('<error>Error while reindexing simplified rights table.</error>');
						}
					} else {
						$this->out('<error>Error while syncing Acos.</error>');
					}
				} else {
					$this->out('<error>Error while syncing Aros.</error>');
				}
			} else {
				$this->out('<error>Error while purging Acls.</error>');
			}
		} else {
			$this->out('<error>Error while normalizing simplified rights table.</error>');
		}
	}

	/**
	 *
	 */
	private function _clear_logs() {
		App::uses('Folder', 'Utility');
		$logs_folder = new Folder(TMP . 'logs');
		$logs = $logs_folder->read(true, false, true);
		foreach ($logs[1] as $file) {
			if (file_put_contents($file, '') !== false) {
				$this->out('File ' . $file . ' cleared');
			}
		}
	}

	/**
	 * CakePHP cache cleaner
	 *
	 * @return void
	 */
	private function _clear_cache() {
		if (Cache::clear()) {
			$this->out('Cached data : cleared');
		}
		$cachePaths = array('js', 'css', 'menus', 'views', 'persistent', 'models');
		foreach ($cachePaths as $config) {
			if (clearCache(null, $config)) {
				$this->out('Cache ' . $config . ' : cleared');
			} else {
				$this->out('Cache ' . $config . ' : no files removed');
			}
		}
	}

	/**
	 *
	 * @param type $pass
	 */
	private function _hash_pass($pass) {
		$this->out(AuthComponent::password($pass));
	}

	/**
	 *
	 * @param type $with_acl
	 */
	private function _clean_tree_tables($with_acl = false) {
		$this->out(__('**** Cleaning tree tables starts. Please wait ... ****'));
		$modelNames = $this->_modelsList();
		foreach ($modelNames as $modelName) {
			if ($this->_hasTreeBehavior($modelName)) {
				$modelClass = ClassRegistry::init($modelName);
				$status = $modelClass->recover();
				$this->out($modelName . ': ' . ($status ? 'ok' : 'error'));
			}
		}

		if ($with_acl) {
			$this->Acl = new AclComponent(new ComponentCollection());
			$this->Acl->startup(null);
			$this->out('Aro :' . ($this->Acl->Aro->recover() ? 'ok' : 'error' ));
			$this->out('Aco :' . ($this->Acl->Aco->recover() ? 'ok' : 'error'));

			App::uses('DataAclComponent', 'DataAcl.Controller/Component');

			$this->DataAcl = new DataAclComponent(new ComponentCollection());
			$this->DataAcl->startup(null);
			$this->DataAcl->Daro->recover();
			$this->DataAcl->Daco->recover();
		}
		$this->out(__('==== Cleaning tree table done. ===='));
	}

	/**
	 *
	 * @param type $modelName
	 * @return boolean
	 */
	protected function _hasTreeBehavior($modelName) {
		/* if( in_array( $modelName, array( 'Aco', 'Aro', 'Daro', 'Daco' ) ) ) {
		  return false; // FIXME
		  } */
		$file = APP . DS . 'Model' . DS . $modelName . '.php';
		if (!file_exists($file)) {
			return false;
		}
		$modelClass = ClassRegistry::init($modelName);
		if (empty($modelClass->actsAs)) {
			return false;
		}
		$behaviors = array_keys(Set::normalize($modelClass->actsAs));
		return in_array('Tree', $behaviors);
	}

	/**
	 *
	 * @param type $accepted
	 * @return type
	 */
	protected function _modelsList($accepted = array()) {
		$models = array();
		$dirName = sprintf('%sModel' . DS, APP);
		$dir = opendir($dirName);
		while (( $file = readdir($dir) ) !== false) {
			$explose = explode('~', $file);
			if (( count($explose) == 1 ) && (!is_dir($dirName . $file) ) && (!in_array($file, array('empty', '.svn')) )) {
				$model = Inflector::classify(preg_replace('/\.php$/', '', $file));
				if (empty($accepted) || in_array($model, $accepted)) {
					$models[] = $model;
				}
			}
		}
		closedir($dir);
		sort($models);
		return $models;
	}

	/**
	 *
	 */
	protected function _clean() {
		$this->out(__('**** Running maintenance tasks, Please wait ... ****'));
		$this->_clean_tree_tables(true);
		$this->_clear_logs();
		$this->_clear_cache();
		$this->out(__('==== Maintenance tasks done. ===='));
	}

	/**
	 *
	 */
	protected function _reset() {
		$this->out(__('**** Running reset tasks, Please wait ... ****'));
		$this->_clean();
		$this->_rights();
		$this->out(__('==== Reset tasks done. ===='));
	}

	/**
	 *
	 */
	protected function _setLightRights() {
		$ret = false;
		$this->out(__('**** Setting rights starts. Please wait ... ****'));

		$this->out(__(' -- Base groups --'));
		$baseGroupAros = $this->Acl->Aro->find('all', array('recursive' => -1, 'conditions' => array('model' => 'Group', 'id > ' => 1, 'id < ' => 8)));
		$this->_setAclFromRights($baseGroupAros, 'baseGroup');

		$this->out(__(' -- Other groups --'));
		$otherGroupAros = $this->Acl->Aro->find('all', array('recursive' => -1, 'conditions' => array('model' => 'Group', 'id >= ' => 8)));
		$this->_setAclFromRights($otherGroupAros, 'otherGroup');

		$this->out(__(' -- Desktops --'));
		$desktopAros = $this->Acl->Aro->find('all', array('recursive' => -1, 'conditions' => array('model' => 'Desktop')));
		$this->_setAclFromRights($desktopAros, 'desktop');

		$this->out(__('==== Setting rights done. ===='));
		$ret = true;
		return $ret;
	}

	/**
	 *
	 * @param type $aros
	 * @param type $mode : 'baseGroup' ,'otherGroup', 'desktop'
	 */
	private function _setAclFromRights($aros, $mode) {
		$Right = ClassRegistry::init('Right');
		$Right->useDbConfig($this->_conn);
		$Group = ClassRegistry::init('Group');
		$Group->useDbConfig($this->_conn);
		$Desktop = ClassRegistry::init('Desktop');
		$Desktop->useDbConfig($this->_conn);


		foreach ($aros as $aro) {

			$rights = array();
			$rightSets = array();

			if ($mode == 'baseGroup') {
				$rightSets = $Right->getRightSetsFromGroupName($aro['Aro']['alias']);
			} else if ($mode == 'otherGroup') {
				$group = $Group->find('first', array('conditions' => array('Group.id' => $aro['Aro']['foreign_key'], 'Group.active' => true)));
				if (!empty($group)) {
					$baseAro = $this->Aro->find('first', array('conditions' => array('Aro.model' => 'Group', 'Aro.foreign_key' => $group['Group']['group_id'])));
					if (!empty($baseAro)) {
						$rightSets = $Right->getRightSetsFromGroupName($baseAro['Aro']['alias']);
					}
				}
				$rights = $Right->getRightSets($aro['Aro']['id']);
			} else if ($mode == 'desktop') {
				$right = $Right->find('first', array('conditions' => array('Right.model' => 'Desktop', 'Right.foreign_key' => $aro['Aro']['foreign_key'])));
				if (!empty($right)) {
					$desktop = $Desktop->find('first', array('conditions' => array('Desktop.id' => $aro['Aro']['foreign_key'])));
					if (!empty($desktop)) {
						$baseAro = $this->Aro->find('first', array('conditions' => array('Aro.model' => 'Group', 'Aro.foreign_key' => $desktop['Desktop']['group_id'])));
						if (!empty($baseAro)) {
							$rightSets = $Right->getRightSetsFromGroupName($baseAro['Aro']['alias']);
						}
					}
					$rights = $Right->getRightSets($aro['Aro']['id']);
				}
			}

			$this->out($aro['Aro']['alias']);

			if ($mode == 'desktop' && empty($rights)) {
				$this->out('  Save lightGrid -> <info>Nothing to do.</info>');
			} else if (empty($rightSets)) {
				$this->out('  Save lightGrid -> <error>right sets not found.</error>');
			} else {
				$valid = array();
				$rights = $Right->saveRightSets($aro['Aro']['id'], array('rightSets' => $rightSets));
				$this->out('  Save lightGrid -> ' . ($rights ? '<success>ok</success>' : '<error>error</error>'));

				foreach ($rights as $right => $val) {
					if ($val == 1) {
						$validAllow = $this->Acl->allow($aro['Aro']['alias'], $right);
						$this->out('    allow ' . $right . ' : ' . ($validAllow ? '<success>ok</success>' : '<error>error</error>'));
						$valid[] = $validAllow;
					} else {
						$validDeny = $this->Acl->deny($aro['Aro']['alias'], $right);
						$this->out('    allow ' . $right . ' : ' . ($validDeny ? '<success>ok</success>' : '<error>error</error>'));
						$valid[] = $validDeny;
					}
				}
				$this->out('  Apply rights -> ' . (!in_array(false, $valid, true) ? '<success>ok</success>' : '<error>error</error>'));
			}
			$this->out();
		}
	}

	/**
	 *
	 */
	protected function _delegation() {
		$date_act = date('Y-m-d');
		$plandelegation = ClassRegistry::init('Plandelegation');
		$desktopsUser = ClassRegistry::init('DesktopsUser');

		$conns = ConnectionManager::enumConnectionObjects();
		foreach (array_keys($conns) as $conn) {
			if ($conn != 'default') {

				$plandelegation->useDbConfig = $conn;
				$desktopsUser->useDbConfig = $conn;
				$desktopsUser->Desktop->useDbConfig = $conn;

				$continue = false;
				try {
					$delegs = $plandelegation->find('all');
					$continue = true;
					$this->out('<info>Base ' . $conn . ' :</info> traitement des délégations planifiées.');
				} catch (Exception $exc) {
					if (Configure::read('debug') > 0) {
						$this->out($exc->getTraceAsString());
					}
					$this->out('<info>Base ' . $conn . ' :</info> <error>les délégations planifiées ne sont pas gérées.</error>');
				}


				if ($continue) {
					if (empty($delegs)) {
						$this->out('Aucune délégation.');
					} else {
						$countDeleg = 1;
						foreach ($delegs as $deleg) {
							$msgDeleg = 'traitement de la délégation ' . $countDeleg . ' : ';

							$date_start = $deleg['Plandelegation']['date_start'];
							$date_end = $deleg['Plandelegation']['date_end'];

							$tmp = explode(' ', $date_start);
							$start = $tmp[0];
							$tmp = explode(' ', $date_end);
							$end = $tmp[0];

							if ($start == $date_act) {
								$data = $desktopsUser->create();
								foreach ($deleg['Plandelegation'] as $key => $val) {
									if (!in_array($key, array('created', 'modified', 'date_start', 'date_end', 'id'))) {
										$data['DesktopsUser'][$key] = $val;
									}
								}
								$data['DesktopsUser']['delegation'] = true;

								$success = false;
								try {
									$desktopsUser->save($data);
									$success = true;
								} catch (Exception $exc) {
									if (Configure::read('debug') > 0) {
										$this->out($exc->getTraceAsString());
									}
								}

								$this->out($msgDeleg . ($success ? '<success>la délégation est établie.</success>' : '<error>impossible d\'établir la délégation.</error>'));
							} else if ($end == $date_act) {
								$qd = array(
									'conditions' => array(
										'DesktopsUser.user_id' => $deleg['Plandelegation']['user_id'],
										'DesktopsUser.desktop_id' => $deleg['Plandelegation']['desktop_id'],
										'DesktopsUser.delegation' => true
									)
								);
								$listDeleg = $desktopsUser->find('all', $qd);

								$success = false;
								try {
									foreach ($listDeleg as $item) {
										$desktopsUser->delete($item['DesktopsUser']['id']);
									}
									$plandelegation->delete($deleg['Plandelegation']['id']);
									$success = true;
								} catch (Exception $exc) {
									if (Configure::read('debug') > 0) {
										$this->out($exc->getTraceAsString());
									}
								}

								$this->out($msgDeleg . ($success ? '<success>la délégation est annulée.</success>' : '<error>impossible d\'annuler la délégation.</error>'));
							} else {
								$this->out($msgDeleg . '<info>la délégation n\'est pas traitée aujourd\'hui.</info>');
							}
							$countDeleg++;
						}
					}
				}
			}
			$this->out();
		}
	}

	/**
	 *  Check if aro is not set
	 *
	 * @param array $aro Aro to check
	 * @return boolean
	 * */
	private function _aro_not_set($aro = null) {
		$retval = false;
		if ($aro != null) {
			if (!empty($aro['Aro']['model']) && !empty($aro['Aro']['foreign_key'])) {
				$conditions = array(
					'model' => $aro['Aro']['model'],
					'foreign_key' => $aro['Aro']['foreign_key']
				);
				if (!empty($aro['Aro']['alias'])) {
					$conditions['alias'] = $aro['Aro']['alias'];
				}
				if (!empty($aro['Aro']['parent_id'])) {
					$conditions['parent_id'] = $aro['Aro']['parent_id'];
				}
				$exists = $this->Aro->find('list', array('conditions' => $conditions));
				if (count($exists) == 0) {
					$retval = true;
				}
			}
		}
		return $retval;
	}

	/**
	 *  Create an Aro
	 *
	 * @param array $aro Aro to create
	 * @return boolean
	 * */
	private function _aro_create($aro = null) {
		$retval = false;
		if ($aro != null) {
			if ($this->_aro_not_set($aro)) {
				$this->Aro->create($aro);
				if ($this->Aro->save()) {
					$this->out($aro['Aro']['model'] . ' ' . $aro['Aro']['alias'] . ' -> <success>ok</success>');
					$retval = true;
				} else {
					$this->out($aro['Aro']['model'] . ' ' . $aro['Aro']['alias'] . ' -> <error>error</error>');
				}
			}
		}
		return $retval;
	}

	/**
	 *  Synchronisation between Aro table and Group table (here: Role)
	 *
	 * @return boolean
	 * */
	private function _aro_sync_group() {
		$retval = true;
		$groups = $this->Group->find('list');
		foreach ($groups as $key => $group) {
			$aro = array('Aro' => array());
			$aro['Aro']['model'] = 'Group';
			$aro['Aro']['foreign_key'] = $key;
			$aro['Aro']['alias'] = $group;
			$this->_aro_create($aro);
		}
		return $retval;
	}

	/**
	 *  Synchronisation between Aro table and User table (here: Utilisateur)
	 *
	 * @return boolean
	 * */
	private function _aro_sync_desktop() {
		$retval = true;
		$desktops = $this->Desktop->find('all');
		foreach ($desktops as $desktop) {
			$aro = array('Aro' => array());
			$aro['Aro']['model'] = 'Desktop';
			$aro['Aro']['foreign_key'] = $desktop['Desktop']['id'];
			$aro['Aro']['alias'] = $desktop['Desktop']['name'];
			$aroParent = $this->Aro->find('first', array('conditions' => array('model' => 'Group', 'foreign_key' => $desktop['Desktop']['group_id'])));
			$aro['Aro']['parent_id'] = $aroParent['Aro']['id'];
			$this->_aro_create($aro);
		}
		return $retval;
	}

	/**
	 *
	 * @return optionParser
	 */
	public function getOptionParser() {
		$actions = array(
			'repair_addressbook_slugs' => 'Recalcule les informations pour la recherche d\'homonymie des contacts.',
			'list_collectivites' => 'Récupère la liste des collectivités, leur identifiant et leur statut.',
			'rights' => 'Génération complète des droits fonction.',
			'clear_cache' => 'Nettoyage du cache CakePHP.',
			'hash_pass' => 'Renvoi la valeur hashée d\'une chaine passée en argument.',
			'clean_tree_tables' => 'Nettoie les tables utilisant le behavior Tree.',
			'clear_logs' => 'Vide les fichiers de logs.',
			'clean' => 'Opérations de maintenance (clean_tree_tables, clear_logs, clear_cache).',
			'reset' => 'Opérations de remises à zéro (clean, rights, data).',
			'install' => "Importation de la structure de la base de données d'administration et création du superadmin.",
			'backup' => "Sauvegarde de la base de données définie par la connexion, toutes les bases, l'application, ou toutes les bases et l'application dans un fichier .tar.bz2.\n" .
			" -m ou --backup_mode\n" .
			"  full (par défaut) : sauvegarde toute l'application et toutes les base de données de l'application\n" .
			"  db : sauvegarde toute la base de données définie par l'option -c ou --connection\n" .
			"  all_db : sauvegarde toutes les base de données de l'application\n" .
			"  app : sauvegarde les fichiers de l'application\n" .
			" -d ou --backup_directory\n" .
			"      répertoire de destination du fichier de sauvegarde (le répertoire courant par défaut)",
			"fix_fs_rights" => "Définis les droits sur les fichiers de l'application et de CakePHP\n" .
			" -o ou --owner : définis le propriétaire (par défaut : www-data)\n" .
			" -g ou --group : définis le groupe (par défaut : www-data)",
			"delegation" => "Vérifie et applique les planification de délégations de rôle",
			"set_light_rights" => "Génération des droits simplifiés"
		);
		ksort($actions);
		$optionParser = parent::getOptionParser();
		$optionParser->description(__("Outils d'administration"));
		foreach ($actions as $action => $description) {
			$optionParser->addSubcommand($action, array('help' => $description));
		}
		$optionParser->addOption('connection', array(
			'short' => 'c',
			'help' => 'connection',
			'default' => 'default',
			'choices' => array_keys(ConnectionManager::enumConnectionObjects())
		));
		$optionParser->addOption('backup_mode', array(
			'short' => 'm',
			'help' => 'backup_mode',
			'default' => 'full',
			'choices' => array('full', 'db', 'all_db', 'app')
		));
		$optionParser->addOption('backup_directory', array(
			'short' => 'd',
			'help' => 'backup_directory'
		));
		$optionParser->addOption('owner', array(
			'short' => 'o',
			'help' => 'propriétaires des fichiers suivant le schéma UNIX'
		));
		$optionParser->addOption('group', array(
			'short' => 'g',
			'help' => 'groupe des fichiers suivant le schéma UNIX'
		));
		return $optionParser;
	}

}
