<?php

/**
 * Carnet d'adresse
 *
 *
 * Addressbooks controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 *
 * @package		app
 * @subpackage		Controller
 */
class AddressbooksController extends AppController {

	/**
	 * Controller uses
	 *
	 * @var array
	 * @access public
	 */
	public $uses = array('Addressbook');

	/**
	 * Gestion des carnets d'adresses et contacts
	 *
	 * @logical-group Carnet d'adresse
	 * @user-profile Admin
	 *
	 *
	 * @access public
	 * @return void
	 */
	public function index() {
		$this->set('ariane', array(
                    '<a href="/environnement/index/0/admin">'.__d('menu', 'Administration', true).'</a>',
                    __d('menu', 'gestionAddressbooks', true)
                ));
	}

	/**
	 * Récupération de la liste des carnets d'adresse
	 *
	 * @logical-group Carnet d'adresse
	 * @user-profile Admin
	 *
	 *
	 * @access public
	 * @return void
	 */
	public function get_addressbooks() {
		$this->Addressbook->recursive = -1;
		$querydata = array(
			'conditions' => array(
				'Addressbook.active' => true
			),
			'order' => array(
				'Addressbook.name'
			)
		);
		$addressbooks = $this->Addressbook->find('all', $querydata);
		$this->set('addressbooks', $addressbooks);
	}

	/**
	 * Ajout de carnet d'adresse
	 *
	 * @logical-group Carnet d'adresse
	 * @user-profile Admin
	 *
	 *
	 * @access public
	 * @return void
	 */
	public function add() {
		$this->Addressbook->recursive = -1;
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
			$this->Addressbook->create($this->request->data);
			if ($this->Addressbook->save()) {
				$this->Jsonmsg->valid();
			}
			$this->Jsonmsg->send();
		}
	}

	/**
	 * Edition de carnet d'adresse
	 *
	 * @logical-group Carnet d'adresse
	 * @user-profile Admin
	 *
	 * @access public
	 * @param string $id
	 * @return void
	 */
	public function edit($id = null) {
		$this->Addressbook->recursive = -1;
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
			$this->Addressbook->create($this->request->data);
			if ($this->Addressbook->save()) {
				$this->Jsonmsg->valid();
			}
			$this->Jsonmsg->send();
		} else if ($id != null) {
			$this->request->data = $this->Addressbook->find('first', array('conditions' => array('Addressbook.id' => $id)));
		}
	}

	/**
	 * Suppression de carnet d'adresse (soft delete)
	 *
	 * @logical-group Carnet d'adresse
	 * @user-profile Admin
	 *
	 * @access public
	 * @param string $id
	 * @return void
	 */
	public function delete($id = null, $real = false) {
		if ($id != null) {
			if ($real) {
				$this->Jsonmsg->init(__d('default', 'delete.error'));
				if ($this->Addressbook->delete($id, true)) {
					$this->Jsonmsg->valid(__d('default', 'delete.ok'));
				}
			} else {
				$this->Jsonmsg->init();
				$this->Addressbook->id = $id;
//        $addressbook = $this->Addressbook->find('first', array('conditions' => array('Addressbook.id' => $id)));
//        $addressbook['Addressbook']['active'] = false;
//        $this->Addressbook->create($addressbook);
//        if ($this->Addressbook->save()) {
				if ($this->Addressbook->saveField('active', false)) {
					$this->Jsonmsg->valid();
				}
			}
			$this->Jsonmsg->send();
		}
	}

}
