<?php

/**
 * Affaires
 *
 * Affaires controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class AffairesController extends AppController {

	/**
	 * Controller name
	 *
	 * @var string
	 * @access public
	 */
	public $name = 'Affaires';

	/**
	 * Controller uses
	 *
	 * @var array
	 * @access public
	 */
	public $uses = array('Affaire');

	/**
	 * Ajout d'une affaire (avec incrémentation du compteur)
	 *
	 * @logical-group Affaires
	 * @user-profile Admin
	 * @user-profile User
	 *
	 * @access public
	 * @return void
	 */
	public function add() {
		if (!empty($this->request->data)) {
			$this->Affaire->save($this->request->data);
		} else {
			$this->loadModel('Compteur');
			$CompteurAffaire = $this->Compteur->find(
				'first',
				array(
					'fields' => array( 'Compteur.id', 'Compteur.nom' ),
					'conditions' => array(
						'Compteur.nom' => 'compteur_affaire'
					),
					'recursive' => -1
				)
			);
			$idAffaire = $CompteurAffaire['Compteur']['id'];
			$this->set('newAffaireName', $this->Compteur->genereCompteur($idAffaire));
		}
		$this->set('dossiers', $this->Affaire->Dossier->find('list'));
		$affaires = $this->Affaire->find('list');
	}

	/**
	 * Edition d'une affaire
	 *
	 * @logical-group Affaires
	 * @user-profile Admin
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $id The id of the Affaire
	 * @return void
	 */
	public function edit($id = null) {
		if ($id != null) {
			if (!empty($this->request->data)) {
				$this->Affaire->save($this->request->data);
			} else {
				$this->request->data = $this->Affaire->find(
                    'first',
                    array(
                        'conditions' => array(
                            'Affaire.id' => $id
                        ),
                        'contain' => false
                    )
                );
			}
			$this->set('dossiers', $this->Affaire->Dossier->find('list'));
		}
	}

	/**
	 * Suppression d'une affaire
	 *
	 * @logical-group Affaires
	 * @user-profile Admin
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $id The id of the Affaire
	 * @return void
	 */
	public function delete($id = null) {
		if ($id != null) {
			$this->Affaire->delete($id);
		}
		$this->autoRender = false;
	}

}

?>