<?php

/**
 * Accusés de réception
 *
 * Ars controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class ArsController extends AppController {

	/**
	 * Controller name
	 *
	 * @var string
	 * @access public
	 */
	public $name = 'Ars';

	/**
	 *
	 * Correspondance type MIME / extensions de fichier
	 *
	 * @access private
	 * @var array
	 */
	private $_allowed = array(
		'audio' => array(
			'audio/x-wav' => 'wav',
			'audio/mpeg' => 'mp3',
			'application/ogg' => 'ogg',
			'audio/ogg' => 'ogg'
		),
		'video' => array(
			'video/x-msvideo' => 'avi',
			'video/mpeg' => 'mpg',
			'video/mp4' => 'mp4',
			'application/ogg' => 'ogg',
			'video/ogg' => 'ogg'
		),
		'ott' => array(
			'application/vnd.oasis.opendocument.text' => 'odt',
			'application/vnd.oasis.opendocument.text-template' => 'ott'
		)
	);

	/**
	 *
	 * Controller's Components
	 *
	 * @access public
	 * @var array
	 */
	public $components = array('Linkeddocs');

	/**
	 * Sélection d'un accusé de réception (marqué comme envoyé / utilisé)
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @logical-group Accusés de réception
	 * @user-profile User
	 *
	 * @access public
	 * @return void
	 */
	public function select() {
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
			$this->Ar->create($this->request->data);
			$saved = $this->Ar->save();
			if (!empty($saved)) {
				$this->Jsonmsg->valid();
			}
			$this->Jsonmsg->send();
		}
	}

	/**
	 * Suppression d'un accusé de réception
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @logical-group Accusés de réception
	 * @user-profile User
	 *
	 * @access public
	 * @return void
	 */
	public function delete($id = null) {
		$this->Jsonmsg->init(__d('default', 'delete.error'));
		if ($this->Ar->delete($id)) {
			$this->Jsonmsg->valid(__d('default', 'delete.ok'));
		}
		$this->Jsonmsg->send();
	}

	/**
	 * Téléchargement d'un accusé de réception
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @logical-group Accusés de réception
	 * @user-profile User
	 *
	 * @access public
	 * @throws NotFoundException
	 * @return void
	 */
	public function download($id) {
		$ar = $this->Ar->find('first', array('conditions' => array('Ar.id' => $id), 'recursive' => -1));
		if (empty($ar)) {
			throw new NotFoundException();
		}


		$this->autoRender = false;
		Configure::write('debug', 0);

		header("Pragma: public");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");

		header("Content-type: application/octet-stream");
		header("Content-Disposition: attachment; filename=\"" . Inflector::slug($ar['Ar']["name"]) . '.' . $this->_allowed[$ar['Ar']['format']][$ar['Ar']['mime']] . "\"");
		header("Content-length: " . $ar['Ar']['size']);

		print $ar['Ar']['content'];
		exit();
	}

	/**
	 * Récupération de la pré-visualisation d'un accusé de réception (génération si nécessaire)
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @logical-group Accusés de réception
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $arId
	 * @throws NotFoundException
	 * @return void
	 */
	public function getPreview($arId) {
		$ar = $this->Ar->find('first', array('conditions' => array('Ar.id' => $arId), 'recursive' => -1));
		if (empty($ar)) {
			throw new NotFoundException();
		}

		$preview = __d('document', 'Document.preview.error');
		if (empty($ar['Ar']['preview'])) {
			$preview = $this->Ar->generatePreview($arId);
		} else {
			$preview = $ar['Ar']['preview'];
		}

		$mime = $ar['Ar']['mime'];


		$tmpPreviewFile = '';
		$tmpPreviewFileBasename = '';
		$previewType = '';

		if ($mime == "application/vnd.oasis.opendocument.text") {
			$previewType = 'flexpaper';
			$tmpPreviewFileBasename = Inflector::slug($this->Session->read('Auth.User.username')) . "_" . Inflector::slug($ar['Ar']['name']) . "_preview.swf";
			$tmpPreviewFile = APP . DS . WEBROOT_DIR . DS . 'files' . DS . 'previews' . DS . $tmpPreviewFileBasename;
			//purge des fichiers de preview précédents
			$this->Linkeddocs->purge();
			file_put_contents($tmpPreviewFile, $preview);
		} else {
			foreach ($this->Ar->otherMimeTypes as $type => $mimes) {
				if (in_array($mime, array_keys($mimes))) {
					$previewType = $type;
					$tmpPreviewFileBasename = Inflector::slug($this->Session->read('Auth.User.username')) . "_" . Inflector::slug($ar['Ar']['name']) . "_preview." . $mimes[$mime];
					$tmpPreviewFile = APP . DS . WEBROOT_DIR . DS . 'files' . DS . 'previews' . DS . $tmpPreviewFileBasename;
					//purge des fichiers de preview précédents
					$this->Linkeddocs->purge();
					file_put_contents($tmpPreviewFile, $preview);
				}
			}
		}

		$this->set('previewType', $previewType);
		$this->set('mime', $mime);
		$this->set('tmpPreviewFileBasename', $tmpPreviewFileBasename);
	}

}
