<?php

/**
 * Administration
 *
 * Checks controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Arnaud AUZOLAT
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 *
 * @package		app
 * @subpackage		Controller
 */
class ChecksController extends AppController {

	/**
	 *
	 * Controller name
	 *
	 * @var string
	 * @access public
	 */
	public $name = 'Checks';

	/**
	 * Vérifications des pré-requis
	 *
	 * @logical-group Administration
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function index() {
		$this->set('ariane', array(
                    '<a href="/environnement">'.__d('menu', 'Administration').'</a>', 
                    __d('menu', 'checkInstall')
                ));

		$verif = array();
		//recupération de la version de cakephp
		$cakeVersion = Configure::version();
		$verif['cake_version'] = $cakeVersion;
		$verif['php_version'] = phpversion();
		$verif['apache_version'] = apache_get_version();
		$this->loadModel('User');
		$verif['postgresql_version'] = $this->_pgVersion($this->User);

		//verification de la présence du patch postgres_bytea pour cake2.0.x (<= 2.0.5)
		$verif['patch_cake2_pg_bytea'] = array('chk' => false, 'patch' => true);
		$tmp_version = explode('.', $cakeVersion);
		$version_patch_unmatch = false;
		if ($tmp_version[0] == "2" && $tmp_version[1] == "0" && (int) $tmp_version[2] < 6) {
			$verif['patch_cake2_pg_bytea']['chk'] = true;
			$file = CAKE . 'Model/Datasource/Database/Postgres.php';
			if (is_file($file)) {
				$handle = fopen($file, "r");
				if ($handle) {
					$content = array();
					$read = false;
					while (($buffer = fgets($handle, 4096)) !== false) {
						if (!$read && preg_match('#function fetchResult#', $buffer)) {
							$read = true;
						}
						if ($read) {
							$content[] = $buffer;
						}
						if ($read && preg_match('#function#', $buffer) && !preg_match('#function fetchResult#', $buffer)) {
							$read = false;
						}
					}
					fclose($handle);
					foreach ($content as $row) {
						if (preg_match('#\$resultRow\[\$table\]\[\$column\] \= is_null\(\$row\[\$index\]\) \? null : stream_get_contents\(\$row\[\$index\]\)\;#', $row)) {
							$verif['patch_cake2_pg_bytea']['patch'] = true;
						}
					}
				}
			}
		} else {
			$version_patch_unmatch = true;
		}
		$this->set('version_patch_unmatch', $version_patch_unmatch);

		//TODO: prevoir un test de génération de document temporaire
		//verification des valeurs dans core.php pour dataacl
		$verif['dataacl'] = array('core' => false);
		if (
				Configure::read('DataAcl.classname') != '' &&
				Configure::read('DataAcl.database') != '' &&
				Configure::read('DataAcl.defaultAccessValue.Daro') != '' &&
				Configure::read('DataAcl.defaultAccessValue.Daco')
		) {
			$verif['dataacl']['core'] = true;
		}


		//Configuration webgfc
		$verif['webgfc'] = array('inc' => false, 'wsdl' => false);
		if (is_file(APP . "Config/webgfc.inc") && is_readable(APP . "Config/webgfc.inc")) {
			$verif['webgfc']['inc'] = true;
		}
		if (is_file(APP . "webroot/files/wsdl/webgfc.wsdl") && is_readable(APP . "webroot/files/wsdl/webgfc.wsdl")) {
			$verif['webgfc']['wsdl'] = true;
		}


		//verification de la presence de l executable pdf2swf
		$verif['swftools'] = array('pdf2swf' => false);
		$result = exec('whereis pdf2swf');
		if (preg_match('#pdf2swf:\s\/(.*\/pdf2swf)#', $result)) {
			$verif['swftools']['pdf2swf'] = true;
		}

		//verification gedooo
		$verif['gedooo'] = array('config' => false, 'up' => false);
		if (
				Configure::read('PROTOCOLE_DL') != '' &&
				Configure::read('USE_GEDOOO') === true &&
				Configure::read('GEDOOO_WSDL') != ''
		) {
			$verif['gedooo']['config'] = true;
		}

		try {
			$client = @new SoapClient(Configure::read('GEDOOO_WSDL'), array('exceptions' => 1));
			$verif['gedooo']['up'] = true;
		} catch (Exception $e) {
			$verif['gedooo']['up'] = false;
		}


		//verification cloudooo
		$verif['cloudooo'] = array('config' => false, 'up' => false);
		if (
				Configure::read('CLOUDOOO_HOST') != '' &&
				Configure::read('CLOUDOOO_PORT') != ''
		) {
			$verif['cloudooo']['config'] = true;
		}

		$cloudoooResult = $this->_socket(Configure::read('CLOUDOOO_HOST'), Configure::read('CLOUDOOO_PORT'));
		$verif['cloudooo']['up'] = $cloudoooResult['success'];


		$this->set('verif', $verif);
	}

	/**
	 *
	 * Vérification de la disponibilité d'un serveur via les sockets
	 *
	 * @access private
	 * @param string $hostname
	 * @param integer $port
	 * @param string $message
	 * @return array
	 */
	private function _socket($hostname, $port, $message = "La machine distante n' est pas accessible (%s)") {
		$timeout = 10;
		Set_Time_Limit(0); //Time for script to run .. not sure how it works with 0 but you need it
// Ignore_User_Abort(True); //this will force the script running at the end
		$handle = @fsockopen($hostname, $port, $errno, $errstr, $timeout);
		$result = array(
			'success' => !empty($handle)
		);
		if (!$result['success']) {
			$result['message'] = sprintf($message, "{$errno}: {$errstr}");
		} else {
			fclose($handle);
		}
		return $result;
	}

	/**
	 *
	 * Récupération de la version de Postgres
	 *
	 * @access private
	 * @param Model $model
	 * @param boolean $full
	 * @return string
	 */
	private function _pgVersion(&$model, $full = false) {
		$psqlVersion = $model->getDataSource($model->useDbConfig)->query('SELECT version();');
		$psqlVersion = Set::classicExtract($psqlVersion, '0.0.version');
		if (!$full) {
			$psqlVersion = preg_replace('/.*PostgreSQL ([^ ]+) .*$/', '\1', $psqlVersion);
		}
		return $psqlVersion;
	}

}

?>
