<?php

App::uses('SimpleCommentAppController', 'SimpleComment.Controller');

/**
 * SimpleComment Plugin
 * SimpleComment App controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 *
 * @package             SimpleComment
 * @subpackage          SimpleComment.Controller
 */
class CommentsController extends AppController {

	/**
	 * Controller name
	 *
	 * @var array
	 */
	public $name = 'Comments';

	/**
	 * Controller uses
	 *
	 * @var array
	 */
	public $uses = array('SimpleComment.Comment');

	/**
	 * Controller uses
	 *
	 * @var array
	 */
	public $helpers = array('Html', 'Form');

	/**
	 *
	 * @param type $flux_id
	 * @return type
	 */
	private function _getAllowedReaders($flux_id) {
		//FIXME : revoir l'utilisation du plugin
		$this->loadModel('Courrier');
		$Reader = $this->Courrier->Comment->Reader;
		$qd = array(
			'fields' => array('id', 'name'),
			'recursive' => -1,
			'joins' => array(
				$Reader->join($Reader->Composition->name),
				$Reader->Composition->join($Reader->Composition->Etape->name),
				$Reader->Composition->Etape->join($Reader->Composition->Etape->Circuit->name),
				$Reader->Composition->Etape->Circuit->join($Reader->Composition->Etape->Circuit->Soustype->name),
				$Reader->Composition->Etape->Circuit->Soustype->join($this->Courrier->name)
			),
			'conditions' => array(
				$this->Courrier->name . '.id' => $flux_id
			),
			'contain' => false
		);

		return $this->Comment->Reader->find('list', $qd);
	}

	/**
	 *
	 * @param type $flux_id
	 */
	public function add($flux_id, $canAddPrivateComment = false) {
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init(__d('default', 'save.error'));
            
            
            
			$this->Comment->create();
			$this->request->data['Comment']['owner_id'] = $this->Session->read('Auth.User.Courrier.Desktop.id');
			$this->request->data['Comment']['target_id'] = $flux_id;
			$success = $this->Comment->save($this->request->data);

			if (!empty($success)) {
				$this->Jsonmsg->valid(__d('default', 'save.ok'));
			}
			$this->Jsonmsg->send();
		} else {
//            $Courrier = ClassRegistry::init('Courrier');
            $this->loadModel('Courrier');
            $flux = $this->Courrier->find(
                'first',
                array(
                    'conditions' => array(
                        'Courrier.id' => $flux_id
                    ),
                    'recursive' => -1
                )
            );
            $isSoustype = !empty($flux['Courrier']['soustype_id']) ? true : false;         

			$this->set('canAddPrivateComment', $canAddPrivateComment);
			$this->set('isSoustype', $isSoustype);
			$this->set('readers', $this->_getAllowedReaders($flux_id));
		}
	}

	/**
	 *
	 * @param type $id
	 * @param type $flux_id
	 * @throws BadMethodCallException
	 */
	public function edit($flux_id, $canAddPrivateComment = false, $id) {
		if (empty($id) || empty($flux_id)) {
			throw new BadMethodCallException();
		}

		if (!empty($this->request->data)) {
			$this->Jsonmsg->init(__d('default', 'save.error'));

			$this->Comment->create();
			$this->request->data['Comment']['id'] = $id;

			$success = $this->Comment->save($this->request->data);

			if (!empty($success)) {
				$this->Jsonmsg->valid(__d('default', 'save.ok'));
			}
			$this->Jsonmsg->send();
		} else {
		
            $this->loadModel('Courrier');
            $flux = $this->Courrier->find(
                'first',
                array(
                    'conditions' => array(
                        'Courrier.id' => $flux_id
                    ),
                    'recursive' => -1
                )
            );
            $isSoustype = !empty($flux['Courrier']['soustype_id']) ? true : false; 
            $this->set('isSoustype', $isSoustype);
            
			$this->request->data = $this->Comment->find('first', array('conditions' => array('Comment.id' => $id), 'contain' => array('Reader')));
			$this->set('canAddPrivateComment', $canAddPrivateComment);
			$this->set('readers', $this->_getAllowedReaders($flux_id));
		}
	}

	/**
	 *
	 * @param type $id
	 */
	public function delete($id) {
		$this->Jsonmsg->init(__d('default', 'delete.error'));
		if ($this->Comment->delete($id)) {
			$this->Jsonmsg->valid(__d('default', 'delete.ok'));
		}
		$this->Jsonmsg->send();
	}

}

?>