<?php

App::uses('File', 'Utility');

class ConnecteursController extends AppController {

	const NS_CMIS_RA = "http://docs.oasis-open.org/ns/cmis/restatom/200908/";    
	const NS_CMIS  = "http://docs.oasis-open.org/ns/cmis/core/200908/";
	/**
	 * Controller name
	 *
	 * @var string
	 * @access public
	 */
	public $name = 'Connecteurs';
    
    public $components = array( 'Iparapheur', 'Cmis');
    
    
	/**
	 * Gestion des connecteurs interface graphique)
	 *
	 * @logical-group Connecteurs
	 * @user-profil Admin
	 *
	 * @access public
	 * @return void
	 */
	public function index() {
		$this->set('ariane', array(
                    '<a href="/environnement/index/0/admin">'.__d('menu', 'Administration', true).'</a>', 
                    __d('menu', 'gestionConnecteurs', true)
                ));
	}

    	/**
	 * Récupération de la liste des connecteurs
	 *
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function getConnecteurs() {
            
        $connecteurs_tmp = $this->Connecteur->find("all", array('order' => 'Connecteur.name')); 
        $connecteurs = array();
		foreach ($connecteurs_tmp as $item) {
			if( $item['Connecteur']['name'] == 'GED (CMIS)' ) {
				$item['url'] = Configure::read('BaseUrl') . '/connecteurs/getGed';
				$isGed = true;
			}
			else {
				$item['url'] = Configure::read('BaseUrl') . '/connecteurs/getCircuitsParapheur';
				$isGed = false;
			}
			
			$item['right_edit'] = true;
			$item['right_check'] = true;
			$connecteurs[] = $item;
		}
		
		$this->set('isGed', $isGed);
		$this->set('connecteurs', $connecteurs);
	}
    
    
    public function edit($id) {
        switch ($id) {
            case Configure::read( 'Parapheur.id' ):
                // Configuration signature
                $protocoles = array('PASTELL' => 'Pastell', 'IPARAPHEUR' => 'i-Parapheur');
                if (!Configure::read('USE_PASTELL')){
                    unset($protocoles['PASTELL']);
                }
                $this->set('protocoles', $protocoles);
                $this->render('signature');
                break;
			case Configure::read( 'Ged.id' ):
                // Connecteur CMIS/GED
                $this->render('cmis');
                break;
            default:
                $this->Session->setFlash('Ce connecteur n\'est pas valide', 'growl', array('type' => 'erreur'));
                return $this->redirect(array('action' => 'index'));
        }
        
    }

    public function _replaceValue($content, $param, $new_value) {
        if (is_bool(Configure::read($param)))
            $valeur = Configure::read($param) === true ? 'true' : 'false';
        else
            $valeur = Configure::read($param);

        $host_b = "Configure::write('$param', '" . $valeur . "');";
        $host_a = "Configure::write('$param', '$new_value');";
        $return = str_replace($host_b, $host_a, $content, $count);
        if ($count === 0) {
            $host_b = "Configure::write('$param', " . $valeur . ");";
            $host_a = "Configure::write('$param', $new_value);";

            $return = str_replace($host_b, $host_a, $content, $count);
        }

        return $return;
    }

    public function makeconf($type) {
        $this->Jsonmsg->init();
        $file = new File(APP . 'Config' . DS . 'webgfc.inc', true);
        $content = $file->read();
        switch ($type) {
            case 'signature' :
                $protocol = strtoupper($this->data['Connecteur']['signature_protocol']);
                $content = $this->_replaceValue($content, 'PARAPHEUR', $protocol);
                $content = $this->_replaceValue($content, 'PASTELL_PARAPHEUR_TYPE', $this->data['Connecteur']['pastell_parapheur_type']);
                $content = $this->_replaceValue($content, 'USE_PARAPHEUR', $this->data['Connecteur']['use_signature']);
                $content = $this->_replaceValue($content, 'USE_IPARAPHEUR', $this->data['Connecteur']['use_signature']);
                $content = $this->_replaceValue($content, 'IPARAPHEUR_HOST', $this->data['Connecteur']['host']);
                $content = $this->_replaceValue($content, 'IPARAPHEUR_LOGIN', $this->data['Connecteur']['login']);
                $content = $this->_replaceValue($content, 'IPARAPHEUR_PWD', $this->data['Connecteur']['pwd']);
                $content = $this->_replaceValue($content, 'IPARAPHEUR_TYPE', $this->data['Connecteur']['type']);
                if ($protocol == 'IPARAPHEUR') {
                    $content = $this->_replaceValue($content, 'IPARAPHEUR_CERTPWD', $this->data['Connecteur']['certpwd']);
                    if( !empty($this->data['Connecteur']['clientcert'])) {
                        if (file_exists($this->data['Connecteur']['clientcert']['tmp_name'])) {
                            $certs = array();
                            $path_dir_parapheur = APP . DS . 'Config' . DS . 'cert_parapheur' . DS;
                            $pkcs12 = file_get_contents($this->data['Connecteur']['clientcert']['tmp_name']);
                            if (openssl_pkcs12_read($pkcs12, $certs, $this->data['Connecteur']['certpwd'])) {
                                file_put_contents($path_dir_parapheur . 'cert.pem', $certs['pkey'] . $certs['cert']);
                                file_put_contents($path_dir_parapheur . 'ac.pem', $certs['extracerts'][0]);
                            } else
                                $this->Jsonmsg->error('Le mot de passe du certificat est erroné');
                        }
                    }
                }
                break;
            case 'cmis' :
                $content = $this->_replaceValue($content, 'USE_GED', $this->data['Connecteur']['use_ged']);
                $content = $this->_replaceValue($content, 'CMIS_HOST', $this->data['Connecteur']['ged_url']);
                $content = $this->_replaceValue($content, 'CMIS_LOGIN', $this->data['Connecteur']['ged_login']);
                $content = $this->_replaceValue($content, 'CMIS_PWD', $this->data['Connecteur']['ged_passwd']);
                $content = $this->_replaceValue($content, 'CMIS_REPO', $this->data['Connecteur']['ged_repo']);
                $content = $this->_replaceValue($content, 'GED_XML_VERSION', $this->data['Connecteur']['ged_xml_version']);
                break;
            case 'debug' :
                $content = $this->_replaceValue($content, 'debug', $this->data['Connecteur']['debug']);
                break;
            case 'all' :
                $content = $this->data['Connecteur']['all'];
                break;
            default :
                $this->Jsonmsg->error('Ce connecteur n\'est pas valide');
                return $this->redirect(array('action' => 'index'));
        }
        if (!$file->writable()) {
            $this->Jsonmsg->error('Impossible de modifier le fichier de configuration, veuillez donner les droits sur fichier webgfc.inc');
        } else {
            //TODO : php_check_syntax
            $success = $file->open('w+');
            $success &= $file->append($content);
            $success &= $file->close();
            if ($success){
                $this->Jsonmsg->valid('La configuration du module &quot;' . $type . '&quot; a été enregistrée');
            }
            else {
                $this->Jsonmsg->error('Un problème est survenu lors de la modification du fichier de configuration webgfc.inc');
            }
        }
        $this->Jsonmsg->send();
// debug($this->request->data);
// die();
        return $this->redirect(array('action' => 'index'));
    }
    
    
    
    public function getCircuitsParapheur() {
        $this->Jsonmsg->init();
        $time_start = microtime(true);
        $circuits = '';
        if (!Configure::read('USE_PARAPHEUR')){
            $this->Session->setFlash('Utilisation du parapheur désactivé', 'growl', array( 'type' => 'info' ) );
            $this->Jsonmsg->send();
            return;
        }
        if (Configure::read('PARAPHEUR') == 'IPARAPHEUR') {
            $Parafwebservice = new IparapheurComponent();
            $circuits = $Parafwebservice->getListeSousTypesWebservice(Configure::read('IPARAPHEUR_TYPE'));
            $time_end = microtime(true);
            if (!empty($circuits["soustype"])) {
                $hostparapheur = "Parapheur contacté : ". Configure::read('IPARAPHEUR_HOST');
                $time = round($time_end - $time_start, 2);
                $nbCircuits = "Réponse du Parapheur : ".count($circuits['soustype'])." circuit(s) du i-parapheur récupéré(s) en $time secondes";
                
                $messageSuccess = $hostparapheur."<br />".$nbCircuits;
                $this->Jsonmsg->valid($messageSuccess);
            }
            else
                $this->Jsonmsg->error('La liste de circuit du i-parapheur est vide');
            $this->Jsonmsg->send();
        }
    }

    
    public function check( $id ) {
		switch ($id) {
            case Configure::read( 'Parapheur.id' ):
                // Configuration signature
                $this->Jsonmsg->init();
				$time_start = microtime(true);
				$circuits = '';
				if (!Configure::read('USE_PARAPHEUR')){
					$this->Session->setFlash('Utilisation du parapheur désactivé', 'growl', array( 'type' => 'info' ) );
					$this->Jsonmsg->send();
					return;
				}
				if (Configure::read('PARAPHEUR') == 'IPARAPHEUR') {
					$Parafwebservice = new IparapheurComponent();
					$circuits = $Parafwebservice->getListeSousTypesWebservice(Configure::read('IPARAPHEUR_TYPE'));
					$time_end = microtime(true);
					if (!empty($circuits["soustype"])) {
						$hostparapheur = "Parapheur contacté : ". Configure::read('IPARAPHEUR_HOST');
						$time = round($time_end - $time_start, 2);
						$nbCircuits = "Réponse du Parapheur : ".count($circuits['soustype'])." circuit(s) du i-parapheur récupéré(s) en $time secondes";
						
						$messageSuccess = $hostparapheur."<br />".$nbCircuits;
						$this->Jsonmsg->valid($messageSuccess);
					}
					else
						$this->Jsonmsg->error('La liste de circuit du i-parapheur est vide');
					$this->Jsonmsg->send();
				}
                break;
			case Configure::read( 'Ged.id' ):
                // Connecteur CMIS/GED
                $this->Jsonmsg->init();
				$time_start = microtime(true);

				// Le connecteur est-il actif
				if (!Configure::read('USE_GED')){
					$this->Jsonmsg->error('Utilisation de la GED désactivée', 'growl', array( 'type' => 'info' ) );
					$this->Jsonmsg->send();
					return;
				}
				
                require_once( ROOT.DS.APP_DIR.DS.'Lib'.DS.'cmis.php' );

                // Le couple login/mdp est-il le bon
                $configured = Cmis::configured();
				if (! $configured){
					$this->Jsonmsg->error("La connexion avec la GED a échoué : couple login/mot de passe erroné.", 'growl', array( 'type' => 'info' ) );
					$this->Jsonmsg->send();
					return;
				}
				

				// Existence ou non du répertoire renseigné
				$folderUsed = $this->testObject();
				if( $folderUsed === false ) {
					$this->Jsonmsg->error("La connexion avec la GED a échoué : le dossier ".Configure::read('CMIS_REPO')." n'a pas été trouvé</ul>", 'growl', array( 'type' => 'info' ) );
					$this->Jsonmsg->send();
					return;
				}
				

				// si tout ok
				$messageSuccess ="La connexion est réussie - web-GFC a récupéré les informations suivantes :<ul>" ;
				foreach($this->getFolderRetrieveInfo() as $repoInfo){
					$messageSuccess .= "<li> $repoInfo  : ".$folderUsed[$repoInfo] ."</li>";
				}
				$messageSuccess .="</ul>";
				
				
				$info = $this->getRepositoryInfo();
				
				$this->Jsonmsg->valid($messageSuccess);
				$this->Jsonmsg->send();

                break;
            default:
                $this->Session->setFlash('Ce connecteur n\'est pas valide', 'growl', array('type' => 'erreur'));
                return $this->redirect(array('action' => 'index'));
        }
    }
    
    
    public function getRepositoryInfo(){
		$xmldata = $this->get(Configure::read('CMIS_HOST'));
		if (! $xmldata){
			return false;
		}
		
		libxml_use_internal_errors(true);
		$xml = simplexml_load_string($xmldata);	
		
		if (! $xml){
			$errors = libxml_get_errors();
			$this->lastError = "Erreur XML : ".$errors[0]->message;
			return false;
		}
		
		$repInfo = $xml->workspace->children(self::NS_CMIS_RA)->repositoryInfo;
		
		$result = array();
		foreach($this->getRepositoryRetrieveInfo() as $infoName){
			$result[$infoName] = strval($repInfo->children(self::NS_CMIS)->$infoName);
		}
		
		$uriTemplate = $xml->workspace->children(self::NS_CMIS_RA)->uritemplate;
		
		foreach ($uriTemplate as $template){
			$type = strval($template->children(self::NS_CMIS_RA)->type);
			$result['template'][$type] = strval($template->children(self::NS_CMIS_RA)->template);
		}

		return $result;
	}
	
		
    private function get($url,$content = false){
    	$session = curl_init($url);

        curl_setopt($session, CURLOPT_HEADER, false);
        curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($session, CURLOPT_USERPWD, Configure::read('CMIS_LOGIN') . ":" . Configure::read('CMIS_PWD'));

        if ($content){
        	curl_setopt($session, CURLOPT_CUSTOMREQUEST, "POST");
        	curl_setopt($session, CURLOPT_POSTFIELDS, $content);
            curl_setopt($session, CURLOPT_HTTPHEADER, array ("Content-Type: application/atom+xml;type=entry"));
        } else {
			curl_setopt($session, CURLOPT_CUSTOMREQUEST, "GET");
        }
        $result =  curl_exec($session);
        $codeResponse = curl_getinfo($session, CURLINFO_HTTP_CODE);
                 
        if ( ! in_array($codeResponse, array('200','201')) ){
        	$this->lastError = curl_error($session);
        	if (! $this->lastError){
        		$this->lastError = "Erreur $codeResponse (la GED a retourné : $result)";
        	}
        	return false;
        }
        
      	return $result;
    }
    
    
    public function getRepositoryRetrieveInfo(){
		return array('repositoryId','repositoryName','repositoryDescription','vendorName','productName','productVersion','rootFolderId');
	}
	
	public function getRootFolder(){
		return Configure::read('CMIS_REPO');
	}
	
	
	public function testObject(){
		return $this->getObjectByPath(Configure::read('CMIS_REPO'));
	}
	
	public function getObjectByPath($path)
    {
    	
    	$repositoryInfo = $this->getRepositoryInfo();
    	
    	if (! $repositoryInfo){
    		return false;
    	}
    	
    	$url_template = $repositoryInfo['template']['objectbypath'];
    	
    	$url = str_replace("{path}", $path, $url_template);
    	$url = preg_replace("/{[a-zA-Z0-9_]+}/", "", $url);
    	
        $xmldata = $this->get($url);
        
        if (! $xmldata){
        	return false;
        }
        
        libxml_use_internal_errors(true);
        $xml = simplexml_load_string($xmldata);	
   		if (! $xml){
			$errors = libxml_get_errors();
			$this->lastError = "Erreur XML : ".$errors[0]->message;
			return false;
		}
        
    	
		$result = array('author' => strval($xml->author->name));
		foreach($this->getFolderRetrieveInfo() as $infoName){
			$result[$infoName] = strval($xml->$infoName);
		}
        
		foreach($xml->link as $link){
			if (! isset($result['link'][strval($link['rel'])])){
				$result['link'][strval($link['rel'])] = strval($link['href']);
			}
		}
		
        return $result;        
    }
    
    	
	public function getFolderRetrieveInfo(){
		return array('content','id','summary','title','published','updated');
	}
}
