<?php

/**
 * Fiches de contact
 *
 * Contactinfos controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class ContactinfosController extends AppController {

	/**
	 * Controller uses
	 *
	 * @var array
	 */
	public $uses = array('Contactinfo');

    
    protected function _setOptions() {
        $options = array();
        $options = $this->Contactinfo->Contact->Addressbook->listTypesVoie();
//        debug($options);
        $this->set( compact ( 'options' ) );
    }
    
	/**
	 * Ajout d'une fiche de contact
	 *
	 * @logical-group Contact
	 * @logical-group Fiche de contact
	 * @user-profile Admin
	 *
	 * @logical-used-by Flux > Ajout d'un flux
	 * @user-profile User
	 *
	 * @logical-used-by Flux > Edition d'un flux
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $contactId
	 * @return void
	 */
	public function add($contactId) {
        $contact = $this->Contactinfo->Contact->find(
            'first',
            array(
                'conditions' => array(
                    'Contact.id' => $contactId
                ),
                'contain' => false
            )
        );
		$this->Contactinfo->recursive = -1;
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
            $options = $this->Contactinfo->Contact->Addressbook->listTypesVoie();
            $typevoie = Hash::get( $options['Addressbook']['typevoie'], Hash::get( $this->request->data, 'Contactinfo.typevoie' ));
            $this->request->data['Contactinfo']['adresse'] = Hash::get( $this->request->data, 'Contactinfo.numvoie' ).' '.$typevoie.' '.Hash::get( $this->request->data, 'Contactinfo.nomvoie' );
            $this->request->data['Contactinfo']['contact_id'] = $contactId;
                    
			$this->Contactinfo->create($this->request->data);
			if ($this->Contactinfo->save()) {
				$this->Jsonmsg->valid();
			}
			$this->Jsonmsg->send();
		}
        else {
            $this->request->data['Contactinfo']['civilite'] = $contact['Contact']['civilite'];
            $this->request->data['Contactinfo']['nom'] = $contact['Contact']['nom'];
            $this->request->data['Contactinfo']['prenom'] = $contact['Contact']['prenom'];
        }
        $this->_setOptions();
	}

	/**
	 *
	 */
	public function getHomonyms() {
		Configure::write('debug', 0);
		$this->autoRender = false;
		$return = array();
		if ($this->RequestHandler->isAjax() && !empty($this->request->data['Contactinfo']['nom']) && !empty($this->request->data['Contactinfo']['prenom'])) {
			$return = $this->Contactinfo->getHomonyms($this->request->data['Contactinfo']['nom'], $this->request->data['Contactinfo']['prenom']);
		}
		echo json_encode($return);
	}

	/**
	 * Edition d'une fiche de contact
	 *
	 * @logical-group Contact
	 * @logical-group Fiche de contact
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id
	 * @return void
	 */
	public function edit($id = null) {
		$this->Contactinfo->recursive = -1;
		if (!empty($this->request->data)) {
            $options = $this->Contactinfo->Contact->Addressbook->listTypesVoie();
            $typevoie = Hash::get( $options['Addressbook']['typevoie'], Hash::get( $this->request->data, 'Contactinfo.typevoie' ));
            $this->request->data['Contactinfo']['adresse'] = Hash::get( $this->request->data, 'Contactinfo.numvoie' ).' '.$typevoie.' '.Hash::get( $this->request->data, 'Contactinfo.nomvoie' );
			$this->Jsonmsg->init();
			$this->Contactinfo->create($this->request->data);
			if ($this->Contactinfo->save()) {
				$this->Jsonmsg->valid();
			}
			$this->Jsonmsg->send();
		} else if ($id != null) {
			$this->request->data = $this->Contactinfo->find('first', array('conditions' => array('Contactinfo.id' => $id)));
		}
        $this->_setOptions();
	}

	/**
	 * Suppression d'une fiche de contact (soft delete - delete)
	 *
	 * @logical-group Contact
	 * @logical-group Fiche de contact
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id
	 * @return void
	 */
	public function delete($id = null, $real = true) {
		if ($id != null) {
			if ($real) {
				$this->Jsonmsg->init(__d('default', 'delete.error'));
				if ($this->Contactinfo->delete($id, true)) {
					$this->Jsonmsg->valid(__d('default', 'delete.ok'));
				}
			} else {
				$this->Jsonmsg->init();
				$this->Contactinfo->id = $id;

				if ($this->Contactinfo->saveField('active', false)) {
					$this->Jsonmsg->valid();
				}
			}
			$this->Jsonmsg->send();
		}
	}

}
