<?php

/**
 * Contacts
 *
 * Contacts controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 *
 * @package		app
 * @subpackage		Controller
 */
class ContactsController extends AppController {

	/**
	 * Controller uses
	 *
	 * @var array
	 */
	public $uses = array('Contact');

     protected function _setOptions() {
        $options = array();
        $options = $this->Contact->Addressbook->listTypesVoie();
        $this->set( compact ( 'options' ) );
    }
    
	/**
	 * Récupération de la liste des contacts
	 *
	 * @logical-group Contact
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $addressbook_id identifiant du carnet d'adresse
	 * @return void
	 */
	public function get_contacts($addressbook_id = null) {
		if ($addressbook_id != null) {
			$this->Contact->recursive = -1;
			$querydata = array(
				'conditions' => array(
					'Contact.addressbook_id' => $addressbook_id,
					'Contact.active' => true
				),
				'order' => array(
					'Contact.name'
				),
				'contain' => array(
					'Contactinfo' => array(
						'conditions' => array(
							'Contactinfo.active' => true
						)
					)
				)
			);
			$contacts = $this->Contact->find('all', $querydata);
			$this->set('contacts', $contacts);
			$this->set('addressbook_id', $addressbook_id);
            $this->_setOptions();
		}
	}

	/**
	 * Ajout d'un contact
	 *
	 * @logical-group Contact
	 * @user-profile Admin
	 *
	 * @logical-used-by Flux > Ajout d'un flux
	 * @user-profile User
	 *
	 * @logical-used-by Flux > Edition d'un flux
	 * @user-profile User
	 *
	 * @access public
	 * @return void
	 */
	public function add() {
//        $this->autoRender = false;
		$this->Contact->recursive = -1;
		if (!empty($this->request->data)) {

			$this->Jsonmsg->init();
			$this->request->data['Contact']['foreign_key'] = $this->Contact->Addressbook->field('foreign_key', array('Addressbook.id' => $this->request->data['Contact']['addressbook_id']));

            $this->Contact->begin();  
            $success = $this->Contact->save($this->request->data['Contact']);

            if( !empty( $this->request->data['Contactinfo'] ) ) {
                $contactinfoValue = array(
                    'Contactinfo' => array(
                        'contact_id' => $this->Contact->id,
                        'civilite' => $this->request->data['Contact']['civilite'],
                        'nom' => $this->request->data['Contact']['nom'],
                        'prenom' => $this->request->data['Contact']['prenom'],
                        'name' => $this->request->data['Contactinfo']['name'],
                        'email' => $this->request->data['Contactinfo']['email'],
                        'tel' => $this->request->data['Contactinfo']['tel'],
                        'numvoie' => $this->request->data['Contactinfo']['numvoie'],
                        'typevoie' => $this->request->data['Contactinfo']['typevoie'],
                        'nomvoie' => $this->request->data['Contactinfo']['nomvoie'],
                        'adresse' => $this->request->data['Contactinfo']['adresse'],
                        'compl' => $this->request->data['Contactinfo']['compl'],
                        'cp' => $this->request->data['Contactinfo']['cp'],
                        'ville' => $this->request->data['Contactinfo']['ville'],
                        'region' => $this->request->data['Contactinfo']['region'],
                        'pays' => $this->request->data['Contactinfo']['pays'],
                        'role' => $this->request->data['Contactinfo']['role'],
                        'organisation' => $this->request->data['Contactinfo']['organisation'],
                    )
                );

                $this->Contact->Contactinfo->create( $contactinfoValue );
                $success = $this->Contact->Contactinfo->save() && $success;
            }
            

            
//            $this->Session->setFlash( $this->Jsonmsg->sendJson(array( 'contact_id' => $this->Contact->id) ) );
//debug( $this->Jsonmsg->valid() );
            $result = null;
            if ($success) {
//                $this->Session->setFlash(__d('default', 'save.ok'), 'growl', array('type' => 'default'));
//                $this->Jsonmsg->sendJson(array( 'contact_id' => $this->Contact->id ) );
				
                $this->autoRender = false;
                $result = $this->Contact->id;
                echo $result;
                $this->Contact->commit(); // FIXME
			} else {
                $result = false;
                echo $result;
				$this->Contact->rollback();
			}
//			$this->Jsonmsg->send();
		} else {
			$this->set('addressbooks', $this->Contact->Addressbook->find('list', array('conditions' => array('Addressbook.active' => true))));
		}
        $this->_setOptions();
	}

	/**
	 * Edition d'un contact
	 *
	 * @logical-group Contact
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id
	 * @return void
	 */
	public function edit($id = null) {
		$this->Contact->recursive = -1;
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
//			$this->Contact->create($this->request->data);
            $this->Contact->begin();  
            $success = $this->Contact->save($this->request->data['Contact']);

            if( !empty( $this->request->data['Contactinfo'] ) ) {
//    debug( $this->request->data );
                $contact_id = $this->request->data['Contact']['id'];
                $contactinfoValue = array(
                    'Contactinfo' => array(
                        'id' => $this->request->data['Contactinfo']['id'],
                        'contact_id' => $contact_id,
                        'civilite' => $this->request->data['Contact']['civilite'],
                        'nom' => $this->request->data['Contact']['nom'],
                        'prenom' => $this->request->data['Contact']['prenom'],
                        'name' => $this->request->data['Contactinfo']['name'],
                        'email' => $this->request->data['Contactinfo']['email'],
                        'tel' => $this->request->data['Contactinfo']['tel'],
                        'numvoie' => $this->request->data['Contactinfo']['numvoie'],
                        'typevoie' => $this->request->data['Contactinfo']['typevoie'],
                        'nomvoie' => $this->request->data['Contactinfo']['nomvoie'],
                        'adresse' => $this->request->data['Contactinfo']['adresse'],
                        'compl' => $this->request->data['Contactinfo']['compl'],
                        'cp' => $this->request->data['Contactinfo']['cp'],
                        'ville' => $this->request->data['Contactinfo']['ville'],
                        'region' => $this->request->data['Contactinfo']['region'],
                        'pays' => $this->request->data['Contactinfo']['pays'],
                        'role' => $this->request->data['Contactinfo']['role'],
                        'organisation' => $this->request->data['Contactinfo']['organisation'],
                    )
                );

//                $this->Contact->Contactinfo->create( $contactinfoValue );
                $success = $this->Contact->Contactinfo->save($contactinfoValue) && $success;
            }
            if ($success) {
				$this->Contact->commit();
				$this->Jsonmsg->valid();
			} else {
				$this->Contact->rollback();
			}
//			if ($this->Contact->save()) {
//				$this->Jsonmsg->valid();
//			}
			$this->Jsonmsg->send();
		} else if ($id != null) {
			$this->request->data = $this->Contact->find(
                'first', array(
                    'fields' => array_merge(
                        $this->Contact->fields(),
                        $this->Contact->Addressbook->fields(),
                        $this->Contact->Contactinfo->fields()
                    ),
                    'conditions' => array(
                        'Contact.id' => $id
                    ),
                    'joins' => array(
                        $this->Contact->join($this->Contact->Addressbook->alias),
                        $this->Contact->join($this->Contact->Contactinfo->alias)
                    ),
                    'contain' => false,
                    'recursive' => -1
                )
            );
			$this->set('addressbooks', $this->Contact->Addressbook->find('list'));
		}
        $this->_setOptions();
	}

	/**
	 * Suppression d'un contact
	 *
	 * @logical-group Contact
	 * @logical-group Fiche de contact
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id
	 * @return void
	 */
	public function delete($id = null, $real = true) {
		if ($id != null) {
			if ($real) {
				$this->Jsonmsg->init(__d('default', 'delete.error'));
				if ($this->Contact->delete($id, true)) {
					$this->Jsonmsg->valid(__d('default', 'delete.ok'));
				}
			} else {
				$this->Jsonmsg->init();
				$this->Contact->id = $id;

				if ($this->Contact->saveField('active', false)) {
					$this->Jsonmsg->valid();
				}
			}
			$this->Jsonmsg->send();
		}
	}

	/**
	 * Importation de contact et de fiches de contact à partir d'un fichier vcard
	 *
	 * @logical-group Contact
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function import() {
		if (!empty($this->request->data)) {
			if (!empty($this->request->data['Import']['file'])) {
				$tmpName = $this->request->data['Import']['file']['tmp_name'];
				$filename = TMP . $this->request->data['Import']['file']['name'];

				if (move_uploaded_file($tmpName, $filename)) {
					if ($this->Contact->Addressbook->importContactVcards($filename, $this->request->data['Import']['addressbook_id'])) {
						$this->Session->setFlash(__d('default', 'save.ok'), 'growl', array('type' => 'default'));
					} else {
						$this->Session->setFlash(__d('default', 'save.error'), 'growl', array('type' => 'error'));
					}
				}

				if (is_file($filename)) {
					unlink($filename);
				}
				$this->redirect(array('controller' => 'addressbooks', 'action' => 'index'));
			}
		} else {
			$this->set('addressbooks', $this->Contact->Addressbook->find('list', array('conditions' => array('Addressbook.active' => 1), 'order' => array('Addressbook.name'))));
		}
        $this->_setOptions();
	}

	/**
	 * Récupération de la liste des contacts
	 *
	 * @logical-group Flux
	 * @logical-group Contact
	 *
	 * @logical-used-by Flux > Edition d'un flux
	 * @user-profile User
	 *
	 * @access public
	 * @return void
	 */
	public function search() {
		if ($this->RequestHandler->isAjax()) {
			Configure::write('debug', 0);
			$this->autoRender = false;
			$response = array();
			$qdContact = array(
				'recursive' => -1,
				'contain' => false,
				'fields' => array(
					'Contact.name',
					'Contact.id',
					'Contact.citoyen',
					'Addressbook.id',
					'Addressbook.name'
				),
				'joins' => array(
					$this->Contact->join($this->Contact->Addressbook->alias)
				),
				'conditions' => array(
					'Contact.name ILIKE' => '%' . $_GET['term'] . '%'
				),
				'order' => array(
					'Addressbook.name',
					'Contact.name'
				)
			);
			$contacts = $this->Contact->find('all', $qdContact);
			$i = 0;
			foreach ($contacts as $contact) {
				$response[$i]['id'] = $contact['Contact']['id'];
				$response[$i]['value'] = $contact['Contact']['name'];
				$response[$i]['label'] = $contact['Contact']['name'];
				$response[$i]['img'] = $contact['Contact']['citoyen'] ? 'citoyen' : 'organisme';
				$response[$i]['category'] = $contact['Addressbook']['name'];
				$i++;
			}
			echo json_encode($response);
		}
        $this->_setOptions();
	}

	/**
	 * Récupération de la liste des fiches de contact
	 *
	 * @logical-group Flux
	 * @logical-group Contact
	 *
	 * @logical-used-by Flux > Edition d'un flux
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $contactId
	 * @return void
	 */
	public function searchContactinfo($contactId) {
		if ($this->RequestHandler->isAjax()) {
			Configure::write('debug', 0);
			$this->autoRender = false;
			$qdContactinfo = array(
				'recursive' => -1,
				'fields' => array(
					'Contactinfo.id',
					'Contactinfo.name',
					'Contactinfo.civilite',
					'Contactinfo.prenom',
					'Contactinfo.nom',
					'Contactinfo.email',
					'Contactinfo.adresse',
					'Contactinfo.compl',
					'Contactinfo.cp',
					'Contactinfo.ville',
					'Contactinfo.region',
					'Contactinfo.pays',
					'Contactinfo.tel',
					'Contactinfo.role',
					'Contactinfo.organisation'
				),
				'conditions' => array(
					'Contactinfo.contact_id' => $contactId
				),
				'order' => array(
					'Contactinfo.name'
				)
			);

			$contactinfos = $this->Contact->Contactinfo->find('all', $qdContactinfo);
			$return = array();
			for ($i = 0; $i < count($contactinfos); $i++) {
				$cInfos = array();
				foreach ($contactinfos[$i]['Contactinfo'] as $key => $val) {
					if ($val != '') {
						$cInfos[$key != 'id' && $key != 'name' ? __d('addressbook', 'Contactinfo.' . $key) : $key] = $val;
					}
				}
				$return[] = array('Contactinfo' => $cInfos);
			}
			echo json_encode($return);
		}
        $this->_setOptions();
	}

	/**
	 *
	 */
	public function getHomonyms() {
		Configure::write('debug', 0);
		$this->autoRender = false;
		$return = array();
		if ($this->RequestHandler->isAjax() && !empty($this->request->data['Contact']['name'])) {
			$return = $this->Contact->getHomonyms($this->request->data['Contact']['name']);
		}
		echo json_encode($return);
	}

}
