<?php

App::import('Sanitize');
App::uses( 'Folder', 'Utility' );
App::uses( 'File', 'Utility' );
/**
 * Flux
 *
 * Courriers controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class CourriersController extends AppController {

    /**
     * Controller name
     *
     * @var array
     */
    public $name = 'Courriers';

    /**
     * Controller uses
     *
     * @var array
     */
    public $uses = array('Courrier');

    /**
     * Controller components
     *
     * @var array
     */
    public $components = array('Email', 'DataAuthorized', 'Cmis', 'Conversion');

    /**
     * Controller helpers
     *
     * @var array
     */
    public $helpers = array('Notification', 'GFCComment');
    

    /**
     *
     */
    public function beforeFilter() {
        parent::beforeFilter();
        $this->Auth->allow('acquisition');
        
    }

    /**
     * Vérifie si un flux est présent dans les bannettes de tous les rôles d'un utilisateur
     *
     * @access private
     * @param array $flux
     * @param integer $desktopId si défini, la vérification se fait pour le rôle correspondant à cet identifiant
     * @return integer
     */
    private function _inBannette($flux, $desktopId = null) {
        if ($desktopId != null) {
            $desktops = array($desktopId);
        } else {
            $desktops = $this->SessionTools->getDesktopList();
        }
        $querydata = array(
            'conditions' => array(
                'Bancontenu.courrier_id' => $flux['Courrier']['id'],
                'Bancontenu.desktop_id' => $desktops,
                'Bancontenu.etat' => '1'
            )
        );
        $result = $this->Courrier->Bancontenu->find('first', $querydata);
        return !empty($result);
    }

    /**
     * Vérifie si un flux est présent dans la bannette "pour copie" de tous les rôles d'un utilisateur
     *
     * @access private
     * @param array $flux
     * @param integer $desktopId si défini, la vérification se fait pour le rôle correspondant à cet identifiant
     * @return integer
     */
    private function _inBannetteCopy($flux, $desktopId = null) {
        if ($desktopId != null) {
            $desktops = array($desktopId);
        } else {
            $desktops = $this->SessionTools->getDesktopList();
        }
        $querydata = array(
            'conditions' => array(
                'Bancontenu.courrier_id' => $flux['Courrier']['id'],
                'Bancontenu.desktop_id' => $desktops,
                'Bancontenu.bannette_id' => BAN_COPY
            )
        );
        $result = $this->Courrier->Bancontenu->find('first', $querydata);
        return !empty($result);
    }

    /**
     * Vérifie si un flux est présent dans les bannettes de tous les rôles d'un utilisateur avec un statut 2 (historique)
     *
     * @access private
     * @param array $flux
     * @param integer $desktopId si défini, la vérification se fait pour le rôle correspondant à cet identifiant
     * @return integer
     */
    private function _inHistorique($flux, $desktopId = null) {
        if ($desktopId != null) {
            $desktops = array($desktopId);
        } else {
            $desktops = $this->SessionTools->getDesktopList();
        }

        $querydata = array(
            'conditions' => array(
                'Bancontenu.courrier_id' => $flux['Courrier']['id'],
                'Bancontenu.desktop_id' => $desktops,
                'Bancontenu.etat' => array('0', '2')
            )
        );
        $result = $this->Courrier->Bancontenu->find('first', $querydata);
        return !empty($result);
    }

    /**
     * Vérifie si l'utilisateur à le droit d'accèder au flux parent du flux réponse
     *
     * @access private
     * @param array $flux
     * @param integer $desktopId
     * @return boolean
     */
    private function _isAuthorizedResponse($flux, $desktopId = null) {
        $newFlux = $this->Courrier->find('first', array('conditions' => array('Courrier.parent_id' => $flux['Courrier']['id'])));
        return ($this->_inBannette($newFlux, $desktopId) || $this->_inHistorique($newFlux, $desktopId));
    }

    /**
     * Visualisation / Edition d'un flux (interface graphique)
     *
     * @logical-group Flux
     * @logical-group Visualisation d'un flux
     * @logical-group Edition d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant du flux
     * @param integer $desktopId identifiant du rôle de l'utilisateur
     * @param integer $tab numéro de l'onglet à réouvrir une fois les modifications effectuées (javascript)
     * @throws NotFoundException
     * @return void
     */
    public function view($id, $desktopId = null, $tab = null) {
        $this->Courrier->recursive = -1;
        $qdCourrier = array(
            'conditions' => array(
                'Courrier.id' => $id
            ),
            'contain' => array(
                'Document' => array(
                    'fields' => $this->Courrier->Document->getLightFields()
                ),
                'Soustype'
            )
        );

        $courrier = $this->Courrier->find('first', $qdCourrier);

        if (empty($courrier)) {
            throw new NotFoundException();
        }

        $fromTask = Configure::read('fromTask');

// echo '<pre>';var_dump($this->Courrier->createRefus($courrier));echo '</pre>';

        $fromMesServices = false;
        $fluxMesServices = $this->Session->read('Auth.User.Courrier.mesServices');

        // Ajout du contrôle sur le profil Admin. Si l'utilisateur est Admin alors il voit tout
        $isAdmin = false;
        $groupName = $this->Session->read('Auth.User.Desktop.Group');
        if( !empty( $groupName ) && $groupName['name'] == 'Admin') {
            $isAdmin = true;
        }
        $isUserInit = false;
        $secondaryDesktops = $this->Session->read('Auth.User.SecondaryDesktops');
        $desktopName = Hash::extract( $secondaryDesktops, '{n}.Group.name' );
        if( in_array( 'Initiateur', $desktopName  ) || $groupName['name'] == 'Initiateur' )  {
            $isUserInit = true;
        }
        $this->set( 'isUserInit', $isUserInit );
        
        $isUserDisp = false;
        if( in_array( 'Aiguilleur', $desktopName  ) || $groupName['name'] == 'Aiguilleur' )  {
            $isUserDisp = true;
        }
        $this->set( 'isUserDisp', $isUserDisp );
        
        if (!empty($fluxMesServices)) {
            if (in_array($id, $fluxMesServices) && !in_array($desktopId, $this->SessionTools->getDesktopList())) {
                $fromMesServices = true;
            }
        }
        $fromHistorique = preg_match('#historique$#', $_SERVER['HTTP_REFERER']);

        $viewParentMode = false;

        
        $fromMesCopies = false;
        $fluxMesCopies = $this->Session->read('Auth.User.Courrier.mesCopies');
//        debug($this->SessionTools->getDesktopList());
//        debug($id);
        if (!empty($fluxMesCopies)) {
            if (in_array($id, $fluxMesCopies) /*&& !in_array($desktopId, $this->SessionTools->getDesktopList())*/) {
                $fromMesCopies = true;
            }
        }
//debug($fromMesCopies);
        $isViewAllowed = array(
            'mesServices' => $fromMesServices,
//            'mesCopies' => $fromMesCopies,
            'historique' => $fromHistorique && $this->_inHistorique($courrier),
            'bannette' => $this->_inBannette($courrier),
            'responseParent' => $viewParentMode && $this->_isAuthorizedResponse($courrier),
            'fromTask' => $fromTask,
            'isAdmin' => $isAdmin
        );
        

        if (!in_array(true, $isViewAllowed, true)) {
            throw new UnauthorizedException();
        }

        $fromBannetteCopy = $this->_inBannetteCopy($courrier);
        $ariane = array(
            '<a href="/environnement/userenv/0">'.__d('menu', 'Courrier').'</a>',
            '<a href="/environnement/userenv/0">'.__d('courrier', 'View Courrier').'</a>',
            $courrier['Courrier']['name']
        );
        $parent = array();
        if (!empty($courrier['Courrier']['parent_id'])) {
            $qdParent = array(
                'fields' => array(
                    'Courrier.id',
                    'Courrier.name'
                ),
                'conditions' => array(
                    'Courrier.id' => $courrier['Courrier']['parent_id']
                ),
                'contain' =>(
                    'Desktop'
                )
            );
            $parent = $this->Courrier->find('first', $qdParent);
            $ariane = array(
                '<a href="/environnement/userenv/0">'.__d('menu', 'Courrier').'</a>',
                '<a href="/environnement/userenv/0">'.__d('courrier', 'View Courrier').'</a>',
                '<a href="/courriers/historiqueGlobal/'.$parent['Courrier']['id'].'">'.$parent['Courrier']['name'].'</a>',
                $courrier['Courrier']['name']
            );
        }

        
        // En cas de refus 
        $fluxRefus = array();
        if (!empty($courrier['Courrier']['ref_ancien'])) {
            $qdFluxRefus = array(
                'fields' => array(
                    'Courrier.id',
                    'Courrier.name'
                ),
                'conditions' => array(
                    'Courrier.id' => $courrier['Courrier']['ref_ancien']
                )
            );
            $fluxRefus = $this->Courrier->find('first', $qdFluxRefus);
        }
        $this->set('fluxRefus', $fluxRefus);
        
        //
        
        $sortant = false;
        if (isset($courrier['Soustype']['entrant']) && $courrier['Soustype']['entrant'] == 0) {
            $sortant = true;
        }
        $this->set('sortant', $sortant);
        $this->set('parent', $parent);
        $this->set('ariane', $ariane);
        $this->set('courrier', $courrier);
        $this->set('mainDoc', $this->Courrier->getMainDoc($courrier['Courrier']['id']));

        //on verifie si le courrier est un courrier cloturé
        $isClosed = $this->Courrier->isClosed($courrier['Courrier']['id']);
        $this->set('isClosed', $isClosed);

        //on verifie si le courrier est un courrier refusé
        $this->set('isRefused', $this->Courrier->isRefused($courrier['Courrier']['id']));

        //on determine si le courrier est dans un circuit
        $this->set('inCircuit', $this->Courrier->isInCircuit($courrier['Courrier']['id']));
        //on determine si le courrier est a l etape de fin
        $endCircuit = false;
        $etape = $this->Courrier->Traitement->whichIsNext($courrier['Courrier']['id']);
        if ($this->Courrier->Traitement->Circuit->Etape->estDerniereEtape($etape['Etape']['id'])) {
            $endCircuit = true;
        }
        $this->set('endCircuit', $endCircuit);

        if (empty($desktopId)) {
            $desktopId = $this->Session->read('Auth.User.Desktop.id');
        }

        if (!$fromHistorique && !$fromMesServices /* && !$fromTask */) {
            //On enleve les notifications concernant ce flux
            $this->Courrier->Notifieddesktop->Notification->setReadByDesktopCourrier($desktopId, $courrier['Courrier']['id']);
            $this->Courrier->Bancontenu->setRead($courrier['Courrier']['id'], $desktopId);
        }


        $userId = $this->Session->read('Auth.User.Courrier.Desktop.id');

        $traitement = $this->Courrier->Traitement->find( 'first', array( 'fields' => array( 'circuit_id'), 'conditions' => array( 'Traitement.target_id' => $courrier['Courrier']['id']), 'contain' => false) );
        $circuitId = $traitement['Traitement']['circuit_id'];
        $isUserInCircuit = $this->Courrier->Traitement->Circuit->userInCircuit( $userId, $circuitId );
        $this->set( compact( 'isUserInCircuit' ));

        $userDesktops = $this->getSessionAllDesktops();
        if (in_array($desktopId, array_keys($userDesktops))) {
            $this->Session->write('Auth.User.Courrier.Desktop.id', $desktopId);
            $aro = array('model' => 'Desktop', 'foreign_key' => $desktopId);
        } else if ($fromHistorique || $fromMesServices || $fromTask) {
            $aro = array('model' => 'Desktop', 'foreign_key' => $this->Session->read('Auth.User.Desktop.id'));
        } else {
            throw new ForbiddenException();
        }

        $rights = array(
            'getInfos' => $this->Acl->check($aro, 'controllers/Courriers/getInfos'),
            'getMeta' => $this->Acl->check($aro, 'controllers/Courriers/getMeta'),
            'getTache' => $this->Acl->check($aro, 'controllers/Courriers/getTache'),
            'getAffaire' => $this->Acl->check($aro, 'controllers/Courriers/getAffaire'),
            'getFlowControls' => $this->Acl->check($aro, 'controllers/Courriers/getFlowControls'),
            'setInfos' => $this->Acl->check($aro, 'controllers/Courriers/setInfos'),
            'setMeta' => $this->Acl->check($aro, 'controllers/Courriers/setMeta'),
            'setTache' => $this->Acl->check($aro, 'controllers/Courriers/setTache'),
            'setAffaire' => $this->Acl->check($aro, 'controllers/Courriers/setAffaire'),
            'getArs' => $this->Acl->check($aro, 'controllers/Courriers/getArs'),
            'getRelements' => $this->Acl->check($aro, 'controllers/Courriers/getRelements'),
            'getDocuments' => $this->Acl->check($aro, 'controllers/Courriers/getDocuments'),
            'getCircuit' => $this->Acl->check($aro, 'controllers/Courriers/getCircuit'),
            'edit' => true
        );

        if (!$rights['setInfos'] && !$rights['setMeta'] && !$rights['setTache'] && !$rights['setAffaire']) {
            $rights['edit'] = false;
        }

        $viewOnlyTabs = array(
            'getInfos' => ($rights['getInfos'] && !$rights['setInfos']) || $isClosed || $fromHistorique || /*$fromMesServices ||*/ $fromBannetteCopy || $fromTask,
            'getTache' => ($rights['getTache'] && !$rights['setTache']) || $isClosed || $fromHistorique || /*$fromMesServices ||*/ $fromBannetteCopy || $fromTask,
            'getMeta' => ($rights['getMeta'] && !$rights['setMeta']) || $isClosed || $fromHistorique || /*$fromMesServices ||*/ $fromBannetteCopy || $fromTask,
            'getAffaire' => ($rights['getAffaire'] && !$rights['setAffaire']) || $isClosed || $fromHistorique || /*$fromMesServices ||*/ $fromBannetteCopy || $fromTask
        );

        $this->Session->delete('Auth.User.Courrier.viewOnlyTabs');
        $this->Session->write('Auth.User.Courrier.viewOnlyTabs', $viewOnlyTabs);
// debug(
// 	array(
// 		'service' => $fromMesServices,
// 		'copy' => $fromBannetteCopy,
// 		'closed' => $isClosed,
// 		'historique' => $fromHistorique,
// 		'task' => $fromTask,
// 		'viewOnly' => $viewOnlyTabs
// 	)
// );
        
        // Le circuit possède-t-il comme prochaine étape une délégation vers le parapheur ?
        $circuit = $this->getCircuit($id);
        $IdCircuit= Hash::get( $this->viewVars, 'circuit.Circuit.id');
        $isDelegatedToParapheur = $this->Courrier->Traitement->Circuit->hasEtapeDelegation($IdCircuit);
        $this->set('isDelegatedToParapheur', $isDelegatedToParapheur);

        $this->set('fromTask', $fromTask);
        $this->set('fromBannetteCopy', $fromBannetteCopy);
        $this->set('fromMesServices', $fromMesServices);
        $this->set('fromMesCopies', $fromMesCopies);
        $this->set('fromHistorique', $fromHistorique);
        $this->set('rights', $rights);
        $this->set('tab', $tab);
// debug($this->Session->read('Auth.User.Courrier'));
        $this->set( 'desktopidactuel', $this->Session->read( 'Auth.User.Courrier.Desktop.id'));
        $this->set( 'envactuel', $this->Session->read( 'Auth.User.Env.actual.name' ) );
        $this->set('viewOnly', $this->Session->read('Auth.User.Courrier.viewOnlyTabs.getInfos'));
    }

    /**
     * @logical-group Flux
     * @logical-group Edition d'un flux
     *
     * @logical-used-by Flux > Edition d'un flux
     * @userProfile User
     *
     * Edition d'un flux (uniquement pour la gestion des habilitations
     *
     * @access public
     * @return void
     */
    //FIXME: supprimer cette fonction et trouver une méthode pour indiquer le droit d'édition dans la gestion des ACL
    public function edit(/* $id = null */) {
//		$this->action = 'edit';
//		if ($id != null) {
//			$this->set('ariane', array(__d('menu', 'Courrier', true), __d('courrier', 'Edit Courrier', true)));
//
//			$this->Courrier->id = $id;
//			$courrier = $this->Courrier->read();
//			if (!empty($courrier)) {
//				$this->set('courrier', $courrier);
//
//				//on verifie si le courrier est un courrier refusé
//				$this->set('isRefused', $this->Courrier->isRefused($courrier['Courrier']['id']));
//
//				//on determine si le courrier est dans un circuit
//				$this->set('inCircuit', $this->Courrier->isInCircuit($courrier['Courrier']['id']));
//
//				//on determine si le courrier est a l etape de fin
//				$endCircuit = false;
//				$etape = $this->Courrier->Traitement->whichIsNext($courrier['Courrier']['id']);
//				if ($this->Courrier->Traitement->Circuit->Etape->estDerniereEtape($etape['Etape']['id'])) {
//					$endCircuit = true;
//				}
//				$this->set('endCircuit', $endCircuit);
//			} else {
//				throw new NotFoundException();
//			}
//		}
    }

    /**
     * Création d'un flux
     *
     * @logical-group Flux
     * @logical-group Création d'un flux
     * @user-profile User
     *
     * @param type $desktopId
     * @throws ForbiddenException
     * @return void
     */
    public function add($desktopId, $serviceId) {
        $this->set('ariane', array(
            '<a href="/environnement/userenv/0">'.__d('menu', 'Courrier', true).'</a>', 
            __d('courrier', 'New Courrier', true)
        ));

        $aro = array('model' => 'Desktop', 'foreign_key' => $desktopId);
        $rights = array(
            'createInfos' => $this->Acl->check($aro, 'controllers/Courriers/createInfos'),
            'setInfos' => $this->Acl->check($aro, 'controllers/Courriers/setInfos'),
            'setMeta' => $this->Acl->check($aro, 'controllers/Courriers/setMeta'),
            'setTache' => $this->Acl->check($aro, 'controllers/Courriers/setTache'),
            'setAffaire' => $this->Acl->check($aro, 'controllers/Courriers/setAffaire'),
            'getArs' => $this->Acl->check($aro, 'controllers/Courriers/getArs'),
            'getRelements' => $this->Acl->check($aro, 'controllers/Courriers/getRelements'),
            'getDocuments' => $this->Acl->check($aro, 'controllers/Courriers/getDocuments'),
//			'setCircuit' => $this->Acl->check($aro, 'controllers/Courriers/setCircuit'),
            'getCircuit' => $this->Acl->check($aro, 'controllers/Courriers/getCircuit')
        );

        if (!$rights['createInfos'] /* || !$rights['setQualification'] */) {
            throw new ForbiddenException();
        }

        $this->set('fluxId', '');
        $this->set('rights', $rights);
        $this->Session->write('Auth.User.Courrier.Desktop.id', $desktopId);
        $this->Session->write('Auth.User.Courrier.Service.id', $serviceId);
    }

    /**
     * Accès à l'onglet "Informations" (ajax)
     *
     * @logical-group Flux
     * @logical-group Visualisation d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $id
     * @throws NotFoundException
     * @return void
     */
    public function getInfos($id = null) {
        $querydata = array(
            'conditions' => array(
                'Courrier.id' => $id
            ),
            'contain' => array(
                'Contactinfo' => array(
                    'Contact'
                ),
                'Soustype' => array(
                    'Type'
                ),
                'Affairesuivie'
            )
        );

        $flux = $this->Courrier->find('first', $querydata);
//debug($flux);
        if (empty($flux)) {
            throw new NotFoundException();
        }

        $this->set('types', $this->Courrier->Soustype->Type->find('list'));
        $this->set('soustypes', $this->Courrier->Soustype->find('all'));
        $this->set('flux', $flux);
        $this->set('viewOnly', $this->Session->read('Auth.User.Courrier.viewOnlyTabs.getInfos'));
        $allDesktops = $this->Courrier->Desktop->getDesktopsByService( 'all' );
        ksort($allDesktops);
//        debug($allDesktops);
        $this->set( 'allDesktops', $allDesktops);
    }

    /**
     *
     * @param type $id
     * @throws NotFoundException
     */
    //TODO: supprimer les fonctions commentées
//	public function getQualification($id = null) {
//		if ($id != null) {
//			$this->Courrier->recursive = 1;
//			$this->Courrier->id = $id;
//			$courrier = $this->Courrier->read();
//			if (!empty($courrier)) {
//				$this->set('courrier', $courrier);
//				$this->set('types', $this->Courrier->Soustype->Type->find('list'));
//				$this->set('soustypes', $this->Courrier->Soustype->find('all'));
//				$this->set('viewOnly', $this->Session->read('Auth.User.Courrier.viewOnlyTabs.getQualification'));
//			} else {
//				throw new NotFoundException();
//			}
//		}
//	}

    /**
     * Récupération du schéma du circuit emprunté par le flux
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @param integer $id
     * @throws NotFoundException
     * @return void
     */
    public function getCircuit($fluxId) {
//		if (empty($fluxId)) {
//			throw new BadMethodCallException();
//		}

        $qdCircuit = array(
            'fields' => array(
                'Circuit.id',
                'Circuit.nom'
            ),
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'contain' => false,
            'joins' => array(
                $this->Courrier->join($this->Courrier->Soustype->alias),
                $this->Courrier->Soustype->join($this->Courrier->Soustype->Circuit->alias),
            )
        );
        $circuit = $this->Courrier->find('first', $qdCircuit);

        $etapes = null;
        $visas = null;
        if (!empty($circuit)) {
            //definition de l identifiant du circuit
            $circuitId = $circuit['Circuit']['id'];

            //recuperation de la structure initiale du circuit
            $qdEtapes = array(
                'fields' => array(
                    'Etape.nom',
                    'Etape.type',
                    'Etape.ordre',
                    'Etape.description'
                ),
                'contain' => array(
                    'Composition.type_validation',
                    'Composition.trigger_id',
                    'Composition' => array(
                        CAKEFLOW_TRIGGER_MODEL
                    )
                ),
                'conditions' => array(
                    'Etape.circuit_id' => $circuitId
                ),
                'order' => 'Etape.ordre'
            );
            $etapes = $this->Courrier->Traitement->Circuit->Etape->find('all', $qdEtapes);

            //recuperation du traitement du circuit
            $qdVisas = array(
                'fields' => array(
                    'Traitement.id',
                    'Traitement.target_id',
                    'Traitement.numero_traitement',
                    'Traitement.treated',
                    'Visa.etape_type',
                    'Visa.etape_nom',
                    'Visa.trigger_id',
                    'Visa.action',
                    'Visa.numero_traitement',
                    'Desktop.name'
                ),
                'contain' => false,
                'joins' => array(
                    array(
                        'table' => 'wkf_visas',
                        'alias' => 'Visa',
                        'type' => 'right',
                        'conditions' => array(
                            'Visa.traitement_id = Traitement.id'
                        )
                    ),
                    array(
                        'table' => 'desktops',
                        'alias' => 'Desktop',
                        'type' => 'left',
                        'conditions' => array(
                            'Visa.trigger_id = Desktop.id'
                        )
                    )
                ),
                'conditions' => array(
                    'Traitement.target_id' => $fluxId
                ),
                'order' => array(
                    'Traitement.target_id',
                    'Visa.numero_traitement',
                    'Visa.id'
                )
            );
            $visas = $this->Courrier->Traitement->find('all', $qdVisas);
        }

        $compiledVisas = array();
        $compiledVisasRef = array();
        foreach ($visas as $kVisa => $visa) {
            if (in_array($visa['Visa']['numero_traitement'], $compiledVisasRef)) {
                $key = array_search($visa['Visa']['numero_traitement'], $compiledVisasRef);
                $compiledVisas[$key]['Composition'][] = array(
                    'Desktop' => array(
                        'name' => $visa['Desktop']['name'],
                        'action' => $visa['Visa']['action']
                    )
                );
            } else {
                $compiledVisasRef[$kVisa] = $visa['Visa']['numero_traitement'];
                $visa['Composition'] = array(
                    array(
                        'Desktop' => array(
                            'name' => $visa['Desktop']['name'],
                            'action' => $visa['Visa']['action']
                        )
                    )
                );
                $compiledVisas[$kVisa] = $visa;
            }
        }

        $this->set('circuit', $circuit);
        $this->set('etapes', $etapes);
        $this->set('visas', $compiledVisas);
        $this->set('viewOnly', $this->Session->read('Auth.User.Courrier.viewOnlyTabs.getCircuit'));
    }

    /**
     * Accès à l'onglet "Méta-données" (ajax)
     *
     * @logical-group Flux
     * @logical-group Visualisation d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @return void
     */
    public function getMeta($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}

        $this->set('metas', $this->DataAuthorized->getAuthMetadonnees(array('fluxId' => $fluxId)));
        $this->set('viewOnly', $this->Session->read('Auth.User.Courrier.viewOnlyTabs.getMeta'));
    }

    /**
     * Accès à l'onglet "Tâches" (ajax)
     *
     * @logical-group Flux
     * @logical-group Visualisation d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function getTache($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}

        $qd = array(
            'conditions' => array(
                'Courrier.id' => $fluxId
            )
        );
        $courrier = $this->Courrier->find('first', $qd);
        if (empty($courrier)) {
            throw new NotFoundException();
        }

        $this->set('canAddTask', !empty($courrier['Courrier']['soustype_id']));

        $querydata = array(
            'contain' => array(
                $this->Courrier->Tache->Desktop->alias
            ),
            'conditions' => array(
                "Tache.courrier_id" => $fluxId
            ),
            'order' => array(
                'Tache.created'
            )
        );
        $taches_tmp = $this->Courrier->Tache->find('all', $querydata);

        $taches = array();
        //on verifie que l utilisateur fasse partie des personnes pouvant valider la tache
        $userDesktops = $this->SessionTools->getDesktopList();
        foreach ($taches_tmp as $tache) {
            //calcul du retard
            $retard = false;
            if ($tache['Tache']['statut'] != 2) {
                $tmpTime = split(' ', $tache['Tache']['created']);
                $tmpTime1 = split('-', $tmpTime[0]);
                if ($tache['Tache']['delai_unite'] == 0) {
                    $endTime = mktime(0, 0, 0, $tmpTime1[1], $tmpTime1[2] + $tache['Tache']['delai_nb'], $tmpTime1[0]);
                } else if ($tache['Tache']['delai_unite'] == 1) {
                    $endTime = mktime(0, 0, 0, $tmpTime1[1], $tmpTime1[2] + (7 * $tache['Tache']['delai_nb']), $tmpTime1[0]);
                } else if ($tache['Tache']['delai_unite'] == 2) {
                    $endTime = mktime(0, 0, 0, $tmpTime1[1] + $tache['Tache']['delai_nb'], $tmpTime1[2], $tmpTime1[0]);
                }
                if ($endTime < mktime()) {
                    $retard = true;
                }
            }
            $tache['retard'] = $retard;

            $tache['canGrab'] = false;
            $tache['canRelease'] = false;
            $tache['canValid'] = false;

            if ($tache['Tache']['statut'] == 0) {
                if (count($tache['Desktop']) == 1) {
                    if (in_array($tache['Desktop'][0]['id'], $userDesktops)) {
                        $tache['canValid'] = true;
                    }
                } else if (count($tache['Desktop'] > 1)) {
                    foreach ($tache['Desktop'] as $dktp) {
                        if (in_array($dktp['id'], $userDesktops)) {
                            $tache['canGrab'] = true;
                        }
                    }
                }
            } else if ($tache['Tache']['statut'] == 1) {
                if (in_array($tache['Tache']['act_desktop_id'], $userDesktops)) {
                    $tache['canValid'] = true;
                    $tache['canRelease'] = true;
                }
            }

            if ($retard) {
                array_unshift($taches, $tache);
            } else {
                array_push($taches, $tache);
            }
        }
        $this->set('taches', $taches);
        $this->set('userDesktops', $userDesktops);
        $this->set('viewOnly', $this->Session->read('Auth.User.Courrier.viewOnlyTabs.getTache'));
    }

    /**
     * Accès à l'onglet "Dossiers/Affaires" (ajax)
     *
     * @logical-group Flux
     * @logical-group Visualisation d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function getAffaire($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}

        $querydata = array(
            'contain' => array(
                'Affaire' => array(
                    'Dossier'
                )
            ),
            'conditions' => array(
                'Courrier.id' => $fluxId
            )
        );
        $courrier = $this->Courrier->find('first', $querydata);

        if (!empty($courrier)) {
            $this->set('courrier', $courrier);
            $this->set('viewOnly', $this->Session->read('Auth.User.Courrier.viewOnlyTabs.getAffaire'));
        } else {
            throw new NotFoundException();
        }
    }

    /**
     * Accès à l'onglet "Informations" (ajax)
     *
     * @logical-group Flux
     * @logical-group Création d'un flux
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function createInfos() {
        if (!empty($this->request->data)) {
            $this->Courrier->begin();

            $valid = false;
            $this->request->data['Courrier']['user_creator_id'] = $this->Session->read('Auth.User.id');
            $this->request->data['Courrier']['desktop_creator_id'] = $this->Session->read('Auth.User.Courrier.Desktop.id');
            $this->request->data['Courrier']['service_creator_id'] = $this->Session->read('Auth.User.Courrier.Service.id');

            if (empty($this->request->data['Courrier']['reference']) && !empty($this->request->data['Courrier']['soustype_id'])) {
//                $this->request->data['Courrier']['reference'] = $this->Courrier->genRef($this->request->data['Courrier']['soustype_id']);
                $this->request->data['Courrier']['reference'] = $this->Courrier->genRef2($this->request->data);
                $this->request->data['Courrier']['direction'] = $this->Courrier->Soustype->field('Soustype.entrant', array('Soustype.id' => $this->request->data['Courrier']['soustype_id']));
            }


            $this->Courrier->create($this->request->data);
            $saved = $this->Courrier->save();

            if (!empty($saved)) {
                $newCourrierId = $this->Courrier->id;
                if ($this->Courrier->Bancontenu->add($this->Session->read('Auth.User.Courrier.Desktop.id'), BAN_DOCS, $newCourrierId, true)) {
                    $valid = true;
                }
            }

            if ($valid) {
                $this->Jsonmsg->valid(__d('courrier', 'Courrier.create_ok'));
                $this->Jsonmsg->json['newCourrierId'] = $newCourrierId;
                $this->Courrier->commit();
            } else {
                $this->Courrier->rollback();
            }

            $this->Jsonmsg->send();
        } else {
            $allDesktops = $this->Courrier->Desktop->getDesktopsByService( 'all' );
            ksort($allDesktops);
            $this->set( 'allDesktops', $allDesktops);
            $this->set('isInCircuit', false);
            $this->set('types', $this->DataAuthorized->getAuthTypes(array('withSoustypes' => true)));

            $this->set('contactname', !empty($this->request->data['Contactinfo']['Contact']) ? $this->request->data['Contactinfo']['Contact']['name'] : "");
        }
    }

    /**
     * Accès à l'onglet "Informations" (ajax)
     *
     * @logical-group Flux
     * @logical-group Edition d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant du flux
     * @return void
     */
    public function setInfos($id = null) {
        //enregistrement des données
        if (!empty($this->request->data)) {
            $msg = 'Onglet ' . __d('courrier', 'Informations') . ' : ';
            $this->Jsonmsg->init($msg . __d('default', 'save.error'));

            if (empty($this->request->data['Courrier']['reference']) && !empty($this->request->data['Courrier']['soustype_id'])) {
				$this->request->data['Courrier']['reference'] = $this->Courrier->genRef2($this->request->data);
                $this->request->data['Courrier']['direction'] = $this->Courrier->Soustype->field('Soustype.entrant', array('Soustype.id' => $this->request->data['Courrier']['soustype_id']));
            }

            $this->Courrier->create($this->request->data);
            if ($this->Courrier->save()) {
                $this->Jsonmsg->valid($msg . __d('default', 'save.ok'));
            }
            $this->Jsonmsg->json['loadContext'] = 'reload';
            $this->Jsonmsg->send();
        } else {
//			if (!is_integer($fluxId) || $fluxId <= 0) {
//				throw new BadMethodCallException();
//			}
            //affichage du formulaire
            $qd = array(
                'conditions' => array(
                    'Courrier.id' => $id
                ),
                'contain' => array(
                    'Contactinfo' => array(
                        'Contact'
                    ),
                    'Soustype' => array(
                        'Type'
                    )
                )
            );

            $this->request->data = $this->Courrier->find('first', $qd);

            //contactinfos
            $qdContactInfo = array(
                'fields' => array(
                    'Contactinfo.id',
                    'Contactinfo.name',
                    'Contactinfo.civilite',
                    'Contactinfo.prenom',
                    'Contactinfo.nom',
                    'Contactinfo.email',
                    'Contactinfo.numvoie',
                    'Contactinfo.typevoie',
                    'Contactinfo.nomvoie',
                    'Contactinfo.adresse',
                    'Contactinfo.compl',
                    'Contactinfo.cp',
                    'Contactinfo.ville',
                    'Contactinfo.region',
                    'Contactinfo.pays',
                    'Contactinfo.tel',
                    'Contactinfo.role',
                    'Contactinfo.organisation'
                ),
                'recursive' => -1,
                'conditions' => array(
                    'Contactinfo.contact_id' => $this->request->data['Contactinfo']['contact_id']
                ),
                'order' => array(
                    'Contactinfo.name'
                )
            );
            $contactinfos = $this->Courrier->Contactinfo->find('all', $qdContactInfo);

            $contactinfosTreated = array();
            for ($i = 0; $i < count($contactinfos); $i++) {
                $cInfos = array();
                foreach ($contactinfos[$i]['Contactinfo'] as $key => $val) {
                    if ($val != '') {
                        $cInfos[$key != 'id' && $key != 'name' ? str_replace("'", "&apos;", __d('addressbook', 'Contactinfo.' . $key)) : $key] = str_replace("'", "&apos;", $val);
                    }
                }
                $contactinfosTreated[] = array('Contactinfo' => $cInfos);
            }


            $this->set('contactinfos', $contactinfosTreated);
            $this->set('isInCircuit', $this->Courrier->isInCircuit($this->request->data['Courrier']['id']));
            $this->set('types', $this->DataAuthorized->getAuthTypes(array('withSoustypes' => true)));
            //contact name
            $this->set('contactname', !empty($this->request->data['Contactinfo']['Contact']) ? $this->request->data['Contactinfo']['Contact']['name'] : "");
            $this->set('contactid', !empty($this->request->data['Contactinfo']['Contact']) ? $this->request->data['Contactinfo']['Contact']['id'] : "");
            $allDesktops = $this->Courrier->Desktop->getDesktopsByService( 'all' );
            ksort($allDesktops);
            $this->set( 'allDesktops', $allDesktops);
        }
        $this->set( 'envactuel', $this->Session->read( 'Auth.User.Env.actual.name' ) );
    }

    /**
     *
     * @param type $id
     */
    public function setComments($id) {
        $this->set('flux_id', $id);
        $soustype_id = $this->Courrier->field('soustype_id', array('Courrier.id' => $id));
        $this->set('canAddPrivateComment', !empty($soustype_id));
        $this->set('comments', $this->_getComments($id));
    }

    /**
     *
     * @param type $id
     */
    public function getComments($id) {
        $soustype_id = $this->Courrier->field('soustype_id', array('Courrier.id' => $id));
        $this->set('canAddPrivateComment', !empty($soustype_id));
        $this->set('comments', $this->_getComments($id));
    }

    /**
     *
     * @param type $id
     * @return type
     */
    private function _getComments($id) {
        //definition d'une nouvelle relation à la volée (pour contourner le problème de nom de classe en double)
        $bindParams = array(
            'belongsTo' => array(
                'SecondaryUser' => array(
                    'className' => 'User',
                    'foreignKey' => 'user_id',
                    'conditions' => '',
                    'fields' => '',
                    'order' => ''
                )
            )
        );
        $this->Courrier->Comment->Owner->DesktopsUser->bindModel($bindParams, false);

        //querydata générique
        $qdComment = array(
            'fields' => array(
                'Comment.id',
                'Comment.created',
                'Comment.content',
                'Comment.private',
                'Owner.id',
                'Owner.name',
                'User.prenom',
                'User.nom',
                'SecondaryUser.prenom',
                'SecondaryUser.nom'
            ),
            'conditions' => array(
                'Comment.target_id' => $id,
//                'or' => array( //FIXME Arnaud
//                    'DesktopsUser.delegation' => false,
//                    'DesktopsUser.delegation' => null
//                )
            ),
            'order' => array('Comment.created desc'),
            'joins' => array(
                $this->Courrier->Comment->join($this->Courrier->Comment->Owner->alias),
                $this->Courrier->Comment->Owner->join($this->Courrier->Comment->Owner->User->alias),
                $this->Courrier->Comment->Owner->join($this->Courrier->Comment->Owner->DesktopsUser->alias),
                $this->Courrier->Comment->Owner->DesktopsUser->join($this->Courrier->Comment->Owner->DesktopsUser->SecondaryUser->alias)
            ),
            'contain' => false,
            'recursive' => -1
        );

        //public
        $qdPublic = $qdComment;
        $qdPublic['conditions']['Comment.private'] = false;

        //private
        $qdPrivate = $qdComment;
        $qdPrivate['conditions']['Comment.private'] = true;
        $sqParams = array(
            'alias' => 'comments_readers',
            'fields' => array(
                'comments_readers.comment_id'
            ),
            'conditions' => array(
                'comments_readers.reader_id' => $this->Session->read('Auth.User.Courrier.Desktop.id')
            ),
            'contain' => false,
            'recursive' => -1
        );
        $qdPrivate['conditions'][] = 'Comment.id IN (' . $this->Courrier->Comment->SCCommentReader->sq($sqParams) . ')';


        //written
        $qdWritten = $qdComment;
        $qdWritten['conditions']['Owner.id'] = $this->SessionTools->getDesktopList();

        $return = array(
            'public' => $this->Courrier->Comment->find('all', $qdPublic),
            'private' => $this->Courrier->Comment->find('all', $qdPrivate),
            'written' => $this->Courrier->Comment->find('all', $qdWritten)
        );

        for ($i = 0; $i < count($return['written']); $i++) {
            $found = $this->Courrier->Comment->find('first', array(
                'fields' => array(
                    'Comment.id',
                    'Comment.slug'
                ),
                'conditions' => array(
                    'Comment.id' => $return['written'][$i]['Comment']['id']
                ),
                'contain' => array(
                    'Reader' => array(
                        'fields' => array(
                            'Reader.name'
                        ),
                        'User' => array(
                            'fields' => array(
                                'User.id',
                                'User.prenom',
                                'User.nom'
                            )
                        ),
                        'SecondaryUser' => array(
                            'fields' => array(
                                'SecondaryUser.id',
                                'SecondaryUser.prenom',
                                'SecondaryUser.nom'
                            ),
                            'conditions' => array(
                                'or' => array(
                                    'DesktopsUser.delegation' => false,
                                    'DesktopsUser.delegation' => null
                                )
                            )
                        )
                    )
                )
                    )
            );

            $readers = array();
            foreach ($found['Reader'] as $reader) {
                $name = '';
                if (!empty($reader['User'][0]['nom']) && !empty($reader['User'][0]['prenom'])) {
                    $name .= $reader['User'][0]['nom'] . ' ' . $reader['User'][0]['prenom'];
                } else if (!empty($reader['SecondaryUser'][0]['nom']) && !empty($reader['SecondaryUser'][0]['prenom'])) {
                    $name .= $reader['SecondaryUser'][0]['nom'] . ' ' . $reader['SecondaryUser'][0]['prenom'];
                }
                $name .= (empty($name) ? '' : ' (') . $reader['name'] . (empty($name) ? '' : ')');
                $readers[] = array('id' => $reader['id'], 'name' => $name);
            }

            $return['written'][$i]['readers'] = $readers;
        }

        //suppression de la nouvelle relation
        $this->Courrier->Comment->Owner->DesktopsUser->unbindModel(array('belongsTo' => array('SecondaryUser')));
        return $return;
    }

    /**
     *
     */
    //TODO: supprimer les fonction commentées
    public function getContacts() {
        $querydata = array(
            'fields' => array(
                'Contact.id',
                'Contact.name',
                'Addressbook.name'
            ),
            'conditions' => array(
                'Addressbook.active' => true,
                'Contact.active' => true
            ),
            'contain' => false,
            'joins' => array(
                $this->Courrier->Contactinfo->Contact->Addressbook->join($this->Courrier->Contactinfo->Contact->alias)
            )
        );
        $this->set('contacts', $this->Courrier->Contactinfo->Contact->Addressbook->find('list', $querydata));
    }

    /**
     * Récupération des boutons d'action concernant les circuits de traitement (ajax)
     *
     * @logical-group Flux
     * @logical-group CakeFlow
     * @user-profile User
     *
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     */
    public function getFlowControls($fluxId, $cpyOnly = false) {
//		if (!is_integer($fluxId) || $fluxId < 0) {
//			throw new BadMethodCallException();
//		}


        $this->Courrier->id = $fluxId;
        $courrier = $this->Courrier->find(
            'first', array(
            'fields' => array(
                'Courrier.id',
                'Courrier.soustype_id',
                'Soustype.id',
                'Soustype.circuit_id',
                'Document.id'
            ),
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'joins' => array(
                $this->Courrier->join('Soustype', array('type' => 'LEFT OUTER')),
                $this->Courrier->join('Document', array('type' => 'LEFT OUTER'))
            ),
            'contain' => false
                )
        );

        if (empty($courrier)) {
            throw new NotFoundException();
        }

        //TODO: verifier la pertinence de ce test
        $reponse = false;
        $stypeReponse = $this->Courrier->Soustype->find('first', array('conditions' => array('Soustype.parent_id' => $courrier['Courrier']['soustype_id'])));
        if (!empty($stypeReponse)) {
            $reponse = true;
        }

        $isRefused = $this->Courrier->isRefused($courrier['Courrier']['id']);
        $isInCircuit = $this->Courrier->isInCircuit($courrier['Courrier']['id']);
        $isClosed = $this->Courrier->isClosed($courrier['Courrier']['id']);

        $rebond = false;
        $etapes = $this->Courrier->Traitement->Circuit->Etape->find('list', array('conditions' => array('Etape.circuit_id' => $courrier['Soustype']['circuit_id']), 'contain' => false));
        $nexts = $this->Courrier->Traitement->whoIsNext($courrier['Courrier']['id']);
        if (count($nexts) == 1 && !in_array($nexts[0], array_keys($etapes))) {
            $rebond = true;
        }

        //on determine si le courrier est a l etape de fin
        $endCircuit = false;
        if ($this->Courrier->cakeflowIsEnd($courrier['Courrier']['id'])) {
            $endCircuit = true;
        }

        $this->set('desktopsToSend', $this->Courrier->Desktop->getDesktopsByUser());

        $sendCpyCheck = $this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/sendcpy');
        $sendWkfCheck = $this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/sendwkf');

        
        $userId = $this->Session->read('Auth.User.Courrier.Desktop.id');
        $traitement = $this->Courrier->Traitement->find( 'first', array( 'fields' => array( 'circuit_id'), 'conditions' => array( 'Traitement.target_id' => $courrier['Courrier']['id']), 'contain' => false) );
        $circuitId = $traitement['Traitement']['circuit_id'];
        $isUserInCircuit = $this->Courrier->Traitement->Circuit->userInCircuit( $userId, $circuitId );
        $this->set( compact( 'isUserInCircuit' ));
        
//        debug($courrier['Soustype']['circuit_id']);
//        
//        // Mettre un contrôle sur la vérification de la présence d'un fichier joint ou non avant envoi au parapheur sinon pas de parapheur
//        $listeEtapes = $this->Courrier->Traitement->Circuit->Etape->find(
//            'all',
//            array(
//                'fields' => array(
//                    'Etape.id',
//                    'Etape.soustype'
//                ),
//                'conditions' => array(
//                    'Etape.circuit_id' => $courrier['Soustype']['circuit_id']
//                ),
//            'contain' => false
//            )
//        );
//
//        $hasEtapeParapheur = false;
//        $hasSoustype = Hash::extract($listeEtapes, '{n}.Etape.soustype');
//        foreach($hasSoustype as $key => $value) {
//            if(!empty($value) || $value === (int)0){
//                $hasEtapeParapheur = true;
//            }
//        }

//         debug($this->params);
        
        // Le circuit possède-t-il comme prochaine étape une délégation vers le parapheur ?
        $hasEtapeParapheur = $this->Courrier->Traitement->Circuit->hasEtapeDelegation($courrier['Soustype']['circuit_id']);
        $this->set( compact('hasEtapeParapheur'));
        
        $fairesuivre = true;
        if( empty($courrier['Document']['id'] )){
            $fairesuivre = false;
        }
        $this->set( compact('fairesuivre'));


        // Autorisation de versement
        $isAuthVersGed = false;
        $isAuthVersGed = $this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/sendToGed');
        $this->set( 'isAuthVersGed', $isAuthVersGed );

        
        // Le bureau actuel est-il celui de l'étape du circuit
        $desktopCurrentEtapeCircuit = false;
        $desktopIdInWkf = $this->Courrier->Traitement->whoIs( $courrier['Courrier']['id'], 'current');
		
                //if($desktopIdInWkf == $this->Session->read( 'Auth.User.Courrier.Desktop.id')) { 'array' ne peut jamais égale 'int'.
		if(in_array($this->Session->read( 'Auth.User.Courrier.Desktop.id'),$desktopIdInWkf)){
                    $desktopCurrentEtapeCircuit = true;
		}
        $this->set('desktopCurrentEtapeCircuit', $desktopCurrentEtapeCircuit);
        //$this->set('etapes',$nexts);
        $this->set('sendwkf', $sendWkfCheck);
        $this->set('sendcpy', $sendCpyCheck);
        $this->set('rebond', $rebond);
        $this->set('reponse', $reponse);
        $this->set('courrier', $courrier);
        $this->set('isRefused', $isRefused);
        $this->set('isInCircuit', $isInCircuit);
        $this->set('isClosed', $isClosed);
        $this->set('endCircuit', $endCircuit);
        $this->set('cpyOnly', $cpyOnly && $sendCpyCheck);
        $this->set( 'desktopidactuel', $this->Session->read( 'Auth.User.Courrier.Desktop.id'));
    }

    /**
     * Accès à l'onglet "Méta-données" (ajax)
     *
     * @logical-group Flux
     * @logical-flux Edition d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @return void
     */
    public function setMeta($fluxId = null) {
        //enregistrement des données
        if (!empty($this->request->data)) {
            $msg = 'Onglet ' . __d('courrier', 'Metadonnees') . ' : ';
            $this->Jsonmsg->init($msg . __d('default', 'save.error'));
            //preparation des metadonnee
            $success = true;
            $idsASupprimer = array();
            $aEnregistrer = array();
            foreach ($this->request->data['CourrierMetadonnee'] as $key => $values) {
                $checkboxUnchecked = false;
                if ($this->Courrier->Metadonnee->field('Metadonnee.typemetadonnee_id', array('Metadonnee.id' => $values['metadonnee_id'])) == 3 && $values['valeur'] == 0) {
                    $checkboxUnchecked = true;
                }

                if (trim($values['valeur']) == '' || $checkboxUnchecked) {
                    if (!empty($values['id'])) {
                        $idsASupprimer[] = $values['id'];
                    }
                } else {
                    $aEnregistrer[] = $values;
                }
            }
            if (!empty($idsASupprimer)) {
                $success = $this->Courrier->Metadonnee->CourrierMetadonnee->deleteAll(array('CourrierMetadonnee.id' => $idsASupprimer));
            }
            if (!empty($aEnregistrer)) {
                $success = $this->Courrier->Metadonnee->CourrierMetadonnee->saveAll($aEnregistrer);
            }

            if ($success) {
                $this->Jsonmsg->valid($msg . __d('default', 'save.ok'));
            }
            $this->Jsonmsg->send();
        } else { //affichage du formulaire
//			if (!is_integer($fluxId) || $fluxId <= 0) {
//				throw new BadMethodCallException();
//			}
            //TODO: ameliorer la gestion des thesaurus
            $this->loadModel('Keywordlist');
            $this->loadModel('Keyword');
            $this->Keyword->recursive = -1;
            $this->Keywordlist->recursive = -1;

            $thesaurusList = $this->Keywordlist->find('all', array('conditions' => array('Keywordlist.active' => true)));
            $thesaurus_keywords = array();
            foreach ($thesaurusList as $item) {
                $thesaurus_keywords[] = array(
                    'keywordlist' => array(
                        'id' => $item['Keywordlist']['id'],
                        'nom' => $item['Keywordlist']['nom']
                    ),
                    'keywords' => $this->Keyword->generateTreeList(array('Keyword.keywordlist_id' => $item['Keywordlist']['id']))
                );
            }
            $this->set('thesaurus_keywords', $thesaurus_keywords);

            // On récupère l'ID du sous type présent dans le flux
            $flux = $this->Courrier->find(
                'first',
                array(
                    'fields' => array( 'Soustype.id' ),
                    'conditions' => array(
                        'Courrier.id' => $fluxId
                    ),
                    'contain' => array(
                        'Soustype'
                    )
                )
            );
            $metasIdsBySoustype = array();
            // Si un sous type est lié, on regarde la liste des méta données associées
            if( !empty($flux) ) {
                $sousTypeId = $flux['Soustype']['id'];

                $metasBySoustype = $this->Courrier->Soustype->MetadonneeSoustype->find(
                    'all',
                    array(
                        'conditions' => array(
                            'MetadonneeSoustype.soustype_id' => $sousTypeId
                        ),
                        'recursive' => -1
                    )
                );
            
                $metasIdsBySoustype = Hash::extract( $metasBySoustype, '{n}.MetadonneeSoustype.metadonnee_id' );
            }

            
            $metas = $this->DataAuthorized->getAuthMetadonnees(array('action' => 'update'));
                        
            $metas_values = $this->DataAuthorized->getAuthMetadonnees(array('action' => 'update', 'fluxId' => $fluxId));

            $compiledMetas = array();
            foreach ($metas as $meta) {
                if (!empty($metasIdsBySoustype) ) {
                    //Si des métadonnées sont liées au sous-typr, on réduit la liste uniquement à ces métadonnées
                    if(in_array( $meta['Metadonnee']['id'], $metasIdsBySoustype ) ){ // réduction de la liste des métadonnées selon le sous-type défini dans le flux
                        foreach ($metas_values as $meta_value) {
                            if ($meta['Metadonnee']['id'] == $meta_value['Metadonnee']['id']) {
                                if (!empty($meta_value['CourrierMetadonnee'])) {
                                    $meta['CourrierMetadonnee'] = $meta_value['CourrierMetadonnee'];
                                }
                            }
                        }
                        $compiledMetas[] = $meta;
                    }
                }
                else {
                //sinon, on affiche toutes les métadonnées
                    foreach ($metas_values as $meta_value) {
                        if ($meta['Metadonnee']['id'] == $meta_value['Metadonnee']['id']) {
                            if (!empty($meta_value['CourrierMetadonnee'])) {
                                $meta['CourrierMetadonnee'] = $meta_value['CourrierMetadonnee'];
                            }
                        }
                    }
                    $compiledMetas[] = $meta;
                }
                
            }
// debug($compiledMetas);
            $this->set('metas', $compiledMetas);
        }
    }

    /**
     * Accès à l'onglet "Tâches" (ajax)
     *
     * @logical-group Flux
     * @logical-flux Edition d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function setTache($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}

        $qd = array(
            'recursive' => -1,
            'conditions' => array(
                'Courrier.id' => $fluxId
            )
        );
        $courrier = $this->Courrier->find('first', $qd);
        if (empty($courrier)) {
            throw new NotFoundException();
        }

        $canAddTask = true;
        if (empty($courrier['Courrier']['soustype_id'])) {
            $canAddTask = false;
        }
        $this->set('canAddTask', $canAddTask);

        $querydata = array(
            'contain' => array(
                $this->Courrier->Tache->Desktop->alias
            ),
            'conditions' => array(
                "Tache.courrier_id" => $fluxId
            ),
            'order' => array(
                'Tache.created'
            )
        );
        $taches_tmp = $this->Courrier->Tache->find('all', $querydata);

        $taches = array();
        //on verifie que l utilisateur fait partie des personnes pouvant valider la tache
        $userDesktops = $this->SessionTools->getDesktopList();
        foreach ($taches_tmp as $tache) {
            //calcul du retard
            $retard = false;
            if ($tache['Tache']['statut'] != 2) {
                $tmpTime = split(' ', $tache['Tache']['created']);
                $tmpTime1 = split('-', $tmpTime[0]);
                if ($tache['Tache']['delai_unite'] == 0) {
                    $endTime = mktime(0, 0, 0, $tmpTime1[1], $tmpTime1[2] + $tache['Tache']['delai_nb'], $tmpTime1[0]);
                } else if ($tache['Tache']['delai_unite'] == 1) {
                    $endTime = mktime(0, 0, 0, $tmpTime1[1], $tmpTime1[2] + (7 * $tache['Tache']['delai_nb']), $tmpTime1[0]);
                } else if ($tache['Tache']['delai_unite'] == 2) {
                    $endTime = mktime(0, 0, 0, $tmpTime1[1] + $tache['Tache']['delai_nb'], $tmpTime1[2], $tmpTime1[0]);
                }
                if ($endTime < mktime()) {
                    $retard = true;
                }
            }
            $tache['retard'] = $retard;
            $tache['canGrab'] = false;
            $tache['canRelease'] = false;
            $tache['canValid'] = false;

            if ($tache['Tache']['statut'] == 0) {
                if (count($tache['Desktop']) == 1) {
                    if (in_array($tache['Desktop'][0]['id'], $userDesktops)) {
                        $tache['canValid'] = true;
                    }
                } else if (count($tache['Desktop'] > 1)) {
                    foreach ($tache['Desktop'] as $dktp) {
                        if (in_array($dktp['id'], $userDesktops)) {
                            $tache['canGrab'] = true;
                        }
                    }
                }
            } else if ($tache['Tache']['statut'] == 1) {
                if (in_array($tache['Tache']['act_desktop_id'], $userDesktops)) {
                    $tache['canValid'] = true;
                    $tache['canRelease'] = true;
                }
            }

            if ($retard) {
                array_unshift($taches, $tache);
            } else {
                array_push($taches, $tache);
            }
        }
        $this->set('taches', $taches);
        $this->set('userDesktops', $userDesktops);

        $rights = array(
            'add' => ($this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Taches/add') ? true : false),
            'edit' => ($this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Taches/edit') ? true : false),
            'delete' => ($this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Taches/delete') ? true : false)
        );
        $this->set('rights', $rights);
        $this->set('id', $fluxId);
    }

    /**
     * Accès à l'onglet "Dossiers/Affaires" (ajax)
     *
     * @logical-group Flux
     * @logical-flux Edition d'un flux
     * @user-profile User
     *
     * @access public
     * @param integer $courrierId identifiant du flux
     * @param integer $affaireId identifiant de l'affaire
     * @throws NotFoundException
     * @return void
     */
    public function setAffaire($courrierId = null, $affaireId = null) {
        //enregistrement des données
        //association d un courrier a une affaire
        if ($courrierId != null && $affaireId != null) {
            $courrier = $this->Courrier->find('first', array('conditions' => array('Courrier.id' => $courrierId)));
            $msg = 'Onglet ' . __d('courrier', 'Affaires') . ' : ';
            $this->Jsonmsg->init($msg . __('default', 'save.error'));
            $this->Courrier->begin();

            if ($courrier['Courrier']['affaire_id'] == $affaireId) { //le courrier et l affaire sont liés, on va les dissocier
                $courrier['Courrier']['affaire_id'] = null;
            } else { //sinon on les associe
                $courrier['Courrier']['affaire_id'] = $affaireId;
            }

            if ($this->Courrier->save($courrier)) {
                $this->Courrier->commit();
                $this->Jsonmsg->valid($msg . __('default', 'save.ok'));
            } else {
                $this->Courrier->rollback();
            }
            $this->Jsonmsg->send();
        } else {
            //formulaire de creation d un affaire ( et d un dossier)
            if (!empty($this->request->data)) {
                if (!empty($this->request->data['Affaire'])) {
                    if (!empty($this->request->data['Dossier'])) {
                        $this->request->data['Affaire']['dossier_id'] = '';
                    }
                    $this->Jsonmsg->init();
                    if ($this->Courrier->Affaire->saveAll($this->request->data)) {
                        $this->Courrier->id = $courrierId;
                        if ($this->Courrier->saveField('affaire_id', $this->Courrier->Affaire->id)) {
                            $this->Jsonmsg->valid();
                        }
                    }
                    $this->Jsonmsg->send();
                }
            }

            //affichage du formulaire
            if ($courrierId != null) {
                $querydata = array(
                    'contain' => array(
                        $this->Courrier->Affaire->alias => array(
                            $this->Courrier->Affaire->Dossier->alias
                        )
                    ),
                    'conditions' => array(
                        'Courrier.id' => $courrierId
                    )
                );
                $courrier = $this->Courrier->find('first', $querydata);

                if (!empty($courrier)) {
                    $this->Courrier->Affaire->recursive = 1;
                    $this->Courrier->Affaire->Dossier->recursive = 1;

                    $this->loadModel('Compteur');
                    // select nom from compteurs where id= '1'; => compteur_affaire
//                     select id from compteurs where nom ='compteur_affaire';
                    $CompteurAffaire = $this->Compteur->find(
						'first',
						array(
							'fields' => array( 'Compteur.id', 'Compteur.nom' ),
							'conditions' => array(
								'Compteur.nom' => 'compteur_affaire'
							),
							'recursive' => -1
						)
					);
					$idAffaire = $CompteurAffaire['Compteur']['id'];
					
					$CompteurDossier = $this->Compteur->find(
						'first',
						array(
							'fields' => array( 'Compteur.id', 'Compteur.nom' ),
							'conditions' => array(
								'Compteur.nom' => 'compteur_dossier'
							),
							'recursive' => -1
						)
					);
					$idDossier = $CompteurDossier['Compteur']['id'];

                    $this->set('newAffaireName', $this->Compteur->genereCompteur($idAffaire));
                    // select nom from compteurs where id= '2'; => compteur_dossier
                    $this->set('newDossierName', $this->Compteur->genereCompteur($idDossier));

                    $this->set('dossiers', $this->Courrier->Affaire->Dossier->getDossiersAffairesTree());
                    $this->set('listDossiers', $this->Courrier->Affaire->Dossier->find('list'));
                    $this->set('selectedAffaire', $courrier['Affaire']['id']);
                    $this->request->data = $courrier;
                } else {
                    throw new NotFoundException();
                }
            }
        }
    }

    /**
     * Renvoi la liste des sous-types liés au sous-type du flux en fin de circuit
     *
     * @logical-group Flux
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant du flux
     * @return void
     */
    public function reponse($id = null) {
        if ($id != null) {
            $soustype_id = $this->Courrier->field('soustype_id', array('Courrier.id' => $id));
            $soustypes_lies = $this->Courrier->Soustype->find('list', array('conditions' => array('Soustype.parent_id' => $soustype_id)));
            $this->set('soustypes_lies', $soustypes_lies);
        }
    }

    /**
     * Validation d'une étape d'un circuit
     *
     * @logical-group Flux
     * @logical-group CakeFlow
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux
     * @param integer $parent identifiant du fluw parent
     * @param integer $stype_id identifiant du sous-type
     * @param integer $insertAuto
     * @return void
     */
    public function valid($fluxId, $parent = 0, $stype_id = null, $insertAuto = 0) {
        $this->autoRender = false;
        $desktop_id = $this->Session->read('Auth.User.Courrier.Desktop.id');
        if ($this->Courrier->cakeflowExecuteNext($desktop_id, $fluxId, $parent, $stype_id, $insertAuto)) {
            $this->Session->setFlash(__d('default', 'save.ok'), 'growl');
        } else {
            $this->Session->setFlash(__d('default', 'save.error'), 'growl');
        }
    }

    /**
     * Insertion d'un flux dans un circuit
     *
     * @logical-group Flux
     * @logical-group CakeFlow
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux
     * @throws BadFunctionCallException
     * @return void
     */
    public function insert($fluxId) {
//		if ($fluxId == null) {
//			throw new BadFunctionCallException();
//		}
// debug($this->Session->read('Auth.User.Courrier.Desktop.id'));
        $isEtape = $this->Courrier->isEtape($fluxId);
        $this->autoRender = false;
        if($isEtape == true){
            if ($this->Courrier->cakeflowInsert($fluxId, $this->Session->read('Auth.User.Courrier.Desktop.id'))) {
                $this->Session->setFlash(__d('courrier', 'Courrier.insert.ok'), 'growl', array('type' => 'default'));
            } else {
                $this->Session->setFlash(__d('courrier', 'Courrier.insert.error'), 'growl', array('type' => 'error'));
            }
        }
        else {
            $this->Session->setFlash(__d('courrier', 'Courrier.insert.emptyStep'), 'growl');
        }
        $this->redirect(array('controller' => 'environnement', 'action' => 'userenv'));
    }

    /**
     * Refus d'un flux inséré dans un circuit
     *
     * @logical-group Flux
     * @logical-group CakeFlow
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant du flux
     * @throws BadFunctionCallException
     * @return void
     */
    public function refus($id = null) {
        if ($id != null) {
            $flux = $this->Courrier->find('first', array('conditions' => array('Courrier.id' => $id)));
            $this->set('courrier', $flux);
            if (!empty($this->request->data)) {
//				$this->autoRender = false;
                $msg = array(
                    'success' => false,
                    'message' => ''
                );
                $this->Courrier->begin();
                //Enregistrement du motif de refus dans le courrier
                $this->Courrier->updateAll(
                    array('Courrier.motifrefus' => "'".Sanitize::clean( $this->request->data['Notification']['message'], array( 'encode', false ) )."'"),
                    array( 'Courrier.id' => $id)
                );
//debug(__LINE__);
                //action de refus pour le traitement
                $this->Courrier->Traitement->execute('KO', $this->Session->read('Auth.User.Courrier.Desktop.id'), $flux['Courrier']['id']);
                //duplication du courrier
                $newFlux = $this->Courrier->createRefus($flux);

//debug(__LINE__);
//debug($flux['Courrier']['id']);
//debug($newFlux['Courrier']['id']);
// die();
//debug($newFlux);
//var_dump($newFlux);
                if (!empty($newFlux)) {
//					$newFlux = $this->Courrier->find('first', array('conditions' => array('Courrier.id' => $newFluxId)));
                    //on change l etat du flux dans toutes les bannettes
                    $this->Courrier->Bancontenu->cancel($flux['Courrier']['id']);
                    //on ajoute le flux dupliqué dans la bannette du rédacteur
                    if ($this->Courrier->Bancontenu->add($newFlux['Courrier']['desktop_creator_id'], BAN_REFUS, $newFlux['Courrier']['id'])) {
                        //notification au rédacteur
                        $notifyDesktops = array($flux['Courrier']['desktop_creator_id'] => array($newFlux['Courrier']['id']));
                        if ($this->Courrier->Notifieddesktop->Notification->add(__d('notification', 'ban_cancel.name'), __d('notification', 'ban_cancel.description'), BAN_REFUS, $this->Session->read('Auth.User.Courrier.Desktop.id'), $notifyDesktops)) {
                            
            
                        // Ajout pour les notifs
                        $destinataires = $this->Courrier->Traitement->whoIs($flux['Courrier']['id'], 'in', array('OK','RI','IN'));
                        $destinataires = Hash::merge( $destinataires, $flux['Courrier']['desktop_creator_id']);
//                        $destinataires = $this->Courrier->Traitement->whoIs($flux['Courrier']['id']);

                            foreach ($destinataires as $destinataire_id) {
                                $user = $this->Courrier->User->find(
                                    'first',
                                    array(
//                                        'fields' => array(
//                                            'User.id',
//                                            'User.desktop_id',
//                                            'User.mail'
//                                        ),
                                        'recursive' => -1,
                                        'conditions' => array(
                                            'User.desktop_id' => $destinataire_id
                                        )
                                    )
                                );
                                $user_id = $user['User']['id'];
                                $this->Courrier->User->notifier($newFlux['Courrier']['id'], $user_id, 'refus');
                            }


                            $msg['message'] = 'Le flux "' . $newFlux['Courrier']['name'] . '" a été refusé avec le motif suivant : <br />' . $this->request->data['Notification']['message'];
                            $msg['success'] = true;
                        } else {
                            $msg['message'] .= '<br />Notification: error';
                        }
                    } else {
                        $msg['message'] .= '<br />Bannette add: error';
                    }
                } else {
                    $msg['message'] .= '<br />Courrier duplicate: error';
                }

                if ($msg['success']) {
                    $this->Courrier->commit();
                    $this->Session->setFlash('Le flux a été refusé', 'growl');
                } else {
                    $this->Courrier->rollback();
                    if (Configure::read('debug') > 0) {
                        $msg['message'] .= '<br />Le flux n\'a pas été refusé.';
                    } else {
                        $msg['message'] = 'Le flux n\'a pas été refusé.';
                    }
                    $this->Session->setFlash($msg['message'], 'growl');
                }
            }
        }
    }

    /**
     * Envoi d'un flux en copie
     *
     * @logical-group Flux
     * @logical-group Edition de flux
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux
     * @return void
     */
     public function sendcpy() {
        if (!empty($this->request->data)) {
            //cas d envoi pour copie
            $this->Jsonmsg->init();
            if ($this->Courrier->copy($this->Session->read('Auth.User.id'), $this->request->data['Send']['flux_id'], $this->request->data['Send']['copy_id'])) {
                $success = true;
                if (!empty($this->request->data['Comment']['content'])) {
                    $success = $this->Courrier->Comment->save($this->request->data);
                }
                if (!empty($success)) {
                    $this->Jsonmsg->valid();
                }
            }
            $this->Jsonmsg->send();
        }
    }

    /**
     * Envoi d'un flux à une étape hors circuit
     *
     * @logical-group Flux
     *
     * @logical-used-by CakeFlow > Envoi d'un flux (avec ou sans rebond) à une nouvelle étape
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux
     * @return void
     */
    public function sendwkf() {
        if (!empty($this->request->data)) {
            $this->autoRender = false;
            //rebond
            $newDesktopId = $this->request->data['Send']['desktop_id'];
            $fluxId = $this->request->data['Send']['flux_id'];

            $newDesktop = $this->Courrier->Desktop->find('first', array('conditions' => array('Desktop.id' => $newDesktopId)));
            $desktopId = $this->Session->read('Auth.User.Courrier.Desktop.id');

            // initialisation des visas a ajouter au traitement
            $options = array(
                'insertion' => array(
                    '0' => array(
                        'Etape' => array(
                            'etape_nom' => $newDesktop['Desktop']['name'],
                            'etape_type' => 1
                        ),
                        'Visa' => array(
                            '0' => array(
                                'trigger_id' => $newDesktopId,
                                'type_validation' => 'V'
                            )
                        )
                    )
                )
            );
            $action = $this->request->data['Send']['getback'] ? 'IL' : 'IP';

            $success = false;
            if ($this->Courrier->cakeflowExecuteSend($action, $desktopId, $fluxId, $options)) {
                $success = true;
                if (!empty($this->request->data['Comment']['content'])) {
                    $datas = $this->request->data;
                    $datas['Comment']['owner_id'] = $this->Session->read('Auth.User.Courrier.Desktop.id');
                    $datas['Comment']['private'] = true;
                    $datas['Comment']['target_id'] = $fluxId;
                    $datas['Reader'] = array('Reader' => array($newDesktopId));
                    
                    $success = $this->Courrier->Comment->save($datas);
                    
                }
            }

            if (!empty($success)) {
                $this->Session->setFlash(__d('default', 'save.ok'), 'growl');
            } else {
                $this->Session->setFlash(__d('default', 'save.error'), 'growl');
            }
        }
    }

    /**
     *
     */
    public function send($fluxId = null, $cpyOnly = false) {
        $this->set('sendwkf', $this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/sendwkf'));
        $this->set('sendcpy', $this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/sendcpy'));
        /*if (!empty($this->request->data)) {

            //gestion des commentaires
            $this->request->data['Comment']['owner_id'] = $this->Session->read('Auth.User.Courrier.Desktop.id');
            $this->request->data['Comment']['private'] = true;
            $this->request->data['Comment']['target_id'] = $this->request->data['Send']['flux_id'];
            $this->request->data['Reader'] = array('Reader' => array($this->request->data['Send']['desktop_id']));

            if (isset($this->request->data['Send']['copy']) && $this->request->data['Send']['copy']) {
                $this->sendcpy();
            } else {
                $this->sendwkf();
            }
        } else {
            $desktopsToSend = $this->Courrier->Desktop->getDesktopsByUser('rebond');
            $desktopsToCopy = $this->Courrier->Desktop->getDesktopsByUser('all');
            $this->set('desktopsToSend', $desktopsToSend);
            $this->set('desktopsToCopy', $desktopsToCopy);
            $this->set('cpyOnly', $cpyOnly);
            $this->set('fluxId', $fluxId);
        }*/

        if (!empty($this->request->data)) {
            
            if (isset($this->request->data['Send']['copy']) && $this->request->data['Send']['copy']) {
                $valid = array();
                $this->Jsonmsg->init();
                $copyIds = Hash::extract( $this->request->data, 'Send.copy_id.{n}' );
                foreach( $copyIds as $key => $value) {
                    $success = true;
//                    debug($this->request->data);
                    $content = Hash::get( $this->request->data, 'Comment.content');
                    if( !empty( $content ) ) {
                        $datas = $this->request->data;
                        $datas['Comment']['owner_id'] = $this->Session->read('Auth.User.Courrier.Desktop.id');
                        $datas['Comment']['private'] = true;
                        $datas['Comment']['target_id'] = $datas['Send']['flux_id'];
                        $datas['Reader'] = array('Reader' => $value);

                        $success = $this->Courrier->Comment->save($datas);
                        if (!empty($success)) {
                            $this->Jsonmsg->valid();
                        }  
                    }
                    else {
                        $this->Jsonmsg->valid();
                    }
                    
                    // Envoi des notifications pour chacun des bureaux en copie
                    $destId = array();
                    $notifInfos = $this->Courrier->Notifieddesktop->Notification->setNotification('ban_copy');
                    $desktopId = $this->Session->read('Auth.User.desktop_id');
                    $fluxId = $this->request->data['Send']['flux_id'];

                    $notifyDesktops = array(
                        $value => array( $fluxId )
                    );                     
                    $notified = $this->Courrier->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId, $notifyDesktops);

                    // On ajoute le flux dans la bannette copie
                    $valid[$this->request->data['Send']['flux_id']] = array(
                        'add' => $this->Courrier->Bancontenu->add($value, BAN_COPY, $this->request->data['Send']['flux_id'])
                    );
                    
//                    var_dump($notifyDesktops);
                    // Envoi du mail au destinataire présent
                    $User = ClassRegistry::init( 'User' );
                    $user = $this->User->find(
                        'first',
                        array(
                            'recursive' => -1,
                            'conditions' => array(
                                'OR' => array(
                                    'User.desktop_id' => $value,
                                    'SecondaryDesktop.id' => $value,
                                )
                            ),
                           'joins' => array(
                                $this->User->join( 'Desktop', array( 'type' => 'INNER' ) ),
                                $this->User->Desktop->join( 'Group', array( 'type' => 'INNER' ) ),
                                $this->User->join( 'DesktopsUser', array( 'type' => 'LEFT OUTER' ) ),
                                alias_querydata(
                                    $this->User->DesktopsUser->join( 'Desktop', array( 'type' => 'LEFT OUTER' ) ),
                                    array( 'Desktop' => 'SecondaryDesktop' )
                                ),
                                alias_querydata(
                                    $this->User->DesktopsUser->Desktop->join( 'Group', array( 'type' => 'LEFT OUTER' ) ),
                                    array( 'Desktop' => 'SecondaryDesktop', 'Group' => 'SecondaryGroup' )
                                )
                            )
                        )
                    );
//        var_dump($user);
                    $user_id = $user['User']['id'];
                    $this->User->notifier($this->request->data['Send']['flux_id'], $user_id, 'copy');

                }
                $this->Jsonmsg->send();
                
            }
            // envoi à la signature manuelle
            // ajout d'une étape "fictive"
            else if (isset($this->request->data['Send']['signature_manuelle']) && $this->request->data['Send']['signature_manuelle']) {
//                $this->request->data['Send']['getback'] = fa;
                $this->request->data['Send']['desktop_id'] = $this->Session->read('Auth.User.Courrier.Desktop.id');
                $this->sendwkfsignaturemanuelle($this->request->data);
            }
            else {
                $this->sendwkf();
            }     
        } else {
//            $desktopsToSend = $this->Courrier->Desktop->getDesktopsByUser('rebond');
            $desktopsToCopy = $this->Courrier->Desktop->getDesktopsByUser('all');
            
            $allDesktops = $this->Courrier->Desktop->getDesktopsByService( 'all' );
            $desktopsToSend = $this->Courrier->Desktop->getDesktopsByService( 'rebond' );
//            debug($allDesktops);
            $this->set('desktopsToSend', $desktopsToSend);
            $this->set('desktopsToCopy', $desktopsToCopy);
            $this->set('allDesktops', $allDesktops);
            $this->set('cpyOnly', $cpyOnly);
            $this->set('fluxId', $fluxId);
        }
 
    }

    /**
     * Envoi d'un flux à un aiguilleur
     *
     * @logical-group Flux
     * @user-profile User
     * @user-profile Aiguilleur
     *
     * @access public
     * @return void
     */
    public function dispsend() {
        $this->autoRender = false;
        if (!empty($this->request->data)) {
            $desktopId = $this->request->data['DispForm']['initDesktopService'];
            $flux = array();
            foreach ($this->request->data['DispForm'] as $key => $val) {
                if ($key != 'initDesktopService') {
                    if ($val == true) {
                        $tmp = explode('_', $key);
                        if ($tmp[0] == 'checkItem') {
                            $flux[] = $tmp[1];
                        }
                    }
                }
            }

            $groupId = $this->Courrier->Bancontenu->Desktop->field('Desktop.group_id', array('Desktop.id' => $desktopId));
            $dispDesktops = array_keys($this->getSessionDispDesktops());
            $senderDispId = $dispDesktops[0];
            $notifyDesktops = array($desktopId => $flux);
            if ($groupId == DISP_GID) {
                $valid = array();
                for ($i = 0; $i < count($flux); $i++) {
                    $valid[$flux[$i]] = array(
                        'end' => $this->Courrier->Bancontenu->end($senderDispId, BAN_AIGUILLAGE, $flux[$i]),
                        'add' => $this->Courrier->Bancontenu->add($desktopId, BAN_AIGUILLAGE, $flux[$i])
                    );
                }
                //TODO: gérer les notifications
//				$notifInfos = $this->Courrier->Notifieddesktop->Notification->setNotification('ban_xnew');
//				$this->Courrier->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $this->Session->read('Auth.User.Courrier.Desktop.id'), $notifyDesktops);
            } else if ($groupId == INIT_GID) {
                $valid = array();
                for ($i = 0; $i < count($flux); $i++) {
                    $courrier = $this->Courrier->find('first', array('conditions' => array('Courrier.id' => $flux[$i])));
                    $courrier['Courrier']['desktop_creator_id'] = $desktopId;
                    $this->Courrier->create();
                    $this->Courrier->save($courrier);
                    $valid[$flux[$i]] = array(
                        'end' => $this->Courrier->Bancontenu->end($senderDispId, BAN_AIGUILLAGE, $flux[$i]),
                        'add' => $this->Courrier->Bancontenu->add($desktopId, BAN_DOCS, $flux[$i])
                    );
                }
                $notifInfos = $this->Courrier->Notifieddesktop->Notification->setNotification('ban_xnew');
                $this->Courrier->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $this->Session->read('Auth.User.Courrier.Desktop.id'), $notifyDesktops);
            }
        }
    }

    /**
     * Suppression d'un flux
     *
     * @logical-group Flux
     * @user-profile User
     *
     * @param integer $id identifiant du flux
     * @throws NotFoundException
     */
    public function delete($id = null) {
        $this->Courrier->id = $id;
        if (!$this->Courrier->exists()) {
            throw new NotFoundException();
        }

        $deleteAll = array();
        $bancontenus = array_keys($this->Courrier->Bancontenu->find('list', array('conditions' => array('Bancontenu.courrier_id' => $id))));
        $this->Courrier->begin();

        for ($i = 0; $i < count($bancontenus); $i++) {
            $deleteAll[] = $this->Courrier->Bancontenu->delete($bancontenus[$i]);
        }
        $deleteAll[] = $this->Courrier->delete($id);

        if (!in_array(false, $deleteAll, true)) {
            $this->Courrier->commit();
            $this->Session->setFlash(__d('courrier', 'Courrier deleted', true), 'growl');
        } else {
            $this->Courrier->rollback();
            $this->Session->setFlash(__d('courrier', 'Courrier was not deleted', true), 'growl');
        }
        $this->redirect(array('controller' => 'environnement', 'action' => 'index'));
    }

    /**
     *
     * @param type $name
     * @param type $ext
     */
    //TODO: supprimer les fonctions commentées
//	public function download($name, $ext) {
//		$this->viewClass = 'Media';
//		$params = array(
//			'id' => $name . "." . $ext,
//			'name' => $name,
//			'download' => true,
//			'extension' => $ext
//		);
//		if (is_file(ASSOCIATED_DIR . $name . "." . $ext)) {
//			$params['path'] = ASSOCIATED_DIR;
//		}
//		if (is_file(GENERATED_DIR . $name . "." . $ext)) {
//			$params['path'] = GENERATED_DIR;
//		}
//		if (is_file(VIDEO_DIR . $name . "." . $ext)) {
//			$params['path'] = VIDEO_DIR;
//		}
//		if (is_file(WAV_DIR . $name . "." . $ext)) {
//			$params['path'] = WAV_DIR;
//		}
//		if ($ext == "odt") {
//			$params['mimeType'] = array('odt' => "application/vnd.oasis.opendocument.text");
//		}
//		$this->set($params);
//	}

    /**
     * Vérification de la possibilité d'insertion d'un flux dans un circuit à la fin du wizard de création
     *
     * @logical-group Flux
     * @logical-group Création de flux
     * @user-profile User
     *
     * @access public
     * @param integer $id identifiant du flux
     * @return void
     */
    public function getInsertEnd($id) {
        $this->Courrier->id = $id;
        $courrier = $this->Courrier->find(
                'first', array(
            'fields' => array(
                'Courrier.id',
                'Courrier.soustype_id',
                'Soustype.id',
                'Soustype.circuit_id',
                'Document.id'
            ),
            'conditions' => array(
                'Courrier.id' => $id
            ),
            'joins' => array(
                $this->Courrier->join('Soustype', array('type' => 'LEFT OUTER')),
                $this->Courrier->join('Document', array('type' => 'LEFT OUTER'))
            ),
            'contain' => false
                )
        );

        // Le circuit possède-t-il comme prochaine étape une délégation vers le parapheur ?
        $hasEtapeParapheur = $this->Courrier->Traitement->Circuit->hasEtapeDelegation($courrier['Soustype']['circuit_id']);
        $this->set( compact('hasEtapeParapheur'));
//    debug($hasEtapeParapheur);
        $soustypeDefined = false;
        $canInsert = false;
        if (!empty($courrier['Courrier']["soustype_id"])) {
            $soustypeDefined = true;

            if (!empty($courrier['Soustype']["circuit_id"])) {
                $canInsert = true;
            }
            if ($hasEtapeParapheur) {
                if (!empty($courrier['Document']['id'])) {   
                    $canInsert = true;
                }
                else {
                    $canInsert = false;
                }
            }
        }

        $this->set('courrier', $courrier);
        $this->set('soustypeDefined', $soustypeDefined);
        $this->set('canInsert', $canInsert);
    }

    /**
     * Récupération de la liste des accusés de réception
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function getArs($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}

        $qdFlux = array(
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'contain' => array(
                'Soustype' => array(
                    'Armodel' => array(
                        'fields' => $this->Courrier->Soustype->Armodel->getLightFields(),
                        'order' => array('Armodel.name')
                    )
                )
            )
        );
        $flux = $this->Courrier->find('first', $qdFlux);

        if (empty($flux)) {
            throw new NotFoundException();
        }

        $qdAr = array(
            'fields' => $this->Courrier->Ar->getLightFields(),
            'recursive' => -1,
            'conditions' => array(
                'Ar.courrier_id' => $fluxId
            ),
            'order' => 'Ar.created DESC'
        );
        $ar = $this->Courrier->Ar->find('first', $qdAr);

        $this->set('ar', $ar);
        $this->set('armodels', !empty($flux['Soustype']['Armodel']) ? $flux['Soustype']['Armodel'] : array());
        $this->set('fluxId', $fluxId);

        $this->set('fromForbiddenLocation', $this->_isFromFobiddenLocation($flux));
    }

    /**
     * Génération d'un accusé de réception
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function setArs($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}

        $qdFlux = array(
            'fields' => array('Courrier.id'),
            'recursive' => -1,
            'conditions' => array(
                'Courrier.id' => $fluxId
            )
        );
        $flux = $this->Courrier->find('first', $qdFlux);
        if (empty($flux)) {
            throw new NotFoundException();
        }

        $qdArmodel = array(
            'recursive' => -1,
            'conditions' => array(
                'Armodel.id' => $this->request->data['Armodel']['choice']
            )
        );
        $armodel = $this->Courrier->Soustype->Armodel->find('first', $qdArmodel);
        if (empty($armodel)) {
            throw new NotFoundException();
        }

        if (!empty($this->request->data)) {
            $this->Jsonmsg->init(__d('ar', 'Ar.generated.error'));
            $this->Courrier->Ar->begin();
            $this->Courrier->Ar->deleteAll(array('Ar.courrier_id' => $fluxId));
            $ar = array(
                'Ar' => array(
                    'name' => $armodel['Armodel']['name'] . " " . date('YmdHis'),
                    'format' => $armodel['Armodel']['format'],
                    'email_title' => $armodel['Armodel']['email_title'],
                    'email_content' => $armodel['Armodel']['email_content'],
                    'sms_content' => $armodel['Armodel']['sms_content'],
                    'courrier_id' => $fluxId
                )
            );

            if ($armodel['Armodel']['format'] == "ott") {
                $gedoooResult = $this->_generate($fluxId, array('name' => $armodel['Armodel']['name'], 'content' => $armodel['Armodel']['content']), null, $this->Session->read('Auth.User.id'));
                $ar['Ar']['content'] = $gedoooResult['content'];
                $ar['Ar']['size'] = $gedoooResult['size'];
                $ar['Ar']['mime'] = $gedoooResult['mime'];
                $ar['Ar']['ext'] = $gedoooResult['ext'];
            } else if ($armodel['Armodel']['format'] == "video" || $armodel['Armodel']['format'] == "audio") {
                //TODO: prendre en charge la gestion de la ged
                $ar['Ar']['content'] = $armodel['Armodel']['content'];
                $ar['Ar']['size'] = $armodel['Armodel']['size'];
                $ar['Ar']['mime'] = $armodel['Armodel']['mime'];
                $ar['Ar']['ext'] = $armodel['Armodel']['ext'];
            }

            $this->Courrier->Ar->create();
            $saved = $this->Courrier->Ar->save($ar);
            if (!empty($saved)) {
                $this->Courrier->Ar->commit();
                $this->Jsonmsg->valid(__d('ar', 'Ar.generated.ok'));
            } else {
                $this->Courrier->Ar->rollback();
            }
            $this->Jsonmsg->send();
        }
    }

    /**
     * Génération d'un document réponse à partir du modèle de document initial et des éléments (webdav) de réponse (composants de réponse)
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function genResponse() {
        if ($this->request->data) {
            $this->Jsonmsg->init();
            $fluxId = $this->request->data['ResponseDoc']['courrier_id'];
            $rmodelId = $this->request->data['ResponseDoc']['rmodel_id'];

            $this->Courrier->Soustype->Rmodel->id = $rmodelId;
            $rmodel = array(
                'name' => $this->Courrier->Soustype->Rmodel->field('name', array('id' => $rmodelId)),
                'content' => $this->Courrier->Soustype->Rmodel->getFileContent($rmodelId)
            );

            $relements = array();
            foreach ($this->request->data['ResponseDoc']['Relement'] as $key => $val) {
                if ($val == 1) {
                    $name = $this->Courrier->Relement->field('Relement.name', array('Relement.id' => $key));

                    $tab = array(
                        'name' => $name,
                        'content' => file_get_contents(WEBDAV_DIR . DS . $fluxId . DS . $name)
                    );
                    $relements[] = $tab;
                }
            }

            $document = array(
                'Document' => array(
                    'name' => 'Document_généré_' . date('YmdHis') . ".odt",
                    'courrier_id' => $fluxId
                )
            );
            $gedoooResult = $this->_generate($fluxId, $rmodel, $relements, $this->Session->read('Auth.User.id'));
            if (!empty($gedoooResult)) {
                $document['Document']['content'] = $gedoooResult['content'];
                $document['Document']['size'] = $gedoooResult['size'];
                $document['Document']['mime'] = $gedoooResult['mime'];
                $document['Document']['ext'] = $gedoooResult['ext'];
                // Passage en doc principal pour les modèles de réponse générés
                // car ils finiront par être adressés au parapheur
                $document['Document']['main_doc'] = true;

                $docPrincipalFlux = $this->Courrier->getMainDoc($fluxId);
                if( !empty($docPrincipalFlux)) {
                    $this->Courrier->Document->updateAll(
                        array( 'Document.main_doc' => false),
                        array( 'Document.id' => $docPrincipalFlux['Document']['id'])
                    );
                    
                }
                $this->Courrier->Document->create($document);
                $docSaved = $this->Courrier->Document->save();
                if (!empty($docSaved)) {
                    $this->Jsonmsg->valid();
                }
            }
            $this->Jsonmsg->send();
        }
    }

    /**
     * Génération de document odt via GED'OOo
     *
     * @access private
     * @param integer $courrier_id identifiant du flux
     * @param integer $soustype_id identifiant du sous-type
     * @return boolean
     */
    private function _generate($courrier_id, $model, $elements = null, $user_id) {
        $return = array();

        //generation du fichier avec GedOOo
        App::import('Vendor', 'GDO_Utility', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_Utility.class'));
        App::import('Vendor', 'GDO_FieldType', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_FieldType.class'));
        App::import('Vendor', 'GDO_ContentType', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_ContentType.class'));
        App::import('Vendor', 'GDO_IterationType', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_IterationType.class'));
        App::import('Vendor', 'GDO_PartType', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_PartType.class'));
        App::import('Vendor', 'GDO_FusionType', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_FusionType.class'));
        App::import('Vendor', 'GDO_MatrixType', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_MatrixType.class'));
        App::import('Vendor', 'GDO_MatrixRowType', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_MatrixRowType.class'));
        App::import('Vendor', 'GDO_AxisTitleType', array('file' => 'GEDOOo' . DS . 'phpgedooo' . DS . 'GDO_AxisTitleType.class'));

        $u = new GDO_Utility();
//      $oMainPart->addElement(new GDO_FieldType('collectivite_nom', $collectivite['Collectivite']['name'], "text"));
//		$oMainPart->addElement(new GDO_FieldType('collectivite_adresse', utf8_encode($collectivite['Collectivite']['adresse']), "text"));
//		$oMainPart->addElement(new GDO_FieldType('collectivite_codepostal', utf8_encode($collectivite['Collectivite']['codepostal']), "text"));
//		$oMainPart->addElement(new GDO_FieldType('collectivite_ville', utf8_encode($collectivite['Collectivite']['ville']), "text"));
//		$oMainPart->addElement(new GDO_FieldType('collectivite_telephone', utf8_encode($collectivite['Collectivite']['telephone']), "text"));
        //*****************************************
        // Organisation des données
        //*****************************************
        $oMainPart = new GDO_PartType();

        $querydata = array(
            'fields' => array_merge(
                $this->Courrier->fields(),
                $this->Courrier->Contactinfo->fields(),
                $this->Courrier->Contactinfo->Contact->fields(),
                $this->Courrier->Soustype->fields(),
                $this->Courrier->Service->fields(),
                $this->Courrier->Soustype->Type->fields(),
                $this->Courrier->Soustype->Circuit->fields(),
                $this->Courrier->Affairesuivie->fields(),
                $this->Courrier->Affairesuivie->User->fields(),
                array(
                    'Courrier.name',
                    'Courrier.objet',
                    'Courrier.date',
                    'Courrier.datereception',
                    'Courrier.reference',
                    'Contactinfo.name',
                    'Contact.name',
                    'Type.name',
                    'Soustype.name',
                    'Soustype.delai_nb',
                    'Soustype.delai_unite',
                    'Circuit.nom'
                )
            ),
            'contain' => false,
            'joins' => array(
                $this->Courrier->join($this->Courrier->Contactinfo->alias),
                $this->Courrier->Contactinfo->join($this->Courrier->Contactinfo->Contact->alias),
                $this->Courrier->join($this->Courrier->Soustype->alias),
                $this->Courrier->join($this->Courrier->Service->alias),
                $this->Courrier->Soustype->join($this->Courrier->Soustype->Type->alias),
                $this->Courrier->Soustype->join($this->Courrier->Soustype->Circuit->alias),
                $this->Courrier->join($this->Courrier->Affairesuivie->alias),
                $this->Courrier->Affairesuivie->join($this->Courrier->Affairesuivie->User->alias)
            ),
            'conditions' => array(
                'Courrier.id' => $courrier_id
            )
        );
        $courrier = $this->Courrier->find('first', $querydata);
        
        $courrier['Affairesuivie']['name'] = $courrier['User']['prenom'].' '.$courrier['User']['nom'];
//debug($courrier);
//die();
        $metadonnees = $this->Courrier->CourrierMetadonnee->find(
                'all', array(
            'fields' => array_merge(
                    $this->Courrier->CourrierMetadonnee->fields(), $this->Courrier->CourrierMetadonnee->Metadonnee->fields()
            ),
            'conditions' => array(
                'CourrierMetadonnee.courrier_id' => $courrier_id
            ),
            'joins' => array(
                $this->Courrier->CourrierMetadonnee->join('Metadonnee', array('type' => 'LEFT OUTER'))
            ),
            'recursive' => -1
                )
        );
        if (!empty($metadonnees)) {
            $courrier['Courrier']['listemetadonnees'] = implode(', ', Hash::extract($metadonnees, '{n}.Metadonnee.name'));
        }

        // Récupération de l'utilisateur connecté
        $user = $this->Courrier->User->find(
            'first',
            array(
                'fields' => array(
                    'User.username',
                    'User.nom',
                    'User.prenom',
                    'User.mail',
                    'User.numtel',
                    'Desktop.name',
                    'Service.name'
                ),
                'conditions' => array(
                    'User.id' => $user_id
                ),
                'joins' => array(
                    $this->Courrier->User->join( 'Desktop' ),
                    $this->Courrier->User->Desktop->join( 'DesktopsService' ),
                    $this->Courrier->User->Desktop->DesktopsService->join( 'Service' )
                ),
                'contain' => false
            )
        );
        $courrier = Hash::merge( $courrier, $user);
        // Infos sur la collectivité
        $collectivite = ClassRegistry::init('Collectivite')->find(
                'first', array(
            'fields' => array(
                'Collectivite.name',
                'Collectivite.logo',
                'Collectivite.siren',
                'Collectivite.adresse',
                'Collectivite.codeinsee',
                'Collectivite.codepostal',
                'Collectivite.ville',
                'Collectivite.complementadresse',
                'Collectivite.telephone'
            ),
            'conditions' => array(
                'Collectivite.conn' => $this->Session->read('Auth.User.connName')
            ),
            'contain' => false
                )
        );
//debug($courrier);
//die();
        // Information sur le courrier envoyée à Gedooo
        $oMainPart->addElement(new GDO_FieldType('courrier_name', $courrier['Courrier']['name'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('courrier_objet', $courrier['Courrier']['objet'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('courrier_reference', $courrier['Courrier']['reference'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('affairesuivie_name', $courrier['Affairesuivie']['name'], 'text'));

        $date_tmp = explode("-", $courrier['Courrier']['date']);
        $date = $date_tmp[2] . "/" . $date_tmp[1] . "/" . $date_tmp[0];
        $oMainPart->addElement(new GDO_FieldType('courrier_date', $date, 'date'));

        $date_reception_tmp = explode("-", $courrier['Courrier']['datereception']);
        $date_reception = $date_reception_tmp[2] . "/" . $date_reception_tmp[1] . "/" . $date_reception_tmp[0];
        $oMainPart->addElement(new GDO_FieldType('courrier_datereception', $date_reception, 'date'));

        $oMainPart->addElement(new GDO_FieldType('courrier_emetteur', $courrier['Contact']['name'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('courrier_type', $courrier['Type']['name'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('courrier_soustype', $courrier['Soustype']['name'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('courrier_circuit', $courrier['Circuit']['nom'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('service_name', $courrier['Service']['name'], 'text'));
        if (!empty($metadonnees)) {
            $oMainPart->addElement(new GDO_FieldType('courrier_metadonnee', $courrier['Courrier']['listemetadonnees'], 'text'));
        }

        // Information sur la personne ayant contacté la collectivité
        $oMainPart->addElement(new GDO_FieldType('contact_name', $courrier['Contact']['name'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_civilite', $courrier['Contactinfo']['civilite'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_nom', $courrier['Contactinfo']['nom'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_prenom', $courrier['Contactinfo']['prenom'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_numvoie', $courrier['Contactinfo']['numvoie'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_typevoie', $courrier['Contactinfo']['typevoie'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_nomvoie', $courrier['Contactinfo']['nomvoie'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_adresse', $courrier['Contactinfo']['adresse'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_compl', $courrier['Contactinfo']['compl'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_cp', $courrier['Contactinfo']['cp'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_ville', $courrier['Contactinfo']['ville'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_region', $courrier['Contactinfo']['region'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_pays', $courrier['Contactinfo']['pays'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_tel', $courrier['Contactinfo']['tel'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_organisation', $courrier['Contactinfo']['organisation'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('contactinfo_role', $courrier['Contactinfo']['role'], 'text'));

        // Information sur l'utilsiateur connecté
        $oMainPart->addElement(new GDO_FieldType('user_username', $user['User']['username'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('user_nom', $user['User']['nom'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('user_prenom', $user['User']['prenom'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('user_mail', $user['User']['mail'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('user_numtel', $user['User']['numtel'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('desktop_name', $user['Desktop']['name'], 'text'));

        // Information sur la collectivité de l'utilsiateur
        $oMainPart->addElement(new GDO_FieldType('collectivite_name', $collectivite['Collectivite']['name'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('collectivite_logo', $collectivite['Collectivite']['logo'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('collectivite_siren', $collectivite['Collectivite']['siren'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('collectivite_adresse', $collectivite['Collectivite']['adresse'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('collectivite_codeinsee', $collectivite['Collectivite']['codeinsee'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('collectivite_codepostal', $collectivite['Collectivite']['codepostal'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('collectivite_ville', $collectivite['Collectivite']['ville'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('collectivite_complementadresse', $collectivite['Collectivite']['complementadresse'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('collectivite_telephone', $collectivite['Collectivite']['telephone'], 'text'));
        
        // Information sur le type/sous type du courrier
        $delaiunite = '';
        if($courrier['Soustype']['delai_unite'] == '0') {
            $delaiunite = 'jour(s)';
        }
        else if($courrier['Soustype']['delai_unite'] == '1') {
            $delaiunite = 'semaine(s)';
        }
        if($courrier['Soustype']['delai_unite'] == '2') {
            $delaiunite = 'mois';
        }
        
        $direction = 'entrant';
        if( $courrier['Soustype']['entrant'] == '0') {
            $direction = 'sortant';
        }
        $oMainPart->addElement(new GDO_FieldType('type_name', $courrier['Type']['name'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('soustype_name', $courrier['Soustype']['name'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('soustype_delai_nb', $courrier['Soustype']['delai_nb'], 'text'));
        $oMainPart->addElement(new GDO_FieldType('soustype_delai_unite', $delaiunite, 'text'));
        $oMainPart->addElement(new GDO_FieldType('soustype_direction', $direction, 'text'));
        

        //inclusion des fichiers webdav
        if (!empty($elements)) {
            $documents = new GDO_IterationType("documents");
            foreach ($elements as $element) {
                $oDevPart = new GDO_PartType();
                $oDevPart->addElement(new GDO_ContentType("document", "", "application/vnd.oasis.opendocument.text", "binary", $element['content']));
                $documents->addPart($oDevPart);
            }
            $oMainPart->addElement($documents);
        }

        //On charge le fichier
        //TODO: prevoir la gestion de la ged
        $oTemplate = new GDO_ContentType("", Inflector::slug($model['name']) . $courrier_id . "_" . date('YmdHis') . ".odt", "application/vnd.oasis.opendocument.text", "binary", $model['content']);

        
        
        // Debug: plugin Gedooo2
        // Gedooo
        /*CakePlugin::load('Gedooo2');
        App::uses( 'Gedooo2Debugger', 'Gedooo2.Utility' );
        Configure::write(
            'Gedooo2',
            array(
                'debugHashPathsToCsv' => true,
                'debugAllPathsToCsv' => true,
                'debugAllPathsValuesToCsv' => true,
            )
        );
        debug( Gedooo2Debugger::hashPathsToCsv( $oMainPart ) );
        debug( Gedooo2Debugger::allPathsToCsv( $oMainPart, Configure::read( 'Gedooo2.debugAllPathsValuesToCsv' ) === true ) );
        die(); */
        
        //Fusion
        $oFusion = new GDO_FusionType($oTemplate, "application/vnd.oasis.opendocument.text", $oMainPart);
        $oFusion->process();

        $success = ( $oFusion->getCode() == 'OK' );
        if ($success) {
            $content = $oFusion->getContent();
            $return['content'] = $content->binary;
            $return['size'] = strlen($return['content']);
            $return['ext'] = 'odt';
            $return['mime'] = 'application/vnd.oasis.opendocument.text';
        }


        return $return;
    }

    /**
     * Récupération du panneau contextuel
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function getContext($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}

        $qdFlux = array(
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'contain' => array(
                'Document' => array(
                    'conditions' => array('Document.main_doc' => false)
                )
            ),
            'fields' => array('Courrier.id')
        );
        $flux = $this->Courrier->find('first', $qdFlux);

        if (empty($flux)) {
            throw new NotFoundException();
        }

//		TODO: utiliser $this->Acl->check
        $rights = array('getDocuments' => ($this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/getDocuments') ? true : false));

        $this->set('mainDoc', $this->Courrier->getMainDoc($fluxId));
        $this->set('rights', $rights);
        $this->set('fluxId', $fluxId);
        $this->set('documents', $flux['Document']);


//		BAN_AIGUILLAGE
        $qd = array(
            'recursive' => -1,
            'conditions' => array(
                'Bancontenu.bannette_id' => BAN_AIGUILLAGE,
                'Bancontenu.courrier_id' => $fluxId,
                'Bancontenu.desktop_id' => $this->SessionTools->getDesktopList(),
                'Bancontenu.etat' => 1
            )
        );
        $result = $this->Courrier->Bancontenu->find('first', $qd);
        if (!empty($result) && count($result) == 1) {
            $this->Courrier->Bancontenu->setRead($result['Bancontenu']['courrier_id'], $result['Bancontenu']['desktop_id']);
        }
    }

    /**
     * Récupération des composants de réponse
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function getRelements($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}
        $qdFlux = array(
            'recursive' => -1,
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'fields' => array(
                'Courrier.id',
                'Courrier.soustype_id'
            )
        );
        $flux = $this->Courrier->find('first', $qdFlux);

        if (empty($flux)) {
            throw new NotFoundException();
        }

        $qdRmodels = array(
            'recursive' => -1,
            'fields' => $this->Courrier->Soustype->Rmodel->getLightFields(),
            'conditions' => array(
                'Rmodel.soustype_id' => $flux['Courrier']['soustype_id']
            )
        );
        $rmodels = $this->Courrier->Soustype->Rmodel->find('all', $qdRmodels);
        $this->set('rmodels', $rmodels);

//
//        $qdRelements = array(
//            'recursive' => -1,
//            'conditions' => array(
//                'Relement.courrier_id' => $flux['Courrier']['id']
//            )
//        );
//        $relements = $this->Courrier->Relement->find('all', $qdRelements);


        // 2. Recherche des fichiers déjà liés*
        $qdRelements = array(
            'recursive' => -1,
            'conditions' => array(
                'Relement.courrier_id' => $flux['Courrier']['id']
            ),
            'fields' => array(
                'Relement.id',
                'Relement.name',
                '( \''.WEBDAV_DIR . DS.'\' || "Relement"."courrier_id" || \''.DS.'\' || "Relement"."name" ) AS "Relement__path"'
            )
        );
        $relements = $this->Courrier->Relement->find('all', $qdRelements);

        
        // Ajouter un modèle ODT "type" par défaut
        $fichiers = array();
        $modeleDir = WEBDAV_DIR . DS . 'modeles';
        $Folder = new Folder( $modeleDir, false, 0777 );
        foreach( $Folder->find('.+\.odt$') as $i => $fileName ) {
            $fichiers[$i] = array(
                'id' => '',
                'name' => $fileName,
                'path' => $modeleDir . DS . $fileName
            );
            $relementsName = Hash::extract($relements, '{n}.Relement.name');
            if( in_array( $fichiers[$i]['name'], $relementsName ) ) {
                unset($fichiers[$i]['name']);
            }
        }
        $fichiersName = Hash::extract($fichiers, '{n}.name');
        $this->set('fichiersName', $fichiersName);
        $sessionId = $this->Session->id();
        $this->set('sessionId', $sessionId);
        
        $aro = array('model' => 'Desktop', 'foreign_key' => $this->Session->read('Auth.User.Desktop.id'));
        $this->set('right_upload', $this->Acl->check($aro, 'controllers/Relements/add') && !$this->_isFromFobiddenLocation($flux));
        
        $inCircuit = $this->Courrier->isInCircuit($fluxId);
        $this->set('right_delete', $this->Acl->check($aro, 'controllers/Relements/delete') && !$this->_isFromFobiddenLocation($flux) && !$inCircuit);
        $this->set('right_sendmail', $this->Acl->check($aro, 'controllers/Courriers/sendmail') && !$this->_isFromFobiddenLocation($flux) );
        $this->set('right_sendged', $this->Acl->check($aro, 'controllers/Courriers/sendDocumentToGed') && !$this->_isFromFobiddenLocation($flux) );
        $this->set('relements', $relements);
        $this->set('fichiers', $fichiers);
        $this->set('fluxId', $fluxId);
    }

    /**
     * Vérification de la présence du flux dans une bannette ne donnant pas lieu à la modification du flux (Mes services, En copie, Historique)
     *
     * @param type $flux
     * @return boolean
     */
    private function _isFromFobiddenLocation($flux) {
        $fromMesServices = false;
        $fluxMesServices = $this->Session->read('Auth.User.Courrier.mesServices');
        if (!empty($fluxMesServices)) {
            if (in_array($flux['Courrier']['id'], $fluxMesServices) && !in_array($this->Session->read('Auth.User.Courrier.Desktop.id'), $this->SessionTools->getDesktopList())) {
                $fromMesServices = true;
            }
        }
        $fromHistorique = preg_match('#historique$#', $_SERVER['HTTP_REFERER']);
        $fromBannetteCopy = $this->_inBannetteCopy($flux);
        
        return $fromHistorique || $fromBannetteCopy || $fromBannetteCopy;
    }

    /**
     * Récupération du document principal et des pièces jointes d'un flux
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function getDocuments($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}
        $qdFlux = array(
            'recursive' => -1,
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'fields' => array(
                'Courrier.id'
            )
        );
        $flux = $this->Courrier->find('first', $qdFlux);

        if (empty($flux)) {
            throw new NotFoundException();
        }

        $documents = $this->Courrier->getDocs($flux['Courrier']['id']);
        $desktopCreatorId = array();
        foreach( $documents as $i => $document) {
            $desktopCreatorId[] = $document['Document']['desktop_creator_id'];
        }
        $this->set('documents', $documents );
                
        //document principal
        $this->set('mainDoc', $this->Courrier->getMainDoc($flux['Courrier']['id']));
        
        
        $this->set('documentssignes', $this->Courrier->getDocsSignes($flux['Courrier']['id']));
        $this->set('fluxId', $fluxId);
        $aro = array('model' => 'Desktop', 'foreign_key' => $this->Session->read('Auth.User.Desktop.id'));
        $this->set('right_upload', $this->Acl->check($aro, 'controllers/Documents/upload') && !$this->_isFromFobiddenLocation($flux));
        $inCircuit = $this->Courrier->isInCircuit($fluxId);
        $this->set('right_delete', $this->Acl->check($aro, 'controllers/Documents/delete') && !$this->_isFromFobiddenLocation($flux) && !$inCircuit);
        $this->set('right_sendmail', $this->Acl->check($aro, 'controllers/Courriers/sendmail') && !$this->_isFromFobiddenLocation($flux) );
        $this->set('right_sendged', $this->Acl->check($aro, 'controllers/Courriers/sendDocumentToGed') && !$this->_isFromFobiddenLocation($flux) );
        $this->set('right_setAsDefault', $this->Acl->check($aro, 'controllers/Documents/setMainDoc') && !$this->_isFromFobiddenLocation($flux));
        $this->set('fromDispEnv', $this->Session->read('Auth.User.Env.actual.group_id') == DISP_GID);
    }

    /**
     * Récupération des informations du flux parent dans le cas d'une réponse à un flux
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux

     * @throws NotFoundException
     * @return void
     */
    public function getParent($fluxId) {
//		if (!is_integer($fluxId) || $fluxId <= 0) {
//			throw new BadMethodCallException();
//		}
        $querydata = array(
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'contain' => array(
                'Contactinfo' => array(
                    'Contact'
                ),
                'Soustype' => array(
                    'Type'
                ),
                'Metadonnee',
                'Affaire' => array(
                    'Dossier'
                )
            )
        );
        $flux = $this->Courrier->find('first', $querydata);

        if (empty($flux)) {
            throw new NotFoundException();
        }

        $metadonnees = Hash::extract($flux, 'Metadonnee.{n}');
        $this->set(compact('metadonnees'));

        // Ajout pour récupérer les documents liés au flux d'origine
        $this->set('documents', $this->Courrier->getDocs($flux['Courrier']['id']));
        $this->set('mainDoc', $this->Courrier->getMainDoc($flux['Courrier']['id']));
        $aro = array('model' => 'Desktop', 'foreign_key' => $this->Session->read('Auth.User.Desktop.id'));
        $this->set('right_setAsDefault', $this->Acl->check($aro, 'controllers/Documents/setMainDoc') && !$this->_isFromFobiddenLocation($flux));

        $this->set('flux', $flux);
    }

    /**
     *
     * @throws NotFoundException
     *
     * exmeple de crontab :
     * 		0 13 * * * /var/www/webgfc/lib/Cake/Console/cake --app /var/www/webgfc/app CreateFromFiles process
     *
     */
    public function acquisition() { //TODO: revoir le principe d acquisition (mettre le code du shell dans le modele)
        if (!$this->Session->read('Auth.User.id')) {
            throw new NotFoundException();
        }

        $this->loadModel('Collectivite');
        $this->Collectivite->useDbConfig = 'default';
        $scan_desktop_id = $this->Collectivite->field('scan_desktop_id', array('conn' => $this->Session->read('Auth.User.connName')));
        $desktops = $this->SessionTools->getDesktopList();

        if (!in_array($scan_desktop_id, $desktops)) {
            throw new NotFoundException();
        }

//		App::uses('ShellDispatcher', 'Console');
//		$msg = __d('courrier', 'import.error');
//		$command = CAKE . 'Console/cake -app ' . APP . ' CreateFromFiles process -c ' . $this->Session->read('Auth.User.connName') . ' --no-color';
//		exec(escapeshellcmd($command), $output, $return_var);
//		debug($output);
//		debug($return_var);
//		if ($return_var == 0) {
//			$msg = __d('courrier', 'import.success');
//		}
//		$this->Session->setFlash($msg);
//		$this->redirect('/environnement');
    }
    /**
     * Avant Ré-aiguillage d'un flux à un aiguilleur ou un initiateur, vérifier 
     * si ce flux est déjà inserer dans un circuit
     *
    
     */
    
    public function isInTraite(){
        /*
        $inCircuit=$this->Courrier->isInCircuit($fluxId);
        return json_encode(array('inCircuit'=>$inCircuit));
         * 
         */
        
       
        
        $this->autoRender = false;
        $fluxId=$this->request->data('fluxId');
        $inCircuit=$this->Courrier->isInCircuit($fluxId);
        /*
        $test = array(
            "foo" => "bar",
            "bar" => "foo",
            "test" => $inCircuit,
        );
        echo json_encode($test);
         * 
         */
         return json_encode(array('inCircuit'=>$inCircuit));
        //echo 'what';
        //exit;
    }
    
    /**
     * Réaliser un list de fluxId en traitement
     *
    
     */
    
    public function fluxIdInTraite(){
        $this->autoRender = false;
        $listFluxId =array_values($this->Courrier->Traitement->find( 'list', array( 'fields' => array( 'Traitement.target_id'), 'contain' => false) ));
        echo json_encode($listFluxId);
         
        //echo json_encode(array('test'=>'test'));
    }
    
    /**
     * Ré-aiguillage d'un flux à un aiguilleur ou un initiateur, depuis l'initiateur
     *
     * @logical-group Flux
     * @user-profile User
     * @user-profile Initiateur
     *
     * @access public
     * @return void
     */
    public function sendBackToDisp() {

        // Liste des services concernés (aiguilleur + initiateur)
        $this->loadModel('Desktop');
        $this->Desktop->setDataSource(Configure::read('conn'));
        $this->set('userId', $this->Session->read('Auth.User.id'));

        $inits = $this->Desktop->getInitsByService();
        $disps = $this->Desktop->getDispsByService();
//FIXME: pourquoi l'aiguilleur s.vilardell n'apparaît pas dans la liste des aiguilleurs lorsqu'on est connecté en b.jauberthie(?)
        $desktops = $this->SessionTools->getDesktopList();
        $otherDisps = array();
        foreach ($disps as $srv => $disp) {
            foreach ($disp as $key => $val) {
                if (!in_array($key, $desktops)) {
                    $otherDisps[$srv] = array($key => $val);
                }
            }
        }
        $services = array_merge_recursive($inits, $otherDisps);
        ksort($services);
        $this->set('services', $services);

        if (!empty($this->request->data)) {

            if (!empty($this->request->data['checkItem'])) {
                $this->set('checkItem', $this->request->data);
            }

            if (!empty($this->request->data['Courrier'])) {
                $desktopId = $this->request->data['Courrier']['dispInitDesktop'];
                $flux = array();

                foreach ($this->request->data['checkItem'] as $key => $val) {
                    if ($val == true) {
                        $flux[] = $val;
                    }
                }

                $groupId = $this->Courrier->Bancontenu->Desktop->field('Desktop.group_id', array('Desktop.id' => $desktopId));

                $initDesktops = array_keys($this->getSessionInitDesktops());
                $senderInitId = $initDesktops[0]; // FIXME
                $notifyDesktops = array($desktopId => $flux);

                if ($groupId == DISP_GID) {
                    $valid = array();
                    for ($i = 0; $i < count($flux); $i++) {
                        $valid[$flux[$i]] = array(
                            'end' => $this->Courrier->Bancontenu->end($senderInitId, BAN_DOCS, $flux[$i]),
                            'add' => $this->Courrier->Bancontenu->add($desktopId, BAN_AIGUILLAGE, $flux[$i])
                        );
                    }
                } else if ($groupId == INIT_GID) {
                    $valid = array();
                    for ($i = 0; $i < count($flux); $i++) {
                        $courrier = $this->Courrier->find('first', array('conditions' => array('Courrier.id' => $flux[$i]), 'contain' => false));
                        $courrier['Courrier']['desktop_creator_id'] = $desktopId;
                        $this->Courrier->create();
                        $this->Courrier->save($courrier);
                        $valid[$flux[$i]] = array(
                            'end' => $this->Courrier->Bancontenu->end($senderInitId, BAN_DOCS, $flux[$i]),
                            'add' => $this->Courrier->Bancontenu->add($desktopId, BAN_DOCS, $flux[$i])
                        );
                    }
                    $notifInfos = $this->Courrier->Notifieddesktop->Notification->setNotification('ban_xnew');
                    $this->Courrier->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $this->Session->read('Auth.User.Courrier.Desktop.id'), $notifyDesktops);
                }
                $this->Session->setFlash(__d('default', 'save.ok'), 'growl');
            }
        }
    }
    
    
    /**
     * Envoi de plusieurs flux depuis un aiguilleur
     *
     * @logical-group Flux
     * @user-profile User
     * @user-profile Aiguilleur
     *
     * @access public
     * @return void
     */
    public function dispmultiplesend( $fluxIds = array(), $cpyOnly = false ) {

        $this->set('sendwkf', $this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/sendwkf'));
        $this->set('sendcpy', $this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/sendcpy'));
    

        if (!empty($this->request->data['Send'])) {
            $flux = array();
            // Liste des services à qui s'adressent le ou les flux
            $desktopIds = Hash::extract( $this->request->data, 'Send.initDesktopService.{n}' );
            $desktopCopyIds = Hash::get( $this->request->data, 'Send.copy_id' );
            $desktopCopyIds = ( empty( $desktopCopyIds ) ? array() : (array)$desktopCopyIds );
            

            foreach ($this->request->data['DispForm']['itemToLink'] as $key => $fluxId) {
                $flux[] = $fluxId;
                
                // Enregistrement des commentaires pour les flux (copie ou aiguillage)
                $readers = array_merge(@$desktopIds,@$desktopCopyIds);
                if( !empty( $this->request->data['Comment']['content'] ) ) {
                    $this->request->data['Comment']['owner_id'] = $this->Session->read('Auth.User.Courrier.Desktop.id');
                    $this->request->data['Comment']['private'] = false;
                    $this->request->data['Comment']['target_id'] = $fluxId;
                    $this->request->data['Reader'] = array('Reader' => $readers);
                    $this->request->data['Send']['flux_id'] = $fluxId;
                    $this->Courrier->Comment->saveAll($this->request->data);
                }
            }
            // Pour chacun de ces services, on regarde le groupe auquel il est lié
            foreach( $desktopIds as $key => $desktopId ) {
                $groupsIds[] = $this->Courrier->Bancontenu->Desktop->field('Desktop.group_id', array('Desktop.id' => $desktopId));

                $dispDesktops = array_keys($this->getSessionDispDesktops());
                $senderDispId = $dispDesktops[0];
//debug($fluxId);
                // Pour chacun de ces groupes, on vérifie s'il est INIT ou AIGUILLEUR
                foreach( $groupsIds as $i => $groupId) {
                    $valid = array();
                    for ($i = 0; $i < count($flux); $i++) {
                        if ($groupId == INIT_GID) {
                            $courrier = $this->Courrier->find('first', array('conditions' => array('Courrier.id' => $flux[$i])));
                            $courrier['Courrier']['desktop_creator_id'] = $desktopId;
                            $this->Courrier->create();
                            $this->Courrier->save($courrier);
                            $bannetteAdd = BAN_DOCS;
                        }
                        else {
                            $bannetteAdd = BAN_AIGUILLAGE;
                        }

                        
                        
                        foreach( $desktopIds as $key => $value) {
                            
                            // Envoi des notifications pour chacun des bureaux en copie
                            $destId = array();
                            $notifInfos = $this->Courrier->Notifieddesktop->Notification->setNotification('ban_xnew');
                            $desktopId = $this->Session->read('Auth.User.desktop_id');
                            $fluxId = $flux[$i];
//UPDATE notifications SET desktop_id = ( SELECT users.desktop_id FROM users where users.id = notifications.desktop_id ) WHERE notifications.desktop_id IS NOT NULL;
                            $notifyDesktops = array(
                                $value => array( $fluxId )
                            );   
                            $notified = $this->Courrier->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId, $notifyDesktops);

                            
                            
                            
                            $valid[$flux[$i]] = array(
                                'end' => $this->Courrier->Bancontenu->end($senderDispId, BAN_AIGUILLAGE, $flux[$i]),
                                'add' => $this->Courrier->Bancontenu->add($value, $bannetteAdd, $flux[$i])
                            );
                            
                            $user_id = $this->Courrier->User->userIdByDesktop($value);
							$this->Courrier->User->notifier($flux[$i], $user_id, 'aiguillage');
                        }
                        
                        
                        // AIGUILLAGE POUR COPIE
                        // Gestion des desktops auxquels sont envoyés les flux
                        if( !empty($this->request->data['Send']['copy_id']) && isset($this->request->data['Send']['copy']) && $this->request->data['Send']['copy'] ) {
                            $copyIds = Hash::extract( $this->request->data, 'Send.copy_id.{n}' );
                            foreach( $desktopCopyIds as $key => $value) {
                                
                                // Envoi des notifications pour chacun des bureaux en copie
                                $destId = array();
                                $notifInfos = $this->Courrier->Notifieddesktop->Notification->setNotification('ban_copy');
                                $desktopId = $this->Session->read('Auth.User.desktop_id');
                                $fluxId = $flux[$i];

                                $notifyDesktops = array(
                                    $value => array( $fluxId )
                                );                     
                                $notified = $this->Courrier->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId, $notifyDesktops);

                                $valid[$flux[$i]] = array(
                                    'add' => $this->Courrier->Bancontenu->add($value, BAN_COPY, $flux[$i])
                                );
                                
                                $user_id = $this->Courrier->User->userIdByDesktop($value);
								$this->Courrier->User->notifier($flux[$i], $user_id, 'copy');
                                
                            }
                        }
                    }

                }
 
            }    
        }
        else {
            
//            $desktopsToSend = $this->Courrier->Desktop->getDesktopsByUser('rebond');
            $desktopsToCopy = $this->Courrier->Desktop->getDesktopsByUser('all');
            $allDesktops = $this->Courrier->Desktop->getDesktopsByService( 'all' );
            ksort($allDesktops);
            $desktopsToSend = $this->Courrier->Desktop->getDesktopsByService( 'rebond' );
            
            $this->set('desktopsToSend', $desktopsToSend);
            $this->set('desktopsToCopy', $desktopsToCopy);
            $this->set('allDesktops', $allDesktops);
            $this->set('cpyOnly', $cpyOnly);
            $this->set('fluxIds', $fluxIds);
            $this->set('itemList', $this->request->data);
        }
        
        $inits = $this->Courrier->User->Desktop->getInitsByService();
        $disps = $this->Courrier->User->Desktop->getDispsByService();

        $desktops = $this->SessionTools->getDesktopList();
        $otherDisps = array();
        foreach ($disps as $srv => $disp) {
            foreach ($disp as $key => $val) {
                if (!in_array($key, $desktops)) {
                    $otherDisps[$srv] = array($key => $val);
                }
            }
        }
//        $services = array_merge_recursive($inits, $otherDisps);
        
        $services = $this->Courrier->User->Desktop->getInitsDispsByService();
        ksort($services);

        $this->set('services', $services);
        $this->set('bannettes', $this->_bannettes);
    }
    
    
    
       
    /**
     * Fonction permettant de retourner l'historique global  d'un flux
     * @user-profile User
     * 
     *
     * @access public
     * @return void
     */
    
    public function historiqueGlobal( $fluxId = null ) {

        $desktops = $this->getSessionAllDesktops();
        $historiqueGlobal = array();
        $namedModel = Hash::get($this->request->params, 'named.model');
        //ajouter ariane
        $domain = strrchr($_SERVER['HTTP_REFERER'], '/');
        $courrier = $this->Courrier->find('first',array('conditions'=>array('Courrier.id'=>$fluxId),'contain'=>false));
        if($domain=='/historique'){
            $arianeParent = __d('courrier', 'Bannette.archives');
        }else if($domain=='/recherches'){
           $arianeParent = __d('menu', 'Recherche', true);
        }else{
            
            $arianeParent = __d('courrier', 'Bannette.archives');
        }
        $ariane = array(
            '<a href="/environnement/userenv/0">'.__d('menu', 'Courrier').'</a>',
            '<a href="/environnement/historique">'.$arianeParent.'</a>',
            $courrier['Courrier']['name']
        );
        $parent = array();
        if (!empty($courrier['Courrier']['parent_id'])) {
            $qdParent = array(
                'fields' => array(
                    'Courrier.id',
                    'Courrier.name'
                ),
                'conditions' => array(
                    'Courrier.id' => $courrier['Courrier']['parent_id']
                )
            );
            $parent = $this->Courrier->find('first', $qdParent);
            
            $ariane = array(
                '<a href="/environnement/userenv/0">'.__d('menu', 'Courrier').'</a>',
                '<a href="/environnement/historique">'.$arianeParent.'</a>',
                '<a href="/courriers/historiqueGlobal/'.$parent['Courrier']['id'].'">'.$parent['Courrier']['name'].'</a>',
                $courrier['Courrier']['name']
            );
        }
        $this->set('ariane',$ariane);
        
        // Données liées au courrier
        $querydata = array(
            'fields' => array_merge(
                $this->Courrier->fields(),
                $this->Courrier->Contactinfo->fields(),
                $this->Courrier->Contactinfo->Contact->fields(),
                $this->Courrier->Soustype->fields(),
                $this->Courrier->Service->fields(),
                $this->Courrier->Soustype->Type->fields(),
                $this->Courrier->Soustype->Circuit->fields(),
                array(
                    'Courrier.name',
                    'Courrier.objet',
                    'Courrier.date',
                    'Courrier.datereception',
                    'Contactinfo.name',
                    'Contact.name',
                    'Type.name',
                    'Soustype.name',
                    'Circuit.nom',
                    'Service.name',
                    'Affairesuivie.name'
                )
            ),
            'contain' => false,
            'joins' => array(
                $this->Courrier->join($this->Courrier->Contactinfo->alias, array( 'type' => 'LEFT OUTER')),
                $this->Courrier->Contactinfo->join($this->Courrier->Contactinfo->Contact->alias, array( 'type' => 'LEFT OUTER')),
                $this->Courrier->join($this->Courrier->Soustype->alias, array( 'type' => 'LEFT OUTER')),
                $this->Courrier->join($this->Courrier->Service->alias, array( 'type' => 'LEFT OUTER')),
                $this->Courrier->Soustype->join($this->Courrier->Soustype->Type->alias, array( 'type' => 'LEFT OUTER')),
                $this->Courrier->Soustype->join($this->Courrier->Soustype->Circuit->alias, array( 'type' => 'LEFT OUTER')),
                $this->Courrier->join($this->Courrier->Affairesuivie->alias, array( 'type' => 'LEFT OUTER'))
            ),
            'conditions' => array(
                'Courrier.id' => $fluxId
            )
        );
        $courrier = $this->Courrier->find('first', $querydata);


        // Données nécessaire pour récupérer les différents documents liés au flux
        $qdCourrier = array(
            'conditions' => array(
                'Document.courrier_id' => $fluxId
            ),
            'fields' => array(
                'Document.name'
            ),
            'contain' => false
        );
        $documents = $this->Courrier->Document->find('all', $qdCourrier);
        
        $rights = array(
            'getDocuments' => true,
            'getRelements' => true,
            'getArs' => true,
            'getCircuit' => true
        );

        // Si le sous-type est sortant, on affiche les composants de réponse sinon les ARs
        $sortant = false;
        if (isset($courrier['Soustype']['entrant']) && $courrier['Soustype']['entrant'] == 0) {
            $sortant = true;
        }
        $this->set('sortant', $sortant);
        
        $qdParent = array(
            'fields' => array(
                'Courrier.id',
                'Courrier.name'
            ),
            'conditions' => array(
                'Courrier.id' => $courrier['Courrier']['parent_id']
            )
        );
        $parent = $this->Courrier->find('first', $qdParent);
            
//         $metas_values = $this->DataAuthorized->getAuthMetadonnees(array( 'fluxId' => $fluxId));
        $this->set('metas', $this->DataAuthorized->getAuthMetadonnees(array('fluxId' => $fluxId)));

        
//       debug($this->params);  
        // En cas de refus 
        $fluxRefus = array();
        if (!empty($courrier['Courrier']['ref_ancien'])) {
            $qdFluxRefus = array(
                'fields' => array(
                    'Courrier.id',
                    'Courrier.name'
                ),
                'conditions' => array(
                    'Courrier.id' => $courrier['Courrier']['ref_ancien']
                )
            );
            $fluxRefus = $this->Courrier->find('first', $qdFluxRefus);
        }
        $this->set('fluxRefus', $fluxRefus);
        
        
        
        // Récupération de toutes les infos échangés sur le flux entre les diférentes bannettes
        $bannettes = $this->Courrier->Bancontenu->find(
            'all',
            array(
                'conditions' => array(
                    'Bancontenu.courrier_id' => $fluxId
                ),
                'contain' => array(
                    'Desktop' => array(
                        'SecondaryUser',
                        'User',
                        'Group'
                    ),
                    'Bannette'
                ),
                'order' => array( 'Bancontenu.created ASC' )
            )
        );

        // Récupération des commentaires liés au flux
        $comments = $this->_getComments($fluxId);

        $this->set('mainDoc', $this->Courrier->getMainDoc($fluxId));
        $this->set('documents', $documents);
        $this->set('rights', $rights);
        $this->set('parent', $parent);
        $this->set('courrier', $courrier);
        $this->set('comments', $comments);
        $this->set('bannettes', $bannettes);
    }
    
    
    /** Envoie pour un ged en protocole CMIS
     * 
     * @param type $courrier_id
     */
    public function sendToGed($courrier_id) {
        App::uses('Folder', 'Utility');
        App::uses('File', 'Utility');
        

        $folderTmp = new Folder(TMP.'files'.DS.'export'.DS.$courrier_id, true, 0777);

        try{
                $cmis = new CmisComponent;
                $cmis->CmisComponent_Service();

                $folderSend = new Folder($folderTmp->pwd().DS.'send', true, 0777);
                
                $cmisFolder=$this->{'_sendToGedVersion' . Configure::read('GED_XML_VERSION')}($cmis, $folderTmp, $folderSend, $courrier_id);
                
                $files = $folderSend->find('.*');
                foreach ($files as $file) {
                    $file = new File($folderSend->pwd() . DS . $file);
//                     $this->Progress->at(95, 'Envoi du fichier '.$file->name.'...');
                    $contents = $file->read();
                    $cmis->client->createDocument($cmisFolder->id, $file->name, array(), $file->read(), $file->mime());

                    $file->close();
                }

//                 $this->Progress->at(100, 'Opération terminée. Redirection...');
                $folderTmp->delete();
//                 sleep(3);

                if( $this->params['params']['action'] == 'getFlowControls' ) {
					$desktopidactuel = $this->Session->read( 'Auth.User.Courrier.Desktop.id');
					$this->redirect(array('controler'=>'courriers','action'=>'getFlowControls', $courrier_id, $desktopidactuel));
				}
                $this->redirect(array('controller' => 'courriers', 'action' => 'historiqueGlobal', $courrier_id));
//				$this->autoRender = false;
        
        } catch (Exception $e) {
            switch ($e->getCode()) {
                case '500':
                        $message='Erreur interne';
                    break;
                case '404':
                        $message='Ressource non trouvée';
                    break;
                case '409':
                        $message='Conflit, le courrier existe déjà dans la GED';
                    break;
                default:
                    $message=strip_tags($e->getMessage());
                    break;
            }
                
            $this->log('Export CMIS: Erreur ' . $e->getCode() . " ! \n" . $e->getMessage(), 'error');
            $this->Session->setFlash('CMIS: Erreur ' . $e->getCode() . ' ! ' .  $message, 'growl', array('type' => 'erreur'));

        }
        
//        $this->Progress->end('/courriers/index');

    }
    
     /** Envoi d'un document pour un ged en protocole CMIS
     * 
     * @param type $doc_id
     */
    public function sendDocumentToGed($doc_id) {
        App::uses('Folder', 'Utility');
        App::uses('File', 'Utility');
        
        $doc = $this->Courrier->Document->find(
            'first',
            array(
                'fields' => array(
                    'Document.id',
                    'Courrier.id'
                ),
                'conditions' => array(
                    'Document.id' => $doc_id
                ),
                array(
                    'Courrier'
                )
            )
        );
        
        
        $folderTmp = new Folder(TMP.'files'.DS.'export'.DS.$doc_id, true, 0777);

        try{
                $cmis = new CmisComponent;
                $cmis->CmisComponent_Service();

                $folderSend = new Folder($folderTmp->pwd().DS.'send', true, 0777);
                
                $cmisFolder=$this->_sendDocToGedVersion($cmis, $folderTmp, $folderSend, $doc_id);
                
               $files = $folderSend->find('.*');
                foreach ($files as $file) {
                    $file = new File($folderSend->pwd() . DS . $file);
                    $contents = $file->read();
                    $cmis->client->createDocument($cmisFolder->id, $file->name, array(), $file->read(), $file->mime());

                    $file->close();
                }
                

                $folderTmp->delete();
//debug($this->params['params']);die();

                $this->redirect(array('controller' => 'courriers', 'action' => 'view', $doc['Courrier']['id'], $this->Session->read('Auth.User.Desktop.id')));
        
        } catch (Exception $e) {
            switch ($e->getCode()) {
                case '500':
                        $message='Erreur interne';
                    break;
                case '404':
                        $message='Ressource non trouvée';
                    break;
                case '409':
                        $message='Conflit, le courrier existe déjà dans la GED';
                    break;
                default:
                    $message=strip_tags($e->getMessage());
                    break;
            }
                
            $this->log('Export CMIS: Erreur ' . $e->getCode() . " ! \n" . $e->getMessage(), 'error');
            $this->Session->setFlash('CMIS: Erreur ' . $e->getCode() . ' ! ' .  $message, 'growl', array('type' => 'erreur'));

        }
    }
    
    
    
    /*function _sendToGedVersion1(&$cmis, &$folderTmp, &$folderSend, $seance_id) {

            $this->Conversion = new ConversionComponent();

            // Création du répertoire de séance
            $result = $cmis->client->getFolderTree($cmis->folder->id, 1);

            $this->Progress->at(0, 'Création du répertoire de séance...');

            $this->Seance->Behaviors->attach('Containable');
            $seance = $this->Seance->find('first', array(
                'conditions' => array('Seance.id' => $seance_id),
                'contain' => array('Typeseance.libelle')));

            $this->Progress->at(5, 'Recherche de la séance en base de données...');

            $date_seance = $seance['Seance']['date'];
            $date_convocation = $seance['Seance']['date_convocation'];
            $type_seance = $seance['Typeseance']['libelle'];
            $libelle_seance = $seance['Typeseance']['libelle'] . " " . CakeTime::i18nFormat($seance['Seance']['date'], '%A %d %B %G à %k:%M');

            $this->_deletetoGed($cmis, $libelle_seance);

            $zip = new ZipArchive;
            $zip->open($folderSend->pwd() .DS. 'documents.zip', ZipArchive::CREATE);
            $this->Progress->at(15, 'Création des dossiers...');
            // Création des dossiers
            $my_seance_folder = $cmis->client->createFolder($cmis->folder->id, $libelle_seance);
            // Création des dossiers vides
            $zip->addEmptyDir('Rapports');
            $zip->addEmptyDir('Annexes');

            $delibs_id = $this->Seance->getDeliberationsId($seance_id);
            $this->log(var_export($delibs_id,true),'debug');

            $this->Progress->at(20, 'Création du fichier XML...');
            $dom = new DOMDocument('1.0', 'utf-8');
            $dom->formatOutput = true;
            $idDepot = $seance['Seance']['numero_depot'] + 1;

            $dom_depot = $this->_createElement($dom, 'depot', null, array(
                'idDepot' => $idDepot,
                'xmlns:webdelibdossier' => 'http://www.adullact.org/webdelib/infodossier/1.0',
                'xmlns:xm' => 'http://www.w3.org/2005/05/xmlmine'));

            $dom_courrier = $this->_createElement($dom, 'seance', null, array('idSeance' => $seance_id));
            $dom_courrier->appendChild($this->_createElement($dom, 'typeSeance', $type_seance));
            $dom_courrier->appendChild($this->_createElement($dom, 'dateSeance', $date_seance));
            $dom_courrier->appendChild($this->_createElement($dom, 'dateConvocation', $date_convocation));

            //Noeud document[convocation]
            $this->Progress->at(25, 'Génération de la convocation...');
            $document = $this->_createElement($dom, 'document', null, array('nom' => 'convocation.pdf', 'type' => 'convocation'));
            $document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
            $document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
            $dom_courrier->appendChild($document);
            // fusion du document de convocation
            $zip->addFromString('convocation.pdf', $this->Seance->fusion($seance_id, 'convocation'));

            //Noeud document[odj]
            $this->Progress->at(40, 'Génération de l\'ordre du jour...');
            $document = $this->_createElement($dom, 'document', null, array('nom' => 'odj.pdf', 'type' => 'odj'));
            $document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
            $document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
            $dom_courrier->appendChild($document);
            // fusion du document de ordre du jour
            $zip->addFromString('odj.pdf', $this->Seance->fusion($seance_id, 'ordredujour'));

            //Noeud document[pv_sommaire]
            $this->Progress->at(50, 'Ajout du PV sommaire...');
            $document = $this->_createElement($dom, 'document', null, array('nom' => 'pv.pdf', 'type' => 'pv_sommaire'));
            $document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
            $document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
            $dom_courrier->appendChild($document);
            //Ajout au zip
            if (!empty($seance['Seance']['pv_sommaire']))
                $zip->addFromString('pv.pdf', $seance['Seance']['pv_sommaire']);

            //Noeud document[pv_complet]
            $this->Progress->at(55, 'Ajout du PV complet...');
            $document = $this->_createElement($dom, 'document', null, array('nom' => 'pvcomplet.pdf', 'type' => 'pv_complet'));
            $document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
            $document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
            $dom_courrier->appendChild($document);
            if (!empty($seance['Seance']['pv_complet']))
                $zip->addFromString('pvcomplet.pdf', $seance['Seance']['pv_complet']);
            //Infos supps
            $this->Progress->at(60, 'Ajout des informations supplémentaires de séance...');
            $this->_createElementInfosups($zip, $dom, $dom_courrier, $seance_id, 'Seance');

            //Insertion du noeud xml seance
            $dom_depot->appendChild($dom_courrier);

            $this->Progress->at(66, 'Ajout des délibérations...');
            foreach ($delibs_id as $delib_id) {

                $doc = $this->_createElement($dom, 'dossierActe', null, array('idActe' => $delib_id, 'refSeance' => $seance_id));

                $this->Deliberation->Behaviors->attach('Containable');
                $delib = $this->Deliberation->find('first', array(
                    'conditions' => array('Deliberation.id' => $delib_id),
                    'fields' => array('Deliberation.id', 'Deliberation.num_delib', 'Deliberation.objet_delib', 'Deliberation.titre',
                        'Deliberation.delib_pdf', 'Deliberation.tdt_data_pdf', 'Deliberation.tdt_data_bordereau_pdf', 'Deliberation.deliberation',
                        'Deliberation.deliberation_size', 'Deliberation.signature', 'Deliberation.tdt_ar_date'),
                    'contain' => array(
                        'Service' => array('fields' => array('libelle')),
                        'Theme' => array('fields' => array('libelle')),
                        'Typeacte' => array('fields' => array('nature_id')),
                        'Redacteur' => array('fields' => array('nom', 'prenom')),
                        'Rapporteur' => array('fields' => array('nom', 'prenom')),
                )));

                $nature = $this->Nature->find('first', array(
                    'fields' => array('libelle'),
                    'conditions' => array('Nature.id' => $delib['Typeacte']['nature_id'])));

                $doc->appendChild($this->_createElement($dom, 'natureACTE', $nature['Nature']['libelle']));
                $doc->appendChild($this->_createElement($dom, 'dateACTE', $date_seance));
                $doc->appendChild($this->_createElement($dom, 'numeroACTE', $delib['Deliberation']['num_delib']));
                $doc->appendChild($this->_createElement($dom, 'themeACTE', $delib['Theme']['libelle']));
                $doc->appendChild($this->_createElement($dom, 'emetteurACTE', $delib['Service']['libelle']));
                $doc->appendChild($this->_createElement($dom, 'redacteurACTE', $delib['Redacteur']['prenom'] . ' ' . $delib['Redacteur']['nom']));
                $doc->appendChild($this->_createElement($dom, 'rapporteurACTE', $delib['Rapporteur']['prenom'] . ' ' . $delib['Rapporteur']['nom']));
                $doc->appendChild($this->_createElement($dom, 'typeseanceACTE', $type_seance));
                $doc->appendChild($this->_createElement($dom, 'dateAR', $delib['Deliberation']['tdt_ar_date'])); // utile ??
                //<listeCommissions>
                $seances_id = $this->Deliberation->getSeancesid($delib_id);

                $listeCommissions = '';
                foreach ($seances_id as $commission_id) {
                    if (!$this->Deliberation->Seance->isSeanceDeliberante($commission_id)) {
                        $typeSeance = $this->Deliberation->Seance->Typeseance->getLibelle($this->Deliberation->Seance->getType($commission_id));
                        $listeCommissions .= $typeSeance . ' : ' . $this->Deliberation->Seance->getDate($commission_id) . ', ';
                    }
                }

                $doc->appendChild($this->_createElement($dom, 'listeCommissions', $listeCommissions));
                //</listeCommissions>
                //Infos supps de délibération
                $this->_createElementInfosups($zip, $dom, $doc, $delib_id, 'Deliberation');

                //Noeud document[TexteActe]
                $delib_filename = $delib_id . '-' . $delib['Deliberation']['num_delib'] . '.pdf';
                $document = $this->_createElement($dom, 'document', null, array('nom' => $delib_filename, 'relname' => $delib_filename, 'type' => 'TexteActe'));
                $document->appendChild($this->_createElement($dom, 'titre', $delib['Deliberation']['objet_delib']));
                $document->appendChild($this->_createElement($dom, 'description', $delib['Deliberation']['titre']));
                $document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
                $document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
                $doc->appendChild($document);
                //Ajout au zip
                $zip->addFromString($delib_filename, (!empty($delib['Deliberation']['tdt_data_pdf']) ? $delib['Deliberation']['tdt_data_pdf'] : $delib['Deliberation']['delib_pdf']));

                //Noeud document[Rapport]
                if (!empty($delib['Deliberation']['deliberation_size'])) {
                    $document = $this->_createElement($dom, 'document', null, array('nom' => $delib_filename, 'relname' => $delib_filename, 'type' => 'Rapport'));
                    $document->appendChild($this->_createElement($dom, 'titre', $delib['Deliberation']['objet_delib']));
                    $document->appendChild($this->_createElement($dom, 'description', $delib['Deliberation']['titre']));
                    $document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
                    $document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
                    $doc->appendChild($document);
                    // fusion du rapport et ajout au zip
                    $zip->addFromString('Rapports' . DS . $delib_filename, $this->Deliberation->fusion($delib['Deliberation']['id'], 'rapport'));
                }

                //Ajout de la signature (XML+ZIP)
                $signatureName = $delib['Deliberation']['id'] . '-signature.zip';
                //Création du noeud XML
                $document = $this->_createElement($dom, 'document', null, array('nom' => $signatureName, 'relname' => $signatureName, 'type' => 'Signature'));
                $document->appendChild($this->_createElement($dom, 'mimetype', 'application/zip'));
                $doc->appendChild($document);
                //Ajout à l'archive
                $zip->addFromString('Annexes' . DS . $signatureName, $delib['Deliberation']['signature']);

                //Ajout du bordereau (XML+ZIP)
                $bordereauName = $delib['Deliberation']['id'] . '-bordereau.pdf';
                //Création du noeud XML
                $document = $this->_createElement($dom, 'document', null, array('nom' => $bordereauName, 'relname' => $bordereauName, 'type' => 'Bordereau'));
                $document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
                $doc->appendChild($document);
                //Ajout à l'archive
                $zip->addFromString('Annexes' . DS . $bordereauName, $delib['Deliberation']['tdt_data_bordereau_pdf']);


                //Ajout des annexes
                $annexes_id =  $this->Deliberation->Annex->getAnnexesFromDelibId($delib_id, true);
                if (!empty($annexes_id)) {
                    foreach ($annexes_id as $annex_id) {
                        $annex_id = $annex_id['Annex']['id'];
                        $annex = $this->Deliberation->Annex->find('first', array(
                            'conditions' => array('Annex.id' => $annex_id),
                            'fields' => array('Annex.id', 'Annex.titre', 'Annex.filename', 'Annex.filetype', 'Annex.data_pdf', 'Annex.data'),
                            'recursive' => -1));

                        switch ($annex['Annex']['filetype']) {
                            case 'application/pdf' :
                                $annexe_content = $annex['Annex']['data'];
                                $annexe_filetype = 'application/pdf';
                                $annexe_filename = $annex['Annex']['filename'];
                                break;

                            case 'application/vnd.oasis.opendocument.text' :
                                $annexe_content = $this->Conversion->convertirFlux($annex['Annex']['data'], 'odt', 'pdf');
                                $annexe_filetype = 'application/pdf';
                                $annexe_filename = str_replace('odt', 'pdf', $annex['Annex']['filename']);
                                break;

                            default:
                                $annexe_content = $annex['Annex']['data'];
                                $annexe_filetype = $annex['Annex']['filetype'];
                                $annexe_filename = $annex['Annex']['filename'];
                                break;
                        }
                        //Création du noeud XML <document> de l'annexe
                        $document = $this->_createElement($dom, 'document', null, array('nom' => $annexe_filename, 'relname' => $annex['Annex']['id'] . '.pdf', 'type' => 'Annexe'));
                        $document->appendChild($this->_createElement($dom, 'titre', $annex['Annex']['titre']));
                        $document->appendChild($this->_createElement($dom, 'mimetype', $annexe_filetype));
                        $document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
                        $doc->appendChild($document);
                        //Ajout du fichier annexe à l'archive
                        $zip->addFromString('Annexes' . DS . $annex['Annex']['id'] . '.pdf', $annexe_content);
                    }
                }
                $dom_depot->appendChild($doc);
            }
           
            $zip->close();
            $dom->appendChild($dom_depot);

            $file = new File($folderSend->pwd().DS.'XML_DESC_'.$seance_id.'.xml',true, 0777);
            $file->append($dom->saveXML());
            $file->close();
            
            $this->Seance->id = $seance_id;
            $this->Seance->saveField('numero_depot', $idDepot);
            $this->Session->setFlash('Le dossier \"' . $libelle_seance . '\" a été ajouté (Depot n°' . $idDepot . ')', 'growl', array('type' => 'important'));
            
            return $my_seance_folder;
    }*/
    
    /**
     * Création des fichiers pour l'export GED en version 2 et 3
     * @param type $cmis
     * @param type $folderTmp
     * @param type $folderSend
     * @param type $courrier_id
     * @param type $message_tdt
     * @return type
     * @throws Exception
     */
    function _sendToGedVersion2(&$cmis, &$folderTmp, &$folderSend, $courrier_id, $message_tdt=false) {

        try
        {
            $this->Conversion = new ConversionComponent();
            // Création du répertoire de séance
            $result = $cmis->client->getFolderTree($cmis->folder->id, 1);

//             $this->Progress->at(0, 'Création du répertoire de séance...');

//             $this->Courrier->Behaviors->attach('Containable');
            $courrier = $this->Courrier->find(
                'first',
                array(
                    'conditions' => array('Courrier.id' => $courrier_id),
                    'contain' => array(
                        'Document',
                        'Metadonnee' => array(
                            'Selectvaluemetadonnee'
                        )
                    )
                )
            );

            // On récupère les valeurs des métadonnées
           foreach( $courrier['Metadonnee'] as $m => $flux ) {
               $idSelect = Configure::read('Selectvaluemetadonnee.id');
               if( $flux['typemetadonnee_id'] == $idSelect ) {
                   foreach( $flux['Selectvaluemetadonnee'] as $s => $selectedValue ) {
                       if( $flux['CourrierMetadonnee']['valeur'] == $selectedValue['id'] ) {
                           $courrier['Metadonnee'][$m]['CourrierMetadonnee']['valeur'] = $selectedValue['name'];
                       }
                   }
               }
           }
           
           /*     $svM = $this->Courrier->Metadonnee->Selectvaluemetadonnee->find(
                    'first',
                    array(
                        'conditions' => array(
                            'Selectvaluemetadonnee.id' => $value['CourrierMetadonnee']['valeur']
                        ),
                        'recursive' => -1
                    )
                );
                
            } */
            /*foreach( $courrier['Metadonnee'] as $metaKey => $value ) {
//                if( is_int( $value['CourrierMetadonnee']['valeur'] ) ) {
                    $svM = $this->Courrier->Metadonnee->Selectvaluemetadonnee->find(
                        'all',
                        array(
                            'conditions' => array(
                                'Selectvaluemetadonnee.id' => $value['CourrierMetadonnee']['valeur']
                            ),
                            'recursive' => -1
                        )
                    );
                    $courrier['Metadonnee'][$metaKey]['valeurSelected'] = Hash::extract($svM,'{n}.Selectvaluemetadonnee.name');
//
            }*/
            

//debug($courrier['Metadonnee']);
//die();

//             $this->Progress->at(5, 'Recherche de la séance en base de données...');

            $date_courrier = $courrier['Courrier']['date'];
            $date_reception = $courrier['Courrier']['datereception'];
            $type_courrier = $courrier['Courrier']['name'];
            $libelle_courrier = $courrier['Courrier']['reference'] . " " . date('Ymd_His');//CakeTime::i18nFormat($courrier['Courrier']['date'], '%A %d %B %G à %k:%M');

            $this->_deletetoGed($cmis, $libelle_courrier);

            $zip = new ZipArchive;
            $zip->open($folderSend->pwd() .DS. 'documents.zip', ZipArchive::CREATE);

//             $this->Progress->at(15, 'Création des dossiers...');
            // Création des dossiers
            $my_courrier_folder = $cmis->client->createFolder($cmis->folder->id, $libelle_courrier);
            // Création des dossiers vides
            $zip->addEmptyDir('Document');
            $zip->addEmptyDir('Annexes');

            $courrier_id = $courrier['Courrier']['id'];

//             $this->Progress->at(20, 'Création du fichier XML...');
            $dom = new DOMDocument('1.0', 'utf-8');
            $dom->formatOutput = true;
            
            $idDepot = $courrier['Courrier']['numero_depot'] + 1;

            $dom_depot = $this->_createElement($dom, 'depot', null, array(
                'versionDepot' => 2,
                'idDepot' => $idDepot,
                'dateDepot' => date("Y-m-d H:i:s"),
                'xmlns:webgfcdossier' => 'http://www.adullact.org/webgfc/infodossier/1.0',
                'xmlns:xm' => 'http://www.w3.org/2005/05/xmlmine'));

            $dom_courrier = $this->_createElement($dom, 'courrier', null, array('idCourrier' => $courrier_id));
            $dom_courrier->appendChild($this->_createElement($dom, 'typeCourrier', $type_courrier));
            $dom_courrier->appendChild($this->_createElement($dom, 'dateCourrier', $date_courrier));
            $dom_courrier->appendChild($this->_createElement($dom, 'dateReception', $date_reception));
           
			$document = $this->_createElement($dom, 'document', null, array('nom' => 'courrier.pdf', 'type' => 'courrier'));
			$document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
			$document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
            $dom_courrier->appendChild($document);
            
            // Documents attachés au courrier
            $aDocuments = array();
            $aDocuments = $this->Courrier->getDocumentsForDelegation($courrier_id);

			$courrier_filename = $courrier_id . '-' . $aDocuments['docPrincipale']['Document']['name'];
            $document = $this->_createElement($dom, 'document', null, array(
				'idDocument' => $aDocuments['docPrincipale']['Document']['id'],
				'nom' => $courrier_filename,
				'relName' => 'Document' . DS . $courrier_filename,
				'type' => 'Document'));
// 			$document->appendChild($this->_createElement($dom, 'signature', 'false'));
			$document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
			$document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
			$dom_courrier->appendChild($document);
			
			// fusion du documentet ajout au zip
			$zip->addFromString('Document' . DS . $courrier_filename, $aDocuments['docPrincipale']['Document']['content']);

            // Fichiers supplémentaires
            foreach($aDocuments['annexes'] as $i => $annexe ) {
				$annexe_filename = $annexe['filename'];
				$document = $this->_createElement($dom, 'annexes', null, array(
					'idAnnexe' => $i,
					'nom' => $annexe_filename,
					'relName' => 'Annexe' . DS . $annexe_filename,
					'type' => 'Annexe'));
				$document->appendChild($this->_createElement($dom, 'signature', 'false'));
				$document->appendChild($this->_createElement($dom, 'mimetype', $annexe['mimetype']));
				$document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
			
				$zip->addFromString('Annexes' . DS . $annexe_filename, $annexe['content']);
				
				$dom_courrier->appendChild($document);
			}
			
            // Méta-données
            foreach($courrier['Metadonnee'] as $m => $meta ) {
				$document = $this->_createElement($dom, 'metadonnees', null, array(
					'idMeta' => $m,
					'nom' => $meta['name'],
					'relName' => 'Métadonnée' . DS . $meta['name'],
					'type' => 'Metadonnee')
                );
				$document->appendChild($this->_createElement($dom, 'name', $meta['name']));
				$document->appendChild($this->_createElement($dom, 'value', $meta['CourrierMetadonnee']['valeur']));

				$dom_courrier->appendChild($document);
			}
            
            
            //Insertion du noeud xml courrier
            $dom_depot->appendChild($dom_courrier);

            $zip->close();
            $dom->appendChild($dom_depot);
            
            $file = new File($folderSend->pwd().DS.'XML_DESC_'.$courrier_id.'.xml');
            $file->append($dom->saveXML());
            $file->close();

            $this->Courrier->id = $courrier_id;
            $this->Courrier->saveField('numero_depot', $idDepot);

            $this->Session->setFlash('Le dossier \"' . $libelle_courrier . '\" a été ajouté (Depot n°' . $idDepot . ')', 'growl', array('type' => 'success'));

        
        }
        catch (Exception $e)
        {
            $this->log('Export CMIS: Erreur ' . $e->getCode() . "! \n File:" .$e->getFile(). ' Line:'.$e->getLine(), 'error');
            throw $e;
        }
        
        return !empty($my_courrier_folder)?$my_courrier_folder:false;
    }
    
    /**
     * Création des fichiers pour l'export GED en version 3
     * @param type $cmis
     * @param type $folderTmp
     * @param type $folderSend
     * @param type $courrier_id
     * @return type
     */
    function _sendToGedVersion3(&$cmis, &$folderTmp, &$folderSend, $courrier_id) {
        
       return $this->_sendToGedVersion2($cmis, $folderTmp, $folderSend, $courrier_id, true);
    }
    
    
    
    function _deletetoGed(&$cmis, $libelle_courrier) {
        
        // Règle de gestion on écrase les documents existants
        try {
            //On recherche le dossier
                $objet_cmis = $cmis->client->getObjectByPath(Configure::read('CMIS_REPO').'/'.$libelle_courrier);
            
            if (is_object($objet_cmis)) {
                //On recherche tous les enfants du dossier
                $children = $cmis->client->getChildren($objet_cmis->id);
                //On boucle sur les enfants
                foreach ($children->objectList as $child) {
                    //On supprimer l'enfant selectionné
                    $cmis->client->deleteObject($child->id);
                }
                //On peut maitenant supprimer le dossier
                $cmis->client->deleteObject($objet_cmis->id);
            }
        } catch (CmisObjectNotFoundException $e) {
            // L'objet n'existe pas encore : ne rien faire
        }
    }
    
    
    
    function _createElement($domObj, $tag_name, $value = NULL, $attributes = NULL) {
        try
        {
            $element = ($value != NULL ) ? $domObj->createElement($tag_name, AppTools::xml_entity_encode($value)) : $domObj->createElement($tag_name);

            if ($attributes != NULL) {
                foreach ($attributes as $attr => $val) {
// 			debug($attributes);
// 			debug($attr);
// 			debug($val);
//             die();
                    $element->setAttribute($attr, $val);
                }
            }
        }  catch (Exception $e){
            $this->log('Export CMIS: Erreur _createElement ('.$tag_name.') '. $e->getCode() . "! \n" . $e->getMessage(). ' line:'.$e->getLine(), 'error');
            throw $e;
        }
        return $element;
    }
    
    
    /**
     * Envoi d'un document par mail
     */
    public function sendmail( $documentId = null ) {

        $document = $this->Courrier->Document->find(
            'first',
            array(
                'recursive' => -1,
//                'fields' => $this->Courrier->Document->getLightFields(),
                'conditions' => array(
                    'Document.id' => $documentId
                )
            )
        );
        if( !empty($this->request->data)) {
            $success = $this->Courrier->docParMail($this->request->data['Sendmail']['email'], $document);
            
            $this->Jsonmsg->init();
            if( $success ) {
                $this->Jsonmsg->valid( "Le mail a bien été envoyé");
            }
            $this->Jsonmsg->send();
        }
//        var_dump($user);
//        $user_id = $user['User']['id'];
//        $this->User->notifier($this->request->data['Send']['flux_id'], $user_id, 'copy');

    }
    
    
    
    /**
     * Envoi d'un flux à une étape hors circuit
     *
     * @logical-group Flux
     *
     * @logical-used-by CakeFlow > Envoi d'un flux (avec ou sans rebond) à une nouvelle étape
     * @user-profile User
     *
     * @access public
     * @param integer $fluxId identifiant du flux
     * @return void
     */
    public function sendwkfsignaturemanuelle( $data ) {
        if (!empty($data)) {
            $this->autoRender = false;
            //rebond
            $newDesktopId = $data['Send']['desktop_id'];
            $fluxId = $data['Send']['flux_id'];
//debug($data);
//die();
            $newDesktop = $this->Courrier->Desktop->find('first', array('conditions' => array('Desktop.id' => $newDesktopId)));
            $desktopId = $this->Session->read('Auth.User.Courrier.Desktop.id');

            // initialisation des visas a ajouter au traitement
            $options = array(
                'insertion' => array(
                    '0' => array(
                        'Etape' => array(
                            'etape_nom' => 'En cours de signature',
                            'etape_type' => 1
                        ),
                        'Visa' => array(
                            '0' => array(
                                'trigger_id' => $newDesktopId,
                                'type_validation' => 'V'
                            )
                        )
                    )
                )
            );
            $action = $data['Send']['getback'] ? 'IL' : 'IP';

            $success = false;
            if ($this->Courrier->cakeflowExecuteSend($action, $desktopId, $fluxId, $options)) {
                $success = true;
            }

            if (!empty($success)) {
                $this->Session->setFlash(__d('default', 'save.ok'), 'growl');
            } else {
                $this->Session->setFlash(__d('default', 'save.error'), 'growl');
            }
        }
    }
    
    
    /**
     * Création des fichiers pour l'export GED en version 2 et 3
     * @param type $cmis
     * @param type $folderTmp
     * @param type $folderSend
     * @param type $courrier_id
     * @param type $message_tdt
     * @return type
     * @throws Exception
     */
    function _sendDocToGedVersion(&$cmis, &$folderTmp, &$folderSend, $doc_id, $message_tdt=false) {
//debug($cmis->folder);
//die();
        try
        {
            $this->Conversion = new ConversionComponent();
            // Création du répertoire de séance
            $result = $cmis->client->getFolderTree($cmis->folder->id, 1);
            
        $qdDoc = array(
			'recursive' => -1,
            'fields' => array(
                'Document.id',
                'Document.courrier_id',
                'Document.name',
                'Document.ext',
                'Document.content',
                'Document.gedpath',
                'Document.size',
                'Document.mime',
                'Document.main_doc',
                'Document.desktop_creator_id'
            ),
			'conditions' => array(
				'Document.id' => $doc_id
			),
            'contain' => false
		);


		$doc = $this->Courrier->Document->find('first', $qdDoc);
            
        $courrier = $this->Courrier->find(
            'first',
            array(
                'conditions' => array('Courrier.id' => $doc['Document']['courrier_id']),
                'contain' => array(
                    'Document',
                    'Metadonnee' => array(
                        'Selectvaluemetadonnee'
                    )
                )
            )
        );

              // On récupère les valeurs des métadonnées
           foreach( $courrier['Metadonnee'] as $m => $flux ) {
               $idSelect = Configure::read('Selectvaluemetadonnee.id');
               if( $flux['typemetadonnee_id'] == $idSelect ) {
                   foreach( $flux['Selectvaluemetadonnee'] as $s => $selectedValue ) {
                       if( $flux['CourrierMetadonnee']['valeur'] == $selectedValue['id'] ) {
                           $courrier['Metadonnee'][$m]['CourrierMetadonnee']['valeur'] = $selectedValue['name'];
                       }
                   }
               }
           }
           
            $date_courrier = $courrier['Courrier']['date'];
            $date_reception = $courrier['Courrier']['datereception'];
            $type_courrier = $courrier['Courrier']['name'];
            $libelle_courrier = $courrier['Courrier']['reference'] . " " . date('Ymd_His');//CakeTime::i18nFormat($courrier['Courrier']['date'], '%A %d %B %G à %k:%M');

            $this->_deletetoGed($cmis, $libelle_courrier);

//            $zip = new ZipArchive;
//            $zip->open($folderSend->pwd() .DS. 'documents.zip', ZipArchive::CREATE);

//             $this->Progress->at(15, 'Création des dossiers...');
            // Création des dossiers
            $my_courrier_folder = $cmis->client->createFolder($cmis->folder->id, $libelle_courrier);
            // Création des dossiers vides
//            $zip->addEmptyDir('Document');
//            $zip->addEmptyDir('Annexes');

            $courrier_id = $courrier['Courrier']['id'];


            $dom = new DOMDocument('1.0', 'utf-8');
            $dom->formatOutput = true;
            
            $idDepot = $courrier['Courrier']['numero_depot'] + 1;

            $dom_depot = $this->_createElement($dom, 'depot', null, array(
                'versionDepot' => 2,
                'idDepot' => $idDepot,
                'dateDepot' => date("Y-m-d H:i:s"),
                'xmlns:webgfcdossier' => 'http://www.adullact.org/webgfc/infodossier/1.0',
                'xmlns:xm' => 'http://www.w3.org/2005/05/xmlmine'));

            $dom_courrier = $this->_createElement($dom, 'courrier', null, array('idCourrier' => $courrier_id));
            $dom_courrier->appendChild($this->_createElement($dom, 'typeCourrier', $type_courrier));
            $dom_courrier->appendChild($this->_createElement($dom, 'dateCourrier', $date_courrier));
            $dom_courrier->appendChild($this->_createElement($dom, 'dateReception', $date_reception));
           
			$document = $this->_createElement($dom, 'document', null, array('nom' => 'courrier.pdf', 'type' => 'courrier'));
			$document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
			$document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
            $dom_courrier->appendChild($document);
            
            // Documents attachés au courrier
            $aDocuments = array();
            $aDocuments = $this->Courrier->getDocumentsForDelegation($courrier_id);

			$courrier_filename = $courrier_id . '-' . $aDocuments['docPrincipale']['Document']['name'];
            $document = $this->_createElement($dom, 'document', null, array(
				'idDocument' => $aDocuments['docPrincipale']['Document']['id'],
				'nom' => $courrier_filename,
				'relName' => 'Document' . DS . $courrier_filename,
				'type' => 'Document'));
// 			$document->appendChild($this->_createElement($dom, 'signature', 'false'));
			$document->appendChild($this->_createElement($dom, 'mimetype', 'application/pdf'));
			$document->appendChild($this->_createElement($dom, 'encoding', 'utf-8'));
			$dom_courrier->appendChild($document);
			
			// fusion du documentet ajout au zip
//			$zip->addFromString('Document' . DS . $courrier_filename, $aDocuments['docPrincipale']['Document']['content']);

			
            // Méta-données
            foreach($courrier['Metadonnee'] as $m => $meta ) {
				$document = $this->_createElement($dom, 'metadonnees', null, array(
					'idMeta' => $m,
					'nom' => $meta['name'],
					'relName' => 'Métadonnée' . DS . $meta['name'],
					'type' => 'Metadonnee')
                );
				$document->appendChild($this->_createElement($dom, 'name', $meta['name']));
				$document->appendChild($this->_createElement($dom, 'value', $meta['CourrierMetadonnee']['valeur']));

				$dom_courrier->appendChild($document);
			}
            
            
            //Insertion du noeud xml courrier
            $dom_depot->appendChild($dom_courrier);

//            $zip->close();
            $dom->appendChild($dom_depot);
            
            $file = new File($folderSend->pwd().DS.'XML_DESC_'.$courrier_id.'.xml');
            $file->append($dom->saveXML());
            $file->close();

            // On ne sauve que l fichier envoyé
            $file = new File($folderSend->pwd().DS.$doc['Document']['name']);
            $file->write($doc['Document']['content'], 'w');
            $file->close();
            
            $this->Courrier->id = $courrier_id;
            $this->Courrier->saveField('numero_depot', $idDepot);

            $this->Session->setFlash('Le dossier \"' . $libelle_courrier . '\" a été ajouté (Depot n°' . $idDepot . ')', 'growl', array('type' => 'success'));

        
        }
        catch (Exception $e)
        {
            $this->log('Export CMIS: Erreur ' . $e->getCode() . "! \n File:" .$e->getFile(). ' Line:'.$e->getLine(), 'error');
            throw $e;
        }
        
        return !empty($my_courrier_folder)?$my_courrier_folder:false;
            
    }
}



?>