<?php

App::uses('AppController', 'Controller');

/**
 * Rôles
 *
 * Desktops controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class DesktopsController extends AppController {

	/**
	 * Controller uses
	 *
	 * @access public
	 * @var array
	 */
	public $uses = array('Desktop');

	/**
	 * Controller helpers
	 *
	 * @access public
	 * @var array
	 */
	public $helpers = array('DataAcl.DataAcl');

	/**
	 * Controller components
	 *
	 * @access public
	 * @var array
	 */
	public $components = array('Rights');

	/**
	 * Ajout d'un rôle pour un utilisateur
	 *
	 * @logical-group Rôle
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $userId identifiant de l'utilisateur
	 * @param integer $mainDesktop identifiant du rôle principal
	 * @return void
	 */
	public function add($userId, $mainDesktop = false) {
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
			$this->Desktop->create($this->request->data);

			$this->Desktop->begin();
			if ($this->Desktop->save()) {
				$desktopsuser = $this->Desktop->DesktopsUser->create();
//				$desktopsuser['DesktopsUser']['main_desktop'] = $mainDesktop;
//				$desktopsuser['DesktopsUser']['active'] = true;
				$desktopsuser['DesktopsUser']['desktop_id'] = $this->Desktop->id;
				$desktopsuser['DesktopsUser']['user_id'] = $userId;
//debug($desktopsuser);
//die();
				if ($this->Desktop->DesktopsUser->save($desktopsuser)) {
					$this->Jsonmsg->valid();
				}
			}
			if ($this->Jsonmsg->json['success']) {
				$this->Desktop->commit();
			} else {
				$this->Desktop->rollback();
			}
			$this->Jsonmsg->send();
		} else {
			$this->set('groups', $this->Desktop->Group->find('list', array('conditions' => array('id >' => 2, 'Group.active' => true))));
			$this->set('services', $this->Desktop->Service->generateTreeList(array(), null, null, "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"));
			$this->set('userId', $userId);
		}
	}

	/**
	 * Edition d'un rôle
	 *
	 * @logical-group Rôle
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $desktopId identifiant du rôle
	 * @return void
	 */
	public function edit($desktopId) {
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
			$this->Desktop->create($this->request->data);

			$changeProfil = false;
			$prevProfilId = $this->Desktop->field('Desktop.group_id', array('Desktop.id' => $this->request->data['Desktop']['id']));
			if ($prevProfilId == $this->request->data['Desktop']['group_id']) {
				$changeProfil = true;
			}

			$this->Desktop->begin();
			if ($this->Desktop->save()) {
				$aroId = $this->Acl->Aro->field('Aro.id', array('model' => 'Desktop', 'foreign_key' => $this->request->data['Desktop']['id']));
				$valid = array();
				$hasSelfAcl = $this->Acl->Aro->Permission->find('first', array('conditions' => array('Permission.aro_id' => $aroId)));
				if (!empty($hasSelfAcl)) {
					$valid['Acl'] = $this->Acl->Aro->Permission->deleteAll(array('Permission.aro_id' => $aroId));
				}

				$this->loadModel('Right');
				$hasSelfRight = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $aroId)));
				if (!empty($hasSelfRight)) {
					$valid['AclSimplified'] = $this->Right->deleteAll(array('Right.aro_id' => $aroId));
//					$valid['AclSimplified'] = $this->Right->delete($hasSelfRight['Right']['id']);
				}

				$this->_logDebug($valid);
				if (!in_array(false, $valid, true)) {
					$this->Jsonmsg->valid();
					$this->Acl->flushcache();
				}
			}
			if ($this->Jsonmsg->json['success']) {
				$this->Desktop->commit();
			} else {
				$this->Desktop->rollback();
			}
			$this->Jsonmsg->send();
		} else {
			$this->request->data = $this->Desktop->find(
					'first', array(
				'conditions' => array(
					'Desktop.id' => $desktopId
				),
				'contain' => array(
					'Service'
				)
					)
			);
//			$this->request->data = $this->Desktop->read(null, $desktopId);
			$this->set('groups', $this->Desktop->Group->find('list', array('conditions' => array('id >' => 2, 'active' => true))));
			$services = $this->Desktop->Service->generateTreeList(array(), null, null, "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
			$this->set('services', $services);

			$srvs = $this->Desktop->getServicesList($desktopId);
			$selectedServices = array();
			foreach ($srvs as $k => $v) {
				$selectedServices[$k] = $services[$k];
			}
			$this->set('selectedServices', $selectedServices);
		}
	}

	/**
	 * Gestion des habilitations d'un rôle (interface graphique)
	 *
	 * @logical-group Rôle
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du rôle
	 * @param string $mode choix du type d'habilitations (func, func_classique, data, meta)
	 * @return void
	 */
	public function setRights($id = null, $mode = 'data') {
		if (!empty($this->request->data)) {


			$this->Jsonmsg->init();
			$this->loadModel('Right');
			if (!empty($this->request->data['Acl']['Type'])) {
				//Enregistrement en mode classique (a supprimer dans la prochaine version)
				if ($this->request->data['Acl']['Type'] == 'Acl') {
					if ($this->Acl->setRights($this->request->data['Requester'], $this->request->data['Rights'])) {
						$qdAro = array(
							'conditions' => array(
								'model' => key($this->request->data['Requester']),
								'foreign_key' => current($this->request->data['Requester'])
							)
						);
						$aro = $this->Acl->Aro->find('first', $qdAro);
						$rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $aro['Aro']['id'])));
						if (empty($rights)) {
							$rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $aro['Aro']['parent_id'])));
							$rights['Right']['id'] = null;
							$rights['Right']['aro_id'] = $aro['Aro']['id'];
							$rights['Right']['created'] = null;
							$rights['Right']['modified'] = null;
							$rights['Right']['model'] = 'Desktop';
							$rights['Right']['foreign_key'] = current($this->request->data['Requester']);
						}
						$rights['Right']['mode_classique'] = true;
						$this->Right->create($rights);
						$saved = $this->Right->save();
						if (!empty($saved)) {
							$this->Jsonmsg->valid();
						}
					}
				} else if ($this->request->data['Acl']['Type'] == 'DataAcl') {
					if ($this->SessionDataAcl->setRights($this->request->data['Requester'], $this->request->data['Rights'])) {
						$this->Jsonmsg->valid();
						$this->SessionDataAcl->flushCache();
					}
				}
				//Enregistrement en mode simplifié
			} else if (!empty($this->request->data['Right'])) {



				$aro_id = $this->request->data['Right']['aro_id'];
				$aro = $this->Acl->Aro->find('first', array('recursive' => -1, 'conditions' => array('id' => $aro_id)));

				$computedRights = $this->Right->saveRightSets($aro_id, $this->Rights->parseRightsData($this->request->data['Right']), true);


//					$this->ldebug($computedRights);
//					$this->ldebug($computedRights['controllers/Courriers/getMeta']);
//					$this->ldebug($this->Rights->parseRightsData($this->request->data['Right']));

				if (!empty($computedRights)) {
					$valid = array();
					$this->Acl->Aro->Permission->deleteAll(array('aro_id' => $aro['Aro']['id']));
					foreach ($computedRights as $key => $val) {
						if ($val) {
							$valid[] = $this->Acl->allow($aro['Aro'], $key);
						} else {
							$valid[] = $this->Acl->deny($aro['Aro'], $key);
						}
					}
					if (!in_array(false, $valid, true)) {
						$this->Jsonmsg->valid();
						$this->Acl->flushCache();
					}
				}
			}
			$this->Jsonmsg->send();
		} else {
			if ($id != null) {
				if ($mode == 'func') {
					$this->loadModel('Right');
					$aro = array('model' => 'Desktop', 'foreign_key' => $id);
					$aro_id = $this->Acl->Aro->field('id', $aro);
					$rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $aro_id)));
					if (empty($rights)) {
						$parent_id = $this->Acl->Aro->field('parent_id', $aro);
						$rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $parent_id)));
						$rights['Right']['id'] = null;
						$rights['Right']['aro_id'] = $aro_id;
						$rights['Right']['created'] = null;
						$rights['Right']['modified'] = null;
					}
					$this->set('rights', $rights);

					$qdGid = array(
						'fields' => array(
							'Group.id'
						),
						'conditions' => array(
							'Desktop.id' => $id,
							'NOT' => array(
// 								'Group.id' => array(ADMIN_GID, DISP_GID)
								'Group.id' => array(ADMIN_GID)
							)
						),
						'contain' => array('Group')
					);
					$gid = $this->Desktop->find('first', $qdGid);
					$this->set('editableGroup', !empty($gid));

					$this->set('tabsForSets', json_encode($this->Right->getTabsForSets()));
					$this->set('envsForSets', json_encode($this->Right->getEnvsForSets()));
					$this->set('ctxsForSets', json_encode($this->Right->getCtxsForSets()));
					$this->set('wkfsForSets', json_encode($this->Right->getWkfsForSets()));
				} else if ($mode == 'func_classique') {
					set_time_limit(120);
					//func
					$node = $this->Acl->Aro->node(array('model' => 'Desktop', 'foreign_key' => $id));
					$rights = array();
					$rights[] = $this->Acl->getRightsGrid($node[0]);
					$this->set('rights', $rights);
				} else if ($mode == 'data') {
					//data
					$daros = $this->Desktop->getDaros($id);
					$dataRights = array();
					$options = array(
						'actions' => array('read'),
						'models' => array('Type', 'Soustype')
					);

					$servicesNames = array();
					foreach ($daros as $daro) {
						$dataRights[$daro['Daro']['foreign_key']] = $this->DataAcl->getRightsGrid($daro, array(), $options);
						$serviceId = $this->Desktop->DesktopsService->field('DesktopsService.service_id', array('DesktopsService.id' => $daro['Daro']['foreign_key']));
						$servicesNames[$daro['Daro']['foreign_key']] = $this->Desktop->Service->field('Service.name', array('Service.id' => $serviceId));
					}
					$this->set('dataRights', $dataRights);
					$this->set('servicesNames', $servicesNames);
				} else if ($mode == 'meta') {
					//meta
					$daros = $this->Desktop->getDaros($id);
					$metaRights = array();
					$options = array(
						'actions' => array('read', 'update'),
						'models' => array('Metadonnee')
					);
					$servicesNames = array();
					foreach ($daros as $daro) {
						$metaRights[$daro['Daro']['foreign_key']] = $this->DataAcl->getRightsGrid($daro, array(), $options);
						$serviceId = $this->Desktop->DesktopsService->field('DesktopsService.service_id', array('DesktopsService.id' => $daro['Daro']['foreign_key']));
						$servicesNames[$daro['Daro']['foreign_key']] = $this->Desktop->Service->field('Service.name', array('Service.id' => $serviceId));
					}
					$this->set('metaRights', $metaRights);
					$this->set('servicesNames', $servicesNames);
				}
				$this->set('requester', array('model' => 'Desktop', 'foreign_key' => $id));
				$this->set('mode', $mode);
			}
		}
	}

	/**
	 * Gestion des délégations d'un rôle
	 *
	 * @logical-group Rôle
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du rôle
	 * @param integer $userId identifiant de l'utilisateur
	 * @param boolean $unset détermine si l'on doit appliquer ou terminer une délégation
	 * @param boolean $plandeleg détermin si la délégation est planifiée
	 * @return void
	 */
	public function setDeleg($id = null, $userId = null, $unset = false, $plandeleg = false) {
		if ($id != null) {
			if ($userId != null && $unset) {
				$this->Jsonmsg->init(__d('default', 'delete.error'));
				if ($plandeleg) {
					$this->loadModel('Plandelegation');
					$plandelegationId = $this->Plandelegation->field('Plandelegation.id', array(array('Plandelegation.user_id' => $userId, 'Plandelegation.desktop_id' => $id)));
					if ($this->Plandelegation->delete($plandelegationId)) {
						$this->Jsonmsg->valid(__d('default', 'delete.ok'));
					}
				} else {
					$delegationId = $this->Desktop->DesktopsUser->field('DesktopsUser.id', array(array('DesktopsUser.user_id' => $userId, 'DesktopsUser.desktop_id' => $id, 'DesktopsUser.delegation' => true)));
					if ($this->Desktop->DesktopsUser->delete($delegationId)) {
						$this->Jsonmsg->valid(__d('default', 'delete.ok'));
					}
				}
				$this->Jsonmsg->send();
			} else if (empty($this->request->data)) {
				$qdUsers = array(
					'fields' => array_merge($this->Desktop->User->fields(), $this->Desktop->fields()),
					'order' => 'username',
					'conditions' => array(
						'User.active' => true,
						'Desktop.group_id <>' => ADMIN_GID
					),
					'contain' => false,
					'joins' => array(
						$this->Desktop->User->join($this->Desktop->alias)
					)
				);
				$users = $this->Desktop->User->find('list', $qdUsers);

				$querydata = array(
					'conditions' => array(
						'DesktopsUser.desktop_id' => $id,
						'DesktopsUser.delegation' => true
					)
				);

				$desktopuser = $this->Desktop->DesktopsUser->find('all', $querydata);
				$alldesktopuser = Set::classicExtract($desktopuser, '{n}.DesktopsUser.user_id');

				$chooseusers = array();
				foreach ($users as $userId => $user) {
					if ($userId != $this->Session->read('Auth.User.id')) {

						if (!is_array($alldesktopuser) || is_array($alldesktopuser) && !in_array($userId, $alldesktopuser)) {
							$chooseusers[$userId] = $user;
						}
					}
				}
				$this->set('chooseusers', $chooseusers);


				$qdPlanDeleg = array(
					'order' => array(
						'Plandelegation.date_start, Plandelegation.created, Plandelegation.date_end'
					),
					'conditions' => array(
						'Plandelegation.desktop_id' => $id
					)
				);

				$this->loadmodel('Plandelegation');
				$plandelegations = $this->Plandelegation->find('all', $qdPlanDeleg);


				$this->set('baseDesktops', $this->Desktop->DesktopsUser->getBaseDesktops($this->Session->read('Auth.User.id')));
				$this->set('currentUserDesktops', $this->SessionTools->getDesktopList());
				$this->set('users', $users);
				$this->set('desktopId', $id);
				$this->set('desktopName', $this->Desktop->field('name', array('Desktop.id' => $id)));
				$this->set('desktopsuser', $desktopuser);
				$this->set('plandelegations', $plandelegations);
			} else {
				if (!empty($this->request->data['DesktopsUser']['planification'])) {
					$this->Jsonmsg->init(__d('desktop', 'Desktop.delegation.planification.error'));
					$this->loadmodel('Plandelegation');
					$plandelegation = $this->Plandelegation->create();
					foreach ($this->request->data['DesktopsUser'] as $key => $name) {
						$plandelegation['Plandelegation'][$key] = $name;
					}

					$qdTest = array(
						'conditions' => array(
							'Plandelegation.desktop_id' => $plandelegation['Plandelegation']['desktop_id'],
							'Plandelegation.user_id' => $plandelegation['Plandelegation']['user_id'],
							'Plandelegation.date_end > now()'
						)
					);
					$tests = $this->Plandelegation->find('first', $qdTest);


					if (empty($tests)) {
						$planed = $this->Plandelegation->save($plandelegation);
						if (!empty($planed)) {
							$this->Jsonmsg->valid(__d('desktop', 'Desktop.delegation.planification.ok'));
						}
					} else {
						$this->Jsonmsg->json['message'] .= "<br />" . __d('desktop', 'Desktop.delegation.planned');
					}
					$this->Jsonmsg->send();
				} else {
					$this->Jsonmsg->init(__d('desktop', 'Desktop.delegation.error'));
					$this->Desktop->DesktopsUser->create();
					if ($this->Desktop->DesktopsUser->save($this->request->data)) {
						$this->Jsonmsg->valid(__d('desktop', 'Desktop.delegation.ok'));
					}
					$this->Jsonmsg->send();
				}
			}
		}
	}

	/**
	 * Visualisation des habilitations (Acl simplifiées / Acl / Méta-données / Soustypes)
	 *
	 * @logical-group Rôle
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du rôle
	 * @param string $mode choix du type d'habilitations (func, func_classique, data, meta)
	 * @return void
	 */
	public function getRights($id = null, $mode = 'data') {
		if ($id != null) {
			if ($mode == 'func') {
				$this->loadModel('Right');
				$aro = array('model' => 'Desktop', 'foreign_key' => $id);
				$aro_id = $this->Acl->Aro->field('id', $aro);
				$rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $aro_id)));
				if (empty($rights)) {
					$parent_id = $this->Acl->Aro->field('parent_id', $aro);
					$rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $parent_id)));
					$rights['Right']['id'] = null;
					$rights['Right']['aro_id'] = $aro_id;
					$rights['Right']['created'] = null;
					$rights['Right']['modified'] = null;
				}
				$this->set('rights', $rights);
			} else if ($mode == 'func_classique') {
				//func
				$node = $this->Acl->Aro->node(array('model' => 'Desktop', 'foreign_key' => $id));
				$rights = array();
				$rights[] = $this->Acl->getRightsGrid($node[0]);
				$this->set('rights', $rights);
			} else if ($mode == 'data') {
				//data
				$daros = $this->Desktop->getDaros($id);
				$dataRights = array();
				$options = array(
					'actions' => array('read'),
					'models' => array('Type', 'Soustype')
				);
				foreach ($daros as $daro) {
					$dataRights[$daro['Daro']['foreign_key']] = $this->DataAcl->getRightsGrid($daro, array(), $options);
				}
				$this->set('dataRights', $dataRights);
			} else if ($mode == 'meta') {
				//meta
				$metaRights = array();
				$options = array(
					'actions' => array('read', 'update'),
					'models' => array('Metadonnee')
				);
				foreach ($daros as $daro) {
					$metaRights[$daro['Daro']['foreign_key']] = $this->DataAcl->getRightsGrid($daro, array(), $options);
				}
				$this->set('metaRights', $metaRights);
			}
			$this->set('requester', array('model' => 'Desktop', 'foreign_key' => $id));
			$this->set('mode', $mode);
		}
	}

	/**
	 * Gestion des habilitations d'un rôle (ajax)
	 *
	 * @logical-group Rôle
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du rôle
	 * @param string $mode détermine si on est en visualisation ou en édition (set, get)
	 * @return void
	 */
	public function rights($id = null, $mode = 'set') {
		if ($id != null) {
			$this->set('id', $id);
			$this->set('mode', $mode);
			$this->set('desktopName', $this->Desktop->field('name', array('Desktop.id' => $id)));
		}
	}

	/**
	 * Suppresion d'un rôle pour un utilisateur
	 *
	 * @logical-group Rôle
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $desktopId identifiant du rôle
	 * @param interger $userId identifiant de l'utilisateur
	 * @return void
	 */
	public function delete($desktopId, $userId) {
		$this->Jsonmsg->init(__('default', 'delete.error'));

		$message = '';
		$desktopUserId = $this->Desktop->DesktopsUser->field('DesktopsUser.id', array('DesktopsUser.desktop_id' => $desktopId, 'DesktopsUser.user_id' => $userId));
		if ($this->Desktop->DesktopsUser->delete($desktopUserId)) {
			$message = 'Liaison : ' . __d('default', 'delete.ok');
		}

		$desktopUser = $this->Desktop->DesktopsUser->find('all', array('conditions' => array('DesktopsUser.desktop_id' => $desktopId)));
		if (empty($desktopUser)) {
			if ($this->Desktop->delete($desktopId)) {
				$message .= '<br />Bureau : ' . __d('default', 'delete.ok');
			}
		}

		if ($message != '') {
			$this->Jsonmsg->valid($message);
		}
		$this->Jsonmsg->send();
	}

}
