<?php

/**
 * Documents principaux et pièces jointes
 *
 * Documents controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class DocumentsController extends AppController {

	/**
	 * Controller name
	 *
	 * @access public
	 * @var array
	 */
	public $name = 'Documents';

	/**
	 * Controller components
	 *
	 * @access public
	 * @var array
	 */
	public $components = array('Linkeddocs');

	/**
	 * Définition d'une pièce jointe comme document principal
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $id identifiant du document
	 * @return void
	 */
	public function setMainDoc($id) {
		$this->Jsonmsg->init();
		$this->Document->begin();

		$fluxId = $this->Document->field('Document.courrier_id', array('Document.id' => $id));
		$docs = $this->Document->find('all', array('conditions' => array('Document.courrier_id' => $fluxId)));

		$valid = array();
		foreach ($docs as $doc) {
			$this->Document->create();
			$doc['Document']['main_doc'] = false;
			$saved = $this->Document->save($doc);
			$valid[] = !empty($saved);
		}

		$this->Document->id = $id;
		$valid[] = $this->Document->saveField('main_doc', true);

		if (!in_array(false, $valid, true)) {
			$this->Jsonmsg->valid();
			$this->Document->commit();
		} else {
			$this->Document->rollback();
		}
		$this->Jsonmsg->send();
	}

	/**
	 *
	 * @param type $fluxId
	 * @throws NotFoundException
	 */
//	public function getDocumentList($fluxId) {
//		$qdCourrier = array(
//			'conditions' => array(
//				'Courrier.id' => $fluxId
//			),
//			'contain' => array(
//				'Document' => array(
//					'fields' => $this->Document->getLightFields(),
//					'order' => 'Document.created'
//				)
//			)
//		);
//		$courrier = $this->Document->Courrier->find('first', $qdCourrier);
//
//		if (empty($courrier)) {
//			throw new NotFoundException();
//		}
//
//		$this->set('documentList', $courrier['Document']);
//	}

	/**
	 * Ajout d'un document pricipal ou d'une pièce jointe
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @user-profile User
	 *
	 * @access public
	 * @return void
	 */
	public function upload() {
		if ($this->request->is('post')) {
			$ext = '';

			if (preg_match("/\.([^\.]+)$/", $_FILES['myfile']['name'], $matches)) {
				$ext = $matches[1];
			}
                    
			$docs = $this->Document->find('list', array('conditions' => array('Document.courrier_id' => $this->request->data['courrierId'])));
			$doc = array(
				'Document' => array(
					'size' => $_FILES['myfile']['size'],
					'mime' => $_FILES['myfile']['type'],
					'ext' => $ext,
					'courrier_id' => $this->request->data['courrierId'],
					'desktop_creator_id' => $this->Session->read( 'Auth.User.Desktop.id' ),
					'main_doc' => empty($docs)
				)
			);

			$this->Document->create($doc);
			$docsaved = $this->Document->save();
			$this->set('uploadValid', !empty($docsaved));
			$this->set('fluxId', $this->request->data['courrierId']);
		}
	}

	/**
	 * Suppression d'un document principal ou d'une pièce jointe
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $documentId identifiant du document
	 * @throws NotFoundException
	 * @return void
	 */
	public function delete($documentId) {
		$document = $this->Document->find('first', array('fields' => array('Document.id', 'Document.courrier_id'), 'conditions' => array('Document.id' => $documentId), 'recursive' => -1));
		if (empty($document)) {
			throw new NotFoundException();
		}

		$this->Jsonmsg->init(__d('default', 'delete.error'));
		if ($this->Document->delete($documentId)) {
			$this->Jsonmsg->valid(__d('default', 'delete.ok'));
		}
		$this->Jsonmsg->send();
	}

	/**
	 * Téléchargement d'un document principal ou d'une pièce jointe
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $documentId identifiant du document
	 * @throws NotFoundException
	 * @return void
	 */
	public function download($documentId) {
		$document = $this->Document->find('first', array('conditions' => array('Document.id' => $documentId), 'recursive' => -1));
		if (empty($document)) {
			throw new NotFoundException();
		}

		$this->autoRender = false;
		Configure::write('debug', 0);
		header("Pragma: public");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Content-type: application/octet-stream");
		header("Content-Disposition: attachment; filename=\"" . $document['Document']["name"] . "\"");
		header("Content-length: " . $document['Document']['size']);
		print $document['Document']['content'];
		exit();
	}

	/**
	 * Récupération de la pré-visualisation (miniature) d'un document principal ou d'une pièce jointe
	 *
	 * @logical-group Flux
	 * @logical-group Context
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $documentId identifiant du document
	 * @throws NotFoundException
	 * @return void
	 */
	public function getPreview($documentId) {
		$document = $this->Document->find('first', array('conditions' => array('Document.id' => $documentId), 'recursive' => -1));
		if (empty($document)) {
			throw new NotFoundException();
		}

		$preview = __d('document', 'Document.preview.error');

		if (empty($document['Document']['preview'])) {
			$preview = $this->Document->generatePreview($documentId);
		} else {
			$preview = $document['Document']['preview'];
		}


		$mime = $document['Document']['mime'];


		$tmpPreviewFile = '';
		$tmpPreviewFileBasename = '';
		$previewType = '';

		if (in_array($mime, $this->Document->convertibleMimeTypes) || $mime == "application/pdf") {
			$previewType = 'flexpaper';
			$tmpPreviewFileBasename = Inflector::slug($this->Session->read('Auth.User.username')) . "_" . Inflector::slug($document['Document']['name']) . "_preview.swf";
			$tmpPreviewFile = APP . DS . WEBROOT_DIR . DS . 'files' . DS . 'previews' . DS . $tmpPreviewFileBasename;
			//purge des fichiers de preview précédents
			$this->Linkeddocs->purge();
			file_put_contents($tmpPreviewFile, $preview);
		} else {
			foreach ($this->Document->otherMimeTypes as $type => $mimes) {
				if (in_array($mime, array_keys($mimes))) {
					$previewType = $type;
					$tmpPreviewFileBasename = Inflector::slug($this->Session->read('Auth.User.username')) . "_" . Inflector::slug($document['Document']['name']) . "_preview." . $mimes[$mime];
					$tmpPreviewFile = APP . DS . WEBROOT_DIR . DS . 'files' . DS . 'previews' . DS . $tmpPreviewFileBasename;
					//purge des fichiers de preview précédents
					$this->Linkeddocs->purge();
					file_put_contents($tmpPreviewFile, $preview);
				}
			}
		}

		if( !empty($tmpPreviewFile)) {
			chmod($tmpPreviewFile, 0770);
		}
		$this->set('previewType', $previewType);
		$this->set('mime', $mime);
		$this->set('tmpPreviewFileBasename', $tmpPreviewFileBasename);
	}

}

?>
