<?php

/**
 * Dossiers
 *
 * Dossiers controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class DossiersController extends AppController {

	/**
	 * Controller name
	 *
	 * @access public
	 * @var array
	 */
	public $name = 'Dossiers';

	/**
	 * Gestion des dossiers/affaires
	 *
	 * @logical-group Dossier
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function index() {
		$this->set('ariane', array(
                    '<a href="/environnement/index/0/admin">'.__d('menu', 'Administration', true).'</a>',
                    __d('menu', 'gestionDossiersAffaires', true)));
	}

	/**
	 * Ajout d'un dossier (avec incrémentation du compteur)
	 *
	 * @logical-group Dossier
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function add() {
		if (!empty($this->request->data)) {
			$this->Dossier->save($this->request->data);
		} else {
			$this->loadModel('Compteur');
			$CompteurDossier = $this->Compteur->find(
				'first',
				array(
					'fields' => array( 'Compteur.id', 'Compteur.nom' ),
					'conditions' => array(
						'Compteur.nom' => 'compteur_dossier'
					),
					'recursive' => -1
				)
			);
			$idDossier = $CompteurDossier['Compteur']['id'];
			
			$this->set('newDossierName', $this->Compteur->genereCompteur($idDossier));
		}
	}

	/**
	 * Edition d'un dossier
	 *
	 * @logical-group Dossier
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du dossier
	 * @return void
	 */
	public function edit($id = null) {
		if ($id != null) {
			if (!empty($this->request->data)) {
				$this->Dossier->save($this->request->data);
			} else {
				$this->request->data = $this->Dossier->read(null, $id);
			}
		}
	}

	/**
	 * Suppression d'un dossier
	 *
	 * @logical-group Dossier
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du dossier
	 * @return void
	 */
	public function delete($id = null) {
		if ($id != null) {
			$this->Dossier->delete($id);
		}
	}

	/**
	 * Récupération de la liste des dossiers
	 *
	 * @logical-group Dossier
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function getDossiers() {
		$this->Dossier->recursive = 1;
		$this->set('dossiers', $this->Dossier->getDossiersAffairesTree());
	}

	/**
	 * Récupération de la liste des affaires hors dossier
	 *
	 * @access private
	 * @return void
	 */
	private function _getAffairesHorsDossier() {
		$affaires_tmp = $this->Dossier->Affaire->find("all", array(
			'order' => 'Affaire.name',
			'conditions' => array(
				array(
					'Affaire.dossier_id' => null
				)
			)
				));
		$affaires = array();
		foreach ($affaires_tmp as $item) {
			$item['right_edit'] = true;
			$item['right_delete'] = true;
			$affaires[] = $item;
		}
		$this->set('affaires', $affaires);
	}

}

?>