<?php

/**
 * Environnement
 *
 * Environnement controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
//    @set_time_limit( 0 );
// Mémoire maximum allouée à l'exécution de ce script
//	@ini_set( 'memory_limit', '2048M' );
// Temps maximum d'exécution du script (en secondes)
//	@ini_set( 'max_execution_time', 2000 );
// Temps maximum (en seconde), avant que le script n'arrête d'attendre la réponse de Gedooo
//	@ini_set( 'default_socket_timeout', 12000 );

class EnvironnementController extends AppController {

    /**
     * Controller name
     *
     * @access public
     * @var string
     */
    public $name = 'Environnement';

    /**
     * Controller uses
     *
     * @access public
     * @var array
     */
    public $uses = array('User', 'Notification');

    /**
     * Liste des bannettes
     *
     * @access private
     * @var array
     */
    private $_bannettes = array();

    /**
     * Controller helpers
     *
     * @var array
     * @access public
     */
    public $helpers = array('Paginator');

    /**
     * Liste des rôles de l'utilisateur pouvant ajouter un flux
     *
     * @access private
     * @var array
     */
    private $_add = array();

    /**
     * Liste des rôles de l'utilisateur pouvant ajouter un flux
     *
     * @access private
     * @var array
     */
    private $_addService = array();

    /**
     * Détermine si on a obtenu la liste des flux des services dépendants
     *
     * @access private
     * @var boolean
     */
    private $_initBancontenuService = false;

    /**
     * Traitement des informations brutes d'un flux en vue de l'affichage
     *
     * @access private
     * @param array $flux informations du flux
     * @param boolean $isParent si le flux est un flux parent (des réponses à ce flux ont été
     * @param boolean $delete détermine si l'action de suppression sera disponible à l'affichage
     * @return array les informations du flux traitées
     */
    private function _processFlux($flux, $isParent = false, $delete = true, $parentInfo = array(), $read = true) {
        return array(
            "id" => $flux['Courrier']['id'],
            "priorite" => $flux['Courrier']['priorite'],
//			"retard" => !$this->Bancontenu->Courrier->isInTime($flux['Courrier']['id']),
            "retard" => !$flux['Courrier']['is_in_time'],
//			"ficassoc" => $this->Bancontenu->Courrier->hasAssociatedFile($flux['Courrier']['id']),
            "ficassoc" => $flux['Courrier']['ficassoc'],
//			"direction" => $this->Bancontenu->Courrier->getDirection($flux['Courrier']['id']),
            "direction" => $flux['Courrier']['direction'],
            "etat" => $flux['Bancontenu']['etat'],
            "reference" => $flux['Courrier']['reference'],
            "nom" => $flux['Courrier']['name'],
            "objet" => $flux['Courrier']['objet'],
            "contact" => $flux['Contactinfo']['name'],
            "date" => $flux['Courrier']['datereception'],
            "created" => $flux['Courrier']['created'],
            "typesoustype" => ($flux['Type']['name'] . '/' . $flux['Soustype']['name'] != '/') ? $flux['Type']['name'] . '/' . $flux['Soustype']['name'] : '',
            "bureau" => $flux['Desktop']['name'],
            "modified" => $flux['Courrier']['modified'],
            "desktopId" => $flux['Desktop']['id'],
            "view" => $this->Acl->check(array('model' => 'Desktop', 'foreign_key' => $flux['Desktop']['id']), 'controllers/Courriers/view'),
            "edit" => $this->Acl->check(array('model' => 'Desktop', 'foreign_key' => $flux['Desktop']['id']), 'controllers/Courriers/view'),//Arnaud ('/controllers/Courriers/view')
            "delete" => ($delete) ? $this->Acl->check(array('model' => 'Desktop', 'foreign_key' => $flux['Desktop']['id']), 'controllers/Courriers/delete') : false,
            "isParent" => $isParent,
            "parent_id" => $flux['Courrier']['parent_id'],
            "parentInfo" => $parentInfo,
            'read' => $read,
            "creatorId" => $flux['Courrier']['user_creator_id'],
            "userconnectedId" => $this->Session->read('Auth.User.id'),
            //"mailRetardEnvoye" => $flux['Courrier']['mail_retard_envoye']
        );
    }

    /**
     * Fonction permettant de créer des champs virtuels pour les colonnes présentes dans les différentes bannettes
     *  - fichiers associés oui/non
     *  - sens du flux (entrant, sortant, interne)
     *  - flux en retard ou non
     *
     * @return array
     */
//    protected function _virtualFieldsCourrier() {
//        return array(
//            'EXISTS( SELECT documents.id FROM documents WHERE documents.courrier_id = "Courrier"."id" ) AS "Courrier__ficassoc"',
//            '( CASE WHEN "Courrier"."direction" = \'0\' THEN \'S\' WHEN "Courrier"."direction" = \'1\' THEN \'E\' WHEN "Courrier"."direction" = \'2\' THEN \'I\' ELSE NULL END ) AS "Courrier__direction"',
//            '( CASE
//                WHEN "Courrier"."soustype_id" IS NULL THEN true
//                WHEN "Bancontenu"."etat" IN ( -1, 2 ) THEN true
//                WHEN "Soustype"."delai_unite" = \'0\' THEN ( ( "Courrier"."datereception" + ( "Soustype"."delai_nb" * INTERVAL \'1 day\' ) ) > NOW() )
//                WHEN "Soustype"."delai_unite" = \'1\' THEN ( ( "Courrier"."datereception" + ( "Soustype"."delai_nb" * INTERVAL \'1 week\' ) ) > NOW() )
//                WHEN "Soustype"."delai_unite" = \'2\' THEN ( ( "Courrier"."datereception" + ( "Soustype"."delai_nb" * INTERVAL \'1 month\' ) ) > NOW() )
//                ELSE true
//            END ) AS "Courrier__is_in_time"',
//        );
//    }
    protected function _virtualFieldsCourrier() {
        return array(
            'EXISTS( SELECT documents.id FROM documents WHERE documents.courrier_id = "Courrier"."id" ) AS "Courrier__ficassoc"',
            '( CASE WHEN "Courrier"."direction" = \'0\' THEN \'S\' WHEN "Courrier"."direction" = \'1\' THEN \'E\' WHEN "Courrier"."direction" = \'2\' THEN \'I\' ELSE NULL END ) AS "Courrier__direction"',
            '( CASE
                WHEN "Courrier"."soustype_id" IS NULL THEN true
                WHEN "Bancontenu"."etat" IN ( -1, 2 ) THEN true
                WHEN "Courrier"."delai_unite" = \'0\' THEN ( ( "Courrier"."datereception" + ( "Courrier"."delai_nb" * INTERVAL \'1 day\' ) ) > NOW() )
                WHEN "Courrier"."delai_unite" = \'1\' THEN ( ( "Courrier"."datereception" + ( "Courrier"."delai_nb" * INTERVAL \'1 week\' ) ) > NOW() )
                WHEN "Courrier"."delai_unite" = \'2\' THEN ( ( "Courrier"."datereception" + ( "Courrier"."delai_nb" * INTERVAL \'1 month\' ) ) > NOW() )
                ELSE true
            END ) AS "Courrier__is_in_time"',
        );
    }
    /**
     * Récupération de la liste des flux disponibles via les contenus de bannettes d'un rôle
     *
     * @param array $bannettesParams liste d'array() contenant
     *  - array avec pour clé desktop_id pour les IDs de desktop
     *  - array avec pour clé bannettes pour les noms de bannettes
     * @return array listes des flux par bannettes pour l'ensemble des rôles passés en paramètre
     */
    private function _getAllBancontenus(array $bannettesParams) {      
        $this->loadModel('Bancontenu');

        $bannettes = array();
        foreach ($bannettesParams as $bannetteParams) {
            foreach ($bannetteParams['bannettes'] as $bannetteName) {
                if (!isset($bannettes[$bannetteName])) {
                    $bannettes[$bannetteName] = $bannetteParams['desktop_id'];
                } else {
                    $bannettes[$bannetteName] = array_merge($bannettes[$bannetteName], $bannetteParams['desktop_id']);
                }
            }
        }

//        $this->Session->write( 'paginationModel', null );
        $paginationModel = $this->Session->read('paginationModel');

        $page = Hash::get($this->request->params, 'named.page');
        $namedModel = Hash::get($this->request->params, 'named.model');

        foreach ($bannettes as $bannetteName => $desktopId) {
            $params = array('class' => 'Bannette', 'alias' => Inflector::camelize("bannette_{$bannetteName}"));
            $alias = $params['alias'];
            $this->{$alias} = ClassRegistry::init($params);

            $querydata = array(
                'fields' => array_merge(
                        $this->{$alias}->Bancontenu->fields(),
                        $this->{$alias}->fields(),
                        $this->{$alias}->Bancontenu->Courrier->fields(),
                        $this->{$alias}->Bancontenu->Desktop->fields(),
                        $this->{$alias}->Bancontenu->Courrier->Contactinfo->fields(),
                        $this->{$alias}->Bancontenu->Courrier->Soustype->fields(),
                        $this->{$alias}->Bancontenu->Courrier->Soustype->Type->fields(), 
                        $this->{$alias}->Bancontenu->Courrier->Traitement->Circuit->fields(),
                        $this->_virtualFieldsCourrier()
                ),
                'contain' => false,
                'joins' => array(
                    $this->{$alias}->join($this->{$alias}->Bancontenu->alias),
                    $this->{$alias}->Bancontenu->join($this->{$alias}->Bancontenu->Desktop->alias),
                    $this->{$alias}->Bancontenu->join($this->{$alias}->Bancontenu->Courrier->alias),
                    $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Contactinfo->alias),
                    $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Soustype->alias, array('type' => 'LEFT OUTER')),
                    $this->{$alias}->Bancontenu->Courrier->Soustype->join($this->{$alias}->Bancontenu->Courrier->Soustype->Type->alias, array('type' => 'LEFT OUTER')),
                    $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Traitement->alias, array('type' => 'LEFT OUTER')),
                    $this->{$alias}->Bancontenu->Courrier->Traitement->join($this->{$alias}->Bancontenu->Courrier->Traitement->Circuit->alias, array('type' => 'LEFT OUTER'))
                ),
                'conditions' => array(
                    'Desktop.id' => $desktopId,
                    'Bancontenu.etat' => "1",
                    "{$alias}.name" => $bannetteName,
                ),
                'order' => array(
                    'Courrier.created' => 'DESC',
                    'Courrier.id' => 'DESC'
                ),
//                'limit' => 1 // FIXME: déjà présent dans le paginator
            );

            // Gestion de la pagination pour ne passer de page en page que sur
            // le modèle qui nous intéresse
            if (isset($paginationModel[$alias])) {
                $params = $this->request->params;
                if ($namedModel === $alias) {
                    $querydata['page'] = $params['named']['page'] = $page;
                    if (!empty($params['named']['sort']) && !empty($params['named']['direction'])) {
                        $querydata['order'] = array($params['named']['sort'] => $params['named']['direction']);
                    }
                } else {
                    $querydata['limit'] = $paginationModel[$alias]['limit'];
                    $querydata['order'] = $paginationModel[$alias]['order'];
                    $querydata['page'] = max((int) @$paginationModel[$alias]['page'], 1);
                    $querydata['offset'] = ( ( (int) $paginationModel[$alias]['page'] ) <= 1 ? 0 : ( $paginationModel[$alias]['limit'] * ( $paginationModel[$alias]['page'] - 1 ) ) );
                }

                $params['options'] = (array) Hash::get($paginationModel, "{$alias}.options");

                $this->request->params = $params;
            }

            $paginate = $this->paginate;
            $paginate[$alias] = $querydata;
            // $paginate[$alias]['limit'] = 10; // Afin de n'afficher que 10 lignes par bannettes
            $this->paginate = $paginate;

            $this->request->params['named']['page'] = @$querydata['page'];
            $bannettes[$bannetteName] = $this->paginate($this->{$alias});
            if (!empty($page)) {
                $this->request->params['named']['page'] = $page;
            }

            //preparation de chaque flux
            $parent = array();
            for ($i = 0; $i < count($bannettes[$bannetteName]); $i++) {
                
                if (isset($bannettes[$bannetteName][$i][$alias]) && !empty($bannettes[$bannetteName][$i][$alias])) {

                    $nomBannette = $bannettes[$bannetteName][$i][$alias]['name'];
                    //si parent_id => insertion du parent traité dans la bannette
                    if (!empty($bannettes[$bannetteName][$i]['Courrier']['parent_id'])) {
                        $qdParent = $querydata;
                        $qdParent['conditions'] = array('Courrier.id' => $bannettes[$bannetteName][$i]['Courrier']['parent_id']);
                        $parent = $this->_dataParent($bannettes[$bannetteName][$i]['Courrier']['parent_id']);
                    }

                    $delete = $nomBannette != 'refus' && $nomBannette != 'reponses' && $nomBannette != 'copy';
                    $bannettes[$bannetteName][$i] = $this->_processFlux($bannettes[$bannetteName][$i], false, $delete, $parent, $bannettes[$bannetteName][$i]['Bancontenu']['read']);
                    
                    // Notification de retard
//                    debug($bannettes[$bannetteName][$i]);
//                    die();
                    if( !empty($bannettes[$bannetteName][$i]['retard'] ) && empty($bannettes[$bannetteName][$i]['mailRetardEnvoye']) ){
                        $user_id = $this->{$alias}->Bancontenu->Courrier->User->userIdByDesktop($bannettes[$bannetteName][$i]['desktopId']);
                        $success = $this->{$alias}->Bancontenu->Courrier->User->notifier($bannettes[$bannetteName][$i]['id'], $user_id, 'retard_validation');
                        if( $success) {
                            $this->{$alias}->Bancontenu->Courrier->updateAll(
                                array(
                                    'Courrier.mail_retard_envoye' => true
                                ),
                                array(
                                    'Courrier.id' => $bannettes[$bannetteName][$i]['id']
                                )
                            );
                            $this->Jsonmsg->valid();
                        }
                        else {
                            $this->Jsonmsg->init();
                            $this->Jsonmsg->error();
//                            $this->Jsonmsg->send( 'Erreur d\'envoi du mail');
                        }
                    }
                }
            }
            

        }
        $this->Session->write('paginationModel', $this->request->params['paging']);

        // Récupération de la liste des flux présents dans la bannette "Mes flux en copie"
        for ($i = 0; $i < count($bannettes['copy']); $i++) {
            $fluxCopy[] = $bannettes['copy'][$i]['id'];
        }
        if( !empty($fluxCopy)) {
            $this->Session->delete('Auth.User.Courrier.mesCopies');
            $this->Session->write('Auth.User.Courrier.mesCopies', $fluxCopy);
        }
        
        return $bannettes;
    }

    /**
     *  Fonction permettant de récupérer les données parentes de flux réponses
     *
     */
    protected function _dataParent($courrierParent_id) {
        $courrierParent = $this->Bancontenu->Courrier->find(
                'first', array(
            'conditions' => array(
                'Courrier.id' => $courrierParent_id
            ),
            'contain' => array(
                'Contactinfo' => array(
                    'Contact'
                ),
                'Soustype' => array(
                    'Type'
                ),
                'Desktop'
            )
                )
        );
        return $courrierParent;
    }

    /**
     * Uniquement pour les droits
     */
    public function getBancontenusServices() {

    }

    /**
     * Récupération de la liste des flux des services dépendants
     *
     * @access public
     * @return array liste des flux de la bannette "Mes services"
     */
    private function _getBancontenusServices() {

//        return array( ); // FIXME

        $bannetteService = array();
        $this->loadModel('Desktop');
        $services = array();
        $desktops = $this->SessionTools->getDesktopList();
        foreach ($desktops as $desktop) {
            $qd = array(
                'fields' => array(
                    'DesktopsService.service_id'
                ),
                'conditions' => array(
                    'DesktopsService.desktop_id' => $desktop
                )
            );
            $srvs = $this->Desktop->DesktopsService->find('all', $qd);

            if (!empty($srvs)) {
                $services = array_merge($services, array_unique(Set::classicExtract($srvs, '{n}.DesktopsService.service_id')));
            }
        }

        $allServices = $services;
        foreach ($services as $service) {
            $children = $this->Desktop->Service->children($service);
            if (!empty($children)) {
                $allServices = array_merge($allServices, array_unique(Set::classicExtract($children, '{n}.Service.id')));
            }
        }
        $allServices = array_unique($allServices);
        $childrenDesktops = array();
        foreach ($allServices as $srv) {
            $qd = array(
                'fields' => array(
                    'DesktopsService.desktop_id'
                ),
                'conditions' => array(
                    'DesktopsService.service_id' => $srv
                )
            );
            $dktps = $this->Desktop->DesktopsService->find('all', $qd);
            if (!empty($dktps)) {
                $childrenDesktops = array_merge($childrenDesktops, array_unique(Set::classicExtract($dktps, '{n}.DesktopsService.desktop_id')));
            }
        }
        $childrenDesktops = array_unique($childrenDesktops);
        $childrenDesktops = array_diff($childrenDesktops, $desktops);

        $params = array('class' => 'Bannette', 'alias' => Inflector::camelize("bannette_services"));
        $alias = $params['alias'];
        $this->{$alias} = ClassRegistry::init($params);

        $page = Hash::get($this->request->params, 'named.page');
        $namedModel = Hash::get($this->request->params, 'named.model');

        $querydata = array(
            'fields' => array_merge(
                    $this->{$alias}->Bancontenu->fields(), $this->{$alias}->fields(), $this->{$alias}->Bancontenu->Courrier->fields(), $this->{$alias}->Bancontenu->Desktop->fields(), $this->{$alias}->Bancontenu->Courrier->Contactinfo->fields(), $this->{$alias}->Bancontenu->Courrier->Soustype->fields(), $this->{$alias}->Bancontenu->Courrier->Soustype->Type->fields(), $this->{$alias}->Bancontenu->Courrier->Traitement->Circuit->fields(), $this->_virtualFieldsCourrier()
            ),
            'contain' => false,
            'joins' => array(
                $this->{$alias}->join($this->{$alias}->Bancontenu->alias),
                $this->{$alias}->Bancontenu->join($this->{$alias}->Bancontenu->Desktop->alias),
                $this->{$alias}->Bancontenu->join($this->{$alias}->Bancontenu->Courrier->alias),
                $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Contactinfo->alias),
                $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Soustype->alias, array('type' => 'LEFT OUTER')),
                $this->{$alias}->Bancontenu->Courrier->Soustype->join($this->{$alias}->Bancontenu->Courrier->Soustype->Type->alias, array('type' => 'LEFT OUTER')),
                $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Traitement->alias, array('type' => 'LEFT OUTER')),
                $this->{$alias}->Bancontenu->Courrier->Traitement->join($this->{$alias}->Bancontenu->Courrier->Traitement->Circuit->alias, array('type' => 'LEFT OUTER'))
            ),
            'conditions' => array(
                'Desktop.id' => $childrenDesktops,
                'Bancontenu.etat' => array(1, 0),
                'Bancontenu.bannette_id <>' => BAN_COPY
            ),
            'order' => array(
//                "{$alias}.name" => 'ASC',
//                 'Courrier.created' => 'DESC',
               'Bancontenu.created' => 'DESC'
            )
        );
// debug($childrenDesktops);
        if ($namedModel == $alias && !empty($this->request->params['named']['sort']) && !empty($this->request->params['named']['direction'])) {
            $querydata['order'] = array($this->request->params['named']['sort'] => $this->request->params['named']['direction']); //FIXME
        }
        $bannettes['services'] = $this->{$alias}->find('all', $querydata);
// debug($bannettes['services']);
        //preparation de chaque flux
        $fluxCtrl = array();
        if (isset($this->_bannettes['flux'])) {
            $banFlux = Set::classicExtract($this->_bannettes['flux'], '{n}.id'); //on recupere les flux présent dans la bannette flux
            if (!empty($banFlux)) {
                $fluxCtrl = $banFlux;
            }
        }

        //preparation de chaque flux
        //ici on evite de traiter les flux deja presents dans la bannette flux et les doublons
        $parent = array();
        for ($i = 0; $i < count($bannettes['services']); $i++) {
            if (!in_array($bannettes['services'][$i]['Courrier']['id'], $fluxCtrl)) {
                if (!empty($bannettes['services'][$i]['Courrier']['parent_id'])) {
                    $qdParent = $querydata;
                    $qdParent['conditions'] = array('Courrier.id' => $bannettes['services'][$i]['Courrier']['parent_id']);
//                    $parent = $this->{$alias}->find('first', $qdParent);
                    $parent = $this->_dataParent($bannettes['services'][$i]['Courrier']['parent_id']);
//                    $bannetteService[] = $this->_processFlux($parent, true, false);
                }
                $flux = $this->_processFlux($bannettes['services'][$i], false, false, $parent); //le parametre read est toujours défini a true pour les flux de mes services
                $bannetteService[] = $flux;

                $fluxCtrl[] = $bannettes['services'][$i]['Courrier']['id'];
            }
        }


        $paginationModel = (array) $this->Session->read('paginationModel');

        if (!isset($paginationModel[$alias])) {
            $paginationModel[$alias] = array(
                'page' => 1,
                'current' => null,
                'count' => null,
                'prevPage' => null,
                'nextPage' => null,
                'pageCount' => null,
                'order' => array(),
                'limit' => 20,
                'options' => array(),
                'paramType' => 'named'
            );
        }

        $requestparams = $this->request->params;

        $requestparams['paging'][$alias] = array('page' => 1, 'limit' => 20, 'order' => array(), 'options' => array(), 'paramType' => 'named') + (array) Hash::get($requestparams, "paging.{$alias}");
        if ($namedModel == $alias) {
            $requestparams['paging'][$alias]['page'] = $page;

            if (!empty($requestparams['named']['sort']) && !empty($requestparams['named']['direction'])) {
                $requestparams['paging'][$alias]['order'] = array($requestparams['named']['sort'] => $requestparams['named']['direction']); //FIXME
                $requestparams['paging'][$alias]['options']['order'] = $requestparams['paging'][$alias]['order']; //FIXME
                $requestparams['paging'][$alias]['options']['sort'] = $requestparams['named']['sort']; //FIXME
                $requestparams['paging'][$alias]['options']['direction'] = $requestparams['named']['direction']; //FIXME
            }
        } else {
            $requestparams['paging'][$alias]['page'] = max((int) @$paginationModel[$alias]['page'], 1);
            $requestparams['paging'][$alias]['order'] = $paginationModel[$alias]['order']; //FIXME
        }

//            $requestparams['options'] = (array)Hash::get($paginationModel, "{$alias}.options" ); //FIXME


        $requestparams['paging'][$alias]['limit'] = 20;
        $requestparams['paging'][$alias]['count'] = count($bannetteService);

        $reminder = $requestparams['paging'][$alias]['count'] % $requestparams['paging'][$alias]['limit'];

        $requestparams['paging'][$alias]['pageCount'] = ceil($requestparams['paging'][$alias]['count'] / $requestparams['paging'][$alias]['limit']);
        $requestparams['paging'][$alias]['prevPage'] = ( $requestparams['paging'][$alias]['page'] > 1 );
        $requestparams['paging'][$alias]['nextPage'] = ( $requestparams['paging'][$alias]['page'] < $requestparams['paging'][$alias]['pageCount'] );

        if ($reminder == 0) {
            $requestparams['paging'][$alias]['current'] = $requestparams['paging'][$alias]['limit'];
        } else {
            if ($requestparams['paging'][$alias]['page'] == $requestparams['paging'][$alias]['pageCount']) {
                $requestparams['paging'][$alias]['current'] = $reminder;
            } else {
                $requestparams['paging'][$alias]['current'] = $requestparams['paging'][$alias]['limit'];
            }
        }

        $this->request->params = $requestparams;

        $offset = ( ( $requestparams['paging'][$alias]['page'] <= 1 ) ? 0 : ( ( $requestparams['paging'][$alias]['page'] - 1 ) * $requestparams['paging'][$alias]['limit'] ) );
        $bannetteService = array_slice($bannetteService, $offset, $requestparams['paging'][$alias]['limit']);


        $this->Session->delete('Auth.User.Courrier.mesServices');
        $this->Session->write('Auth.User.Courrier.mesServices', $fluxCtrl);
        $this->_initBancontenuService = true;


        $this->Session->write('paginationModel', $this->request->params['paging']);
// debug($bannetteService[0]);
        return $bannetteService;
    }

    /**
     * Initialisation de l'environnement "Superadmin"
     *
     * @access private
     * @return void
     */
    private function _initSuperadminEnv() {
        $this->Session->write('Auth.User.Env.actual.group_id', SUPERADMIN_GID);
        $this->Session->write('Auth.User.Env.actual.name', 'superadmin');
    }

    /**
     * Initialisation de l'environnement "Admin"
     *
     * @access private
     * @return void
     */
    private function _initAdminEnv() {
        $this->Session->write('Auth.User.Env.actual.group_id', ADMIN_GID);
        $this->Session->write('Auth.User.Env.actual.name', 'admin');
        $conn = $this->Session->read('Auth.User.connName');
        $this->loadModel('Collectivite');
        $this->set('collectivite', $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $conn))));
        $this->loadModel('Service');
        $this->set('nbServices', $this->Service->find('count'));
        $this->loadModel('User');
        $this->set('nbUsers', $this->User->find('count', array('conditions' => array('User.active' => true))));
        $this->loadModel('Group');
        $this->set('nbProfils', $this->Group->find('count'), array('conditions' => array('Group.active' => true)));
        $this->loadModel('Cakeflow.Circuit');
        $this->set('nbCircuits', $this->Circuit->find('count', array('conditions' => array('Circuit.actif' => 1))));
        $this->loadModel('Type');
        $this->set('nbTypes', $this->Type->find('count'));
        $this->set('nbSoustypes', $this->Type->Soustype->find('count'));
        $this->loadModel('Metadonnee');
        $this->set('nbMeta', $this->Metadonnee->find('count'));
        $this->loadModel('Dossier');
        $this->set('nbDossiers', $this->Dossier->find('count'));
        $this->set('nbAffaires', $this->Dossier->Affaire->find('count'));
    }

    /**
     * Initialisation de l'environnement "Initialisateur"
     *
     * @access private
     * @return void
     */
    private function _initInitEnv() {
        $this->Session->write('Auth.User.Env.actual.group_id', INIT_GID);
        $this->Session->write('Auth.User.Env.actual.name', 'init');
        $initDesktops = $this->getSessionInitDesktops();
        $initServices = $this->getSessionInitServices();

        return array(
            'desktop_id' => array_keys($initDesktops),
            'bannettes' => array('docs', 'refus', 'reponses', 'copy')
        );
    }

    /**
     * Initialisation de l'environnement "Valideur"
     *
     * @access private
     * @return void
     */
    private function _initValEnv() {
        $this->Session->write('Auth.User.Env.actual.group_id', VAL_GID);
        $this->Session->write('Auth.User.Env.actual.name', 'val');
        $valDesktops = $this->getSessionValDesktops();

        return array(
            'desktop_id' => array_keys($valDesktops),
            'bannettes' => array('flux', 'copy')
        );
    }

    /**
     * Initialisation de l'environnement "Valideur Editeur"
     *
     * @access private
     * @return void
     */
    private function _initValeditEnv() {
        $this->Session->write('Auth.User.Env.actual.group_id', VALEDIT_GID);
        $this->Session->write('Auth.User.Env.actual.name', 'valedit');
        $valeditDesktops = $this->getSessionValeditDesktops();

        return array(
            'desktop_id' => array_keys($valeditDesktops),
            'bannettes' => array('flux', 'copy')
        );
    }

    /**
     * Initialisation de l'environnement "Documentaliste" (Archiveur)
     *
     * @access private
     * @return void
     */
    private function _initArchEnv() {
        $this->Session->write('Auth.User.Env.actual.group_id', ARCH_GID);
        $this->Session->write('Auth.User.Env.actual.name', 'arch');
        $archDesktops = $this->getSessionArchDesktops();

        return array(
            'desktop_id' => array_keys($archDesktops),
            'bannettes' => array('flux', 'copy')
        );
    }

    /**
     * Initialisation de l'environnement "Aiguilleur" (Dispatcher)
     *
     * @access private
     * @return void
     */
    private function _initDispEnv() {
        $this->Session->write('Auth.User.Env.actual.group_id', DISP_GID);
        $this->Session->write('Auth.User.Env.actual.name', 'disp');
        $dispDesktops = $this->getSessionDispDesktops();

        ///
//        $initDesktops = $this->getSessionInitDesktops();
//        $initServices = $this->getSessionInitServices();
//        $this->set('addservice', $initServices);
//        $this->set('add', $initDesktops);
        ///
        return array(
            'desktop_id' => array_keys($dispDesktops),
            'bannettes' => array('aiguillage', 'copy')
        );
    }

    /**
     * Initialisation des environnements défini dans l'application (profils personnalisés)
     *
     * @access private
     * @return void
     */
    private function _initAppdefinedEnv() {
        $this->Session->write('Auth.User.Env.actual.group_id', 0);
        $this->Session->write('Auth.User.Env.actual.name', 'appdefined');
        $appDefinedDesktops = $this->getSessionAppDefinedDesktops();
        $initServices = $this->getSessionInitServices();

        if ($this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Courriers/add')) {
            $this->_add = $this->getSessionInitServices();
        }

        $listeBannettes = array('flux', 'docs', 'copy');
        return array(
            'desktop_id' => array_keys($appDefinedDesktops),
            'bannettes' => $listeBannettes
        );
    }

    /**
     * Accès à la page d'environnement
     *
     * @logical-group Environnement
     * @user-profile Superadmin
     * @user-profile Admin
     * @user-profile User
     *
     * @access public
     * @param boolean $fromEdit détermine si l'on vient d'enregistrer des informations
     * @param string $env détermine quel environnement sera actif (superadminenv, adminenv, userenv)
     * @return void
     */
    public function index($fromEdit = null, $env = null) {
        if ($fromEdit == 1) {
            $this->Session->setFlash('Les informations ont bien été enregistrées', 'growl');
        }
        if (empty($env)) {
            $env = $this->Session->read('Auth.User.Env.main');
        }
        if ($env == 'superadmin') {
            $this->setAction('superadminenv');
        } else if ($env == 'admin') {
            $this->setAction('adminenv');
        } else if ($env == 'disp') {
            $this->setAction('dispenv');
        } else {
            $this->setAction('userenv');
        }
    }

    /**
     * Initialisation de l'environnement superadmin (affichage graphique)
     *
     * @logical-group Environnement
     * @user-profile Superadmin
     *
     * @access public
     * @return void
     */
    public function superadminenv() {
        $this->set('ariane', array(__d('menu', 'Environnement') . ' ' . __d('menu', 'Env.Superadministrateur', true)));
        $this->_initSuperadminEnv();
    }

    /**
     * Initialisation de l'environnement administrateur (affichage graphique)
     *
     * @logical-group Environnement
     * @user-profile Superadmin
     *
     * @access public
     * @return void
     */
    public function adminenv() {
        $this->set('ariane', array(__d('menu', 'Environnement') . ' ' . __d('menu', 'Env.Administrateur', true)));
        $this->_initAdminEnv();
    }

    /**
     * Récupération des notifications
     *
     * @logical-group Environnement
     * @logical-group Notifications
     * @user-profile Admin
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function getNotifications() {
        $settings = array(
            'read' => false,
            'date_start' => null,
            'date_end' => null,
            'limit' => 10,
            'order' => 'desc'
        );
        $settings = array_merge($settings, $this->request->params['named']);

        $this->loadModel('Notification');
        $querydata = array(
            'fields' => array(
                'Notification.id',
                'Notification.name',
                'Notification.description',
                'Typenotif.name',
                'Courrier.id',
                'Courrier.name',
                'Notifieddesktop.desktop_id',
                'Desktop.name'
            ),
            'contain' => false,
            'joins' => array(
                $this->Notification->join($this->Notification->Typenotif->alias),
                $this->Notification->join($this->Notification->Desktop->alias),
                $this->Notification->join($this->Notification->Notifieddesktop->alias),
                $this->Notification->Notifieddesktop->join($this->Notification->Notifieddesktop->Courrier->alias)
            ),
            'conditions' => array(
                'Notifieddesktop.read' => $settings['read'],
                'Notifieddesktop.desktop_id' => $this->SessionTools->getDesktopList()
            ),
            'limit' => $settings['limit'],
            'order' => 'Notification.created ' . $settings['order']
        );
        if (!empty($settings['start']) && !empty($settings['end']) && $settings['start'] < $settings['end']) {
            $querydata['conditions'] = array_merge($querydata['conditions'], array(array('Notification.created >' => $settings['start']), array('Notification.created <' => $settings['end'])));
        }
        $notifications = $this->Notification->find('all', $querydata);
        $this->set('notifications', $notifications);
    }

    /**
     * Marquer une notification comme lue
     *
     * @logical-group Environnement
     * @logical-group Notifications
     * @user-profile Admin
     * @user-profile User
     *
     * @access public
     * @param integer $notificationId identifiant de la notification
     * @param integer $desktopId identifiant du rôle
     * @param integer $courrierId identifiant du flux
     * @return void
     */
    public function setNotificationRead($notificationId, $desktopId, $courrierId) {
        $this->Jsonmsg->init();
        $this->loadModel('Notifieddesktop');
        $qd = array(
            'conditions' => array(
                'Notifieddesktop.desktop_id' => $desktopId,
                'Notifieddesktop.notification_id' => $notificationId,
                'Notifieddesktop.courrier_id' => $courrierId
            ),
            'recursive' => -1
        );
        $item = $this->Notifieddesktop->find('first', $qd);
        $item['Notifieddesktop']['read'] = true;
        $this->Notifieddesktop->create($item);
        if ($this->Notifieddesktop->save()) {
            $this->Jsonmsg->valid();
        } else {
            $this->Jsonmsg->error();
        }
        $this->Jsonmsg->send();
    }

    /**
     * Marquer toutes les notifications comme lues
     *
     * @logical-group Environnement
     * @logical-group Notifications
     * @user-profile Admin
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function setAllNotificationRead() {
        $verif = array();
        $this->Jsonmsg->init();

        $desktops = $this->SessionTools->getDesktopList();
        foreach ($desktops as $desktop) {
            $verif[] = $this->Notification->setAllReadByDesktop($desktop);
        }

        if (!in_array(false, $verif, true)) {
            $this->Jsonmsg->valid();
        } else {
            $this->Jsonmsg->error();
        }
        $this->Jsonmsg->Send();
    }

    /**
     * Initialisation de l'environnement utilisateur (affichage graphique)
     *
     * @logical-group Environnement
     * @user-profile User
     *
     * @access public
     * @param integer $tab onglet à recharger après modification
     * @return void
     */
    public function userenv($tab = null) {
        $this->set('ariane', array(__d('menu', 'Environnement') . ' ' . __d('menu', 'Env.Utilisateur', true)));
        $envs = array_unique(array_merge(array($this->Session->read('Auth.User.Env.main')), array_unique($this->Session->read('Auth.User.Env.secondary'))));

        $bannettesParams = array();
        foreach ($envs as $env) {
            if ($env != 'admin' && $env != 'disp') {
                if ($env == 'init') {
                    $bannettesParams[] = $this->_initInitEnv();
                } else if ($env == 'val') {
                    $bannettesParams[] = $this->_initValEnv();
                } else if ($env == 'valedit') {
                    $bannettesParams[] = $this->_initValeditEnv();
                } else if ($env == 'arch') {
                    $bannettesParams[] = $this->_initArchEnv();
                } else if ($env == 'appdefined') {
                    $bannettesParams[] = $this->_initAppdefinedEnv();
                }
            }
        }

        $this->_bannettes = $this->_getAllBancontenus($bannettesParams);

        if (!$this->_initBancontenuService && $this->userAclCheck(array('model' => 'User', 'foreign_key' => $this->Session->read('Auth.User.id')), 'controllers/Environnement/getBancontenusServices')) {
            $this->_bannettes['services'] = $this->_getBancontenusServices();
        }


        /////////////////////////////////////////////////////////////
        $this->loadModel('Desktop');
        $this->Desktop->setDataSource(Configure::read('conn'));
        $this->set('userId', $this->Session->read('Auth.User.id'));

        $inits = $this->Desktop->getInitsByService();
        $disps = $this->Desktop->getDispsByService();

        $desktops = $this->SessionTools->getDesktopList();
        $otherDisps = array();
        foreach ($disps as $srv => $disp) {
            foreach ($disp as $key => $val) {
                if (!in_array($key, $desktops)) {
                    $otherDisps[$srv] = array($key => $val);
                }
            }
        }
        $services = array_merge_recursive($inits, $otherDisps);
        ksort($services);
        $this->set('services', $services);

        /////////////////////////////////////////////////////////////
        $this->set('bannettes', $this->_bannettes);
//        $this->set('add', $this->_add);
//        $this->set('addservice', $this->_addService);
        $this->set('tab', $tab);
    }

    /**
     * Initialisation de l'environnement aiguilleur (affichage graphique)
     *
     * @logical-group Environnement
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function dispenv() {
        $this->set('ariane', array(
            __d('menu', 'Environnement') . ' ' . __d('menu', 'Env.Aiguilleur', true)
            ));

        $bannettesParams = array($this->_initDispEnv());
        $this->_bannettes = $this->_getAllBancontenus($bannettesParams);

        $this->loadModel('Desktop');
        $this->Desktop->setDataSource(Configure::read('conn'));
        $this->set('userId', $this->Session->read('Auth.User.id'));

        $inits = $this->Desktop->getInitsByService();
        $disps = $this->Desktop->getDispsByService();

        $desktops = $this->SessionTools->getDesktopList();
        $otherDisps = array();
        foreach ($disps as $srv => $disp) {
            foreach ($disp as $key => $val) {
                if (!in_array($key, $desktops)) {
                    $otherDisps[$srv] = array($key => $val);
                }
            }
        }
        $services = array_merge_recursive($inits, $otherDisps);
        ksort($services);
        $this->set('services', $services);
        $this->set('bannettes', $this->_bannettes);
    }

    /**
     * Récupération de l'historique des flux d'un utilisateur
     *
     * @logical-group Environnement
     * @user-profile User
     *
     * @access public
     * @return void
     */
    public function historique() {
        $this->set('ariane', array(
            '<a href="/environnement/userenv/0">'.__d('menu', 'Environnement').'</a>', 
            __d('menu', 'Historique', true)
        ));
        $this->loadModel('Bancontenu');

        $desktops = $this->getSessionAllDesktops();
        $historique = array();

        $paginationModel = $this->Session->read('paginationModel');
        $page = Hash::get($this->request->params, 'named.page');
        $namedModel = Hash::get($this->request->params, 'named.model');

        foreach ($desktops as $desktopId => $desktopName) {
            if ($this->Session->read('Auth.User.Desktop.id') == $desktopId) {
                $groupId = $this->Session->read('Auth.User.Desktop.Group.id');
            } else {
                $secDktp = ($this->Session->read('Auth.User.SecondaryDesktops'));
                for ($i = 0; $i < count($secDktp); $i++) {
                    if ($secDktp[$i]['id'] == $desktopId) {
                        $groupId = $secDktp[$i]['Group']['id'];
                    }
                }
            }

            $params = array('class' => 'Desktop', 'alias' => Inflector::camelize("bannette_" . Inflector::camelize(Inflector::slug($desktopName))));
            $alias = $params['alias'];
            $this->{$alias} = ClassRegistry::init($params);

            if ($groupId != ADMIN_GID) {
                $historique[$desktopName] = array();
                $querydata = array(
                    'fields' => array_merge(
                            $this->{$alias}->Bancontenu->fields(), $this->{$alias}->fields(), $this->{$alias}->Bancontenu->Courrier->fields(), $this->{$alias}->Bancontenu->Desktop->fields(), $this->{$alias}->Bancontenu->Courrier->Contactinfo->fields(), $this->{$alias}->Bancontenu->Courrier->Soustype->fields(), $this->{$alias}->Bancontenu->Courrier->Soustype->Type->fields(), $this->{$alias}->Bancontenu->Courrier->Traitement->Circuit->fields(), $this->_virtualFieldsCourrier()
                    ),
                    'contain' => false,
                    'joins' => array(
                        $this->{$alias}->join($this->{$alias}->Bancontenu->alias),
                        $this->{$alias}->Bancontenu->join($this->{$alias}->Bancontenu->Desktop->alias),
                        $this->{$alias}->Bancontenu->join($this->{$alias}->Bancontenu->Courrier->alias),
                        $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Contactinfo->alias),
                        $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Soustype->alias, array('type' => 'LEFT OUTER')),
                        $this->{$alias}->Bancontenu->Courrier->Soustype->join($this->{$alias}->Bancontenu->Courrier->Soustype->Type->alias, array('type' => 'LEFT OUTER')),
                        $this->{$alias}->Bancontenu->Courrier->join($this->{$alias}->Bancontenu->Courrier->Traitement->alias, array('type' => 'LEFT OUTER')),
                        $this->{$alias}->Bancontenu->Courrier->Traitement->join($this->{$alias}->Bancontenu->Courrier->Traitement->Circuit->alias, array('type' => 'LEFT OUTER'))
                    ),
                    'conditions' => array(
                        'Desktop.id' => $desktopId,
//                        'Bancontenu.etat <>' => 1
                    ),
                    'order' => array(
                        'Desktop.name' => 'DESC',
                        'Courrier.modified' => 'DESC'
                    )
                );


                // Gestion de la pagination pour ne passer de page en page que sur
                // le modèle qui nous intéresse
                if (isset($paginationModel[$alias])) {
                    $params = $this->request->params;
                    if ($namedModel == $alias) {
                        $querydata['page'] = $params['named']['page'] = $page;
                    } else {
                        $querydata['limit'] = $paginationModel[$alias]['limit'];
                        $querydata['order'] = $paginationModel[$alias]['order'];
                        $querydata['page'] = max((int) @$paginationModel[$alias]['page'], 1);
                        $querydata['offset'] = ( ( (int) $paginationModel[$alias]['page'] ) <= 1 ? 0 : ( $paginationModel[$alias]['limit'] * ( $paginationModel[$alias]['page'] - 1 ) ) );
                    }

                    $this->request->params = $params;
                }

                $paginate = $this->paginate;
                $paginate[$alias] = $querydata;
                $this->paginate = $paginate;

//debug($paginate);
                $this->request->params['named']['page'] = @$querydata['page'];
                $historique[$desktopName] = $this->paginate($this->{$alias});
                if (!empty($page)) {
                    $this->request->params['named']['page'] = $page;
                }

                $parent = array();
                for ($i = 0; $i < count($historique[$desktopName]); $i++) {

                    if (!empty($historique[$desktopName][$i]['Courrier']['parent_id'])) {
                        $qdParent = $querydata;
                        $qdParent['conditions'] = array('Courrier.id' => $historique[$desktopName][$i]['Courrier']['parent_id']);
                        $parent = $this->_dataParent($historique[$desktopName][$i]['Courrier']['parent_id']);
                    }

                    $historique[$desktopName][$i] = $this->_processFlux($historique[$desktopName][$i], false, false, $parent); //le parametre read est a true pour les flux dans l historique
                }
            }
        }

        $this->Session->write('paginationModel', $this->request->params['paging']);
        $this->set('historique', $historique);
    }

}

?>