<?php

/**
 * Profils
 *
 * Groups controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class GroupsController extends AppController {

    /**
     * Controller name
     *
     * @var array
     * @access public
     */
    public $name = 'Groups';

    /**
     * Controller helpers
     *
     * @var array
     * @access public
     */
    public $helpers = array('DataAcl.DataAcl');

    /**
     * Controller components
     *
     * @access public
     * @var array
     */
    public $components = array('Rights', 'Ajaxformvalid');

    /**
     * Controller index method
     *
     * @return void
     */
    //TODO: supprimer les fonctions commentées
//	public function index() {
//		$this->set('ariane', array(__d('menu', 'Administration', true), __d('menu', 'gestionGroups', true)));
//	}

    /**
     * Récupération de la liste des profils
     *
     * @logical-group Profils
     * @user-profile Admin
     *
     * @access public
     * @return void
     */
    public function getGroups() {
        $groups_tmp = $this->Group->find("all", array('conditions' => array('Group.readonly' => false, 'Group.name <>' => 'Superadmin'), 'order' => 'Group.name'));
        $groups = array();
        foreach ($groups_tmp as $item) {
            $item['right_edit'] = true;
            $item['right_delete'] = $this->Group->isDeletable($item['Group']['id']);
            $groups[] = $item;
        }

//         $this->ldebug($groups);

        $groups_base_tmp = $this->Group->find("all", array('conditions' => array('Group.readonly' => true, 'Group.name <>' => 'Superadmin', 'Group.active' => true), 'order' => 'Group.name'));
        $groups_base = array();
        foreach ($groups_base_tmp as $item) {
            $item['right_view'] = true;
            $item['right_edit'] = false;
            $item['right_delete'] = false;
            $groups_base[] = $item;
        }

        $this->set('groups', $groups);
        $this->set('groups_base', $groups_base);
        $this->set('nbItem', count($groups));
    }

    /**
     * Visualisation d'un profil de base
     *
     * @logical-group Profils
     * @user-profile Admin
     *
     * @access public
     * @param integer $id identifiant du profil
     * @return void
     */
    public function view($id) {
        if ($id != null) {
            $this->set('id', $id);
            $group = $this->Group->find(
                    'first', array(
                'fields' => array(
                    'Group.id',
                    'Group.name'
                ),
                'conditions' => array(
                    'Group.id' => $id
                ),
                'contain' => false
                    )
            );
            $this->request->data = $group;
        }
    }

    /**
     * Ajouter un profil personnalisé
     *
     * @logical-group Profils
     * @user-profile Admin
     *
     * @access public
     * @return void
     */
    public function add() {
        if (!empty($this->request->data)) {
            $this->Jsonmsg->init();
            //creation du groupe
            $this->Group->create();
            if ($this->Group->save($this->request->data)) {
                //mise à jour de l'alias de l'aro
                //recuperation de l'aro correspondant au groupe nouvellement créé
                $aro = $this->Acl->Aro->find('first', array('conditions' => array('Aro.model' => 'Group', 'Aro.foreign_key' => $this->Group->id)));
                $aro['Aro']['alias'] = $this->Group->field('name');
                $this->Acl->Aro->create($aro);
                $this->Acl->Aro->save();
                //duplication des droits du parent vers le groupe nouvellement créé
                $parent_group_id = $this->request->data['Group']['group_id'];
                //recuperation de l'aro correspondant au groupe nouvellement créé
                $aroParent = $this->Acl->Aro->find('first', array('conditions' => array('Aro.model' => 'Group', 'Aro.foreign_key' => $parent_group_id)));
                //recuperation des droits du parent
                $rights = $this->Acl->Aro->Permission->find('all', array('conditions' => array('Permission.aro_id' => $aroParent['Aro']['id'])));
                //copie des droits
                $newRights = array();
                foreach ($rights as $kright => $right) {
                    $permission = $right['Permission'];
                    unset($permission['id']);
                    $permission['aro_id'] = $aro['Aro']['id'];
                    $newRights[$kright] = array('Permission' => $permission, 'Aro' => $aro['Aro'], 'Aco' => $right['Aco']);
                }

                $validCopy = $this->Acl->Aro->Permission->saveAll($newRights);
                if (!empty($validCopy)) {
                    $this->loadModel('Right');
                    $rights = $this->Right->find('first', array('conditions' => array('aro_id' => $this->request->data['Group']['group_id'])));
                    $rights['Right']['id'] = '';
                    $rights['Right']['aro_id'] = $aro['Aro']['id'];
                    $rights['Right']['model'] = $aro['Aro']['model'];
                    $rights['Right']['foreign_key'] = $aro['Aro']['foreign_key'];
                    $this->Right->create($rights);
                    $validRight = $this->Right->save();
                    if (!empty($validRight)) {
                        $this->Jsonmsg->valid();
                    }
                }
            }
            $this->Jsonmsg->send();
        }
        $qd = array(
            'conditions' => array(
                'NOT' => array(
                    'Group.id' => array(SUPERADMIN_GID, ADMIN_GID, DISP_GID)
                ),
                'Group.active' => true
            )
        );
        $this->set('groups', $this->Group->find('list', $qd));
    }

    /**
     * Edition d'un profil personnalisé
     *
     * @logical-group Profils
     * @user-profile Admin
     *
     * @access public
     * @param integer $id identifiant du profil
     * @return void
     */
    public function edit($id) {
        if ($id != null) {
            $this->set('id', $id);
//			$this->Group->id = $id;
//			$this->request->data = $this->Group->read();
            $group = $this->Group->find(
                    'first', array(
                'fields' => array(
                    'Group.id',
                    'Group.name',
                    'Group.active'
                ),
                'conditions' => array(
                    'Group.id' => $id
                ),
                'contain' => false
                    )
            );
            $this->request->data = $group;
        }
    }

    /**
     * Edition d'un profil personnalisé
     *
     * @logical-group Profils
     * @user-profile Admin
     *
     * @access public
     * @param integer $id identifiant du profil
     * @return void
     */
    public function setInfos($id = null) {
        if (!empty($this->request->data['Group'])) {
            $this->Jsonmsg->init();
            $canSave = true;
            $this->Group->create($this->request->data);
            //verification de l'autorisation de modification du groupe
            if (isset($this->request->data['Group']['id']) && $this->request->data['Group']['id'] != '') {
                $this->Group->id = $this->request->data['Group']['id'];
                $canSave = !$this->Group->field('readonly');
            }
            if ($canSave) {
                $saved = $this->Group->save();
                if (!empty($saved)) {
                    $this->Jsonmsg->valid(__d('group', 'group.save.infos') . __d('default', 'save.ok'));
                }
            } else {
                $this->Jsonmsg->init(__d('group', 'group.save.notallowed'));
            }
            $this->Jsonmsg->send();
        }
        $this->Group->id = $id;
        $this->request->data = $this->Group->read();
    }

    /**
     *
     * @param integer $id
     */
    //TODO: supprimer les fonctions commentées
//	public function getInfos($id = null) {
//		$this->Group->id = $id;
//		$this->request->data = $this->Group->read();
//	}

    /**
     * Suppression des droits des AROs dépendants
     *
     * @access private
     * @param integer $aroId indetifiant de l'ARO (ACL)
     * @return boolean
     */
    private function _flushDependantRights($aroId) {
        $this->loadModel('Right');
        $qdAros = array(
            'recursive' => -1,
            'fields' => array(
                'Aro.id'
            ),
            'conditions' => array(
                'Aro.parent_id' => $aroId
            )
        );
        $aros = Hash::flatten($this->Acl->Aro->find('all', $qdAros));
        return $this->Acl->Aro->Permission->deleteAll(array('aro_id' => $aros)) && $this->Right->deleteAll(array('aro_id' => $aros));
    }

    /**
     * Edition de la liste des habilitations de fonctionnalités d'un profil personnalisé
     *
     * @logical-group Profils
     * * @logical-group Gestion des habilitations
     *
     * @access public
     * @param integer $id identifiant du profil
     * @param string $mode détermine quel formulaire afficher (func, func_classique)
     * @return void
     */
    public function setDroitsFunc($id = null, $mode = 'func') {
        //mise a jour des droits sur les meta donnees
        if (!empty($this->request->data)) {
            $this->loadModel('Right');
            $this->Jsonmsg->init();
            $this->Acl->Aro->begin();
            $validGlobal = false;

            //Enregistrement en mode classique (a supprimer dans la prochaine version)
            if (!empty($this->request->data['Acl']['Type'])) {
                if ($this->request->data['Acl']['Type'] == 'Acl') {
                    if ($this->Acl->setRights($this->request->data['Requester'], $this->request->data['Rights'])) {
                        $qdAro = array(
                            'conditions' => array(
                                'model' => key($this->request->data['Requester']),
                                'foreign_key' => current($this->request->data['Requester'])
                            )
                        );
                        $aro = $this->Acl->Aro->find('first', $qdAro);
                        $rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $aro['Aro']['id'])));
                        $rights['Right']['mode_classique'] = true;
                        $rights['Right']['model'] = 'Group';
                        $rights['Right']['foreign_key'] = current($this->request->data['Requester']);
                        $this->Right->create($rights);
                        $saved = $this->Right->save();
                        if (!empty($saved) && $this->_flushDependantRights($aro['Aro']['id'])) {
                            $validGlobal = true;
                        }
                    }
                }
                //Enregistrement en mode simplifié
            } else if (!empty($this->request->data['Right'])) {
                $aro_id = $this->request->data['Right']['aro_id'];
                $aro = $this->Acl->Aro->find('first', array('recursive' => -1, 'conditions' => array('id' => $aro_id)));
                $computedRights = $this->Right->saveRightSets($aro_id, $this->Rights->parseRightsData($this->request->data['Right']), true);
                if (!empty($computedRights)) {
                    $valid = array();
                    $this->Acl->Aro->Permission->deleteAll(array('aro_id' => $aro_id));
                    foreach ($computedRights as $key => $val) {
                        if ($val) {
                            $valid[] = $this->Acl->allow($aro['Aro'], $key);
                        } else {
                            $valid[] = $this->Acl->deny($aro['Aro'], $key);
                        }
                    }
                    if (!in_array(false, $valid, true) && $this->_flushDependantRights($aro_id)) {
                        $validGlobal = true;
                    }
                }
            }

            if ($validGlobal) {
                $this->Jsonmsg->valid(__d('group', 'group.save.rights') . __d('default', 'save.ok'));
                $this->Acl->Aro->commit();
                $this->Acl->flushCache();
            } else {
                $this->Acl->Aro->rollback();
            }

            $this->Jsonmsg->send();
        } else {
            if ($mode == 'func') {
                $this->loadModel('Right');
                $aro = array('model' => 'Group', 'foreign_key' => $id);
                $aro_id = $this->Acl->Aro->field('id', $aro);
                $rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $aro_id)));
                $this->set('rights', $rights);
                $this->set('tabsForSets', json_encode($this->Right->getTabsForSets()));
            } else if ($mode == 'func_classique') {
                $this->Group->id = $id;
                $daco = $this->Group->node();
                $rights = $this->Acl->getRightsGrid($daco[0], array(), array('actions' => array('read'), 'models' => array('Group')));
                $this->set('rights', $rights);
                $this->set('id', $id);
            }
            $this->set('mode', $mode);
        }
    }

    /**
     * Vérificatoin de l'autorisation d'accès d'un ARO (profil, rôle) à un ACO (action d'un controlleur)
     *
     * @access private
     * @param array $aros tableau représentant un ARO (ACL)
     * @param string $aco alias d'un ACO (ACL) exemple:  "/controllers/controller/action"
     * @param mixed $action action CRUD ou "*"
     * @return boolean si l'ARO (profil, rôle) et autorisé à accéder à l'ACO (action d'un controlleur) alors on retourne true sinon on retourne false
     */
    private function _funcCheck($aros, $aco, $action = "*") {
        $checks = array();
        foreach ($aros as $aro) {
            $checks[] = $this->Acl->check($aro, $aco, $action);
        }
        return in_array(true, $checks, true);
    }

    /**
     * Récupération de la grille complète des droits pour une hiérarchie d'AROs (profils, rôles)	 *
     *
     * @param array $aros tableau contenants des ARO (profils, rôles) exemple:  $aros = array(array('model' => 'Group', 'foreign_key' => 12), array('model' => 'Desktop', 'foreign_key' => 15))
     * @param array $acos tableau contenants des alias d'ACO (action d'un controlleur) exemple:  array('controllers/controller/action', 'controllers/controller/other_action')
     * @param array $options
     * @return array
     */
    private function _getFuncRightsGrid($aros, $acos = array(), $options = array('actions' => '*', 'alias' => '')) {
        if (empty($acos)) {
            $acos = $this->Acl->Aco->find('threaded');
        }
        $return = array();
        foreach ($acos as $aco) {
            $return[$aco['Aco']['id']] = $this->_getFuncRights($aros, $aco, $options);
        }
        return $return;
    }

    /**
     * Récupération des droits pour un hiérarchie d'AROs
     * cette méthode appelle récursivement _getFuncRightsGrid pour les enfants des AROs
     *
     * @param array $aros
     * @param string $aco
     * @param array $options
     * @return array
     */
    private function _getFuncRights($aros, $aco, $options = array('actions' => '*')) {
        $options['alias'] .= '/' . $aco['Aco']['alias'];
        $options['alias'] = ltrim($options['alias'], '/');

        $actions = $options['actions'];
        $checks = array();
        if ($actions == '*' || is_array($actions) && in_array('*', $actions)) {
            $chk = $this->_funcCheck($aros, $options['alias']);
            $checks['read'] = $chk;
        }
        if ($actions == 'create' || is_array($actions) && in_array('create', $actions)) {
            $checks['create'] = $this->_funcCheck($aros, $options['alias'], 'create');
        }
        if ($actions == 'read' || is_array($actions) && in_array('read', $actions)) {
            $checks['read'] = $this->_funcCheck($aros, $options['alias'], 'read');
        }
        if ($actions == 'update' || is_array($actions) && in_array('update', $actions)) {
            $checks['update'] = $this->_funcCheck($aros, $options['alias'], 'update');
        }
        if ($actions == 'delete' || is_array($actions) && in_array('delete', $actions)) {
            $checks['delete'] = $this->_funcCheck($aros, $options['alias'], 'delete');
        }

        $return = array(
            'checks' => $checks,
            'alias' => $options['alias']
        );
        if (!empty($aco['children'])) {
            $return['children'] = $this->_getFuncRightsGrid($aros, $aco['children'], $options);
            $this->_getFuncRightsGrid($aros, $aco['children']);
        }
        return $return;
    }

    /**
     * Récupération de la liste des habilitations de fonctionnalités (profils de base et personnalisés)
     *
     * @logical-group Profils
     * * @logical-group Gestion des habilitations
     *
     * @user-profile Admin
     *
     * @access public
     * @param integer $id identifiant du rôle
     * @param string $mode détermine quel formulaire afficher (func, func_classique)
     * @return void
     */
    public function getDroitsFunc($id = null, $mode = 'func') {
        if ($mode == 'func') {
            $this->loadModel('Right');
            $aro = array('model' => 'Group', 'foreign_key' => $id);
            $aro_id = $this->Acl->Aro->field('id', $aro);
            $rights = $this->Right->find('first', array('conditions' => array('Right.aro_id' => $aro_id)));
            $this->set('rights', $rights);
        } else if ($mode == 'func_classique') {
            $this->Group->id = $id;
            $this->request->data = $this->Group->read();
            $aros = array(array('model' => 'Group', 'foreign_key' => $id));
            set_time_limit(120);
            $this->set('requester', array('model' => 'Group', 'foreign_key' => $id));
            $this->set('rights', $this->_getFuncRightsGrid($aros));
        }
        $this->set('mode', $mode);
    }

    /**
     * Suppression d'un profil personnalisé
     *
     * @logical-user Profils
     * @user-profile Admin
     *
     * @access public
     * @param integer $id identifiant du rôle
     * @return void
     */
    public function delete($id = null) {
        $group = $this->Group->find('first', array('conditions' => array('Group.id' => $id)));

        if (empty($group)) {
            throw new NotFoundException();
        }

        $this->Jsonmsg->init(__d('default', 'delete.error'));
        $this->Group->begin();
        if ($this->Group->delete($id, true)) {
            $this->Group->commit();
            $this->Jsonmsg->valid(__d('default', 'delete.ok'));
        } else {
            $this->Group->rollback();
        }
        $this->Jsonmsg->send();
    }

    /**
     *
     * @param type $field
     * @param type $collectiviteId
     * @return type
     */
    public function ajaxformvalid($field = null) {
//         $this->ldebug($this->request->data);
        $this->Ajaxformvalid->valid(array('Group'), $this->request->data);
    }

}

?>
