<?php

/**
 * Méta-données
 *
 * Metadonnees controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class MetadonneesController extends AppController {

	/**
	 * Controller name
	 *
	 * @var string
	 * @access public
	 */
	public $name = 'Metadonnees';

	/**
	 * Gestion des méta-données (interface graphique)
	 *
	 * @logical-group Méta-données
	 * @user-profil Admin
	 *
	 * @access public
	 * @return void
	 */
	public function index() {
		$this->set('ariane', array(
                    '<a href="environnement/index/0/admin">'.__d('menu', 'Administration', true).'</a>',
                    __d('menu', 'gestionMeta', true)));
	}

	/**
	 * Récupération de la liste des méta-données (ajax)
	 *
	 * @logical-group Méta-données
	 * @user-profil Admin
	 *
	 * @access public
	 * @return void
	 */
	public function getMetadonnees() {
		$metadonnees_tmp = $this->Metadonnee->find("all", array('order' => 'Metadonnee.name'));
		$metadonnees = array();
		foreach ($metadonnees_tmp as $item) {
			$item['right_edit'] = true;
			$item['right_delete'] = true;
			$metadonnees[] = $item;
		}
		$this->set('metadonnees', $metadonnees);
	}

	/**
	 * Ajout d'une méta-donnée
	 *
	 * @logical-group Méta-données
	 * @user-profil Admin
	 *
	 * @access public
	 * @return void
	 */
	public function add() {
        $listeSoustypes = $this->Metadonnee->Soustype->find(
            'list',
            array(
                'order' => array(
                    'Soustype.name DESC'
                )
            )
        );
        $this->set('listeSoustypes', $listeSoustypes);
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
			$this->Metadonnee->create($this->request->data);
			if ($this->Metadonnee->save()) {
				$this->Jsonmsg->valid();
			}
			$this->Jsonmsg->send();
		} else {
			$this->set('typemetadonnees', $this->Metadonnee->Typemetadonnee->find('list'));
		}
	}

	/**
	 * Edition d'un méta-donnée
	 *
	 * @logical-group Méta-données
	 * @user-profil Admin
	 *
	 * @access public
	 * @param integer $id identifiant de la méta-donnée
	 * @return void
	 */
	public function edit($id = null) {
        $listeSoustypes = $this->Metadonnee->Soustype->find(
            'list',
            array(
                'order' => array(
                    'Soustype.name DESC'
                )
            )
        );
        $this->set('listeSoustypes', $listeSoustypes);


		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
			$this->Metadonnee->create($this->request->data);
// debug($this->request->data);
// die();
			if ($this->Metadonnee->save()) {
				$this->Jsonmsg->valid();
			}
			$this->Jsonmsg->send();
		} else {
			$querydata = array(
				'contain' => array(
					$this->Metadonnee->Typemetadonnee->alias,
					$this->Metadonnee->Courrier->alias,
                    $this->Metadonnee->Soustype->alias,
				),
				'conditions' => array(
					'Metadonnee.id' => $id
				)
			);
			$this->request->data = $this->Metadonnee->find('first', $querydata);
		}
		$this->set('typemetadonnees', $this->Metadonnee->Typemetadonnee->find('list'));
        
       
		
        $selectvalues_tmp = $this->Metadonnee->Selectvaluemetadonnee->find('all', array('conditions' => array('Selectvaluemetadonnee.metadonnee_id' => $id), 'recursive' => -1));
        $selectvalues = array();
		foreach ($selectvalues_tmp as $item) {
			$item['right_edit'] = true;
			$item['right_delete'] = true;
			$selectvalues[] = $item;
		}
        $this->set('selectvalues', $selectvalues);
        $this->set( 'metadonneeId', $id );
	}

	/**
	 * Suppression d'une méta-donnée (delete)
	 *
	 * @logical-group Méta-données
	 * @user-profil Admin
	 *
	 * @access public
	 * @param integer $id identifiant de la méta-donnée
	 * @return void
	 */
	public function delete($id = null) {
		$this->Jsonmsg->init(__d('default', 'delete.error'));
		$this->Metadonnee->begin();
		if ($this->Metadonnee->delete($id)) {
			$this->Metadonnee->commit();
			$this->Jsonmsg->valid(__d('default', 'delete.ok'));
		} else {
			$this->Metadonnee->rollback();
		}
		$this->Jsonmsg->send();
	}

}

?>