<?php

/**
 * Recherches
 *
 * Recherches controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class RecherchesController extends AppController {

	/**
	 * Controller name
	 *
	 * @var string
	 * @access public
	 */
	public $name = 'Recherches';

	/**
	 * Controller uses
	 *
	 * @var array
	 * @access public
	 */
	public $uses = array('Recherche', 'Courrier');

	/**
	 * Controller components
	 *
	 * @var array
	 * @access public
	 */
	public $components = array('DataAuthorized');

	/**
	 * Controller helpers
	 *
	 * @var array
	 * @access public
	 */
	public $helpers = array( 'Csv','Paginator');


	//TODO: enlever le beforeFilter et régénérer les droits
	public function beforeFilter() {
		$this->Auth->allow( array( 'index', 'exportcsv' )); //FIXME
		parent::beforeFilter();

	}


	/**
	 * Index method
	 *
	 * @access public
	 * @return void
	 */
	public function index() {
		$this->set('ariane', array(__d('menu', 'Recherche', true)));
	}

	/**
	 * Formulaire de recherche (enregistrement de recherche)
	 *
	 * @logical-group Recherches
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $id
	 * @param string $forcemode
	 * @return void
	 */
	public function formulaire($id = null, $forcemode = null) {
		//enregistrement des critères de la recherche
		if (!empty($this->request->data)) {
			$this->Jsonmsg->init();
			$data = array();
			$data['Recherche'] = $this->request->data['Recherche'];
			if (!empty($this->request->data['Contact']['Contact'])) {
				$data['Contact'] = array();
				foreach ($this->request->data['Contact']['Contact'] as $kcontact => $contact) {
					$data['Contact']['Contact'][$kcontact] = $contact;
				}
			}
			if (!empty($this->request->data['Type']['Type'])) {
				foreach ($this->request->data['Type']['Type'] as $ktype => $type) {
					$data['Type']['Type'][$ktype] = $type;
				}
			}
			if (!empty($this->request->data['Soustype']['Soustype'])) {
				foreach ($this->request->data['Soustype']['Soustype'] as $ksoustype => $soustype) {
					$data['Soustype']['Soustype'][$ksoustype] = $soustype;
				}
			}
			if (!empty($this->request->data['Circuit']['Circuit'])) {
				$data['Circuit'] = array();
				foreach ($this->request->data['Circuit']['Circuit'] as $kcircuit => $circuit) {
					$data['Circuit']['Circuit'][$kcircuit] = $circuit;
				}
			}
			if (!empty($this->request->data['Dossier']['Dossier'])) {
				$data['Dossier'] = array();
				foreach ($this->request->data['Dossier']['Dossier'] as $kdossier => $dossier) {
					$data['Dossier']['Dossier'][$kdossier] = $dossier;
				}
			}
			if (!empty($this->request->data['Affaire']['Affaire'])) {
				$data['Affaire'] = array();
				foreach ($this->request->data['Affaire']['Affaire'] as $kaffaire => $affaire) {
					$data['Affaire']['Affaire'][$kaffaire] = $affaire;
				}
			}
			if (!empty($this->request->data['Tache']['Tache'])) {
				$data['Tache'] = array();
				foreach ($this->request->data['Tache']['Tache'] as $ktache => $tache) {
					$data['Tache']['Tache'][$ktache] = $tache;
				}
			}
            // Métadonnées
			if (!empty($this->request->data['Metadonnee']['Metadonnee'])) {
				foreach ($this->request->data['Metadonnee']['Metadonnee'] as $kmetadonnee => $metadonnee) {
					$data['Metadonnee']['Metadonnee'][$kmetadonnee] = $metadonnee;
				}
			}
            // Valeur par Métadonnées
            if (!empty($this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'])) {
                foreach ($this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] as $typemetadonneeid => $valueMetadonnee) {
                    $data['Selectvaluemetadonnee']['Selectvaluemetadonnee'][$typemetadonneeid] = $valueMetadonnee;
                }
            }
            /*if (!empty($this->request->data['Metadonnee']['valeur'])) {
                foreach ($this->request->data['Metadonnee']['valeur'] as $typemetadonneeid => $valueMetadonnee) {
                    $data['Metadonnee']['valeur'][$typemetadonneeid] = $valueMetadonnee;
                }
            }*/
            if (!empty($this->request->data['Desktop']['Desktop'])) {
				foreach ($this->request->data['Desktop']['Desktop'] as $kdesktop => $desktop) {
					$data['Desktop']['Desktop'][$kdesktop] = $desktop;
				}
			}
			$data['Recherche']['user_id'] = $this->Session->read('Auth.User.id');
            $data['Recherche']['desktop_id'] = $this->Session->read('Auth.User.Desktop.id');
//debug($data);

            $this->Recherche->begin();          
			$success = $this->Recherche->saveAll($data);
            
            $metasList = $this->Recherche->Metadonnee->find(
                'all',
                array(
                    'recusive' => -1,
                    'contain' => false
                )
            );
            
            $typemetadonneeIds = Hash::extract( $metasList, '{n}.Metadonnee.typemetadonnee_id');
            $valOfSelect =  Configure::read('Selectvaluemetadonnee.id');
            
            // on regarde les anciens enregistrements entre le circut et les sous-types
            /*$records = $this->Recherche->RechercheSelectvaluemetadonnee->find(
                'list',
                array(
                    'fields' => array(
                        "RechercheSelectvaluemetadonnee.id",
                        "RechercheSelectvaluemetadonnee.metadonnee_id"
                    ),
                    'conditions' => array(
                        "RechercheSelectvaluemetadonnee.recherche_id" => $id
                    )
                )
            );
debug($records);

            $oldrecordsids = array_keys( $records );
debug($oldrecordsids);
            $nouveauxids = Hash::filter( (array)Hash::extract( $data, 'Selectvaluemetadonnee.Selectvaluemetadonnee.{n}' ) );
            $idsenmoins = array_diff( $oldrecordsids, $nouveauxids );
            if( !empty( $idsenmoins ) && !empty( $id ) ) {
                $success = $this->Recherche->RechercheSelectvaluemetadonnee->deleteAll(
                    array(
                        "RechercheSelectvaluemetadonnee.recherche_id" => $id,
                        "RechercheSelectvaluemetadonnee.metadonnee_id" => $idsenmoins
                    )
                ) && $success;
            }
debug($idsenmoins);   

            // En plus -> Ajouter
            $idsenplus = array_diff( $nouveauxids, $oldrecordsids );
debug($idsenplus);
die();
            if( !empty( $idsenplus ) ) {
                foreach( $idsenplus as $idenplus ) {
                    $record = array(
                         'RechercheSelectvaluemetadonnee' => array(
                            "recherche_id" => $id,
                            "metadonnee_id" => $idenplus
                        )
                    );

                    $this->Recherche->RechercheSelectvaluemetadonnee->create( $record );
                    $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
                }
            }*/

        // on regarde les nouveautés
//                    $nouveauxids = Hash::extract( $this->request->data, "Soustype.Soustype.{n}" );
        // si des différences existent on note les ids enlevés, les liens coupés entre circuit et sous-types
//                    $idsenmoins = array_diff( $oldrecordsids, $nouveauxids );
                      
            foreach( $data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] as $meta_id => $selectValueMeta ) {
                $metadon = $this->Recherche->Metadonnee->find('first', array('conditions' => array('Metadonnee.id' => $meta_id), 'recursive' => -1 ) );

                if( $metadon['Metadonnee']['typemetadonnee_id'] != $valOfSelect ) {
                    $metaValue = array(
                        'RechercheSelectvaluemetadonnee' => array(
                            'recherche_id' => $this->Recherche->id,
                            'selectvaluemetadonnee_id' => null,
                            'metadonnee_id' => $metadon['Metadonnee']['id'],
                            'valeur' => $selectValueMeta
                        )
                    );
                    
                    $this->Recherche->RechercheSelectvaluemetadonnee->create( $metaValue );
                    $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
                }
                else {
                    $metaValue = array(
                        'RechercheSelectvaluemetadonnee' => array(
                            'recherche_id' => $this->Recherche->id,
                            'selectvaluemetadonnee_id' => $selectValueMeta,
                            'metadonnee_id' => $metadon['Metadonnee']['id'],
                            'valeur' => null
                        )
                    );
                    
                    // Comparaison des ids avant/apres pour mise à jour
                    // on regarde les anciens enregistrements entre la recherche et les valeurs de métadonnées
                    /*$records = $this->Recherche->RechercheSelectvaluemetadonnee->find(
                        'list',
                        array(
                            'fields' => array( "RechercheSelectvaluemetadonnee.id", "RechercheSelectvaluemetadonnee.selectvaluemetadonnee_id" ),
                            'conditions' => array(
                                "RechercheSelectvaluemetadonnee.recherche_id" => $id
                            )
                        )
                    );*/
                    
                    if( empty($id) ){
                        $this->Recherche->RechercheSelectvaluemetadonnee->create( $metaValue );
                        $success = $this->Recherche->RechercheSelectvaluemetadonnee->save() && $success;
                    }
                }
                
            }
//debug($typemetadonneeIds);  
//            debug();
//            die();
//			$this->Recherche->begin();
//            if ($this->Recherche->saveAll($data)) {
            if ($success) {
				$this->Recherche->commit();
				$this->Jsonmsg->valid();
			} else {
				$this->Recherche->rollback();
			}
			$this->Jsonmsg->send();
		} else {
			$mode = "";
			if ($id != null) {
				if ($id == 0) {
					$mode = "add";
				} else {
					$mode = "edit";
					if ($forcemode != null && $forcemode == 'search') {
						$mode = 'search';
						$this->set('recherches', $this->Recherche->find('list', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id')))));
					}
					$this->request->data = $this->Recherche->find(
                        'first',
                        array(
                            'conditions' => array(
                                'Recherche.id' => $id
                            ),
                            'contain' => array(
                                'Desktop',
                                'Metadonnee',
                                'RechercheSelectvaluemetadonnee',
                                'Contact' => array(
                                    'Contactinfo'
                                ),
                                'Type',
                                'Soustype',
                                'Dossier',
                                'Affaire',
                                'Tache',
                                'Circuit'
                            )
                        )
                    );
                    $metaToDisplay = array();
                    foreach( $this->request->data['RechercheSelectvaluemetadonnee'] as $i => $valueSelect ) {
                        $key = $valueSelect['metadonnee_id'];
                        if( $valueSelect['selectvaluemetadonnee_id'] === null && $valueSelect['valeur'] === null ) {
                            $metaToDisplay[$key] = null;
                        }
                        else if( $valueSelect['selectvaluemetadonnee_id'] === null ) {
                            $metaToDisplay[$key] = $valueSelect['valeur'];
                        }
                        else {
                            $metaToDisplay[$key] = $valueSelect['selectvaluemetadonnee_id'];
                        }
                    }
                    $this->request->data['Selectvaluemetadonnee']['Selectvaluemetadonnee'] = $metaToDisplay;
                    
				}
			} else {
				$mode = "search";
				$this->set('recherches', $this->Recherche->find('list', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id')))));
			}
			$this->loadModel('Desktop');
			$desktopId = $this->Session->read('Auth.User.Courrier.Desktop.id');
			$daros = $this->Desktop->getDaros($desktopId);

			$this->set('taches', $this->Recherche->Tache->find('list'));
			$this->set('circuits', $this->Recherche->Circuit->find('list'));
			$this->set('dossiers', $this->Recherche->Dossier->find('list'));
			$this->set('affaires', $this->Recherche->Affaire->find('list'));
            
            $metas = $this->Recherche->Metadonnee->find('all', array( 'contain' => array( 'Selectvaluemetadonnee', 'Typemetadonnee')));
            $this->set('metas', $metas);    

//            $valueByMeta = array();
//            foreach( $metas as $i => $meta) {
//                if( !empty($meta['Selectvaluemetadonnee'])){
//                    $valueByMeta[$meta['Metadonnee']['id']] = $meta['Selectvaluemetadonnee'];
//                }
//            }
//            $this->set( 'valueByMeta', $valueByMeta );

            $valueSelectMeta = array( '0' => 'Non', '1' => 'Oui' );
            $this->set( 'valueSelectMeta', $valueSelectMeta );
			$querydata = array(
				'fields' => array(
					'Contactinfo.id',
					'Contact.name',
					'Addressbook.name'
				),
				'contain' => false,
				'joins' => array(
					$this->Courrier->Contactinfo->Contact->Addressbook->join($this->Courrier->Contactinfo->Contact->alias, array( 'type' => 'INNER' )),
					$this->Courrier->Contactinfo->Contact->join($this->Courrier->Contactinfo->alias, array( 'type' => 'INNER' ))
				)
			);

			$contacts = $this->Courrier->Contactinfo->Contact->Addressbook->find('list', $querydata);

			$this->set('contacts', $contacts);
			$this->set('types', $this->DataAuthorized->getAuthTypes(array('find' => 'list')));
			$this->set('soustypes', $this->DataAuthorized->getAuthSoustypes(array('find' => 'all')));
			$this->set('metadonnees', $this->DataAuthorized->getAuthMetadonnees(array('find' => 'list')));
			$this->set('mode', $mode);
            
            $affairesuiviepars = $this->Courrier->Desktop->getDesktopsByService( 'all' );
            ksort($affairesuiviepars);
            $this->set( 'affairesuiviepars', $affairesuiviepars);
		}
	}

	/**
	 * Récupération de la liste des recherches
	 *
	 * @logical-group Recherches
	 * @user-profile User
	 *
	 * @access public
	 * @return void
	 */
	public function getRecherches() {
		$this->Recherche->recursive = -1;
		$recherches = $this->Recherche->find('all', array('conditions' => array('Recherche.user_id' => $this->Session->read('Auth.User.id'))));
		for ($i = 0; $i < count($recherches); $i++) {
			$recherches[$i]['right_view'] = true;
			$recherches[$i]['right_edit'] = true;
			$recherches[$i]['right_delete'] = true;
		}
		$this->set('recherches', $recherches);
	}

	/**
	 * Suppression d'une recherche enregistrée
	 *
	 * @logical-group Recherches
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $id identifiant de la recherche enregistrée
	 * @return void
	 */
	public function delete($id = null) {
		$this->Jsonmsg->init(__d('default', 'delete.error'));
		$recherche = $this->Recherche->find('first', array('conditions' => array('Recherche.id' => $id)));
		$this->Recherche->begin();
		if ($this->Recherche->delete($id)) {
			$this->Recherche->commit();
			$this->Jsonmsg->valid(__d('default', 'delete.ok'));
		} else {
			$this->Recherche->rollback();
		}
		$this->Jsonmsg->send();
	}

	/**
	 * Lancement de la recherche
	 *
	 * @logical-group Recherches
	 * @user-profile User
	 *
	 * @access public
	 * @param integer $id identifiant de la recherche
	 * @return void
	 */
	public function recherche($id = null) {
//    $this->Recherche->recursive = -1;
		if ($id != null) {
			$this->request->data = $this->Recherche->find('first', array('conditions' => array('Recherche.id' => $id)));
		}

		$params = $this->request->data;
		if ( !empty($params) ) {
// debug($this->request->data);
			//on lance la recherche
			$flux_tmp = array();
			$querydata = $this->Recherche->search( $params, $id );

			// On prépare les conditions en fonction des droits
			$flux = array();
			$this->loadModel('Desktop');
			$desktopId = $this->Session->read('Auth.User.Courrier.Desktop.id');
			$daros = $this->Desktop->getDaros($desktopId);
			$soustypes = $this->Recherche->Soustype->find('list', array( 'contain' => false ));

			$allowedSoustypes = array();
                        $allowedSoustypes  = $this->DataAuthorized->getAuthSoustypes(array('find' => 'all'));
//			foreach ($soustypes as $kSoustype => $soustype) {
//				$daco = $this->DataAcl->Daco->find('first', array('conditions' => array('Daco.model' => 'Soustype', 'Daco.foreign_key' => $kSoustype), 'contain' => false));
//				$chks = array();
//				foreach ($daros as $daro) {
//					$chks[] = $this->DataAcl->check($daro['Daro'], $daco['Daco']);
//				}
//				if (!in_array(false, $chks, true)) {
//					$allowedSoustypes[$kSoustype] = $soustype;
//				}
//			}
//debug( $this->DataAuthorized->getAuthSoustypes(array('find' => 'all')));
			// Si aucune condition, on ne retourne rien
			if( !empty( $querydata ) ) {
				$querydata['limit'] = 10;
                                // Aucune condition sur les utilisateurs du courrier si Admin
                                if( $this->Session->read('Auth.User.Desktop.Group.name') != 'Admin' ) {
                                    $querydata['conditions'][] = array(
                                        'OR' => array(
                                            'Soustype.id' => array_keys( $allowedSoustypes ),
                                            'Courrier.user_creator_id' => $this->Session->read('Auth.User.id'),
                                            'Courrier.desktop_creator_id' => $desktopId
                                        )
                                    );
                                }

				$this->paginate = array( 'Courrier' => $querydata );

				$flux_tmp = $this->paginate( 'Courrier' );
			}
//debug($querydata);
			foreach ($flux_tmp as $kflx => $flx) {
//				if (array_key_exists($flx['Soustype']['id'], $allowedSoustypes) || $flx['Courrier']['user_creator_id'] == $this->Session->read('Auth.User.id')) {
					$flx['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
					$flx['right_read'] = true;
					$flx['right_update'] = true;
//					$flx['right_delete'] = true;
					$flux[] = $flx;
//				}
			}

			$this->set('flux', $flux);
		}
	}

	/**
	 * Export de la recherche au format CSV
	 *
	 * @logical-group Recherches
	 * @user-profile User
	 *
	 * @access public
	 * @return void
	 */
	public function exportcsv() {

		$querydata = $this->Recherche->search( Hash::expand( $this->request->params['named'], '__' ), null );
		unset( $querydata['limit'] );

		$flux = $this->Courrier->find( 'all', $querydata );
		foreach ($flux as $i => $flx) {
			$flx['Courrier']['retard'] = !$this->Courrier->isInTime($flx['Courrier']['id']);
			$flux[$i]['Courrier'] = $flx['Courrier'];
		}

		$this->layout = '';
		$this->set( compact( 'flux' ) );
	}

}

?>
