<?php

/**
 * Scanemails
 *
 * Scanemails controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Arnaud AUZOLAT
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class ScanemailsController extends AppController {

	/**
	 * Controller name
	 *
	 * @access public
	 * @var string
	 */
	public $name = 'Scanemails';

    public $uses = array('Scanemail');
    /**
	 * Gestion des emails interface graphique)
	 *
	 * @logical-group Scanemails
	 * @user-profil Admin
	 *
	 * @access public
	 * @return void
	 */
	public function index() {
		$this->set('ariane', array(
                    '<a href="/environnement/index/0/admin">'.__d('menu', 'Administration', true).'</a>', 
                    __d('menu', 'gestionScanemails', true)
                ));
	}
    
	/**
	 * Ajout d'un email
	 *
	 * @logical-group Scanemails
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function add() {
        $conn = $this->Session->read('Auth.User.connName');
        $this->loadModel('Collectivite');
        $collectivite = $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $conn)));
        $collId = $collectivite['Collectivite']['id'];
        
        $mails = $this->Scanemail->find( 'all' );
        $desktopsIds = Hash::extract( $mails, '{n}.Desktop.id' );

        $listDesktops = $this->Scanemail->Desktop->find(
            'list',
            array(
                'conditions' => array(
                    'Desktop.id NOT' => $desktopsIds,
                    'Desktop.group_id' => array(
                            INIT_GID,
                            DISP_GID
                    )
                ),
                'recursive' => -1
            )
        );
        $this->set('listDesktops', $listDesktops);

        
        
		if (!empty($this->request->data)) {
			$json = array(
				'message' => __d('default', 'save.error'),
				'success' => false
			);

			$scanemail = $this->request->data;
            $scanemail['Scanemail']['collectivite_id'] = $collId; 
            

			$this->Scanemail->begin();
			$this->Scanemail->create($scanemail);
			if ($this->Scanemail->save()) {
				$json['success'] = true;
			}
			if ($json['success']) {
				$this->Scanemail->commit();
				$json['message'] = __d('default', 'save.ok');
			} else {
				$this->Scanemail->rollback();
			}
			$this->Jsonmsg->sendJsonResponse($json);
		}
        
	}

	/**
	 * Edition d'un type
	 *
	 * @logical-group Scanemails
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du type
	 * @throws NotFoundException
	 * @return void
	 */
	public function edit($id) {
       
       
        
		if (!empty($this->request->data)) {
             $conn = $this->Session->read('Auth.User.connName');
            $this->loadModel('Collectivite');
            $collectivite = $this->Collectivite->find('first', array('conditions' => array('Collectivite.conn' => $conn)));
            $collId = $collectivite['Collectivite']['id'];  
            
            $this->Jsonmsg->init();
			$this->Scanemail->create();
            $scanemail = $this->request->data;
            $scanemail['Scanemail']['collectivite_id'] = $collId; 

			if ($this->Scanemail->save($scanemail)) {
				$this->Jsonmsg->valid();
			}
			$this->Jsonmsg->send();
		} else {
			$this->request->data = $this->Scanemail->find('first', array('conditions' => array('Scanemail.id' => $id)));
			if (empty($this->request->data)) {
				throw new NotFoundException();
			}
		}

        $mails = $this->Scanemail->find( 'all' );
        $desktopsIds = Hash::extract( $mails, '{n}.Desktop.id' );
        $listDesktops = $this->Scanemail->Desktop->find(
            'list',
            array(
                'conditions' => array(
                    'Desktop.group_id' => array(
                            INIT_GID,
                            DISP_GID
                    )
                ),
                'recursive' => -1
            )
        );
        $this->set('listDesktops', $listDesktops );
	}

	/**
	 * Suppression d'un email
	 *
	 * @logical-group Scanemails
	 * @user-profil Admin
	 *
	 * @access public
	 * @param integer $id identifiant du scanemail
	 * @return void
	 */
	public function delete($id = null) {
		$this->Jsonmsg->init(__d('default', 'delete.error'));
		$this->Scanemail->begin();
		if ($this->Scanemail->delete($id)) {
			$this->Scanemail->commit();
			$this->Jsonmsg->valid(__d('default', 'delete.ok'));
		} else {
			$this->Scanemail->rollback();
		}
		$this->Jsonmsg->send();
	}

    
    /**
	 * Récupération de la liste des mails à scruter (ajax)
	 *
	 * @logical-group Scanemails
	 * @user-profil Admin
	 *
	 * @access public
	 * @return void
	 */
	public function getScanemails() {
		$scanemails_tmp = $this->Scanemail->find("all", array('order' => 'Scanemail.mail'));
        $scanemails = array();
        foreach ($scanemails_tmp as $i => $item) {
            $item['right_edit'] = true;
            $item['right_delete'] = true;
            $item['Scanemail']['desktopName'] = $scanemails_tmp[$i]['Desktop']['name'];
            $scanemails[] = $item;
        }
        $this->set(compact('scanemails') );
	}
    
    
    
    
    /**
     * Validation d'unicité du nom d'une collectivité (création / édition)
     *
     * @param type $field
     */
    public function ajaxformvalid($field = null, $collectiviteId = 0) {

        $models = array();
        if ($field != null) {
            $models['Scanemail'] = array($field);
        } else {
            $models[] = 'Scanemail';
        }

        $rd = array();

        foreach ($this->request->data as $k => $v) {
            if ($k == 'Scanemail') {
                $rd[$k] = $v;
            }
        }

        $this->Ajaxformvalid->valid($models, $rd, $collectiviteId);
    }
    
}

?>