<?php

/**
 * Sous-types
 *
 * Soustypes controller class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		Controller
 */
class SoustypesController extends AppController {

	/**
	 * Component name
	 *
	 * @access public
	 * @var string
	 */
	public $name = 'Soustypes';

	/**
	 * Controller uses
	 *
	 * @access public
	 * @var array
	 */
	public $uses = array('Soustype');

	/**
	 * Gestion des types et sous-types (interface graphique)
	 *
	 * @logical-group Sous-types
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function index() {

		$this->set('ariane', array(
                    '<a href="environnement/index/0/admin">'.__d('menu', 'Administration', true).'</a>', 
                    __d('menu', 'gestionTypesSoustypes', true)
                ));
		$this->set('listeTypes', $this->Soustype->Type->find('list', array('order' => 'Type.name')));
	}

	/**
	 * Récupération de la liste imbriquée des types et sous-types (ajax)
	 *
	 * @logical-group Sous-types
	 * @user-profile Admin
	 *
	 * @access public
	 * @return void
	 */
	public function getTypesSoustypes() {
		$types_tmp = $this->Soustype->Type->find("all", array('contain' => array($this->Soustype->alias => array('order' => 'Soustype.name')), 'order' => 'Type.name'));
		$types = array();
		foreach ($types_tmp as $item) {
			$item['right_edit'] = true;
			$item['right_delete'] = true;
			$types[] = $item;
		}
		$this->set('types', $types);
	}

	/**
	 * Edition des informations d'un soustype (ajax)
	 *
	 * @logical-group Sous-types
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du sous-type
	 * @return void
	 */
	public function setInfos($id = null) {
		if ($id != null) {
			$this->set('id', $id);
		}
	}

	/**
	 * Récupération de la liste des modèles d'accusé de réception (ajax)
	 *
	 * @logical-group Sous-types
	 * @logical-group Modèles d'accusé de réception
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du sous-type
	 * @return void
	 */
	public function setArmodels($id = null) {
		if ($id != null) {
			$this->set('soustype_id', $id);
			$qd = array(
				'order' => array(
					'Armodel.name'
				),
				'conditions' => array(
					'Armodel.soustype_id' => $id
				),
				'fields' => $this->Soustype->Armodel->getLightFields()
			);
			$armodels = $this->Soustype->Armodel->find('all', $qd);
			$this->set('armodels', $armodels);
		}
	}

	/**
	 * Récupération de la liste des modèles de réponse (ajax)
	 *
	 * @logical-group Sous-types
	 * @logical-group Modèles de réponses
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du sous-type
	 * @return void
	 */
	public function setRmodels($id = null) {
		if ($id != null) {
			$this->set('soustype_id', $id);
			$qd = array(
				'order' => array(
					'Rmodel.name'
				),
				'conditions' => array(
					'Rmodel.soustype_id' => $id
				),
				'fields' => $this->Soustype->Rmodel->getLightFields()
			);
			$rmodels = $this->Soustype->Rmodel->find('all', $qd);
			$this->set('rmodels', $rmodels);
		}
	}

	/**
	 *
	 * @param type $id
	 */
	//TODO: supprimer les fonctions commentées
//	public function view($id) {
//		$soustype = $this->Soustype->find(
//				'first', array(
//			'contain' => array(
//				'Type',
//				'Circuit',
//				'Courrier',
//				'Model'
//			),
//			'conditions' => array('Soustype.id' => $id)
//				)
//		);
//
//		if (empty($soustype)) {
//			throw new NotFoundException();
//		}
//
//		$this->set(compact('soustype'));
//	}

	/**
	 * Ajout d'un sous-type
	 *
	 * @logical-group Sous-types
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $type_id identifiant du type
	 * @return void
	 */
	public function add($type_id = null) {
		if (!empty($this->request->data)) {
			$json = array(
				'message' => __d('default', 'save.error'),
				'success' => false
			);
			//debut des transactions
			$this->Soustype->begin();
			//preparation du tableau des verifications
			$success = array(
				'Soustype' => false
			);
			//enregistrement du soustype
			$stype = $this->Soustype->create($this->request->data);
			if ($this->Soustype->save($stype)) {
				$success['Soustype'] = true;
			}

			//on effectue les transactions si tout est bon
			if (!in_array(false, $success, true)) {
				$this->Soustype->commit();
				$json['success'] = true;
				$json['message'] = __d('default', 'save.ok');
			} else {
				$this->Soustype->rollback();
			}
			$this->Jsonmsg->sendJsonResponse($json);
		} else {
			$this->set('type_id', $type_id);
			$this->set('types', $this->Soustype->Type->find('list', array('order' => array('Type.name'))));
			$this->set('circuits', $this->Soustype->Circuit->find('list', array('order' => array('Circuit.nom'), 'conditions' => array('Circuit.actif' => 1))));
			$querydata = array(
				'fields' => array(
					'Soustype.id',
					'Soustype.name'
				),
				'contain' => false,
				'joins' => array(
					$this->Soustype->Type->join($this->Soustype->alias, array('type' => 'INNER'))
				),
				'order' => array(
					'Soustype.name'
				)
			);
			$this->set('soustypesList', $this->Soustype->Type->find('list', $querydata));
		}
	}

	/**
	 * Edition d'un soustype (interface graphique)
	 *
	 * @logical-group Sous-types
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du sous-type
	 * @throws NotFoundException
	 * @return void
	 */
	public function edit($id = null) {
		$stype = $this->Soustype->find('first', array('recursive' => -1, 'conditions' => array('Soustype.id' => $id), 'fields' => array('Soustype.id', 'Soustype.parent_id')));
		if (empty($stype)) {
			throw new NotFoundException();
		}
		$this->set('id', $id);
		$this->set('response', !empty($stype['Soustype']['parent_id']));
		if (!empty($this->request->data)) {
			$json = array(
				'success' => false,
				'message' => __d('default', 'save.error')
			);
			$this->Soustype->begin();
			$messages = array(
				'soustype' => false,
			);
			if ($this->Soustype->save($this->request->data['Soustype'])) {
				$messages['soustype'] = true;
			}
			if (!in_array(false, $messages, true)) {
				$this->Soustype->commit();
				$json['success'] = true;
				$json['message'] = __d('default', 'save.ok');
			} else {
				$this->Soustype->rollback();
				$json['message'] = 'Préparation de la modification du soustype : ' . ($messages['soustype'] ? 'ok' : 'erreur');
				$json['message'] .= '<br />Préparation de la modification du compteur : ' . ($messages['compteur'] ? 'ok' : 'erreur');
				$json['message'] .= '<br />Préparation de la modification du seuqence : ' . ($messages['sequence'] ? 'ok' : 'erreur');
			}
			$json['message'] = 'Onglet informations<br />' . $json['message'];
			$this->Jsonmsg->sendJsonResponse($json);
		} else {
			$querydata = array(
				'contain' => array(
					'Type',
					'Armodel',
					'Circuit',
					'Formatreponse'
				),
				'conditions' => array('Soustype.id' => $id)
			);
			$this->request->data = $this->Soustype->find('first', $querydata);
			$this->set('models', $this->Soustype->Armodel->find('all', array('order' => array('Armodel.name'))));
			$this->set('circuits', $this->Soustype->Circuit->find('list', array('order' => array('Circuit.nom'), 'conditions' => array('Circuit.actif' => 1))));
			$this->set('types', $this->Soustype->Type->find('list', array('order' => array('Type.name'))));
			$querydata = array(
				'fields' => array(
					'Soustype.id',
					'Soustype.name'
				),
				'contain' => false,
				'joins' => array(
					$this->Soustype->Type->join($this->Soustype->alias, array('type' => 'INNER'))
				),
				'order' => array(
					'Soustype.name'
				)
			);
			$this->set('soustypesList', $this->Soustype->Type->find('list', $querydata));
		}
	}

	/**
	 * Suppression d'un sous-type (delete)
	 *
	 * @logical-group Sous-types
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du sous-types
	 * @throws NotFoundException
	 * @return void
	 */
	public function delete($id = null) {
		$stype = $this->Soustype->find('first', array('conditions' => array('Soustype.id' => $id)));
        $compteur_id = $stype['Soustype']['compteur_id'];
        $sequence_id = $stype['Compteur']['sequence_id'];
		if (empty($stype)) {
			throw new NotFoundException();
		}
        $this->Soustype->begin();
		$this->Jsonmsg->init(__d('default', 'delete.error'));
		$used = false;
		$ref = $this->Soustype->Courrier->find('first', array('conditions' => array('Courrier.soustype_id' => $stype['Soustype']['id'])));

		if (!empty($ref)) {
			$used = true;
		}
		if (!$used) {
			if ($this->Soustype->delete($id, true)) {
                $this->Soustype->Compteur->delete($compteur_id, true);
                $this->Soustype->Compteur->Sequence->delete($sequence_id, true);
                $this->Soustype->commit();
				$this->Jsonmsg->valid(__d('default', 'delete.ok'));
			}
		} else {
            $this->Soustype->rollback();
			$this->Jsonmsg->init(__d('soustype', 'Soustype.delete.notAllowed'));
		}
		$this->Jsonmsg->send();
//		$this->Jsonmsg->init(__d('default', 'delete.error'));
//		$messages = array(
//			'soustype' => false,
//			'rights' => false,
//			'daco' => false
//		);
//		$this->Soustype->begin();
//		if ($this->Soustype->delete($id)) {
//			$messages['soustype'] = true;
//			$daco = $this->Droits->getDaco(array('model' => 'Soustype', 'foreign_key' => $id));
////      $this->_logDebug($daco);
//			if ($this->Droits->deleteDacoRights($daco)) {
//				$messages['rights'] = true;
//				if ($this->Droits->deleteDaco($daco)) {
//					$messages['daco'] = true;
//				}
//			}
//		}
//
//		if (!in_array(false, $messages, true)) {
//			$this->Soustype->commit();
//			$this->Jsonmsg->valid(__d('default', 'delete.ok'));
//		} else {
//			$this->json['message'] = 'Préparation de la suppression du soustype : ' . ($messages['soustype'] ? 'ok' : 'erreur');
//			$this->json['message'] .= '<br />Préparation de la suppression des droits du soustype : ' . ($messages['rights'] ? 'ok' : 'erreur');
//			$this->json['message'] .= '<br />Préparation de la suppression du daco correspondant au soustype : ' . ($messages['daco'] ? 'ok' : 'erreur');
//			$this->Soustype->rollback();
//		}
//		$this->Jsonmsg->send();
	}

	/**
	 * Récupération de la simulation de référence pour un nouveau flux (ajax)
	 *
	 * @logical-group Sous-type
	 *
	 * @logical-group Flux
	 * @logical-group Création d'un flux
	 * @user-profile Admin
	 *
	 * @access public
	 * @param integer $id identifiant du sous-type
	 * @return void
	 */
	public function getRefSimul($id) {
		$this->set('refSimul', $this->Soustype->getRef($id, true));
	}

}

?>