<?php

App::uses('Abaddressbook', 'Addressbook.Model');

/**
 * Addressbook model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Addressbook extends Abaddressbook {

	/**
	 *
	 * @var type
	 */
	public $name = 'Addressbook';

	/**
	 *
	 * @var type
	 */
	public $useTable = 'addressbooks';

	/**
	 *
	 * @var type
	 */
//  public $belongsTo = array(
//      'Collectivite' => array(
//          'className' => 'Collectivite',
//          'foreignKey' => 'foreign_key',
//          'type' => 'INNER',
//          'conditions' => null,
//          'fields' => null,
//          'order' => null
//      )
//  );

	/**
	 *
	 * @var type
	 */
	public $hasAndBelongsToMany = array(
		'Recherche' => array(
			'className' => 'Recherche',
			'joinTable' => 'recherches_addressbooks',
			'foreignKey' => 'addressbook_id',
			'associationForeignKey' => 'recherche_id',
			'unique' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'finderQuery' => null,
			'deleteQuery' => null,
			'insertQuery' => null,
			'with' => 'RechercheAddressbook'
		)
	);

	/**
	 *
	 * @var type
	 */
	public $hasMany = array(
		'Contact' => array(
			'className' => 'Contact',
			'foreignKey' => 'addressbook_id',
			'dependent' => false,
			'conditions' => '',
			'fields' => '',
			'order' => '',
			'limit' => '',
			'offset' => '',
			'exclusive' => '',
			'finderQuery' => '',
			'counterQuery' => ''
		)
	);

	/**
	 *
	 * @param type $lines
	 * @return string
	 */
	private function _convertVcards(&$lines) {
		$cards = array();
		$card = new VCard();

		while (@$card->parse($lines)) {
			$property = $card->getProperty('N');
			if (!$property) {
				return "";
			}
			$n = $property->getComponents();
			$tmp = array();
			if (isset($n[3]))
				$tmp[] = $n[3];   // Mr.
			if (isset($n[1]))
				$tmp[] = $n[1];   // John
			if (isset($n[2]))
				$tmp[] = $n[2];   // Quinlan
			if (isset($n[4]))
				$tmp[] = $n[4];   // Esq.
			$ret = array();
			if (isset($n[0]))
				$ret[] = $n[0];
			$tmp = join(" ", $tmp);
			if ($tmp)
				$ret[] = $tmp;
			$key = join(", ", $ret);
			$cards[$key] = $card;

			// MDH: Create new VCard to prevent overwriting previous one (PHP5)
			$card = new VCard();
		}
		ksort($cards);
		return $cards;
	}

	/**
	 *
	 * @param type $card
	 * @return type
	 */
	private function _checkVcard(&$card) {
		$checks = array(
			'fn' => false,
			'n' => false,
			'version' => false
		);

		if (isset($card->_map['FN']) && isset($card->_map['FN'][0]) && isset($card->_map['FN'][0]->value)) {
			$checks['fn'] = true;
		}
		if (isset($card->_map['N']) && isset($card->_map['N'][0]) && isset($card->_map['N'][0]->value)) {
			$checks['n'] = true;
		}
		if (isset($card->_map['VERSION']) && isset($card->_map['VERSION'][0]) && isset($card->_map['VERSION'][0]->value)) {
			$checks['version'] = true;
		}

		return !in_array(false, $checks, true);
	}

	/**
	 *
	 * @param type $card
	 * @return type
	 */
	private function _processVcard($card, $addressbook_id, $foreign_key) {
		//create
		$names = explode(';', $card->_map['N'][0]->value);


		$contactinfo = array(
			'Contactinfo' => array(
				'name' => $card->_map['FN'][0]->value,
				'civilite' => isset($names[3]) ? $names[3] : '',
				'nom' => isset($names[0]) ? $names[0] : '',
				'prenom' => isset($names[1]) ? $names[1] : ''
			)
		);



		//email
		if (isset($card->_map['EMAIL']) && isset($card->_map['EMAIL'][0])) {
			$contactinfo['Contactinfo']['email'] = $card->_map['EMAIL'][0]->value;
		}

		//adress
		if (isset($card->_map['ADR']) && isset($card->_map['ADR'][0])) {
			$adresses = explode(';', $card->_map['ADR'][0]->value);
			$contactinfo['Contactinfo']['compl'] = isset($adresses[0]) ? $adresses[0] : '';
			$contactinfo['Contactinfo']['compl'] .= isset($adresses[1]) ? ' ' . $adresses[1] : '';
			$contactinfo['Contactinfo']['addresse'] = isset($adresses[2]) ? $adresses[2] : '';
			$contactinfo['Contactinfo']['ville'] = isset($adresses[3]) ? $adresses[3] : '';
			$contactinfo['Contactinfo']['region'] = isset($adresses[4]) ? $adresses[4] : '';
			$contactinfo['Contactinfo']['cp'] = isset($adresses[5]) ? $adresses[5] : '';
			$contactinfo['Contactinfo']['pays'] = isset($adresses[6]) ? $adresses[6] : '';
		}

		//tel
		if (isset($card->_map['TEL']) && isset($card->_map['TEL'][0])) {
			$contactinfo['Contactinfo']['tel'] = $card->_map['TEL'][0]->value;
		}

		//role
		if (isset($card->_map['ROLE']) && isset($card->_map['ROLE'][0])) {
			$contactinfo['Contactinfo']['role'] = $card->_map['ROLE'][0]->value;
		}
		//organisation
		if (isset($card->_map['ORG']) && isset($card->_map['ORG'][0])) {
			$contactinfo['Contactinfo']['organisation'] = $card->_map['ORG'][0]->value;
		}

		return $contactinfo;
	}

	/**
	 *
	 * @param type $filename
	 * @param type $addressbook_id
	 * @param type $foreign_key
	 * @return boolean
	 */
	public function importContactVcards($filename = null, $addressbook_id = null, $foreign_key = null) {
		$return = array();
		if ($addressbook_id != null) {
			if ($filename != null && is_file($filename) && is_readable($filename)) {
				$cards = $this->_convertVcards(file($filename));
				foreach ($cards as $kcard => $card) {
					if ($this->_checkVcard($card)) {
						$contactinfo = $this->_processVcard($card, $addressbook_id, $foreign_key);
						$contact = array(
							'Contact' => array(
								'name' => $contactinfo['Contactinfo']['name'],
								'nom' => $contactinfo['Contactinfo']['nom'],
								'prenom' => $contactinfo['Contactinfo']['prenom'],
								'civilite' => $contactinfo['Contactinfo']['civilite'],
								'addressbook_id' => $addressbook_id
							)
						);

						//save
						$this->Contact->begin();
						$this->Contact->create();
						$savedContact = $this->Contact->save($contact);

						$contactinfo['Contactinfo']['contact_id'] = $savedContact['Contact']['id'];

						$this->Contact->Contactinfo->create();
						$savedContactinfo = $this->Contact->Contactinfo->save($contactinfo);

						$saved = !empty($savedContact) && !empty($savedContactinfo);
						if ($saved) {
							$return[] = true;
							$this->Contact->commit();
						} else {
							$return[] = false;
							$this->Contact->rollback();
						}
					}
				}
			}
		}
		return !in_array(false, $return, true);
	}

    /**
     * 
     */
    public function listTypesVoie() {
        return array(
            'Addressbook' => array(
                'typevoie' => array(
                    'ABE' => 'Abbaye',
                    'ACH' => 'Ancien chemin',
                    'AGL' => 'Agglomération',
                    'AIRE' => 'Aire',
                    'ALL' => 'Allée',
                    'ANSE' => 'Anse',
                    'ARC' => 'Arcade',
                    'ART' => 'Ancienne route',
                    'AUT' => 'Autoroute',
                    'AV' => 'Avenue',
                    'BAST' => 'Bastion',
                    'BCH' => 'Bas chemin',
                    'BCLE' => 'Boucle',
                    'BD' => 'Boulevard',
                    'BEGI' => 'Béguinage',
                    'BER' => 'Berge',
                    'BOIS' => 'Bois',
                    'BRE' => 'Barriere',
                    'BRG' => 'Bourg',
                    'BSTD' => 'Bastide',
                    'BUT' => 'Butte',
                    'CALE' => 'Cale',
                    'CAMP' => 'Camp',
                    'CAR' => 'Carrefour',
                    'CARE' => 'Carriere',
                    'CARR' => 'Carre',
                    'CAU' => 'Carreau',
                    'CAV' => 'Cavée',
                    'CGNE' => 'Campagne',
                    'CHE' => 'Chemin',
                    'CHEM' => 'Cheminement',
                    'CHEZ' => 'Chez',
                    'CHI' => 'Charmille',
                    'CHL' => 'Chalet',
                    'CHP' => 'Chapelle',
                    'CHS' => 'Chaussée',
                    'CHT' => 'Château',
                    'CHV' => 'Chemin vicinal',
                    'CITE' => 'Cité',
                    'CLOI' => 'Cloître',
                    'CLOS' => 'Clos',
                    'COL' => 'Col',
                    'COLI' => 'Colline',
                    'COR' => 'Corniche',
                    'COTE' => 'Côte(au)',
                    'COTT' => 'Cottage',
                    'COUR' => 'Cour',
                    'CPG' => 'Camping',
                    'CRS' => 'Cours',
                    'CST' => 'Castel',
                    'CTR' => 'Contour',
                    'CTRE' => 'Centre',
                    'DARS' => 'Darse',
                    'DEG' => 'Degré',
                    'DIG' => 'Digue',
                    'DOM' => 'Domaine',
                    'DSC' => 'Descente',
                    'ECL' => 'Ecluse',
                    'EGL' => 'Eglise',
                    'EN' => 'Enceinte',
                    'ENC' => 'Enclos',
                    'ENV' => 'Enclave',
                    'ESC' => 'Escalier',
                    'ESP' => 'Esplanade',
                    'ESPA' => 'Espace',
                    'ETNG' => 'Etang',
                    'FG' => 'Faubourg',
                    'FON' => 'Fontaine',
                    'FORM' => 'Forum',
                    'FORT' => 'Fort',
                    'FOS' => 'Fosse',
                    'FOYR' => 'Foyer',
                    'FRM' => 'Ferme',
                    'GAL' => 'Galerie',
                    'GARE' => 'Gare',
                    'GARN' => 'Garenne',
                    'GBD' => 'Grand boulevard',
                    'GDEN' => 'Grand ensemble',
                    'GPE' => 'Groupe',
                    'GPT' => 'Groupement',
                    'GR' => 'Grand(e) rue',
                    'GRI' => 'Grille',
                    'GRIM' => 'Grimpette',
                    'HAM' => 'Hameau',
                    'HCH' => 'Haut chemin',
                    'HIP' => 'Hippodrome',
                    'HLE' => 'Halle',
                    'HLM' => 'HLM',
                    'ILE' => 'Ile',
                    'IMM' => 'Immeuble',
                    'IMP' => 'Impasse',
                    'JARD' => 'Jardin',
                    'JTE' => 'Jetée',
                    'LD' => 'Lieu dit',
                    'LEVE' => 'Levée',
                    'LOT' => 'Lotissement',
                    'MAIL' => 'Mail',
                    'MAN' => 'Manoir',
                    'MAR' => 'Marche',
                    'MAS' => 'Mas',
                    'MET' => 'Métro',
                    'MF' => 'Maison forestiere',
                    'MLN' => 'Moulin',
                    'MTE' => 'Montée',
                    'MUS' => 'Musée',
                    'NTE' => 'Nouvelle route',
                    'PAE' => 'Petite avenue',
                    'PAL' => 'Palais',
                    'PARC' => 'Parc',
                    'PAS' => 'Passage',
                    'PASS' => 'Passe',
                    'PAT' => 'Patio',
                    'PAV' => 'Pavillon',
                    'PCH' => 'Porche - petit chemin',
                    'PERI' => 'Périphérique',
                    'PIM' => 'Petite impasse',
                    'PKG' => 'Parking',
                    'PL' => 'Place',
                    'PLAG' => 'Plage',
                    'PLAN' => 'Plan',
                    'PLCI' => 'Placis',
                    'PLE' => 'Passerelle',
                    'PLN' => 'Plaine',
                    'PLT' => 'Plateau(x)',
                    'PN' => 'Passage à niveau',
                    'PNT' => 'Pointe',
                    'PONT' => 'Pont(s)',
                    'PORQ' => 'Portique',
                    'PORT' => 'Port',
                    'POT' => 'Poterne',
                    'POUR' => 'Pourtour',
                    'PRE' => 'Pré',
                    'PROM' => 'Promenade',
                    'PRQ' => 'Presqu\'île',
                    'PRT' => 'Petite route',
                    'PRV' => 'Parvis',
                    'PSTY' => 'Peristyle',
                    'PTA' => 'Petite allée',
                    'PTE' => 'Porte',
                    'PTR' => 'Petite rue',
                    'QU' => 'Quai',
                    'QUA' => 'Quartier',
                    'R' => 'Rue',
                    'RAC' => 'Raccourci',
                    'RAID' => 'Raidillon',
                    'REM' => 'Rempart',
                    'RES' => 'Résidence',
                    'RLE' => 'Ruelle',
                    'ROC' => 'Rocade',
                    'ROQT' => 'Roquet',
                    'RPE' => 'Rampe',
                    'RPT' => 'Rond point',
                    'RTD' => 'Rotonde',
                    'RTE' => 'Route',
                    'SEN' => 'Sentier',
                    'SQ' => 'Square',
                    'STA' => 'Station',
                    'STDE' => 'Stade',
                    'TOUR' => 'Tour',
                    'TPL' => 'Terre plein',
                    'TRA' => 'Traverse',
                    'TRN' => 'Terrain',
                    'TRT' => 'Tertre(s)',
                    'TSSE' => 'Terrasse(s)',
                    'VAL' => 'Val(lée)(lon)',
                    'VCHE' => 'Vieux chemin',
                    'VEN' => 'Venelle',
                    'VGE' => 'Village',
                    'VIA' => 'Via',
                    'VLA' => 'Villa',
                    'VOI' => 'Voie',
                    'VTE' => 'Vieille route',
                    'ZA' => 'Zone artisanale',
                    'ZAC' => 'Zone d\'aménagement concerte',
                    'ZAD' => 'Zone d\'aménagement différé',
                    'ZI' => 'Zone industrielle',
                    'ZONE' => 'Zone',
                    'ZUP' => 'Zone à urbaniser en priorité'
                )
            )
        );
    }
}

?>