<?php

App::uses('AppModel', 'Model');

/**
 * Armodel model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Armodel extends AppModel {

	/**
	 * Display field
	 *
	 * @var string
	 */
	public $displayField = 'name';

	/**
	 *
	 * @var type
	 */
	public $actsAs = array('Linkeddocs');

	/**
	 *
	 * @var type
	 */
	public $formats = array(
		'ott',
		'audio',
		'video',
		'sms',
		'email'
	);

	/**
	 * Validation rules
	 *
	 * @var array
	 */
	public $validate = array(
		'name' => array(
			'notempty' => array(
				'rule' => array('notempty'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'format' => array(
			'notempty' => array(
				'rule' => array('notempty'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'size' => array(
			'numeric' => array(
				'rule' => array('numeric'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'soustype_id' => array(
			'numeric' => array(
				'rule' => array('numeric'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

	/**
	 * belongsTo associations
	 *
	 * @var array
	 */
	public $belongsTo = array(
		'Soustype' => array(
			'className' => 'Soustype',
			'foreignKey' => 'soustype_id',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		)
	);

	/**
	 *
	 * @var type
	 */
	public $otherMimeTypes = array(
		'audio' => array(
			'audio/x-wav' => 'wav',
			'audio/mpeg' => 'mp3',
			'application/ogg' => 'ogg',
			'audio/ogg' => 'ogg'
		),
		'video' => array(
			'video/x-msvideo' => 'avi',
			'video/mpeg' => 'mpg',
			'video/mp4' => 'mp4',
			'application/ogg' => 'ogg',
			'video/ogg' => 'ogg'
		),
		'image' => array(
			'image/gif' => 'gif',
			'image/jpeg' => 'jpg',
			'image/png' => 'png'
		)
	);

	/**
	 *
	 * @param type $id
	 * @return type
	 * @throws NotFoundException
	 */
	public function generatePreview($id = null) {
		if (!empty($id)) {
			$this->id = $id;
		}

		if (empty($this->id)) {
			throw new NotFoundException();
		}

		$pdfContent = '';
		$this->recursive = -1;
		$armodel = $this->read();

		$return = __d('document', 'Document.preview.error');

		if ($armodel['Armodel']['mime'] == "application/vnd.oasis.opendocument.text") {
			//Conversion en pdf via Gedooo
			require_once 'XML/RPC.php';
			$content = base64_encode($armodel['Armodel']['content']);

			$params = array(new XML_RPC_Value($content, 'string'),
				new XML_RPC_Value($armodel['Armodel']['ext'], 'string'),
				new XML_RPC_Value('pdf', 'string'),
				new XML_RPC_Value(false, 'boolean'),
				new XML_RPC_Value(true, 'boolean'));

			$url = Configure::read('CLOUDOOO_HOST') . ":" . Configure::read('CLOUDOOO_PORT');

			$msg = new XML_RPC_Message('convertFile', $params);
			$cli = new XML_RPC_Client('/', $url);
			$resp = $cli->send($msg);
			$pdfContent = base64_decode($resp->xv->me['string']);
		} else {
			foreach ($this->otherMimeTypes as $type => $mimes) {
				if (in_array($armodel['Armodel']['mime'], array_keys($mimes))) {
					$return = $this->field('Armodel.content', array('Armodel.id' => $id));
				}
			}
		}



		//Conversion en flash
		if (!empty($pdfContent)) {
			$date_suffix = date('Ymd_His');
			$tmp_pdf_to_convert = TMP . "pdf_to_convert_" . $date_suffix;
			$tmp_pdf_converted = TMP . "pdf_converted_" . $date_suffix;

			file_put_contents($tmp_pdf_to_convert, $pdfContent);
			$cmd = "pdf2swf -i " . $tmp_pdf_to_convert . " -o " . $tmp_pdf_converted . " -f -T 9 -t -s storeallcharacters";
			$sys_result = exec($cmd, $output, $return_var);


			if ($return_var == 0) {
				$return = file_get_contents($tmp_pdf_converted);

				$armodel['Armodel']['preview'] = $return;
				$this->create();
				$this->save($armodel);
			}

			unlink($tmp_pdf_converted);
			unlink($tmp_pdf_to_convert);
		}
		return $return;
	}

}
