<?php

App::uses('Abcontact', 'Addressbook.Model');

/**
 * Contact model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Contact extends Abcontact {

	/**
	 *
	 * @var type
	 */
	public $name = 'Contact';

	/**
	 *
	 * @var type
	 */
	public $useTable = 'contacts';

	/**
	 *
	 * @var type
	 */
	public $hasMany = array(
		'Contactinfo' => array(
			'className' => 'Contactinfo',
			'foreignKey' => 'contact_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		)
	);

	/**
	 *
	 * @var type
	 */
	public $hasAndBelongsToMany = array(
		'Recherche' => array(
			'className' => 'Recherche',
			'joinTable' => 'recherches_contacts',
			'foreignKey' => 'contact_id',
			'associationForeignKey' => 'recherche_id',
			'unique' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'finderQuery' => null,
			'deleteQuery' => null,
			'insertQuery' => null,
			'with' => 'RechercheContact'
		)
	);

	/**
	 *
	 * @var type
	 */
	public $belongsTo = array(
		'Addressbook' => array(
			'className' => 'Addressbook',
			'foreignKey' => 'addressbook_id',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		)
	);

	/**
	 *
	 * @param type $nom
	 * @param type $prenom
	 * @return type
	 */
	public function getHomonyms($name) {
		$return = array();
		if (!empty($name)) {
			$slug = strtolower(Inflector::slug($name));
			$qd = array(
				'recursive' => -1,
				'fields' => array(
					'Contact.name',
					'Addressbook.name'
				),
				'contain' => false,
				'joins' => array(
					$this->join($this->Addressbook->alias)
				),
				'conditions' => array("levenshtein('" . $slug . "', Contact.slug) <= " . Configure::read('levenshteinThreshold')),
				'order' => array(
					"levenshtein('" . $slug . "', Contact.slug) DESC",
					'Contact.name'
				)
			);
			$return = $this->find('all', $qd);
		}
		return $return;
	}

	/**
	 *
	 * @param type $created
	 * @return type
	 */
	public function beforeSave($options = array()) {
		parent::beforeSave($options);
		if (!empty($this->data['Contact']['name'])) {
			$this->data['Contact']['slug'] = strtolower(Inflector::slug($this->data['Contact']['name']));
		}
		return true;
	}

}

?>