<?php

App::uses('AppModel', 'Model');
App::uses('SCTarget', 'SimpleComment/Model/Behavior');

/**
 * Courrier model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Courrier extends AppModel {

	/**
	 * Model name
	 *
	 * @var string
	 * @access public
	 */
	public $name = 'Courrier';

	/**
	 * Model Behaviors
	 *
	 * @var mixed
	 * @access public
	 */
//	public $beforesAs = array('Tree'); //TODO: remove if not used

	public $actsAs = array('SimpleComment.SCTarget');

	/**
	 * Validation rules
	 *
	 * @var mixed
	 * @access public
	 */
	public $validate = array(
		'name' => array(
			array(
				'rule' => array('notempty'),
				'allowEmpty' => false,
			),
		),
		'etat' => array(
			array(
				'rule' => array('numeric'),
				'allowEmpty' => true,
			),
		),
		'typeflux' => array(
			array(
				'rule' => array('numeric'),
				'allowEmpty' => true,
			),
		),
		'circuit_id' => array(
			array(
				'rule' => array('numeric'),
				'allowEmpty' => true,
			),
		),
		'soustype_id' => array(
			array(
				'rule' => array('numeric'),
				'allowEmpty' => true,
			),
		),
		'contactinfo_id' => array(
			array(
				'rule' => array('numeric'),
				'allowEmpty' => true,
			),
		),
	);

	/**
	 *
	 * @var type
	 */
	public $hasOne = array(
		'Traitement' => array(
			'className' => 'Cakeflow.Traitement',
			'foreignKey' => 'target_id'
		)
	);

	/**
	 * belongsTo
	 *
	 * @var mixed
	 * @access public
	 */
	public $belongsTo = array(
		'Soustype' => array(
			'className' => 'Soustype',
			'foreignKey' => 'soustype_id',
			'type' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null
		),
		'Contactinfo' => array(
			'className' => 'Contactinfo',
			'foreignKey' => 'contactinfo_id',
			'type' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null
		),
		'Affaire' => array(
			'className' => 'Affaire',
			'foreignKey' => 'affaire_id',
			'type' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null
		),
		'User' => array(
			'className' => 'User',
			'foreignKey' => 'user_creator_id',
			'type' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null
		),
		'Desktop' => array(
			'className' => 'Desktop',
			'foreignKey' => 'desktop_creator_id',
			'type' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null
		),
		'Affairesuivie' => array(
			'className' => 'Desktop',
			'foreignKey' => 'affairesuiviepar_id',
			'type' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null
		),
		'Service' => array(
			'className' => 'Service',
			'foreignKey' => 'service_creator_id',
			'type' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null
		)
	);

	/**
	 * hasMany
	 *
	 * @var mixed
	 * @access public
	 */
	public $hasMany = array(
		'Tache' => array(
			'className' => 'Tache',
			'foreignKey' => 'courrier_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		),
		'Traitement' => array(
			'className' => 'Cakeflow.Traitement',
			'foreignKey' => 'target_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		),
		'Bancontenu' => array(
			'className' => 'Bancontenu',
			'foreignKey' => 'courrier_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		),
		'Notifieddesktop' => array(
			'className' => 'Notifieddesktop',
			'foreignKey' => 'courrier_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		),
		'Document' => array(
			'className' => 'Document',
			'foreignKey' => 'courrier_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		),
		'Ar' => array(
			'className' => 'Ar',
			'foreignKey' => 'courrier_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		),
		'Relement' => array(
			'className' => 'Relement',
			'foreignKey' => 'courrier_id',
			'dependent' => false,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'exclusive' => null,
			'finderQuery' => null,
			'counterQuery' => null
		)
	);

	/**
	 * hasAndBelongsToMany
	 *
	 * @var mixed
	 * @access public
	 */
	public $hasAndBelongsToMany = array(
		'Metadonnee' => array(
			'className' => 'Metadonnee',
			'joinTable' => 'courriers_metadonnees',
			'foreignKey' => 'courrier_id',
			'associationForeignKey' => 'metadonnee_id',
			'unique' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'finderQuery' => null,
			'deleteQuery' => null,
			'insertQuery' => null,
			'with' => 'CourrierMetadonnee'
		),
		'Repertoire' => array(
			'className' => 'Repertoire',
			'joinTable' => 'courriers_repertoires',
			'foreignKey' => 'courrier_id',
			'associationForeignKey' => 'repertoire_id',
			'unique' => null,
			'conditions' => null,
			'fields' => null,
			'order' => null,
			'limit' => null,
			'offset' => null,
			'finderQuery' => null,
			'deleteQuery' => null,
			'insertQuery' => null,
			'with' => 'CourrierRepertoire'
		)
	);

	// Variable permettant de définir le dernier numéro de séqauence d'un courrier généré
	protected $_lastSequenceReference = null;
	protected $_lastSequenceDocument = null;

	/**
	 *
	 * @param type $id
	 * @return boolean
	 */
	public function isInCircuit($id = null) {
		$return = false;
		if ($id != null) {
			$tmp = $this->Traitement->find('first', array('fields' => array('id'), 'conditions' => array('target_id' => $id), 'contain' => false));
			if (!empty($tmp)) {
				$return = true;
			}
		}
		return $return;
	}

	/**
	 *
	 * @param type $id
	 * @return boolean
	 */
	public function isRefused($id = null) {
		$return = false;
		if ($id != null) {
			$courriers = $this->find('first', array('fields' => array('id'), 'conditions' => array('ref_ancien' => $id), 'contain' => false));
			if (!empty($courriers)) {
				$return = true;
			}
		}
		return $return;
	}

	/**
	 *
	 * @param type $id
	 * @return boolean
	 */
	public function isTreated($id = null) {
		$return = false;
		if ($id != null) {
			$traitement = $this->Traitement->find('first', array('fields' => array('treated'), 'conditions' => array('target_id' => $id), 'contain' => false));
			if ($traitement['Traitement']['treated']) {
				$return = true;
			}
		}
		return $return;
	}

	/**
	 *
	 * @param type $id
	 * @return boolean
	 */
	public function isDuplicated($id = null) {
		$return = false;
		if ($id != null) {
			$courrier = $this->read($id);
			if (!empty($courrier) && !empty($courrier['Courrier']['ref_ancien'])) {
				$return = true;
			}
		}
		return $return;
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function getDuplicated($id = null) {
		$return = null;
		if ($id != null) {
			if ($this->isRefused($id)) {
				$return = $this->find('first', array('conditions' => array('ref_ancien' => $id)));
			}
		}
		return $return;
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function getOriginal($id = null) {
		$return = null;
		if ($id != null) {
			$courrier = $this->read($id);
			$return = $this->find('first', array('conditions' => array('id' => $courrier['Courrier']['ref_ancien'])));
		}
		return $return;
	}
        /**
	 * check there is a 'etape' in a 'circuit' correspond a 'flux'
	 * @param type $fluxId
	 */
        public function isEtape($fluxId){
            $isetape = false;
            $querydata = array(
                'fields' => array(
                        'Soustype.circuit_id'
                ),
                'contain' => false,
                'joins' => array(
                        $this->join($this->Soustype->alias)
                ),
                'conditions' => array(
                        'Courrier.id' => $fluxId
                )
            );
            $flux = $this->find('first', $querydata);
            $circuit_id=$flux['Soustype']['circuit_id'];
            $etapes = $this->Soustype->Circuit->Etape->find('all',array(
                'fields' => array(
                    'Etape.id',
                    'Etape.type'
                ),
                'conditions' => array(
                    'Etape.circuit_id' => $circuit_id
                )
            ));
            foreach ($etapes as $etape) {
                if($etape['Etape']['type']!=1){
                    if(count($etape['Composition'])<2){
                        $isetape = false;
                    }else{
                        $isetape = true;
                    }
                }else{
                    if(count($etape['Composition'])<1){
                        $isetape = false;
                    }else{
                        $isetape = true;
                    }
                }
            }
            return $isetape;
        }

	/**
	 * Checks if the Flux is outaded or not
	 *
	 * @param integer $id The id of a Flux
	 * @return boolean if true the Flux is outdated
	 */
	public function isInTime($id = null) {
		$return = true;
		if ($id != null) {

			$this->id = $id;
			$courrier = $this->find(
					'first', array(
				'conditions' => array(
					'Courrier.id' => $id
				),
				'contain' => false
					)
			);
			if (!empty($courrier['Courrier']['soustype_id'])) {

				$soustype = $this->Soustype->find(
						'first', array(
					'conditions' => array(
						'Soustype.id' => $courrier['Courrier']['soustype_id']
					),
					'contain' => false
						)
				);

				if (!empty($soustype['Soustype']['delai_nb']) && !empty($soustype['Soustype']['delai_unite'])) {

					//recuperation de la date du jour
					$date_act = mktime();

					//calcul de la date limite
					$date_courrier_tmp = explode("-", $courrier['Courrier']['datereception']);
					$date_courrier = mktime(0, 0, 0, $date_courrier_tmp[1], $date_courrier_tmp[2], $date_courrier_tmp[0]);

					if ($soustype['Soustype']['delai_unite'] == 0) { // 0 : jour
						$thePHPDate = getdate($date_courrier);
						$thePHPDate['mday'] = $thePHPDate['mday'] + $soustype['Soustype']['delai_nb'];
						$date_valid = mktime($thePHPDate['hours'], $thePHPDate['minutes'], $thePHPDate['seconds'], $thePHPDate['mon'], $thePHPDate['mday'], $thePHPDate['year']); // Convert back to timestamp
					} else if ($soustype['Soustype']['delai_unite'] == 1) { // 1 : semaine
						$thePHPDate = getdate($date_courrier);
						$thePHPDate['mday'] = $thePHPDate['mday'] + ($soustype['Soustype']['delai_nb'] * 7);
						$date_valid = mktime($thePHPDate['hours'], $thePHPDate['minutes'], $thePHPDate['seconds'], $thePHPDate['mon'], $thePHPDate['mday'], $thePHPDate['year']); // Convert back to timestamp
					} else if ($soustype['Soustype']['delai_unite'] == 2) { // 2 : mois
						$thePHPDate = getdate($date_courrier);
						$thePHPDate['mon'] = $thePHPDate['mon'] + $soustype['Soustype']['delai_nb'];
						$date_valid = mktime($thePHPDate['hours'], $thePHPDate['minutes'], $thePHPDate['seconds'], $thePHPDate['mon'], $thePHPDate['mday'], $thePHPDate['year']); // Convert back to timestamp
					}

					//si la date est depassee
					if ($date_valid < $date_act) {
						$return = false;
					}
				}
			}
		}
		return $return;
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function isClosed($id = null) {
		$return = $this->Bancontenu->find('list', array('conditions' => array('Bancontenu.courrier_id' => $id, 'Bancontenu.etat' => 2), 'contain' => false));
		return count($return) > 0;
	}

	/**
	 *
	 * @param type $id
	 * @return string
	 */
	public function getDirection($id) {
		$this->id = $id;
		$directionDef = array(
			0 => "S",
			1 => "E",
			2 => "I"
		);
		$direction = $this->field('direction');
		$return = null;
		if (isset($direction) && in_array($direction, array_keys($directionDef))) {
			$return = $directionDef[$direction];
		}
		return $return;
//		$ref = $this->field('reference');
//		$return = '';
//		if (!empty($ref)) {
//			if (preg_match('#^S.*#', $ref)) {
//				$return = "S";
//			}
//			if (preg_match('#^I.*#', $ref)) {
//				$return = "I";
//			}
//			if (preg_match('#^E.*#', $ref)) {
//				$return = "E";
//			}
//		}
//		return $return;
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function hasAssociatedFile($id) {
		$associatedFiles = $this->Document->find(
				'first', array(
			'fields' => array('id'),
			'conditions' => array('Document.courrier_id' => $id),
			'recursive' => -1
				)
		);
		return !empty($associatedFiles);
	}

	/**
	 * Checks integrity of tree table before modification
	 *
	 * @param mixed $data
	 * @return boolean
	 */
	public function checkParadox($data) {
		if (isset($this->data[$this->alias]['id'])) {
			return $data['parent_id'] != $this->data[$this->alias]['id'];
		}
		return true;
	}

	/**
	 * Creates a response to Flux and store reference to original
	 *
	 * @param type $flux
	 * @param type $soustype_id
	 * @return type
	 */
	public function createResponse($flux, $soustype_id) {
		$newFlux = array(
			'Courrier' => array(
				'name' => 'Réponse à ' . $flux['Courrier']['name'],
				'contact_id' => isset($flux['Courrier']['contact_id']) ? $flux['Courrier']['contact_id'] : null,
				'user_creator_id' => $flux['Courrier']['user_creator_id'],
				'desktop_creator_id' => $flux['Courrier']['desktop_creator_id'],
				'soustype_id' => $soustype_id,
				'parent_id' => $flux['Courrier']['id'],
				'reference' => $this->Soustype->getRef($soustype_id)
			)
		);
		$this->create($newFlux);
		return $this->save();
	}

	/**
	 * Creates a copy of a Flux and store reference to refused one
	 *
	 * @param type $flux
	 * @return type
	 */
	public function createRefus($flux) {
// 		$success = true;
 

        $newFlux = $flux;
		$newFlux['Courrier']['id'] = null;
		$newFlux['Courrier']['ref_ancien'] = $flux['Courrier']['id'];
		$newFlux['Courrier']['desktop_creator_id'] = $flux['Courrier']['desktop_creator_id'];
		$newFlux['Courrier']['parapheur_etat'] = 1;
		$newFlux['Courrier']['parapheur_id'] = null;
// 		$newFlux['Document'] = $flux['Document'];

//        if(!empty($flux['Document']) && isset($flux['Document'])) {
//            $newFlux['Document'] = $flux['Document'];
//            $newFlux['Document']['courrier_id'] = $flux['Document'];
//        }
//        $this->begin();
		$this->create($newFlux);
        $data = $this->save();

        $success = !empty( $data );
        if( $success ) {
            //On récupère les documents liés au courrier nouvellement créé
            if( isset( $flux['Document'] ) && !empty($flux['Document']) ) {
                $newDocument['Document'] = $flux['Document'];

                foreach( $newDocument['Document'] as $i => $document ){
                    $document['id'] = null;
                    $document['courrier_id'] = $this->id;
                    $this->Document->create($document);
                    $success = $this->Document->save() && $success;
                }

            }
        }

//		return $this->save();
        return ( $success ? $data : false );
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	/*public function genRef($soustype_id) {
		$refInOut = array(
			0 => "S_",
			1 => "E_",
			2 => "I_"
		);
		$compteur = ClassRegistry::init('Compteur');
		$compteur->setDataSource($this->useDbConfig);
		$this->Soustype->setDataSource($this->useDbConfig);

		$stype = $this->Soustype->find('first', array('conditions' => array('Soustype.id' => $soustype_id)));
		return $refInOut[$stype['Soustype']['entrant']] . $compteur->genereCompteur($stype['Soustype']['compteur_id']);
	}*/

    /**
     * Génération auto de la référence unique du courrier
     * @param type $id
     * @return string|null
     */
     public function genReferenceUnique( $id =  null, $data=null ) {
        $referenceunique = null;
        $sql = "SELECT nextval('courriers_reference_seq'); /*".mktime(true)." "  . $this->_lastSequenceReference . "*/";

        if( !empty($id) ) {
            $courrier = $this->find(
                'first',
                array(
                    'fields' => array(
                        'Courrier.reference',
                        'Courrier.soustype_id'
                    ),
                    'conditions' => array(
                        'Courrier.id' => $id
                    ),
                    'contain' => array(
                        'Soustype'
                    )
                )
            );
            $referenceunique = $courrier['Courrier']['reference'];

            $soustypeId = $courrier['Courrier']['soustype_id'];
            if( !empty( $data['Courrier']['soustype_id'] ) ) {
                $soustypeId = $data['Courrier']['soustype_id'];
            }

           if( empty($referenceunique) && !empty( $soustypeId ) ) {
               $numSeq = $this->query( $sql );
               if( $numSeq === false ) {
                   return null;
               }
               $referenceunique = date( 'Y' ).sprintf( "%06s", $numSeq[0][0]['nextval'] );
           }
        }
        else {
			$numSeq = $this->query( $sql );
			if( $numSeq === false ) {
				return null; // FIXME: Throw exception
			}
			$this->_lastSequenceReference = $numSeq[0][0]['nextval'];
            $referenceunique = date( 'Y' ).sprintf( "%06s", $numSeq[0][0]['nextval'] );
        }
        return $referenceunique;
    }
    /**
	 *
	 * @param type $id
	 * @return type
	 */
    public function genRef2($data) {
        $id = $data['Courrier']['id'];
		$courrier = $this->find(
            'first',
            array(
                'conditions' => array(
                    'Courrier.id' => $id
                )
            )
        );
		return $this->genReferenceUnique($id, $data);
	}
	/**
	 *
	 * @param type $targetId
	 * @return type
	 */
	public function cakeflowIsEnd($targetId) {
		$before_num_traitement = $this->Traitement->field('numero_traitement', array('Traitement.target_id' => $targetId));
		$last_num_traitement_query = 'select max(V.numero_traitement) from wkf_traitements T, wkf_visas V where T.id = V.traitement_id and T.target_id = ' . $targetId . ';';
		$last_num_traitement = $this->Traitement->query($last_num_traitement_query);
		return ($before_num_traitement === $last_num_traitement[0][0]['max']);
	}

	/**
	 *
	 * @param type $fluxId
	 * @return type
	 */
	private function _cakeflowGetInfo($fluxId) {
		return array(
			'ref_traitement' => $this->Traitement->field('numero_traitement', array('Traitement.target_id' => $fluxId)),
			'desktops' => $this->Traitement->whoIsNext($fluxId),
			'typeEtape' => $this->Traitement->typeEtape($fluxId),
			'treated' => $this->Traitement->field('treated', array('Traitement.target_id' => $fluxId)),
		);
	}

	/**
	 *
	 * @param type $fluxId
	 * @param type $desktopId
	 * @return type
	 * @throws NotFoundException
	 * @throws InvalidArgumentException
	 */
	public function cakeflowInsert($fluxId, $desktopId, $insertAuto = 0) {
		$querydata = array(
			'fields' => array(
				'Courrier.id',
				'Courrier.desktop_creator_id',
				'Courrier.ref_ancien',
				'Courrier.parent_id',
				'Courrier.priorite',
                'Courrier.date',
                'Courrier.datereception',
				'Soustype.circuit_id'
			),
			'contain' => false,
			'joins' => array(
				$this->join($this->Soustype->alias)
			),
			'conditions' => array(
				'Courrier.id' => $fluxId
			)
		);
		$flux = $this->find('first', $querydata);
        
		if (empty($flux)) {
			throw new NotFoundException();
		}

		//mise a jour de champ desktop_creator_id (utilisé en cas de refus du flux dans un circuit)
		$flux['Courrier']['desktop_creator_id'] = $desktopId;
		$this->create();
		$this->save($flux);


		if ($this->Traitement->targetExists($fluxId)) {
			throw new InvalidArgumentException();
		}

		$valid = array(
			'bancontenuEnd' => false,
			'bancontenuAdd' => false,
			'Notification' => true
		);

		$circuitId = $flux['Soustype']['circuit_id'];

		if ($this->Traitement->Circuit->insertDansCircuit($circuitId, $fluxId)) {
			if ($insertAuto != 1) {
				$valid['bancontenuEnd'] = $this->Bancontenu->end($desktopId, BAN_DOCS, $fluxId);
			}
			$after = $this->_cakeflowGetInfo($fluxId);


            // Si le premier bureau est un parapheur, on exécute le traitement pour verser au parpaheur
            if($after['desktops'][0] == -1 ) {
                $traitement = $this->Traitement->find('first', array(
                    'recursive' => -1,
                    'fields' => array('id', 'numero_traitement', 'treated', 'circuit_id'),
                    'conditions' => array(
                        'target_id' => $fluxId,
                        'treated' => '0')));
                $this->Traitement->delegSiBesoin($fluxId, $traitement);
                
                // Ajout dans le bancontenu si parapheur = 1ere etape
                $validBancontenuAdd = array();
                $validBancontenuAdd[] = $this->Bancontenu->add($after['desktops'][0], BAN_FLUX, $fluxId);

                
//                if(!empty($flux['Courrier']['ref_ancien'])) {
                if(!empty($flux['Courrier']['parent_id'])) {
                    $valid['bancontenuEnd'] = $this->Bancontenu->end($desktopId, BAN_REFUS, $fluxId);
                }
                $valid['bancontenuAdd'] = !in_array(false, $validBancontenuAdd, true);
//                $valid['bancontenuAdd'] = true;
                $valid['Notification'] = true;
            }
            else {
                $notifyDesktops = array();
                $validBancontenuAdd = array();
                for ($i = 0; $i < count($after['desktops']); $i++) {
                    $validBancontenuAdd[] = $this->Bancontenu->add($after['desktops'][$i], BAN_FLUX, $fluxId);
                    $notifyDesktops[$after['desktops'][$i]] = array($fluxId);
                }
                // On termine le flux dans la bannette flux refusé à retraiter
//                if(!empty($flux['Courrier']['ref_ancien'])) {
                if(!empty($flux['Courrier']['parent_id']) ){
                    if( !empty($flux['Courrier']['motif_refus'])) {
                        $valid['bancontenuEnd'] = $this->Bancontenu->end($desktopId, BAN_REFUS, $fluxId);
                    }
                    else { //FIXME à vérifier
                        $valid['bancontenuEnd'] = $this->Bancontenu->end($desktopId, BAN_REPONSES, $fluxId);
                    }
                }
                
                $valid['bancontenuAdd'] = !in_array(false, $validBancontenuAdd, true);
                $notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_new');
                $valid['Notification'] = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId, $notifyDesktops);
            }

            // Envoi du mail au destinataire présent
            $destinataires = $this->Traitement->whoIsNext($fluxId);
            foreach ($destinataires as $destinataire_id) {
                $User = ClassRegistry::init( 'User' );
                $user = $this->User->find(
                    'first',
                    array(
                        'recursive' => -1,
                        'conditions' => array(
//                            'User.id' => 84
                            'User.desktop_id' => $destinataire_id
                        )
                    )
                );
                $user_id = $user['User']['id'];
                $this->User->notifier($fluxId, $user_id, 'insertion');
            }
        
        
		}
		return !in_array(false, $valid, true);
	}

	/**
	 *
	 * @param type $code
	 * @param type $desktopId
	 * @param type $fluxId
	 * @param type $options
	 * @return type
	 */
	public function cakeflowExecuteSend($code, $desktopId, $fluxId, $options) {
		return $this->cakeflowExecute($code, $desktopId, $fluxId, $options);
	}

	/**
	 *
	 * @param type $desktopId
	 * @param type $fluxId
	 * @param type $response
	 * @param type $stype_id
	 * @return type
	 */
	public function cakeflowExecuteNext($desktopId, $fluxId, $response = 0, $stype_id = null, $insertAuto = 0) {
		return $this->cakeflowExecute('OK', $desktopId, $fluxId, array(), $response, $stype_id, $insertAuto);
	}

	/**
	 *
	 * @param type $code
	 * @param type $desktopId
	 * @param type $fluxId
	 * @param type $options
	 * @param type $response
	 * @param type $stype_id
	 * @return type
	 * @throws BadMethodCallException
	 * @throws InvalidArgumentException
	 */
	public function cakeflowExecute($code, $desktopId, $fluxId, $options = array(), $response = 0, $stype_id = null, $insertAuto = 0) {
//		if (!in_array($code, array('IP', 'IL', 'OK', 'KO')) || $fluxId <= 0 || $desktopId <= 0) {
		if (!in_array($code, array('IP', 'IL', 'OK', 'KO')) || $fluxId <= 0 || $desktopId < -1) {
			throw new BadMethodCallException();
		}

		if (!$this->Traitement->targetExists($fluxId)) {
			throw new InvalidArgumentException();
		}

		$valid = array(
			'cakeflow' => false,
			'notification' => false,
			'bancontenu_treated' => false
		);

//		$expDesktopId = CakeSession::read('Auth.User.Courrier.Desktop.id');

        $fluxOrigine = $this->find(
            'first',
            array(
                'fields' => array(
                    'Courrier.id',
                    'Courrier.desktop_creator_id'
                ),
                'conditions' => array(
                    'Courrier.id' => $fluxId
                ),
                'contain' => false,
                'recursive' => -1
            )
        );
//         debug($fluxOrigine);
		// -- -- -- -- -- -- --
		// Traitement Cakeflow
		// -- -- -- -- -- -- --
		$before = $this->_cakeflowGetInfo($fluxId);
		$cakeflowReturn = $this->Traitement->execute($code, $desktopId, $fluxId, $options);
		$after = $this->_cakeflowGetInfo($fluxId);

//        $current = $this->Traitement->whoIsNext($fluxId);

        $expDesktopId = $before['desktops'][0];
//        $expDesktopId = $fluxOrigine['Courrier']['desktop_creator_id'];

		$remainingDesktops = array_intersect($before['desktops'], $after['desktops']);
		$diffDesktops = array_diff($before['desktops'], $after['desktops']);

		if ($before['typeEtape'] == ETAPEET) {
			$valid['cakeflow'] = ($before['ref_traitement'] == $after['ref_traitement']) && !empty($diffDesktops);
		} else {
			$valid['cakeflow'] = ($before['ref_traitement'] != $after['ref_traitement']);
		}


		if ($cakeflowReturn) {
			$valid['cakeflow'] = true;
		}

		// -- -- -- -- -- -- -- -- --
		// Notification et Bannettes
		// -- -- -- -- -- -- -- -- --
		//preparation
		$notifyDesktops = array();
		$notifInfos = array(
			'name' => '',
			'description' => '',
			'typeNotif' => ''
		);
		if ($code == 'KO') {
			$desktop = $this->field('Courrier.desktop_creator_id', array('Courrier.id' => $fluxId));
			$notifyDesktops[$desktop] = array($fluxId);
			$notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_cancel');
            // On envoie un mail pour le refus
//            $destinataires = $this->Traitement->whoIs($fluxId, 'in', array('OK','IN'));
//            foreach ($destinataires as $destinataire_id) {
//                $this->User->notifier($nouvelId, $destinataire_id, 'refus');
//            }
		} else if ($code == 'OK') {
			//--------
			//Etape ET
			//--------
			if ($before['typeEtape'] == ETAPEET) {
				for ($i = 0; $i < count($after['desktops']); $i++) {
					$notifyDesktops[$after['desktops'][$i]] = array($fluxId);
				}
				if (empty($remainingDesktops)) {
					$notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_new');
				} else {
					//si en attente
					$notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_wait');
				}
				//--------
				//Etape OU
				//--------
			} else if ($before['typeEtape'] == ETAPEOU) {
				for ($i = 0; $i < count($after['desktops']); $i++) {
					$notifyDesktops[$after['desktops'][$i]] = array($fluxId);
				}
				$notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_new');
				//------------
				//Etape Simple
				//------------
			} else if ($before['typeEtape'] == ETAPESIMPLE) {//Etape simple
				$notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_new'); //FIXME parpaheur ?
				for ($i = 0; $i < count($after['desktops']); $i++) {
					$notifyDesktops[$after['desktops'][$i]] = array($fluxId);
				}
			}
			//----------------
			//Rebond et Retour
			//----------------
		} else if ($code == 'IL' || $code == 'IP') {
			$notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_jmp');
			if ($code == 'IL') {
				$notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_jbk');
			}
			for ($i = 0; $i < count($after['desktops']); $i++) {
				$notifyDesktops[$after['desktops'][$i]] = array($fluxId);
			}
		} else {
			throw new BadMethodCallException();
		}

		// -- -- -- --
		// Traitement
		// -- -- -- --
		//si le flux est traité
		if ($after['treated'] && $code == 'OK') {
            $this->Bancontenu->fullend($fluxId); // TODO: problème lors de la finalisation suite à récupération parapheur

			//reponse
			if ($response != 0) {
				$flux = $this->find('first', array('conditions' => array('Courrier.id' => $fluxId)));
				$newFlux = $this->createResponse($flux, $stype_id);
				$valid['response_copy'] = false;
				if (!empty($newFlux)) {
					$valid['response_copy'] = true;
					if ($insertAuto == 0) {
						$valid['bancontenu_treated'] = $valid['bancontenu'] = $this->Bancontenu->add($newFlux['Courrier']['desktop_creator_id'], BAN_REPONSES, $newFlux['Courrier']['id']);
						$notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_response');
						$notifyDesktops = array($newFlux['Courrier']['desktop_creator_id'] => array($newFlux['Courrier']['id']));
						$valid['notification'] = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $expDesktopId, $notifyDesktops);
                        
                        // Flux à insérer
                        $user_id = $this->User->userIdByDesktop($newFlux['Courrier']['desktop_creator_id']);
                        $this->User->notifier($newFlux['Courrier']['id'], $user_id, 'insertion_reponse');
                        
					} else {
						$this->cakeflowInsert($newFlux['Courrier']['id'], $expDesktopId, $insertAuto);
                        // Flux insérer directement donc à traiter
                        $destinataires = $this->Traitement->whoIsNext($newFlux['Courrier']['id']);

                        foreach ($destinataires as $destinataire_id) {
                            $user_id = $this->User->userIdByDesktop($destinataire_id);
                            $this->User->notifier($newFlux['Courrier']['id'], $user_id, 'traitement');
                        }
                        
					}
				}
			}
            else {
                $valid['notification'] = true;
				$valid['bancontenu_treated'] = true;
                // Flux traité
            }
		} else {
			//Dans le cas d une etape de type ou, il faut marquer comme lues les notifications de tous les utilisateurs des bureaux de l'etape validée.
			//Il faut aussi changer l'etat de bancontenu de la meme maniere
			if ($before['typeEtape'] == ETAPEOU) {
				$validEtapeOuNotif = array();
				$validEtapeOuBan = array();
				foreach ($before['desktops'] as $prevDesktop) {
					$validEtapeOuNotif[] = $this->Notifieddesktop->Notification->setReadByDesktopCourrier($prevDesktop, $fluxId);
					$validEtapeOuBan[] = $this->Bancontenu->end($prevDesktop, BAN_FLUX, $fluxId);
				}
				$valid['readnotification'] = !in_array(false, $validEtapeOuNotif, true);
				$valid['bancontenu_treated'] = !in_array(false, $validEtapeOuBan, true);
			} else { //sinon dans tous les autres cas, on marque comme lue la notification de l'utilisateur ayant validé et on change l'état de bancontenu
				$valid['readnotification'] = $this->Notifieddesktop->Notification->setReadByDesktopCourrier($expDesktopId, $fluxId);
				$valid['bancontenu_treated'] = $this->Bancontenu->end($expDesktopId, BAN_FLUX, $fluxId);
			}
			//Notification pour les bureaux de l'etape en cours (nouvelle etape, ou etape actuelle dans le cas d une etape de type ET)
			if ($this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $expDesktopId, $notifyDesktops)) {
				$valid['notification'] = true;
			}
			//On met à jour les contenus de bannette pour les bureaux de l etape suivante
			//Si on est à une étape de type ET et que tous les bureaux ont validé ou si etape autre que ET
			if (($before['typeEtape'] == ETAPEET && $before['ref_traitement'] != $after['ref_traitement']) || $before['typeEtape'] != ETAPEET) {
				$validEtapeEtBan = array();
				foreach ($after['desktops'] as $nextDesktopId) {
					$validEtapeEtBan[] = $this->Bancontenu->add($nextDesktopId, BAN_FLUX, $fluxId);
				}
				$valid['bancontenu'] = !in_array(false, $validEtapeEtBan, true);
			}
            
            // Ajout pour les notifs durant le traitement
            $destinataires = $this->Traitement->whoIsNext($fluxId);
            foreach ($destinataires as $destinataire_id) {
                $user_id = $this->User->userIdByDesktop($destinataire_id);
                $this->User->notifier($fluxId, $user_id, 'traitement');
            }
		}

		return !in_array(false, $valid, true);
	}

	/**
	 *
	 * @param type $desktopId
	 * @param type $fluxId
	 * @param type $destId
	 * @return type
	 */
	public function copy($desktopId, $fluxId, $destId) {

		$notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_copy');
		$notifyDesktops = array(
			$destId => array($fluxId)
		);

		$notified = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId, $notifyDesktops);

		$valid = array(
			'notification' => !empty($notified),
			'bancontenu' => $this->Bancontenu->add($destId, BAN_COPY, $fluxId),
		);

		return !in_array(false, $valid, true);
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function getMainDoc($id) {
		$qdMainDoc = array(
			'recursive' => -1,
            'fields' => array(
                'Document.id',
                'Document.courrier_id',
                'Document.name',
                'Document.ext',
                'Document.content',
                'Document.gedpath',
                'Document.size',
                'Document.mime',
                'Document.main_doc',
                'Document.desktop_creator_id'
            ),
//			'fields' => $this->Document->getLightFields(),
			'conditions' => array(
				'Document.courrier_id' => $id,
				'Document.main_doc' => true
			),
            'contain' => false
		);
// var_dump( $this->Document->find('first', $qdMainDoc) );

		return $this->Document->find('first', $qdMainDoc);
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function getDocs($id) {
		$qdMainDoc = array(
			'recursive' => -1,
			'fields' => $this->Document->getLightFields(),
			'conditions' => array(
				'Document.courrier_id' => $id,
				'Document.main_doc' => false
			)
		);
		return $this->Document->find('all', $qdMainDoc);
	}

		/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function getDocsSignes($id) {
		$qdDocSigne = array(
			'recursive' => -1,
			'fields' => $this->Document->getLightFields(),
			'conditions' => array(
				'Document.courrier_id' => $id,
				'Document.main_doc' => true,
				'Document.parapheur_bordereau <>' => null
			)
		);
		return $this->Document->find('first', $qdDocSigne);
	}
	/**
	 * Génération d'un document lié (à partir du script lancé à la main)
	 *
	 * @access private
	 * @param base64 $fichier_associe contenu du document lié
	 * @param integer $fluxId identifiant du flux
	 * @param array $return valeur de retour
	 * @return void
	 */
	private function _genFileAssoc($fichier_associe, $fluxId, &$return) {
		$finfo = new finfo(FILEINFO_MIME_TYPE);
		$fileMimeType = $finfo->buffer($fichier_associe);

//		$filename = "fichier_" . date('Ymd_His'). "_" .mktime(true);
        
        $sql = "SELECT nextval('documents_name_seq'); /*".mktime(true)." "  . $this->_lastSequenceDocument . "*/";
        
        
        
		$fileMimeTypeError = false;
		if ($fileMimeType == "application/pdf") {
			$fileExt = "pdf";
		} else if ($fileMimeType == "application/vnd.oasis.opendocument.text") {
			$fileExt = "odt";
		} else if ($fileMimeType == "image/bmp") {
			$fileExt = "bmp";
		} else if ($fileMimeType == "image/tiff") {
			$fileExt = "tif";
		} else if ($fileMimeType == "text/plain") {
			$fileExt = "txt";
		} else if ($fileMimeType == "application/vnd.ms-office") {
			$fileExt = "doc";
		} else if ($fileMimeType == "application/msword") {
			$fileExt = "doc";
		} else if ($fileMimeType == "application/rtf") {
			$fileExt = "rtf";
		} else {
			$fileMimeTypeError = true;
		}


		if ($fileMimeTypeError) {
			$return['CodeRetour'] .= "FILENOK";
			$return['Message'] .= ". Le fichier n'est pas reconnu";
		} else {
            
            $numSeqDoc = $this->query( $sql );
			if( $numSeqDoc === false ) {
				return null; // FIXME: Throw exception
			}
			$this->_lastSequenceDocument = $numSeqDoc[0][0]['nextval'];
            $filename = "fichier_".sprintf( "%06s", $numSeqDoc[0][0]['nextval'] );
            $filename .= "." . $fileExt;
            
//debug($filename);
//debug($fichier_associe);
			$doc = array(
				'Document' => array(
					'name' => $filename,
					'size' => strlen($fichier_associe),
					'mime' => $fileMimeType,
					'content' => $fichier_associe, //TODO: prevoir l utilisation d une GED
					'ext' => $fileExt,
					'courrier_id' => $fluxId,
					'main_doc' => true
				)
			);
			$this->setDataSource(Configure::read('conn'));
			$this->Document->setDataSource(Configure::read('conn'));
			$this->Document->create($doc);
			$docsaved = $this->Document->save();
//debug( $docsaved );
			if (empty($docsaved)) {
				$return['CodeRetour'] .= "NOFILE";
				$return['Message'] .= ".Le fichier n'a pas été envoyé";
			} else {
				$return['CodeRetour'] .= "FILEOK";
				$return['Message'] .= ". Le fichier a été envoyé";
			}
		}
	}

	/**
	 * Fonctionnalité de numérisation de masse (création de flux)
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @param integer $collectiviteId identifiant de la collectivité
	 * @param base64 $fichier_associe contenu du fichier lié
	 * @param string $username identifiant de connexion de l'utilisateur cible
	 * @return array
	 */
	public function remoteCreate($fichier_associe, $username, $dispDesktop = true, $desktopId = null, $filename) {
		$return = array(
			'CodeRetour' => 'KO',
			'CourrierId' => -1,
			'Message' => 'Impossible de créer le flux.',
			'Severite' => 'important'
		);

		$noRoleErrMsg = "Le rôle n'existe pas.";

		if (empty($desktopId)) {
			$GID = $dispDesktop ? DISP_GID : INIT_GID;
			$BAN = $dispDesktop ? BAN_AIGUILLAGE : BAN_DOCS;
			$noRoleErrMsg = $dispDesktop ? "L'utilisateur n'a pas de rôle Aiguilleur." : "L'utilisateur n'a pas de rôle Initiateur.";

			$querydata = array(
				'conditions' => array(
					'User.username' => $username
				),
				'contain' => array(
					$this->Bancontenu->Desktop->alias => array(
						$this->Bancontenu->Desktop->Group->alias
					),
					$this->Bancontenu->SecondaryDesktop->alias => array(
						$this->Bancontenu->SecondaryDesktop->Group->alias
					),
				)
			);
			$user = $this->User->find('first', $querydata);

			if (empty($user)) {
				$return['CodeRetour'] = "KONOUSER";
				$return['Message'] .= " L'utilisateur n'existe pas.";
			} else {
				if ($user['Desktop']['group_id'] == $GID) {
					$desktopId = $user['Desktop']['id'];
				} else {
					$found = false;
					for ($i = 0; $i < count($user['SecondaryDesktop']); $i++) {
						if ($user['SecondaryDesktop'][$i]['group_id'] == $GID && !$found) {
							$desktopId = $user['SecondaryDesktop'][$i]['id'];
							$found = true;
						}
					}
				}
			}
		} else {
			$desktop = $this->Bancontenu->Desktop->find('first', array('conditions' => array('Desktop.id' => $desktopId)));

			if ($desktop['Desktop']['group_id'] == DISP_GID) {
				$BAN = BAN_AIGUILLAGE;
			} else if ($desktop['Desktop']['group_id'] == INIT_GID) {
				$BAN = BAN_DOCS;
			}
		}


		if (empty($desktopId)) {
			$return['CodeRetour'] = "KONODESKTOP";
			$return['Message'] .= $noRoleErrMsg;
		} else if (empty($BAN)) {
			$return['CodeRetour'] = "KONOBAN";
			$return['Message'] .= 'Aucun bannette renseignée';
		} else {
            $pos_point = strpos($filename, '.');
            $courrierName = substr($filename, 0, $pos_point);
            
            $referenceUnique = $this->genReferenceUnique();
            

			$data = array(
				'Courrier' => array(
					'name' => $courrierName.'_'.date('Y-m-d_H:i:s'),
                    'reference' => $referenceUnique
				)
			);
            $this->begin();
			$this->create($data);
			if ($this->save()) {
				$courrierId = $this->id;
				$return['CodeRetour'] = "CREATED";
				$return['Message'] = "Le flux a été créé";
				$return['Severite'] = "normal";
				$return['CourrierId'] = $courrierId;
                
                $this->commit();
                
				//ajout dans la bannette de l aiguilleur
				$this->Bancontenu->add($desktopId, $BAN, $courrierId);
//TODO: permettre la notification d un lot complet et non la notification de chaque flux comme ci dessous
//          $name = __d('notification', 'ban_wsnew.name');
//          $description = __d('notification', 'ban_wsnew.description');
//          $type = 'ban_wsnew';
//          $desktops = array(
//            $desktopId => array(
//                $courrierId
//            )
//          );
//          $this->Courrier->Notifieddesktop->Notification->add($name, $description, $type, $desktops);
                $this->_genFileAssoc($fichier_associe, $courrierId, $return);

                // Notification par mail de la génération du flux
                $user_id = $this->User->userIdByDesktop($desktopId);
                $this->User->notifier($courrierId, $user_id, 'aiguillage');
			}
            else {
                $this->rollback();
            }
		}


		return $return;
	}
    
     /**
     * Construit le tableau à envoyer au parapheur
     * @param $acte_id
     * @return array
     */
    public function getDocumentsForDelegation($courrier_id){
        $docs = array(
            'docPrincipale' => $this->getMainDoc($courrier_id),
            'annexes' => array()
        );

		foreach ($this->Document->getPiecesJointes($courrier_id) as $piecejointe) {
            $docs['annexes'][] = array(
                'content' => $piecejointe['Document']['content'],
                'mimetype' => $piecejointe['Document']['mime'],
                'filename' => $piecejointe['Document']['name']
            );
        }
// debug($docs);
        return $docs;
    }
  
    
    public function majTraitementsParapheur($courrier_id){

        $before = $this->Traitement->whoIsNext($courrier_id, 'current');

        $ret = $this->Traitement->majTraitementsParapheur($courrier_id);

        $msg = 'Traitement en cours Parapheur';
        if( $ret == 'TRAITEMENT_TERMINE_OK'){
            $parent = 0;
            $type_id = null;
            $insertAuto = 0;
            $desktop_id = @$before[0];
            if( !empty($desktop_id)) {
                if ($this->cakeflowExecuteNext($desktop_id, $courrier_id, $parent, $type_id, $insertAuto )) {
                    $msg = 'Flux récupéré et transféré à l\'étape suivante';
                } else {
                    $msg = 'Echec de la transmission à l\'étape suivante';
                }
            }
            else {
                $msg = 'Aucun bureau destinataire défini';
            }
        }
        return $msg;

    }
    
    public function createRefusParapheur( $fluxRefuse = array(), $desktopId ) {       
        $this->Traitement->execute('KO', $desktopId, $fluxRefuse['Courrier']['id']);
        $msg = array();
        
// debug($fluxRefuse);
        //duplication du courrier
        $newFlux = $this->createRefus($fluxRefuse);

        if (!empty($newFlux)) {
        
			//On récupère les documents liés au courrier nouvellement créé
			if( isset( $newFlux['Document'] ) && !empty($newFlux['Document']) ) {
				$this->Document->begin();
				$newDocument['Document'] = $newFlux['Document'];

				foreach( $newDocument['Document'] as $i => $document ){
					$document['id'] = null;
					$document['courrier_id'] = $newFlux['Courrier']['id'];
					$this->Document->create($document);
					$success = $this->Document->save();

					if($success) {
						$this->Document->commit();
						debug( 'OK les documents joints ont bien été créés' );
					}
					else {
						$this->Document->rollback();
						debug( 'KO échec à la création des documents joints' );
					}

				}

			}
            //on change l etat du flux dans toutes les bannettes
            $this->Bancontenu->cancel($fluxRefuse['Courrier']['id']);
            //on ajoute le flux dupliqué dans la bannette du rédacteur
            if ($this->Bancontenu->add($newFlux['Courrier']['desktop_creator_id'], BAN_REFUS, $newFlux['Courrier']['id'])) {
                //notification au rédacteur
                $notifyDesktops = array($fluxRefuse['Courrier']['desktop_creator_id'] => array($newFlux['Courrier']['id']));
                if ($this->Notifieddesktop->Notification->add(__d('notification', 'ban_cancel.name'), __d('notification', 'ban_cancel.description'), BAN_REFUS, $fluxRefuse['Courrier']['desktop_creator_id'], $notifyDesktops)) {
                    $msg['message'] = 'Le flux "' . $newFlux['Courrier']['name'] . '" a été refusé"';
                    $msg['success'] = true;
                } else {
                    $msg['message'] .= '<br />Notification: error';
                }
            } else {
                $msg['message'] .= '<br />Bannette add: error';
            }
        } else {
            $msg['message'] .= '<br />Courrier duplicate: error';
        }
        
        return $msg;

    }
    
    /**
    *	Mise à jour du document avec la récupératin de la signature et du bordereau
    */
    public function majDocument($id, $dossier) {

		$document = $this->Document->find(
			'first',
			array(
				'fields' => array(
					'Document.id',
					'Document.courrier_id',
					'Document.main_doc',
					'Document.size'
				),
				'conditions' => array(
					'Document.courrier_id' => $id,
					'Document.main_doc' => true
				),
				'contain' => false,
				'recursive' => -1
			)
		);

		$this->Document->id= $document['Document']['id'];
		$this->Document->saveField('parapheur_bordereau', base64_decode($dossier['getdossier']['bordereau']));
		$this->Document->saveField('signature', base64_decode($dossier['getdossier']['signature']));
		$this->Document->saveField('signee', true);
        
        $success = true;
        // Affichage du bordereau
        $documentBordereau = array(
			'Document' => array(
				'courrier_id' => $id,
				'size' => strlen(base64_decode($dossier['getdossier']['bordereau'])),
                'main_doc' => false,
				'content' => base64_decode($dossier['getdossier']['bordereau']),
                'name' => 'Bordereau.pdf',
                'ext' => 'pdf',
                'mime' => 'application/pdf'
                
			)
		);
        $this->Document->create($documentBordereau);
        $success = $this->Document->save() && $success;
        
        // Affichage de la signature
        $documentSignature = array(
			'Document' => array(
				'courrier_id' => $id,
				'size' => strlen(base64_decode($dossier['getdossier']['signature'])),
                'main_doc' => false,
				'content' => base64_decode($dossier['getdossier']['signature']),
                'name' => 'Signature_Recuperee.zip',
                'ext' => 'zip',
//                'mime' => 'application/pdf'
                
			)
		);
        $this->Document->create($documentSignature);
        $success = $this->Document->save() && $success;
		
    }
    
    
    
    /**
     * Envoi par mail d'un document lié à un flux 
     *
     * @param integer $courrier_id identifiant du flux
     * @param integer $user_id identifiant de l'utilisateur à notifier
     * @param string $type notification à envoyer
     * @return bool succès de l'envoi
     */
    public function docParMail($mailDest, $doc) {

        App::uses('CakeEmail', 'Network/Email');
        $config_mail = Configure::read('SMTP_USE') ? 'docparmail' : 'default';
        $this->Email = new CakeEmail($config_mail);
        $this->Email->to($mailDest);

        App::uses('Courrier', 'Model');
        $this->Courrier = new Courrier();
        $flux = $this->Courrier->find(
            'first',
            array(
                'recursive' => -1,
                'conditions' => array('Courrier.id' => $doc['Document']['courrier_id']),
                'fields' => array(
                    'Courrier.id',
                    'Courrier.objet',
                    'Courrier.reference',
                    'Courrier.parent_id',
                    'Courrier.name',
                    'Courrier.motifrefus',
                    'Soustype.circuit_id'
                ),
                'joins' => array(
                    $this->Courrier->join('Soustype')
                )
        ));
        $subject = "Un nouveau document vous a été envoyé depuis web-GFC";
        $type = "docparmail";
        

        // création du répertoire local
        $folder = WWW_ROOT . 'files/attachments/'.$doc['Document']['id'].'/';
        if(!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        // création du fichier
        $filename = $doc['Document']['name'];
        $fileToSend = fopen($folder.$filename, "x+");

        $filecontent = $doc['Document']['content'];
        // écriture
        fputs($fileToSend, $filecontent );
        // fermeture
        fclose($fileToSend);
        
        $this->Email->attachments( array(
            $doc['Document']['name'] => array(
                'file' => $folder.$doc['Document']['name']
                )
            )
        );
        
        
        $this->Email->subject($subject);
        $content = $this->_paramMails($type, $flux);
        $success = $this->Email->send($content);
        
        if( $success ) {
            unlink($folder.$doc['Document']['name']);
        }

        return true;
    }

    /**
     * Détermine le contenu du mail à envoyer en fonction du type de mail, le flux et l'utilisateur
     * @param string $type
     * @param array $flux
     * @param array $user
     * @return string
     */
    function _paramMails($type, $flux) {
        $file = new File(APP . "/Config/emails/$type.txt", false);
        $content = $file->read();
        $file->close();
        $addrTraiter = Configure::read('WEBGFC_URL') . '/courriers/historiqueGlobal/' . $flux['Courrier']['id'];

        $searchReplace = array(
            "#IDENTIFIANT_FLUX#" => $flux['Courrier']['id'],
            "#OBJET_FLUX#" => $flux['Courrier']['name'],
            "#LIBELLE_CIRCUIT#" => $this->Courrier->Traitement->Circuit->getLibelle($flux['Soustype']['circuit_id']),
            "#ADRESSE_A_TRAITER#" => $addrTraiter,            "#FLUX_ORIGINE#" => isset($flux['Courrier']['parent_id']) ? $flux['Courrier']['parent_id'] : null,
            "#REFERENCE_FLUX#" => isset($flux['Courrier']['reference']) ? $flux['Courrier']['reference'] : null,
        );

        return str_replace(array_keys($searchReplace), array_values($searchReplace), $content);
    }
    
    
    /**
	 * Fonctionnalité de numérisation de masse (création de flux)
	 *
	 * @logical-group WebService
	 *
	 * @access public
	 * @param integer $collectiviteId identifiant de la collectivité
	 * @param base64 $fichier_associe contenu du fichier lié
	 * @param string $username identifiant de connexion de l'utilisateur cible
	 * @return array
	 */
	public function remoteCreateMail($desktopId, $filename, $collId) {
		$return = array(
			'CodeRetour' => 'KO',
			'CourrierId' => -1,
			'Message' => 'Impossible de créer le flux.',
			'Severite' => 'important'
		);
        

		$noRoleErrMsg = "Le rôle n'existe pas.";

		if (empty($desktopId)) {
			$GID = $dispDesktop ? DISP_GID : INIT_GID;
			$BAN = $dispDesktop ? BAN_AIGUILLAGE : BAN_DOCS;
			$noRoleErrMsg = $dispDesktop ? "L'utilisateur n'a pas de rôle Aiguilleur." : "L'utilisateur n'a pas de rôle Initiateur.";

			$querydata = array(
				'conditions' => array(
					'User.username' => $username
				),
				'contain' => array(
					$this->Bancontenu->Desktop->alias => array(
						$this->Bancontenu->Desktop->Group->alias
					),
					$this->Bancontenu->SecondaryDesktop->alias => array(
						$this->Bancontenu->SecondaryDesktop->Group->alias
					),
				)
			);
			$user = $this->User->find('first', $querydata);

			if (empty($user)) {
				$return['CodeRetour'] = "KONOUSER";
				$return['Message'] .= " L'utilisateur n'existe pas.";
			} else {
				if ($user['Desktop']['group_id'] == $GID) {
					$desktopId = $user['Desktop']['id'];
				} else {
					$found = false;
					for ($i = 0; $i < count($user['SecondaryDesktop']); $i++) {
						if ($user['SecondaryDesktop'][$i]['group_id'] == $GID && !$found) {
							$desktopId = $user['SecondaryDesktop'][$i]['id'];
							$found = true;
						}
					}
				}
			}
		} else {
			$desktop = $this->Bancontenu->Desktop->find('first', array('conditions' => array('Desktop.id' => $desktopId)));

			if ($desktop['Desktop']['group_id'] == DISP_GID) {
				$BAN = BAN_AIGUILLAGE;
			} else if ($desktop['Desktop']['group_id'] == INIT_GID) {
				$BAN = BAN_DOCS;
			}
		}


		if (empty($desktopId)) {
			$return['CodeRetour'] = "KONODESKTOP";
			$return['Message'] .= $noRoleErrMsg;
		} else if (empty($BAN)) {
			$return['CodeRetour'] = "KONOBAN";
			$return['Message'] .= 'Aucun bannette renseignée';
		} else {
            
            $referenceUnique = $this->genReferenceUnique();

			$data = array(
				'Courrier' => array(
					'name' => '[MAIL]'.$filename->subject,
					'objet' => !empty($filename->body) ? utf8_encode( $filename->body ) : null,
                    'reference' => $referenceUnique
				)
			);
            $this->begin();
            $this->create($data);

            if ($this->save()) {
                $courrierId = $this->id;
                $return['CodeRetour'] = "CREATED";
                $return['Message'] = "Le flux a été créé";
                $return['Severite'] = "normal";
                $return['CourrierId'] = $courrierId;
//debug($return);
                $this->commit();

                //ajout dans la bannette de l aiguilleur
                $this->Bancontenu->add($desktopId, $BAN, $courrierId);
                
//                $fileToAdd = !empty( $filename->pj ) ? $filename->pj : null;
                $this->_genFileAssoc($filename->pj, $courrierId, $return);
                // Notification par mail de la génération du flux
                $user_id = $this->User->userIdByDesktop($desktopId);
                $this->User->notifier($courrierId, $user_id, 'aiguillage');
//debug($return);
//die();

            }
            else {
                $this->rollback();
            }
		}
//debug($return);

		return $return;
	}
}

?>