<?php

App::uses('AppModel', 'Model');

/**
 * Document model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Document extends AppModel {

	/**
	 * Display field
	 *
	 * @var string
	 */
	public $displayField = 'name';

    protected $_lastSequenceDocument = null;
	/**
	 *
	 * @var type
	 */
	public $actsAs = array('Linkeddocs');

	/**
	 *
	 * @var type
	 */
	public $convertibleMimeTypes = array(
//		"application/vnd.oasis.opendocument.text",
//		"application/vnd.oasis.opendocument.spreadsheet",
		"image/bmp",
		"image/tiff",
		"text/plain",
		"application/msword",
		"application/x-octetstream",
		"application/msexcel",
		"application/xls",
		"application/vnd.ms-office",
		"application/vnd.ms-excel",
		"application/rtf",
		"application/binary"
	);

	/**
	 *
	 * @var type
	 */
	public $otherMimeTypes = array(
		'audio' => array(
			'audio/x-wav' => 'wav',
			'audio/mpeg' => 'mp3',
			'application/ogg' => 'ogg',
			'audio/ogg' => 'ogg'
		),
		'video' => array(
			'video/x-msvideo' => 'avi',
			'video/mpeg' => 'mpg',
			'video/mp4' => 'mp4',
			'application/ogg' => 'ogg',
			'video/ogg' => 'ogg'
		),
		'image' => array(
			'image/gif' => 'gif',
			'image/jpeg' => 'jpg',
			'image/png' => 'png'
		),
        'file' => array(
    		'application/vnd.oasis.opendocument.text' => 'odt',
    		'application/vnd.oasis.opendocument.spreadsheet' => 'ods'
        )
	);

	/**
	 * Validation rules
	 *
	 * @var array
	 */
	public $validate = array(
		'name' => array(
			'notempty' => array(
				'rule' => array('notempty'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'size' => array(
			'numeric' => array(
				'rule' => array('numeric'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'courrier_id' => array(
			'numeric' => array(
				'rule' => array('numeric'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

	/**
	 * belongsTo associations
	 *
	 * @var array
	 */
	public $belongsTo = array(
		'Courrier' => array(
			'className' => 'Courrier',
			'foreignKey' => 'courrier_id',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		)
	);

	/**
	 *
	 * @param type $id
	 * @return type
	 * @throws NotFoundException
	 */
	public function generatePreview($id = null) {
		if (!empty($id)) {
			$this->id = $id;
		}

		if (empty($this->id)) {
			throw new NotFoundException();
		}

		$pdfContent = '';
		$fileContent = '';
		$this->recursive = -1;
		$document = $this->read();
		$return = __d('document', 'Document.preview.error');

		//Conversion en pdf via Gedooo
		if (in_array($document['Document']['mime'], $this->convertibleMimeTypes)) {
			require_once 'XML/RPC.php';
			$content = base64_encode($document['Document']['content']);

			$params = array(new XML_RPC_Value($content, 'string'),
				new XML_RPC_Value($document['Document']['ext'], 'string'),
				new XML_RPC_Value('pdf', 'string'),
				new XML_RPC_Value(false, 'boolean'),
				new XML_RPC_Value(true, 'boolean'));

			$url = Configure::read('CLOUDOOO_HOST') . ":" . Configure::read('CLOUDOOO_PORT');

			$msg = new XML_RPC_Message('convertFile', $params);
			$cli = new XML_RPC_Client('/', $url);
			$resp = $cli->send($msg);
			$pdfContent = base64_decode($resp->xv->me['string']);
		} else if ($document['Document']['mime'] == 'application/pdf') {
			$pdfContent = $this->field('Document.content', array('Document.id' => $id));
		} else {
			foreach ($this->otherMimeTypes as $type => $mimes) {
				if (in_array($document['Document']['mime'], array_keys($mimes))) {
					$return = $this->field('Document.content', array('Document.id' => $id));
				}
			}
		}

		//Conversion en flash
		if (!empty($pdfContent)) {
			$date_suffix = date('Ymd_His');
			$tmp_pdf_to_convert = TMP . "pdf_to_convert_" . $date_suffix;
			$tmp_pdf_converted = TMP . "pdf_converted_" . $date_suffix;

			file_put_contents($tmp_pdf_to_convert, $pdfContent);
			$cmd = "pdf2swf -i " . $tmp_pdf_to_convert . " -o " . $tmp_pdf_converted . " -f -T 9 -t -s storeallcharacters";
			$sys_result = exec($cmd, $output, $return_var);


			if ($return_var == 0) {
				$return = file_get_contents($tmp_pdf_converted);

				$document['Document']['preview'] = $return;
				$this->create();
				$this->save($document);
			}

			unlink($tmp_pdf_converted);
			unlink($tmp_pdf_to_convert);
		}

		return $return;
	}
	
	
	/**
	* Récupère toutes les pièces jointes (associées à 1 flux) ayant la propriété main_doc à false
	* @param null|int|array $flux_id
	* @return mixed tableau d'annexes
	*/
	public function getPiecesJointes($flux_id = null){
		$conditions = array('main_doc' => false);

		if (!empty($flux_id)) $conditions['courrier_id'] = $flux_id;

		$piecesjointes = $this->find('all', array(
			'conditions' => $conditions,
			'order' => array('id' => 'ASC'),
			'recursive' => -1
		));

		return $piecesjointes;
	}
    
    
    /**
     * Génération auto de la référence unique du courrier
     * @param type $id
     * @return string|null
     */
     public function genNameUnique( $id =  null, $data=null ) {
        $filename = null;
        $sql = "SELECT nextval('documents_name_seq'); /*".mktime(true)." "  . $this->_lastSequenceDocument . "*/";

        
        $numSeqDoc = $this->query( $sql );
        if( $numSeqDoc === false ) {
            return null; // FIXME: Throw exception
        }
        $this->_lastSequenceDocument = $numSeqDoc[0][0]['nextval'];
        $filename = "fichier_".sprintf( "%06s", $numSeqDoc[0][0]['nextval'] );  
        return $filename;
    }


}
