<?php

App::uses('AppModel', 'Model');

/**
 * Service model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Service extends AppModel {

	/**
	 * Model name
	 *
	 * @access public
	 */
	public $name = 'Service';

	/**
	 *
	 * @var type
	 */
//  public $actsAs = array('Tree');
	public $actsAs = array('DataAcl.DataAcl' => array('type' => 'requester'), 'Tree');

	/**
	 * Validation rules
	 *
	 * @access public
	 */
	public $validate = array(
		'name' => array(
			array(
				'rule' => array('notempty'),
				'allowEmpty' => false,
			),
		),
		'parent_id' => array(
			'rule' => 'checkParadox',
			'on' => 'update',
			'message' => "Un service ne peut pas dépendre de lui même."
		)
	);

	/**
	 * hasAndBelongsToMany
	 *
	 * @access public
	 */
	public $hasAndBelongsToMany = array(
		'Init' => array(
			'className' => 'Desktop',
			'joinTable' => 'inits_services',
			'foreignKey' => 'service_id',
			'associationForeignKey' => 'desktop_id',
			'unique' => true,
			'conditions' => '',
			'fields' => '',
			'order' => '',
			'limit' => '',
			'offset' => '',
			'finderQuery' => '',
			'deleteQuery' => '',
			'insertQuery' => '',
			'with' => 'InitsService'
		),
		'Desktop' => array(
			'className' => 'Desktop',
			'joinTable' => 'desktops_services',
			'foreignKey' => 'service_id',
			'associationForeignKey' => 'desktop_id',
			'unique' => true,
			'conditions' => '',
			'fields' => '',
			'order' => '',
			'limit' => '',
			'offset' => '',
			'finderQuery' => '',
			'deleteQuery' => '',
			'insertQuery' => '',
			'with' => 'DesktopsService'
		)
	);

    
    public $hasMany = array(
        'Courrier' => array(
			'className' => 'Courrier',
			'foreignKey' => 'service_creator_id',
			'dependent' => false,
			'conditions' => '',
			'fields' => '',
			'order' => '',
			'limit' => '',
			'offset' => '',
			'exclusive' => '',
			'finderQuery' => '',
			'counterQuery' => ''
		),
    );
	/**
	 *
	 * @var type
	 */
//  private $_defaultSaveMsg = array(
//      'state' => 'add',
//      'serviceAdded' => false,
//      'serviceDaroAdded' => false,
//      'serviceDaroRights' => false,
//      'debug' => ''
//  );

	/**
	 *
	 * @var type
	 */
//  public $saveMsg = array();

	/**
	 *
	 */
//  private function _initSaveMsg() {
//    $this->saveMsg = $this->_defaultSaveMsg;
//  }

	/**
	 *
	 * @return boolean
	 */
//  public function beforeSave() {
//    $this->_initSaveMsg();
//    return true;
//  }

	/**
	 *
	 * @param type $created
	 */
//  public function afterSave($created) {
//    parent::afterSave($created);
//    $this->saveMsg['serviceAdded'] = $created;
//    //initilisation du modele Daro
//    $this->Daro = ClassRegistry::init('Daro');
//    //si le service n est pas enregistré dans la table daros
//    $serviceDaro = $this->Daro->find('first', array('conditions' => array('model' => 'Service', 'foreign_key' => $this->id)));
//    if (empty($serviceDaro)) {
//      //prepration de l aro
//      $daro = array(
//          'Daro' => array(
//              'model' => 'Service',
//              'foreign_key' => $this->id,
//              'alias' => $this->field('name')
//          )
//      );
//      $daro['Daro']['parent_id'] = 0;
//      $daroParent = $this->Daro->find('first', array('conditions' => array('Daro.model' => 'Service', 'Daro.foreign_key' => $this->field('parent_id'))));
//      if (!empty($daroParent)) {
//        $daro['Daro']['parent_id'] = $daroParent['Daro']['id'];
//      }
//      //sauvegarde
//      if ($this->Daro->save($daro)) {
//        $this->saveMsg['serviceDaroAdded'] = true;
//        //preparation des droits
//        $rights = array(
//            'DarosDaco' => array(
//                'daro_id' => $this->Daro->id,
//                'daco_id' => 1,
//                '_create' => true,
//                '_read' => true,
//                '_update' => true,
//                '_delete' => true
//            )
//        );
//        $this->Daro->DarosDaco->create($rights);
//        //sauvegarde des droits
//        if ($this->Daro->DarosDaco->save()) {
//          $this->saveMsg['serviceDaroRights'] = true;
//        }
//      }
//    }
//  }

	/**
	 *
	 * @param type $data
	 * @return boolean
	 */
	public function checkParadox($data) {
		if (isset($this->data[$this->alias]['id'])) {
			return $data['parent_id'] != $this->data[$this->alias]['id'];
		}
		return true;
	}

	/**
	 * parentNode override
	 *
	 * @return null
	 */
	function parentNode() {
		if (!$this->id && empty($this->data)) {
			return null;
		}
		if (isset($this->data['Service']['parent_id'])) {
			$serviceId = $this->data['Service']['parent_id'];
		} else {
			$serviceId = $this->field('parent_id');
		}
		if (!$serviceId) {
			return null;
		} else {
			return array('Service' => array('id' => $serviceId));
		}
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function getDesktops($id = null) {
		$return = array();
		$services = $this->find('first', array('conditions' => array('Service.id' => $id)));
		foreach ($services['Desktop'] as $desktop) {
			if (!in_array($desktop['id'], $return)) {
				$return[] = $desktop['id'];
			}
		}
		return $return;
	}

	/**
	 *
	 * @param type $id
	 * @return type
	 */
	public function getUsers($id = null) {
		$users = array();
		$desktops = $this->getDesktops($id);
		foreach ($desktops as $desktop) {
			$desktopUsers = $this->Desktop->getUsers($desktop);
			foreach ($desktopUsers as $user) {
				if (!in_array($user, $users)) {
					$users[] = $user;
				}
			}
		}
		return $users;
	}

}

?>