<?php

/**
 * SimpleComment Plugin
 * SimpleComment SCComment helper class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 *
 * @package             SimpleComment
 * @subpackage          SimpleComment.View.Helper
 */
class SCCommentHelper extends Helper {

    /**
     *
     * @var type
     */
    public $helpers = array('Html');

    /**
     *
     * @param type $comment
     * @param type $author
     * @return type
     * @throws BadMethodCallException
     */
    protected function _drawComment($comment, $editParams = array()) {
        if (empty($comment)) {
            throw new BadMethodCallException();
        }

        $content = '';
        if (!empty($comment['editable'])) {

//			$publicImg = $this->Html->image('/img/comment_public_16.png', array('title' => __d('comment', 'Comment.publicComment'), 'alt' => __d('comment', 'Comment.publicComment')));
            $privateImg = $this->Html->image('/img/comment_private_16.png', array('title' => __d('comment', 'Comment.privateComment'), 'alt' => __d('comment', 'Comment.privateComment')));

            $title = __d('comment', 'Comment.' . (!empty($comment['private']) ? 'privateComment' : 'publicComment'));

            $editImg = $this->Html->image('/img/edit_16.png', array('class' => 'sc-edit-btn', 'title' => __d('default', 'Button.edit'), 'alt' => __d('default', 'Button.edit'), 'idComment' => $comment['id']));
            $deleteImg = $this->Html->image('/img/delete_16.png', array('class' => 'sc-delete-btn', 'title' => __d('default', 'Button.delete'), 'alt' => __d('default', 'Button.delete'), 'idComment' => $comment['id']));

            $content .= $this->Html->tag('div', $deleteImg . $editImg, array('class' => array('sc-header')));
        }

        $trContent = $this->Html->tag('td', $comment['comment'], array('class' => 'sc-content'));

        $dlContent = $this->Html->tag('dt', __d('comment', 'Comment.date'), array('class' => 'ui-state-focus')) . $this->Html->tag('dd', $comment['date'], array('class' => 'sc-date'));
        $dlContent .= $this->Html->tag('dt', __d('comment', 'Comment.author'), array('class' => 'ui-state-focus')) . $this->Html->tag('dd', $comment['author'], array('class' => 'sc-author'));

        if (!empty($comment['readers'])) {
            $ulContent = '';
            foreach ($comment['readers'] as $reader) {
                $ulContent .= $this->Html->tag('li', $reader['name']);
            }
            $dlContent .= $this->Html->tag('dt', __d('comment', 'Comment.readers'), array('class' => 'ui-state-focus')) . $this->Html->tag('dd', $this->Html->tag('ul', $ulContent), array('class' => 'sc-author'));
        }

        $trContent .= $this->Html->tag('td', $this->Html->tag('dl', $dlContent), array('class' => 'sc-infos'));


        $tableContent = $this->Html->tag('tr', $trContent);
        $content .= $this->Html->tag('table', $tableContent);

        return $this->Html->tag('div', $content, array('class' => 'sc-comment' . (!empty($comment['private']) ? ' sc-private' : '') . (!empty($comment['editable']) ? ' sc-editable' : '')));
    }

    /**
     *
     *
     * @param type $comments
     * @param type $options
     * @return type
     * @throws BadMethodCallException
     */
    public function draw($comments, $options) {
        if (is_null($options)) {
            throw new BadMethodCallException();
        }

        if (!is_array($options)) {
            throw new BadMethodCallException();
        }

        if (empty($options)){
            $options = array();
        }


        $params = array_merge(array('empty_message' => 'No comment found.', 'edit' => array('url' => '')), $options);

        $content = !empty($params['title']) ? $this->Html->tag('div', $params['title'], array('class' => 'sc-title')) : '';
        if (empty($comments)) {
            $content .= $this->Html->tag('div', $params['empty_message'], array('class' => 'sc-empty'));
        } else {
            foreach ($comments as $comment) {
                $content .= $this->_drawComment($comment, $params['edit']);
            }
        }

        $tagAttrs = array('class' => 'sc-list');
        if (!empty($params['html'])) {
            foreach ($params['html'] as $key => $val) {
                if ($key == 'class') {
                    $tagAttrs['class'] .= ' ' . $val;
                } else {
                    $tagAttrs['class'] = $val;
                }
            }
        }
        return $this->Html->tag('div', $content, $tagAttrs);
    }

}
