<?php

/**
 *
 * Bannette helper class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		cake.app
 * @subpackage	cake.app.view.helper
 */
class BannetteHelper extends Helper {

	/**
	 *
	 * @var type
	 */
	public $helpers = array('String', 'Html', 'Form', 'Paginator', 'Js');

	/**
	 *
	 * @param type $priority
	 * @return type
	 */
	private function _convertPriority($priority) {
		$img = 'priority_unknown.png';
		$alt = $priority;
		$title = __d('courrier', 'Courrier.priorite_indefinie');
		if ($priority == 0) {
			$img = 'priority_low.png';
			$title = __d('courrier', 'Courrier.priorite_basse');
		} else if ($priority == 1) {
			$img = 'priority_mid.png';
			$title = __d('courrier', 'Courrier.priorite_moyenne');
		} else if ($priority == 2) {
			$img = 'priority_high.png';
			$title = __d('courrier', 'Courrier.priorite_haute');
		}
		return $this->Html->image('/img/' . $img, array('alt' => $alt, 'title' => $title));
	}

	/**
	 *
	 */
	private function _convertRetard($retard) {
		return $retard ? $this->Html->image('/img/retard_16.png', array('alt' => 'retard', 'title' => __d('courrier', 'Courrier.retard'))) : '';
	}

	/**
	 *
	 * @param type $fileassoc
	 * @return type
	 */
	private function _convertFileassoc($fileassoc) {
		return $fileassoc ? $this->Html->image('/img/fileassoc_16.png', array('alt' => 'présent', 'alt' => __d('courrier', 'Courrier.fileassoc'))) : '';
	}

	/**
	 *
	 * @param type $direction
	 * @return type
	 */
	private function _convertDirection($direction) {
		$return = "";
		$directionDef = array(
			'E' => array(
//				'img' => 'flux_entrant.png',
				'img' => 'folder_inbox.png',
				'title' => __d('courrier', 'Courrier.courrier_entrant')
			),
			'S' => array(
//				'img' => 'flux_sortant.png',
				'img' => 'folder_outbox.png',
				'title' => __d('courrier', 'Courrier.courrier_sortant')
			),
			'I' => array(
				'img' => 'flux_interne.png',
				'title' => __d('courrier', 'Courrier.courrier_interne')
			)
		);

		if (!empty($direction) && in_array($direction, array_keys($directionDef))) {
			$return = $this->Html->image('/img/' . $directionDef[$direction]['img'], array('alt' => $direction, 'title' => $directionDef[$direction]['title']));
		}
		return $return;
	}

	/**
	 *
	 * @param type $etat
	 * @return type
	 */
	private function _convertEtat($etat) {
		$img = 'flux_valide.png';
		$title = __d('courrier', 'Courrier.courrier_valide');
		$alt = 'validé';
		if ($etat == -1) {
			$img = 'flux_refuse.png';
			$title = __d('courrier', 'Courrier.courrier_refuse');
			$alt = 'refusé';
		} else if ($etat == 1 || $etat == 0 ) {
			$img = 'flux_a_traite.png';
			$title = __d('courrier', 'Courrier.courrier_a_traite');
			$alt = 'traitement';
		} else if ($etat == 2) {
			$img = 'flux_cloture.png';
			$title = __d('courrier', 'Courrier.courrier_cloture');
			$alt = 'cloturé';
		}
		return $this->Html->image('/img/' . $img, array('alt' => $alt, 'title' => $title));
	}

	/**
	 *
	 * @param type $objet
	 * @return type
	 */
	private function _convertObjet($objet) {
		return $this->String->troncateStr($objet, 90);
	}

	/**
	 *
	 * @param type $date
	 * @return type
	 */
	private function _convertDate($date) {
		return strftime("%d/%m/%Y", strtotime($date));
	}

	/**
	 *
	 * @param type $date
	 * @return type
	 */
	private function _convertDateTime($date) {
		return strftime("%d/%m/%Y %T", strtotime($date));
	}

	/**
	 *
	 * @param type $edit
	 * @param type $fluxId
	 * @return type
	 */
	private function _convertEdit($edit, $fluxId, $desktopId = null) { //Arnaud
		return $edit ? $this->Html->image('/img/edit_16.png', array('alt' => __d('default', 'Button.edit'), 'title' => __d('default', 'Button.edit'), 'url' => array('plugin' => null, 'controller' => 'courriers', 'action' => 'view', $fluxId, $desktopId))) : '';
	}
        
	/**
	 *
	 * @param type $view
	 * @param type $fluxId
	 * @return type
	 */
	private function _convertView($view, $fluxId, $desktopId = null) {
        $url = array('plugin' => null, 'controller' => 'courriers', 'action' => 'view', $fluxId, $desktopId);
        if($this->action == 'historique' ) {
            $url = array('plugin' => null, 'controller' => 'courriers', 'action' => 'historiqueGlobal', $fluxId );
        }
//		return $view ? $this->Html->image('/img/view_16.png', array('alt' => __d('default', 'Button.view'), 'title' => __d('default', 'Button.view'), 'url' => array('plugin' => null, 'controller' => 'courriers', 'action' => 'view', $fluxId, $desktopId))) : '';
		return $view ? $this->Html->image('/img/view_16.png', array('alt' => __d('default', 'Button.view'), 'title' => __d('default', 'Button.view'), 'url' => $url )) : '';
	}

	/**
	 *
	 * @param type $delete
	 * @param type $fluxId
	 * @return type
	 */
	private function _convertDelete($delete, $fluxId) {
		return $delete ? $this->Html->image('/img/delete_16.png', array('alt' => __d('default', 'Button.delete'), 'title' => __d('default', 'Button.delete'), 'url' => array('plugin' => null, 'controller' => 'courriers', 'action' => 'delete', $fluxId))) : '';
	}

	/**
	 *
	 * @param type $flux
	 * @return type
	 */
	private function _convertFlux($flux, $date, $bannetteName = null) {
		if ($date == 'datereception') {
			$keydate = __d('bannette', 'Courrier.datereception');
			if (!empty($flux['date'])) {
				$valdate = $this->_convertDate($flux['date']);
			} else {
				$valdate = $this->_convertDateTime($flux['created']);
				$tmp = explode(' ', $valdate);
				$valdate = $tmp[0];
			}
		} else if ($date == 'created') {
			$keydate = __d('bannette', 'Courrier.created');
			$valdate = $this->_convertDateTime($flux['created']);
		}

		$alias = Inflector::camelize("bannette_" . Inflector::camelize(Inflector::slug($bannetteName)));

		$options = array(
			'model' => $alias,
			'url' => (array) $this->passedArgs + array('model' => $alias),
		);

		if ($this->request->params['action'] == 'dispenv') {
			$return = array(
				"id" => $flux['id'],
				"desktopId" => $flux['desktopId'],
                "creatorId" => $flux['creatorId'],
                "userconnectedId" => $flux['userconnectedId'],
				"read" => $flux['read'],
				__d('bannette', 'Courrier.priorite') => $this->_convertPriority($flux['priorite']),
				__d('bannette', 'Courrier.retard') => $this->_convertRetard($flux['retard']),
				__d('bannette', 'Courrier.fichier_associe') => $this->_convertFileassoc($flux['ficassoc']),
				__d('bannette', 'Courrier.direction') => $this->_convertDirection($flux['direction']),
				__d('bannette', 'Courrier.etat') => $this->_convertEtat($flux['etat']),
				__d('bannette', 'Courrier.reference') => $flux['reference'],
				__d('bannette', 'Courrier.name') => $flux['nom'],
				__d('bannette', 'Courrier.objet') => $this->_convertObjet($flux['objet']),
				__d('bannette', 'Courrier.contact') => $flux['contact'],
				$keydate => $valdate,
				__d('bannette', 'Courrier.typesoustype') => $flux['typesoustype'],
				__d('bannette', 'Courrier.bureau') => $flux['bureau'],
				__d('bannette', 'Courrier.modified') => $flux['modified'],
				__d('bannette', 'Courrier.view') => $this->_convertView($flux['view'], $flux['id'], $flux['desktopId']),
				__d('bannette', 'Courrier.edit') => $this->_convertEdit($flux['edit'], $flux['id'], $flux['desktopId']),//Arnaud
				__d('bannette', 'Courrier.delete') => $this->_convertDelete($flux['delete'], $flux['id']),
				'isParent' => $flux['isParent'],
				'hasParent' => !empty($flux['parent_id']),
				'parentInfo' => $this->_convertParentInfo($flux['parentInfo'])
			);
		} else {

			$return = array(
				"id" => $flux['id'],
				"desktopId" => $flux['desktopId'],
                "creatorId" => $flux['creatorId'],
                "userconnectedId" => $flux['userconnectedId'],
				"read" => $flux['read'],
				$this->Paginator->sort('Courrier.priorite', __d('bannette', 'Courrier.priorite'), array_merge($options, array('escape' => false))) => $this->_convertPriority($flux['priorite']),
				__d('bannette', 'Courrier.retard') => $this->_convertRetard($flux['retard']),
				__d('bannette', 'Courrier.fichier_associe') => $this->_convertFileassoc($flux['ficassoc']),
				__d('bannette', 'Courrier.direction') => $this->_convertDirection($flux['direction']),
				__d('bannette', 'Courrier.etat') => $this->_convertEtat($flux['etat']),
				$this->Paginator->sort('Courrier.reference', __d('bannette', 'Courrier.reference'), $options) => $flux['reference'],
				__d('bannette', 'Courrier.name') => $flux['nom'],
				__d('bannette', 'Courrier.objet') => $this->_convertObjet($flux['objet']),
				$this->Paginator->sort('Contactinfo.name', 'Exp. / Dest.', $options) => $flux['contact'],
				$this->Paginator->sort('Courrier.created', $keydate, $options) => $valdate, //FIXME
//                $keydate => $valdate, //FIXME
				$this->Paginator->sort('Type.name', __d('bannette', 'Courrier.typesoustype'), $options) => $flux['typesoustype'],
				$this->Paginator->sort('Desktop.name', __d('bannette', 'Courrier.bureau'), $options) => $flux['bureau'],
				$this->Paginator->sort('Courrier.modified', __d('courrier', 'Courrier.modified'), $options) => $this->_convertDateTime($flux['modified']), //FIXME
				__d('bannette', 'Courrier.view') => $this->_convertView($flux['view'], $flux['id'], $flux['desktopId']),
				__d('bannette', 'Courrier.delete') => $this->_convertDelete($flux['delete'], $flux['id']),
				'isParent' => $flux['isParent'],
				'hasParent' => !empty($flux['parent_id']),
				'parentInfo' => $this->_convertParentInfo($flux['parentInfo']),
				//'mailRetardEnvoye' => $flux['mailRetardEnvoye']
			);
		}

		return $return;
	}

	/**
	 *
	 * @param type $parentInfo
	 */
	private function _convertParentInfo($parentInfo) {
		$return = '';
		if (!empty($parentInfo)) {
//debug( $parentInfo );
			$tableBody = '';
			$thContent = '';
			$trContent = '';
			$tableTitle = '';

			$tableTitle .= 'Informations sur le flux original';
			$thContent .= $this->Html->tag('th', __d('courrier', 'Courrier.reference'));
			$thContent .= $this->Html->tag('th', __d('courrier', 'Courrier.name'));
			$thContent .= $this->Html->tag('th', __d('courrier', 'Courrier.objet'));
			$thContent .= $this->Html->tag('th', __d('courrier', 'Courrier.contact'));
//            $thContent .= $this->Html->tag( 'th', __d( 'courrier', 'Courrier.datereception' ) );
			$thContent .= $this->Html->tag('th', 'Date de réception');
			$thContent .= $this->Html->tag('th', __d('bannette', 'Courrier.typesoustype'));
			$thContent .= $this->Html->tag('th', __d('bannette', 'Courrier.bureau'));
			$thContent .= $this->Html->tag('th', __d('bannette', 'Courrier.modified'));
			$tableBody .= $this->Html->tag('tr', $thContent);

			$trContent .= $this->Html->tag('td', $parentInfo['Courrier']['reference']);
			$trContent .= $this->Html->tag('td', $parentInfo['Courrier']['name']);
			$trContent .= $this->Html->tag('td', $parentInfo['Courrier']['objet']);
			$trContent .= $this->Html->tag('td', $parentInfo['Contactinfo']['name']);
			$trContent .= $this->Html->tag('td', $this->_convertDate($parentInfo['Courrier']['datereception']));
			$trContent .= $this->Html->tag('td', $parentInfo['Soustype']['Type']['name'] . ' / ' . $parentInfo['Soustype']['name']);
			$trContent .= $this->Html->tag('td', $parentInfo['Desktop']['name']);
			$tableBody .= $this->Html->tag('tr', $trContent);

			$return = $this->Html->tag(
					'div', $this->Html->tag(
							'table', $this->Html->tag(
									'caption', $tableTitle, array('style' => 'font-weight: bold;')
							)
							. $this->Html->tag('tbody', $tableBody)
					), array('class' => 'parentInfo')
			);
		}
		return $return;
	}

	/**
	 *
	 * @param type $bannette
	 * @return type
	 */
	private function _convertBannette($bannette, $date, $bannetteName = null) {
		for ($i = 0; $i < count($bannette); $i++) {
			$bannette[$i] = $this->_convertFlux($bannette[$i], $date, $bannetteName);
		}
		return $bannette;
	}

	/**
	 *
	 * @param type $bannetteContent
	 * @param type $bannetteName
	 * @return type
	 */
	public function create($bannetteContent, $bannetteName, $options = array('sortable' => false, 'checkable' => false, 'filter' => false, 'translateBannetteName' => true, 'date' => 'datereception', 'disp' => false, 'unwantedFields' => array())) {


		$checkable = $options['checkable'];
		$sortable = $options['sortable'];
		$filter = $options['filter'];

		if (!isset($options['disp'])) {
			$options['disp'] = false;
		}
// var_dump($options);
// die();
		$idSuffix = str_replace(array(' ', '.'), '', microtime());
		$bannetteContent = $this->_convertBannette($bannetteContent, $options['date'], $bannetteName);



		$alias = Inflector::camelize("bannette_" . Inflector::camelize(Inflector::slug($bannetteName)));
		$path = "paging.{$alias}.count";
		$countFlux = Hash::get($this->request->params, $path);

		$unwantedFields = array();
		if (!empty($options['unwantedFields'])) {
			foreach ($options['unwantedFields'] as $field) {
				$unwantedFields[] = __d('bannette', 'Courrier.' . $field);
			}
		}

		$header = $this->Html->tag('h3', $this->Html->link(($options['translateBannetteName'] ? __d('bannette', 'Bannette.' . $bannetteName) : $bannetteName) . $this->Html->tag('span', 'total : ' . $countFlux), '#', array('escape' => false)), array('class' => 'bannetteHead'));
		$table = $this->Html->tag('div', __d('bannette', 'Bannette.void'), array('class' => 'ui-state-highlight bannetteVoid'));
		if (!empty($bannetteContent)) {

			$ths = array_keys($bannetteContent[0]);
			$thsContent = '';
			foreach ($ths as $th) {
				if ($th != 'id' && $th != 'creatorId'  && $th != 'mailRetardEnvoye' && $th != 'userconnectedId' && $th != 'read' && $th != 'desktopId' && $th != 'hasParent' && $th != 'isParent' && $th != __d('bannette', 'Courrier.delete') && $th != 'parentInfo' && !in_array($th, $unwantedFields) && $th != __d('bannette', 'Courrier.edit')) { //Arnaud

//					debug($th);
					if (preg_match("#abbr title='Priorit#", $th)) {
						$thOptions = array(
							'class' => 'thPriorite thState'
						);
					} else if ($th == __d('bannette', 'Courrier.retard')) {
						$thOptions = array(
							'class' => 'thRetard thState'
						);
					} else if ($th == __d('bannette', 'Courrier.fichier_associe')) {
						$thOptions = array(
							'class' => 'thFicassoc thState'
						);
					} else if ($th == __d('bannette', 'Courrier.direction')) {
						$thOptions = array(
							'class' => 'thDirection thState'
						);
					} else if ($th == __d('bannette', 'Courrier.etat')) {
						$thOptions = array(
							'class' => 'thEtat thState'
						);
					} else if ($th == __d('bannette', 'Courrier.reference')) {
						$thOptions = array(
							'class' => 'thRef'
						);
					} else if ($th == __d('bannette', 'Courrier.name')) {
						$thOptions = array(
							'class' => 'thName'
						);
					} else if ($th == __d('bannette', 'Courrier.objet')) {
						$thOptions = array(
							'class' => 'thObjet'
						);
					} else if ($th == __d('bannette', 'Courrier.contact')) {
						$thOptions = array(
							'class' => 'thContact'
						);
					} else if ($th == __d('bannette', 'Courrier.datereception')) {
						$thOptions = array(
							'class' => 'thDate'
						);
					} else if ($th == __d('bannette', 'Courrier.created')) {
						$thOptions = array(
							'class' => 'thDate'
						);
					} /* else if ($th == __d('bannette', 'Courrier.circuit')) {
					  $thOptions = array(
					  'class' => 'thCircuit'
					  );
					  } */ else if ($th == __d('bannette', 'Courrier.typesoustype')) {
						$thOptions = array(
							'class' => 'thTypesoustype'
						);
					} else if ($th == __d('bannette', 'Courrier.bureau')) {
						$thOptions = array(
							'class' => 'thBureau'
						);
					} else if ($th == __d('bannette', 'Courrier.modified')) {
						$thOptions = array(
							'class' => 'thModified'
						);
					} else if ($th == __d('bannette', 'Courrier.view')) {
						$th = 'Actions';
                        $colspan = 2;//Arnaud
                        if( $options['disp'] ) {// Si aiguilleur alors 3 actions possibles (view, edit, delete)
                            $colspan = 3;//Arnaud
                        }//Arnaud
						$thOptions = array(
							'class' => 'thActions',
							'colspan' => $colspan//Arnaud
						);
					}

					$thsContent .= $this->Html->tag('th', $th, $thOptions);
				}
			}
			if ($checkable) {
				$thsContent .= $this->Html->tag('th', 'Tous', array('state' => 'unchecked', 'class' => 'thCheckable', 'id' => 'thCheckable_' . $idSuffix));
			}
			$tableHead = $this->Html->tag('tr', $thsContent);


			$tableBody = '';
			$odd = false;
			$trgroup = 0;
			$start = true;

			foreach ($bannetteContent as $flux) {
				$trContent = '';

				foreach ($flux as $th => $td) {
					if ($th != 'id' && $th != 'creatorId' && $th != 'userconnectedId' && $th != 'mailRetardEnvoye' && $th != 'read' && $th != 'desktopId' && $th != 'hasParent' && $th != 'isParent' && $th != 'parentInfo' && !in_array($th, $unwantedFields)) {
						if (preg_match("#abbr title='Priorit#", $th)) {
							$tdOptions = array(
								'class' => 'tdPriorite tdState',
								'title' => __d('bannette', 'Courrier.priorite.long')
							);
						} else if ($th == __d('bannette', 'Courrier.retard')) {
							$tdOptions = array(
								'class' => 'tdRetard tdState',
								'title' => __d('bannette', 'Courrier.retard.long')
							);
						} else if ($th == __d('bannette', 'Courrier.fichier_associe')) {
							$tdOptions = array(
								'class' => 'tdFicassoc tdState',
								'title' => __d('bannette', 'Courrier.fichier_associe.long')
							);
						} else if ($th == __d('bannette', 'Courrier.direction')) {
							$tdOptions = array(
								'class' => 'tdDirection tdState',
								'title' => __d('bannette', 'Courrier.direction.long')
							);
						} else if ($th == __d('bannette', 'Courrier.etat')) {
							$tdOptions = array(
								'class' => 'tdEtat tdState',
								'title' => __d('bannette', 'Courrier.etat.long')
							);
						} else if ($th == __d('bannette', 'Courrier.reference')) {
							$tdOptions = array(
								'class' => 'tdRef'
							);
						} else if ($th == __d('bannette', 'Courrier.name')) {
							$tdOptions = array(
								'class' => 'tdName'
							);

							if ($flux['hasParent']) {
								$viewParent = $this->Html->image('/img/help-hint.png', array(
									//'title' => 'Informations sur le flux d\'origine',
									'alt' => 'Informations sur le flux d\'origine',
									'class' => 'btnParentInfo'
										)
								);
//								$td = '&nbsp;' . $this->Html->image('/img/ban_flux_reponse.png') . '&nbsp;&nbsp;' . $td;
								$td = '&nbsp;' . $viewParent . '&nbsp;&nbsp;' . $td . $flux['parentInfo'];
							}
						} else if ($th == __d('bannette', 'Courrier.objet')) {
							$tdOptions = array(
//								'class' => 'tdObjet'
								'class' => 'tdEtat tdState',
							);
						} else if ($th == __d('bannette', 'Courrier.contact')) {
							$tdOptions = array(
								'class' => 'tdContact'
							);
						} else if ($th == __d('bannette', 'Courrier.datereception')) {
							$tdOptions = array(
								'class' => 'tdDate'
							);
						} else if ($th == __d('bannette', 'Courrier.created')) {
							$tdOptions = array(
								'class' => 'tdDate'
							);
						} /* else if ($th == __d('bannette', 'Courrier.circuit')) {
						  $tdOptions = array(
						  'class' => 'tdCircuit'
						  );
						  } */ else if ($th == __d('bannette', 'Courrier.typesoustype')) {
							$tdOptions = array(
								'class' => 'tdTypesoustype'
							);
						} else if ($th == __d('bannette', 'Courrier.bureau')) {
							$tdOptions = array(
								'class' => 'tdBureau'
							);
						} else if ($th == __d('bannette', 'Courrier.modified')) {
							$tdOptions = array(
								'class' => 'tdModified'
							);
						} else if ($th == __d('bannette', 'Courrier.view')) {
							$tdOptions = array(
								'class' => 'tdView tdAction'
							);
							if ($flux['isParent']) {
								$td = '';
							}
						} else if ($th == __d('bannette', 'Courrier.edit')) {//Arnaud
							$tdOptions = array(
								'class' => 'tdEdit tdAction'
							);
							if ($flux['isParent']) {
								$td = '';
							}
						} else if ($th == __d('bannette', 'Courrier.delete')) {
							$tdOptions = array(
								'class' => 'tdDelete tdAction'
							);
//                            debug($flux);
//                            debug($flux['creatorId']);
//                            die();
							if( !$options['disp'] ) { // Si non aiguillage, suppression uniquement pour le propriétaire du flux
								if ($flux['isParent'] || $flux['creatorId'] != $flux['userconnectedId']) {
									$td = '';
								}
							}
						}
						$trContent .= $this->Html->tag('td', $td, $tdOptions);
					}
				}
				if ($filter && $checkable && !$flux['isParent']) {
					$checkableOptions = array(
						'type' => 'checkbox',
						'class' => 'checkItem',
						'itemId' => $flux['id'],
						'value' => '',
						'div' => false,
						'label' => false
					);
					$trContent .= $this->Html->tag('td', $this->Form->input('checkItem_' . $flux['id'], $checkableOptions), array('class' => 'checkable'));
				}

				$sepTr = false;
				if ($options['date'] == 'created') {
					$curFluxDate = DateTime::createFromFormat('d/m/Y H:i:s', $flux[__d('bannette', 'Courrier.created')]);
					if (isset($prevFluxDate)) {
						$interval = $curFluxDate->diff($prevFluxDate);
						if ($interval->i >= 5) {
							$sepTr = true;
							$trgroup++;
						}
					}
					$prevFluxDate = DateTime::createFromFormat('d/m/Y H:i:s', $flux[__d('bannette', 'Courrier.created')]);
				}

				$tableBody .= $this->Html->tag('tr', $trContent, array('class' => (!$flux['read'] ? 'unread ' : '') . ($flux['isParent'] ? 'isParent ' : '') . ($flux['hasParent'] ? 'hasParent ' : '') . 'tooltips ' . 'tableBodyTr tr_' . $flux['id'] . ($odd ? ' odd' : ' even') . ($sepTr || $start ? ' sepTr' : '') . ' trgroup_' . $trgroup, 'trgroup' => $trgroup));
				$odd = !$odd;
				$start = false;
			}

			$tableOptions = array(
				'class' => 'innerTableTrigger bannetteTable ui-widget ui-corner-all ui-widget-content tablesorter' . ($options['disp'] ? ' dispBannetteTable' : ''),
				'id' => 'filterTable_' . $idSuffix
			);

			$table = $this->Html->tag('table', $this->Html->tag('thead', $tableHead) . $this->Html->tag('tbody', $tableBody), $tableOptions);
		}

		$filterDiv = '';
		if ($filter) {
			$filterDivContent = $this->Html->tag('span', $this->Html->image('/img/filter_16.png') . $this->Html->image('/img/filter_down_16.png'), array('class' => 'bttnFilter'));
			$filterDivContent .= $this->Html->tag('span', $this->Html->image('/img/filter_off_16.png'), array('class' => 'statusFilter'));
			$filterDiv = $this->Html->tag('div', $filterDivContent, array('class' => 'controlsFilter', 'id' => 'filters_' . $idSuffix));
		}




		$script = '<script type="text/javascript">
            $("#filterTable_' . $idSuffix . ' tbody tr").hover(function(){
                if (!$(this).hasClass("isParent")){
                  $(this).addClass("ui-state-highlight");
                  $(this).css( "cursor", "pointer");
                }
              }, function(){
                if (!$(this).hasClass("isParent")){
                  $(this).removeClass("ui-state-highlight");
                }
              });


            $("#filterTable_' . $idSuffix . ' tbody tr td.tdName").click(function(){
                    $(".tdView img", $(this).parent()).trigger("click");
              });
          ';
		if ($filter) {
			$script .='gui.filter({
                element: $("#filters_' . $idSuffix . '"),
                table: $("#filterTable_' . $idSuffix . '")
              });
            ';
		}
		if ($checkable) {
			$script .= '$("#thCheckable_' . $idSuffix . '").click(function(){
                    if ($(this).attr("state") == "checked"){
                        $("#filterTable_' . $idSuffix . ' .checkItem").removeAttr("checked");
                        $(this).attr("state", "unchecked");
                        var bannette;
                        $("#bannettesContainer .bannette_panel").each(function() {
                            if ($(this).css("display") != "none") {
                                    bannette = $(this);
                            }
                        });
                        var tabChecked = new Array();
                        $(".checkItem", bannette).each(function() {
                            if ($(this).attr("checked",false)) {
                                    tabChecked.push($(this).attr("itemId"));     
                            }
                        });
                        $.ajax({
                            type: "post",
                            dataType: "json",
                            url: "/Courriers/fluxIdInTraite",
                            success: function(data) {
                                for(var i=0;i<tabChecked.length;i++){
                                    if($.inArray( parseInt(tabChecked[i]), data)!=-1 && $.inArray( parseInt(tabChecked[i]), inCircuitId)==-1){
                                        var index = inCircuitId.indexOf(tabChecked[i]);
                                        if(index!=-1){
                                            inCircuitId.splice(index,1);
                                        }
                                    }
                                }
                                nbInCircuit = inCircuitId.length;
                                isButtonAiguillage(nbInCircuit);
                            }
                        });
                        
                    } else {
                        $("#filterTable_' . $idSuffix . ' .checkItem").attr("checked", "checked");
                        $(this).attr("state", "checked");
                        var bannette;
                        $("#bannettesContainer .bannette_panel").each(function() {
                            if ($(this).css("display") != "none") {
                                    bannette = $(this);
                            }
                        });
                        var tabChecked = new Array();
                        $(".checkItem", bannette).each(function() {
                            if ($(this).attr("checked",true)) {
                                    tabChecked.push($(this).attr("itemId"));     
                            }
                        });
                        
                        $.ajax({
                                type: "post",
                                dataType: "json",
                                url: "/Courriers/fluxIdInTraite",
                                success: function(data) {
                                    for(var i=0;i<tabChecked.length;i++){
                                        if($.inArray( parseInt(tabChecked[i]), data)!=-1 && $.inArray( parseInt(tabChecked[i]), inCircuitId)==-1){
                                            inCircuitId.push(tabChecked[i]);
                                        }
                                    }
                                    nbInCircuit = inCircuitId.length;
                                    isButtonAiguillage(nbInCircuit);
                                }
                            });
                        }
                       
                });
            ';
		}

		if ($sortable) {
			$script .= '$("#filterTable_' . $idSuffix . '").tablesorter();';
		}


		/* Pagination */
		// Paramètres pour la pagination
		$pagination = null;
//        $alias = Inflector::camelize( "bannette_{$bannetteName}" );
		$alias = Inflector::camelize("bannette_" . Inflector::camelize(Inflector::slug($bannetteName)));
		$path = "paging.{$alias}.pageCount";
		$pageCount = Set::classicExtract($this->params, $path);

		$options = array(
			'model' => $alias,
			'url' => (array) $this->passedArgs + array('model' => $alias),
		);

		if ($pageCount > 1) {
			$pagination = '<div class="paging">
                    ' . implode(
							' | ', Set::filter(
									array(
										$this->Paginator->first('<<', $options, null, array('class' => 'disabled')),
										$this->Paginator->prev('<', $options, null, array('class' => 'disabled')),
										$this->Paginator->numbers($options),
										$this->Paginator->next('>', $options, null, array('class' => 'disabled')),
										$this->Paginator->last('>>', $options, null, array('class' => 'disabled'))
									)
							)
					) . '
                </div>';
		}

		// Pour la partie Ajax
//        $this->Js->writeBuffer();
		/* Pagination */

		$script .= '
			$("#filterTable_' . $idSuffix . ' .tdDelete").each(function(){
				var link = $("a", this);
				var url = link.attr("href");
				var img = $("img", link).click(function(){
                    gui.message({
                        title: "' . __d('default', 'Confirmation de suppression') . '",
                        msg: "' . __d('default', 'Voulez-vous supprimer cet élément ?') . '",
                        buttons: {
                            "' . __d('default', 'Button.confirm') . '": function(){
                                window.location.href = url;
                            },
                            "' . __d('default', 'Button.cancel') . '": function(){
                                $(this).dialog("close");
                            }
                        }
                    });
				});

				img.css("cursor", "pointer");
				$(this).append(img);
				link.remove();


                $(".btnParentInfo").hover(function(){
                    $(".parentInfo", $(this).parent()).css("display", "block");
                },function(){
                    $(".parentInfo", $(this).parent()).css("display", "none");
                });

			});
		';
		$script .= "</script>";
		$div = $this->Html->tag('div', $filterDiv . $pagination . $table . $script . $pagination, array('class' => 'bannette_panel'));

		return $header . $div;
	}

	/**
	 *
	 * @param type $courriers
	 * @param type $titre
	 * @param type $actions
	 * @param type $checkable
	 */
	public function draw($courriers, $titre, $actions = array(), $checkable = false) {
		$idSuffix = str_replace(array(' ', '.'), '', microtime());

//		$nbItem = count($courriers);

		$controller = Hash::get($this->request->params, 'controller'); // FIXME
		$nbItem = Hash::get($this->request->params, 'paging.Courrier.count');
		if ($controller == 'repertoires') { // FIXME
			$nbItem = Hash::get($this->request->params, 'paging.CourrierRepertoire.count'); // FIXME
		} // FIXME
		$nbAction = count($actions);
//debug($this->Paginator);
		echo '<h3 class="bannette"><a href="#">' . $titre . ' (' . $nbItem . ')</a></h3>
            <div class="bannette_panel">';
		if ($nbItem > 0) {
			echo '<div class="controlsFilter" id="filters_' . $idSuffix . '">';
			echo '<span class="bttnFilter">';
			echo $this->Html->image('icons/filtre.png');
			echo $this->Html->image('/img/filter_down_16.png');
			echo '</span>';
			echo '<span class="statusFilter">';
			echo $this->Html->image('/img/filter_on_16.png');
			echo '</span>';
			echo '</div>';

			/* Pagination */
			$pagination = null;
			$idRecherche = "#infos";
			if ($controller == 'recherches') {
				$idRecherche = "#repertoires_skel_infos";
			}
			$this->Paginator->options(array(
				'update' => "$idRecherche .content",
				'evalScripts' => true,
				'before' => 'gui.loader({ element: $("' . $idRecherche . ' .content"), message: gui.loaderMessage });',
				'complete' => '$(".loader").remove()',
//				'url' => $this->passedArgs,
				'url' => $this->request->params['pass'],
				'data' => http_build_query($this->request->data),
				'method' => 'POST'
			));

			if ($nbItem > 1) {
				$pagination = '<div class="paging">
						' . implode(
								' | ', Set::filter(
										array(
											$this->Paginator->first('<<', array(), null, array('class' => 'disabled')),
											$this->Paginator->prev('<', array(), null, array('class' => 'disabled')),
											$this->Paginator->numbers(),
											$this->Paginator->next('>', array(), null, array('class' => 'disabled')),
											$this->Paginator->last('>>', array(), null, array('class' => 'disabled'))
										)
								)
						) . '
					</div>';
			}

			if (!empty($pagination)) {
				echo $pagination;
			}
			/* Pagination */

			echo '<table class="bannette_table search ui-widget ui-corner-all ui-widget-content" id="filterTable_' . $idSuffix . '">
            <thead>
            <th class="reference">' . __d('courrier', 'Courrier.reference') . '</th>
            <th class="nom">' . __d('courrier', 'Courrier.name') . '</th>
            <th class="objet">' . __d('courrier', 'Courrier.objet') . '</th>
            <th class="expediteur">' . __d('courrier', 'Courrier.contact') . '</th>
            <th class="datereception">' . __d('courrier', 'Courrier.datereception') . '</th>
            <th class="retard">' . __d('courrier', 'Courrier.retard') . '</th>';
			if ($nbAction != 0) {
				echo '<th class="actions" colspan="' . $nbAction . '">Actions</th>';
			}


			if ($checkable) {
				echo "<th state='unchecked' class='thCheckable' id='thCheckable_" . $idSuffix . "'>Tous</th>";
			}


			echo '</thead>
            <tbody>';
			foreach ($courriers as $courrier) {
				$idSuffixcreated = strtotime($courrier['Courrier']['datereception']);
				echo '<tr onmouseover="$(this).addClass(\'ui-state-highlight\');" onmouseout="$(this).removeClass(\'ui-state-highlight\');">
                    <td class="reference">';
				echo $courrier['Courrier']["reference"] . '</td>
                    <td class="nom">' . $courrier['Courrier']['name'] . '</td>
                    <td class="expediteur">' . $this->String->troncateStr($courrier['Courrier']["objet"], 90) . '</td>
                    <td class="expediteur">' . $courrier['Contact']["name"] . '</td>
                    <td class="datereception">' . strftime("%d/%m/%Y", $idSuffixcreated) . '</td>';
				echo '<td class="retard">';
				if (isset($courrier['retard']) && $courrier['retard']) {
					echo $this->Html->image("/img/retard_16.png", array('height' => '16px;', 'style' => 'vertical-align: middle;', 'alt' => __d('courrier', 'Courrier.alertRetard'), 'title' => __d('courrier', 'Courrier.alertRetard')));
				}
				echo '</td>';
				if ($nbAction != 0) {

					if (isset($courrier['right_read'])) {
						echo '<td class="action view">';
						if ($courrier['right_read']) {
							echo $this->Html->image('/img/view.png', array(
								'title' => __d('default', 'Button.view'),
								'alt' => __d('default', 'Button.view'),
								'url' => array(
									'controller' => 'courriers',
									'action' => $actions['view'],
									$courrier['Courrier']['id']
								)
									)
							);
						}
						echo '</td>';
					}

					if (isset($courrier['right_delete'])) {
						echo '<td class="action delete">';
						if ($courrier['right_delete']) {
                                echo $this->Html->image('/img/delete.png', array(
                                    'title' => __d('default', 'Button.delete'),
                                    'alt' => __d('default', 'Button.delete'),
                                    'url' => array(
                                        'controller' => 'courriers',
                                        'action' => $actions['delete'],
                                        $courrier['Courrier']['id']
                                    )
                                )
							);
						}
						echo '</td>';
					}

					if ($checkable) {
						echo "<td class='checkable'><input type='checkbox' class='checkItem' itemId='" . $courrier['Courrier']['id'] . "' value='' /></td>";
					}
				}
				echo '</tr>';
			}
			echo '</tbody>
            </table>';
//			if (!empty($pagination)) {
//				echo $pagination;
//			}
		} else {
			echo '<div style="padding: 10px;">';
			echo __d('courrier', 'Courrier.void');
			echo "</div>";
		}

		echo '<script type="text/javascript">';
		echo 'gui.filter({ element: $("#filters_' . $idSuffix . '"), table: $("#filterTable_' . $idSuffix . '") });';
        
        echo '$("#filterTable_' . $idSuffix . ' tbody tr td.nom").css( "cursor", "pointer");';
         echo '$("#filterTable_' . $idSuffix . ' tbody tr td.nom").click(function(){
                    $(".view img", $(this).parent()).trigger("click");
              });';

         echo '
			$("#filterTable_' . $idSuffix . ' .delete").each(function(){
				var link = $("a", this);
				var url = link.attr("href");
				var img = $("img", link).click(function(){
                    return confirm("Confirmez-vous la suppression ?" );
                });
            });';
        /*echo '
			$("#filterTable_' . $idSuffix . ' .delete").each(function(){
				var link = $("a", this);
				var url = link.attr("href");
				var img = $("img", link).click(function(){
                    gui.message({
                        title: "' . __d('default', 'Confirmation de suppression') . '",
                        msg: "' . __d('default', 'Voulez-vous supprimer cet élément ?') . '",
                        buttons: {
                            "' . __d('default', 'Button.confirm') . '": function(){
                                window.location.href = "/recherches/recherche";
                            },
                            "' . __d('default', 'Button.cancel') . '": function(){
                                $(this).dialog("close");
                            }
                        }
                    });
				});

			});
		';*/
                 
		if ($checkable) {
			echo '$("#thCheckable_' . $idSuffix . '").click(function(){
                        if ($(this).attr("state") == "checked"){
                            $("#filterTable_' . $idSuffix . ' .checkItem").removeAttr("checked");
                            $(this).attr("state", "unchecked");
                        } else {
                            $("#filterTable_' . $idSuffix . ' .checkItem").attr("checked", "checked");
                            $(this).attr("state", "checked");
                        }
                    });';
		}

		echo '</script>';
		echo '</div>';

		echo $this->Js->writeBuffer();
	}

}

?>
