<?php

/**
 *
 * Liste helper class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		cake.app
 * @subpackage	cake.app.view.helper
 */
class listeHelper extends Helper {

    /**
     *
     * @var type
     */
    var $helpers = array('String', 'Html', 'Paginator');

    /**
     *
     * @var type
     */
    var $treeTrId = 0;

    /*
     * Affichage d une liste non hierarchisée avec un systeme de filtre
     *
     * array $tab :  resultat d un find all
     * string $model : nom du model utilisé
     * array(string) $labels: champ qui vont etre affichés
     * array(string) $actions: tableau associatif des urls des actions a effectuer
     *  ex:
     *      array(
     *          "edit" => "/controllers/action",
     *          "delete" => "/controllers/action"
     *
     *      }
     * array(string) $updateElements : tableau associatif decrivant les elements jQuery a mettre a jour apres les requetes ajax effectuée
     *  ex:
     *      array(
     *          "read" => "$('#infos .content')",
     *          "delete" => "$('#liste .content')"
     *      )
     */

    //$actions = array(
    //      'view' => array(
    //              'url' => '/controllers/action',
    //              'updateElement' => '$("#jQuery_selected_element")'
    //      ),
    //      'edit' => array(
    //              'url' => '/controllers/action',
    //              'updateElement' => '$("#jQuery_selected_element")'
    //      ),
    //      'delete' => array(
    //              'url' => '/controllers/action',
    //              'updateElement' => '$("#jQuery_selected_element")'
    //      )
    //);

    /**
     *
     * @param type $tab
     * @param type $model
     * @param type $fields
     * @param type $actions
     * @param type $options
     */
    public function draw($tab, $model, $fields = array(), $actions = array(), $options = array()) {
        //parametrage par défaut
        $settings = array(
            'filter' => true,
            'title' => '',
            'noTitle' => false,
            'paginator' => false
        );
        //application des parametre personnalisé
        foreach ($options as $key => $val) {
            $settings[$key] = $val;
        }
        //initilisation des variables
        $idSuffix = str_replace(array(' ', '.'), '', microtime());
        $nbItem = count($tab);
        $nbAction = count($actions);
        $applyEditButton = false;
        $applyViewButton = false;
        $applyDeleteButton = false;
        // Ajout pour la vérification du connecteur parapheur
        $applyCheckButton = false;
        // Ajout pour la copie de Circuit
        $applyCopieCircuitButton = false;

        //preparation du titre
        if (!$settings['paginator'] && !$settings['noTitle']) {
            if ($settings['title'] != '') {
                $settings['title'] .= ' (total : ' . $nbItem . ')';
            } else {
                $settings['title'] = 'Total : ' . $nbItem;
            }
        }

        //affichage du titre
        echo '<h3 class="bannette">' . $settings['title'] . '</h3>
            <div class="bannette_panel"' . (!empty($settings['panel_id']) ? ' id="' . $settings['panel_id'] . '"' : "") . '>';
        //affichage de la table html
        if ($nbItem > 0) {
            //affichage du systeme de filtre
            if ($settings['filter']) {
                echo '<div class="controlsFilter" id="filters_' . $idSuffix . '">';
                echo '<span class="bttnFilter">';
                echo $this->Html->image('icons/filtre.png');
                echo $this->Html->image('/img/filter_down_16.png');
                echo '</span>';
                echo '<span class="statusFilter">';
                echo $this->Html->image('/img/filter_off_16.png');
                echo '</span>';
                echo '</div>';
            }

            echo '<table class="bannette_table ui-widget ui-corner-all ui-widget-content" id="filterTable_' . $idSuffix . '">
            <thead>';

            //affichage des entetes de la table html
            foreach ($fields as $field) {
                $fieldName = __d(strtolower($model), $model . "." . $field);
                if (!$settings['paginator']) {
                    echo '<th class="' . $field . '">' . $fieldName . '</th>';
                } else {
                    echo '<th class="' . $field . '">' . $this->Paginator->sort($model . "." . $field, $fieldName) . '</th>';
                }
            }

            if ($nbAction != 0) {
                echo '<th class="actions" colspan="' . $nbAction . '">Actions</th>';
            }
            echo '</thead>
            <tbody>';
            //parcours des elements
            foreach ($tab as $item) {
                $idSuffixcreated = strtotime($item[$model]['created']);

                //debut de la ligne du tableau html parametrage de l identifiant et des classes de chaque ligne
                echo '<tr id="listTrId_' . $item[$model]['id'] . '" onmouseover="$(this).addClass(\'ui-state-highlight\');" onmouseout="$(this).removeClass(\'ui-state-highlight\');" itemId="' . $item[$model]['id'] . '" class="listTr';
                if (isset($item['retard']) && $item['retard']) {
                    echo 'ui-state-error ';
                }

                //verification de l inactivité d un élément
                if (!empty($settings['check_inactive'])) {
                    if (!empty($item[$settings['inactive_model']]) && isset($item[$settings['inactive_model']][$settings['inactive_field']])) {
                        if ($item[$settings['inactive_model']][$settings['inactive_field']] == $settings['inactive_value']) {
                            echo ' ui-state-inactive ';
                        }
                    }
                }
                //elements enfants dans un tableau cakephp (permet d effectuer des liaisons javascript qui suivent la structure de la base de données)
                // Ajout de right_check pour la vérification du connecteur parapheur
                // 20150408: Ajout de right_copieCircuit pour la copie de circuit
                foreach ($item as $linkedItemKey => $linkedItemVal) {
                    if ($linkedItemKey != $model && $linkedItemKey != "right_view" && $linkedItemKey != "right_edit" && $linkedItemKey != "right_delete" && $linkedItemKey != "right_check" && $linkedItemKey != "right_copieCircuit") {
                        if (isset($linkedItemVal['id']) && !empty($linkedItemVal['id'])) {
                            echo " " . $linkedItemKey . "_" . $linkedItemVal['id'];
                        } else {
                            foreach ($linkedItemVal as $linkedItem2) {
                                if (!empty($linkedItem2['id'])) {
                                    echo " " . $linkedItemKey . "_" . $linkedItem2['id'];
                                }
                            }
                        }
                    }
                }
                echo '">';
                //affichage du contenu de chaque champ
                foreach ($fields as $field) {
                    echo '<td>' . $item[$model][$field] . '</td>';
                }
                //definition des icones d action
                if ($nbAction != 0) {
                    if (isset($item['right_view']) && $item['right_view']) {
                        echo '<td class="action">';
                        $applyViewButton = true;
                        echo $this->Html->image('/img/view.png', array(
                            'title' => __d('default', 'Button.view'),
                            'alt' => __d('default', 'Button.view'),
                            'class' => "itemView",
                            'itemId' => $item[$model]['id']
                                )
                        );
                        echo '</td>';
                    }

                    if (isset($item['right_edit']) && $item['right_edit']) {
                        echo '<td class="action">';
                        $applyEditButton = true;
                        echo $this->Html->image('/img/edit.png', array(
                            'title' => __d('default', 'Button.edit'),
                            'alt' => __d('default', 'Button.edit'),
                            'class' => "itemEdit",
                            'itemId' => $item[$model]['id']
                                )
                        );
                        echo '</td>';
                    }
//debug($item);
                    $isAdmin = false;
                    if( isset($item['Desktop']['name'] ) ) {
                        // FIXME: trouver une meilleure identification du bureau admin
                        if( $item['Desktop']['name'] === 'Bureau administrateur' ){
                            $isAdmin = true;
                        }
                        
                    }
//debug($item);
                    if (isset($item['right_delete']) && !$isAdmin ) {
                        if( $item['right_delete'] ){
                            echo '<td class="action">';
                                $applyDeleteButton = true;
                                echo $this->Html->image('/img/delete.png', array(
                                    'title' => __d('default', 'Button.delete'),
                                    'alt' => __d('default', 'Button.delete'),
                                    'class' => "itemDelete",
                                    'itemId' => $item[$model]['id']
                                        )
                                );
                            echo '</td>';
                        }
                        else {
                            echo '<td class="action"></td>';
                        }
                    }
                        
                    
                    
                    
                    // Ajout pour la vérification du connecteur parapheur
                    if (isset($item['right_check']) && $item['right_check']) {
                        echo '<td class="action">';
                        $applyCheckButton = true;
                        echo $this->Html->image('/img/menu_administration_check.png', array(
                            'title' => __d('default', 'Button.check'),
                            'alt' => __d('default', 'Button.check'),
                            'class' => "itemCheck",
                            'itemId' => $item[$model]['id']
                                )
                        );
                        echo '</td>';
                    }


                     // Ajout pour l'icône de copie de circuit
                    if (isset($item['right_copieCircuit']) && $item['right_copieCircuit']) {
                        echo '<td class="action">';
                        $applyCopieCircuitButton = true;
                        echo $this->Html->image('/img/tab-duplicate.png', array(
                            'title' => __d('default', 'Button.copieCircuit'),
                            'alt' => __d('default', 'Button.copieCircuit'),
                            'class' => "itemCopieCircuit",
                            'itemId' => $item[$model]['id']
                                )
                        );
                        echo '</td>';
                    }
                }
                echo '</tr>';
            }
            echo '</tbody>
            </table>';
        } else {
            //si aucun element dans le tableau passe en parametre
            echo '<div style="padding: 10px;">';
            echo __d(strtolower($model), $model . '.void');
            echo "</div>";
        }


        echo '<script type="text/javascript">';
        //utilisation du system de filtre
        if ($settings['filter']) {
            echo 'gui.filter({ element: $("#filters_' . $idSuffix . '"), table: $("#filterTable_' . $idSuffix . '") });';
        }
        //définition des actions des boutons d actions
        if ($applyViewButton) {
            echo '

                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemView").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
            if (isset($actions['view']['formMessage']) && $actions['view']['formMessage']) {
                //popup avec formulaire
                echo '
                    gui.formMessage({
                        updateElement: ' . $actions['view']['updateElement'] . ',
                        ' . ((!empty($actions['view']['formMessageWidth'])) ? 'width: ' . $actions['view']['formMessageWidth'] . ',' : '') . '
                        ' . ((!empty($actions['view']['formMessageHeight'])) ? 'height: ' . $actions['view']['formMessageHeight'] . ',' : '') . '
                        loader: true,
                        loaderMessage: gui.loaderMessage,
                        title: "' . __d(strtolower($model), 'View ' . $model, true) . '",
                        url: "' . $actions['view']['url'] . '/" + itemId,
                        buttons: {
                            "' . __d('default', 'Button.close') . '": function(){
                                $(this).dialog("close");
                            }
                        }
                    });
                ';
            } else {
                if (!empty($actions['view']['updateElement'])) {
                    //requete ajax mettant a jour un element existant
                    echo '
                    gui.request({
                        url: "' . $actions['view']['url'] . '/" + itemId,
                        updateElement: ' . $actions['view']['updateElement'] . ',
                        loader: true,
                        loaderMessage: gui.loaderMessage
                    });
                ';
                } else {
                    //requete ajax basique
                    echo '
                    gui.request({
                        url: "' . $actions['view']['url'] . '/" + itemId
                    });
                ';
                }
                echo '$("td.action img.itemView").parent().parent().removeClass("ui-state-focus");
                     $(item).parent().parent().addClass("ui-state-focus");
            ';
            }
            echo '});';
        }
        if ($applyEditButton) {
            echo '

                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemEdit").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
            if (isset($actions['edit']['message']) && $actions['edit']['message']) {
                //popup sans formulaire
                echo '
                gui.formMessage({
                    updateElement: ' . $actions['edit']['updateElement'] . ',
                    ' . ((!empty($actions['edit']['formMessageWidth'])) ? 'width: ' . $actions['edit']['formMessageWidth'] . ',' : '') . '
                    ' . ((!empty($actions['edit']['formMessageHeight'])) ? 'height: ' . $actions['edit']['formMessageHeight'] . ',' : '') . '
                    loader: true,
                    loaderMessage: gui.loaderMessage,
                    title: "' . __d(strtolower($model), 'Edit ' . $model, true) . '",
                    url: "' . $actions['edit']['url'] . '/" + itemId,
                    buttons: {
                        "' . __d('default', 'Button.close') . '": function(){
                            $(this).dialog("close");
                            ' . (!empty($actions['edit']['refreshAction']) ? $actions['edit']['refreshAction'] : '') . '
                        }
                    }
                });
                ';
            } else if (isset($actions['edit']['formMessage']) && $actions['edit']['formMessage']) {
                //popup avec formulaire
                echo '
                gui.formMessage({
                    updateElement: ' . $actions['edit']['updateElement'] . ',
                    ' . ((!empty($actions['edit']['formMessageWidth'])) ? 'width: ' . $actions['edit']['formMessageWidth'] . ',' : '') . '
                    ' . ((!empty($actions['edit']['formMessageHeight'])) ? 'height: ' . $actions['edit']['formMessageHeight'] . ',' : '') . '
                    loader: true,
                    loaderMessage: gui.loaderMessage,
                    title: "' . __d(strtolower($model), 'Edit ' . $model, true) . '",
                    url: "' . $actions['edit']['url'] . '/" + itemId,
                    buttons: {
                        "' . __d('default', 'Button.submit') . '": function(){
                          var errorFormTabs = [];
                          var areValid = true;
                          $("form", $(this).parents(".ui-dialog")).each(function(){
                              if ($(this).attr("action") !== undefined && $(this).attr("enctype") != "multipart/form-data"){
                                  if (!$(this).valid()){
                                      areValid = false;
                                      var panelId;
                                      //if ($(this).parents(".ui-tabs-panel").length > 0){
                                        panelId = $(this).parents(".ui-tabs-panel").attr("id");
                                        errorFormTabs.push(panelId.substring(panelId.lastIndexOf("-")));
                                      //}
                                  }
                              }
                          });
                          if (areValid){
                            var submits = {};
                            var index = 0;
                            $(".ui-tabs-panel").each(function(){
                            //$("form", $(this).parents(".ui-dialog")).each(function(){
                                if ($("form", this).attr("action") !== undefined){
                                    submits[index] = {"url": $("form", this).attr("action"), "data": $("form", this).serialize()};
                                    index++;
                                }
                            });
                            submits[index] = "end";
                            gui.submitAll({
                                submits: submits,
                                index: 0,
                                endFunction: "' . $actions['edit']['refreshAction'] . '",
                                loader: true,
                                loaderElement: ' . $actions['edit']['updateElement'] . ',
                                loaderMessage: gui.loaderMessage
                            });
                            $(this).dialog("close");
                          } else {
                            $(".ui-dialog .ui-tabs").tabs("select", errorFormTabs[0]);
                          }
                        },
                        "' . __d('default', 'Button.cancel') . '": function(){
                            $(this).dialog("close");
                        }
                    }
                });
                ';
            } else {
                if (!empty($actions['edit']['updateElement'])) {
                    echo '
                    gui.request({
                        url: "' . $actions['edit']['url'] . '/" + itemId,
                        updateElement: ' . $actions['edit']['updateElement'] . ',
                        loader: true,
                        loaderMessage: gui.loaderMessage
                    });
                ';
                } else {
                    echo '
                    gui.request({
                        url: "' . $actions['edit']['url'] . '/" + itemId
                    });
                ';
                }
                echo '$("td.action img.itemEdit").parent().parent().removeClass("ui-state-focus");
                    $(item).parent().parent().addClass("ui-state-focus");

            ';
            }
            echo '});';
        }
        if ($applyDeleteButton) {
            echo '
                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemDelete").unbind("click");
                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemDelete").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
                    if (!empty($actions['delete']['updateElement'])) {
                        echo '
                            gui.message({
                                title: "' . __d('default', 'Confirmation de suppression') . '",
                                msg: "' . __d('default', 'Voulez-vous supprimer cet élément ?') . '",
                                buttons: {
                                    "' . __d('default', 'Button.confirm') . '": function(){
                                        $(this).dialog("close");
                                        gui.request({
                                            url: "' . $actions['delete']['url'] . '/" + itemId,
                                            updateElement: ' . $actions['delete']['updateElement'] . ',
                                            loader: true,
                                            loaderMessage: gui.loaderMessage
                                        }, function(data){
                                            getJsonResponse(data);
                                            ' . $actions['delete']['refreshAction'] . '
                                        });
                                    },
                                    "' . __d('default', 'Button.cancel') . '": function(){
                                        $(this).dialog("close");
                                    }
                                }
                            });
                        ';
                    } else {
                        echo '
                            gui.request({
                                url: "' . $actions['delete']['url'] . '/" + itemId
                            }, function(data){
                              getJsonResponse(data);
                            });
                        ';
                    }
                    echo '
                });
            ';
        }
        
        // Ajout pour la vérification du connecteur parapheur
        if ($applyCheckButton) {
             echo '
               $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemCheck").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';

                        echo '
                         gui.request({
                                url: "' . $actions['check']['url'] . '/ " + itemId
                            }, function(data){
                              getJsonResponse(data);
                            });
                        ';

                     echo '
                });
            ';
        }
 
        // Ajout pour la copie de Circuit
        if ($applyCopieCircuitButton) {
             echo '
               $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemCopieCircuit").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';

//                         echo '
//                          gui.request({
//                                 url: "' . $actions['copieCircuit']['url'] . '/ " + itemId
//                             }, function(data){
//                               getJsonResponse(data);
//                             });
//                         ';



                    if (!empty($actions['copieCircuit']['updateElement'])) {
                        echo '
                            gui.message({
                                title: "' . __d('default', 'Validation de la copie') . '",
                                msg: "' . __d('default', 'Etes-vous sur de vouloir copier ce circuit ?') . '",
                                buttons: {
                                    "' . __d('default', 'Button.confirm') . '": function(){
                                        $(this).dialog("close");
                                        gui.request({
                                            url: "' . $actions['copieCircuit']['url'] . '/" + itemId,
                                            updateElement: ' . $actions['copieCircuit']['updateElement'] . ',
                                            loader: true,
                                            loaderMessage: gui.loaderMessage
                                        }, function(data){
                                            getJsonResponse(data);
                                            ' . $actions['copieCircuit']['refreshAction'] . '
                                        });
                                    },
                                    "' . __d('default', 'Button.cancel') . '": function(){
                                        $(this).dialog("close");
                                    }
                                }
                            });
                        ';
                    } else {
                        echo '
                            gui.request({
                                url: "' . $actions['copieCircuit']['url'] . '/" + itemId
                            }, function(data){
                              getJsonResponse(data);
                            });
                        ';
                    }
             echo '
                });
            ';
        }
        echo '
            </script>
            </div>';
    }

    /*
     * Affichage d une liste hierarchisée
     *
     * array $tab :  resultat d un find threaded
     * string $model : nom du model utilisé
     * array(string) $labels: champ qui va etre affiché
     * array(string) $updateElements : tableau associatif decrivant les elements jQuery a mettre a jour apres les requetes ajax effectuée
     *  ex :
     *      array(
     *              "read" => "$('#infos .content')",
     *              "delete" => "$('#liste .content')"
     *      )
     */

    /**
     *
     * @param type $tab
     * @param type $model
     * @param type $labels
     * @param type $actions
     * @param int $options
     */
    public function tree($tab, $model, $labels = array(), $actions = array(), $options = array(), $echo = true) {
        $retval = '';
        if (empty($options['nbItem'])) {
            $options['nbItem'] = 0;
        }

        $idSuffix = str_replace(array(' ', '.'), '', microtime());
        $nbAction = count($actions);

        $rights = array();

        if (isset($tab[0]['right_view'])) {
            $rights['view'] = $tab[0]['right_view'];
        }
        if (isset($tab[0]['right_edit'])) {
            $rights['read'] = $tab[0]['right_edit'];
        }
        if (isset($tab[0]['right_delete'])) {
            $rights['delete'] = $tab[0]['right_delete'];
        }


        $retval .= '<h3 class="bannette">Total : ' . $options['nbItem'] . '</h3>
            <div class="bannette_panel">';

        if ($options['nbItem'] > 0) {
            $retval .= '<table class="bannette_table ui-widget ui-corner-all ui-widget-content" id="filterTable_' . $idSuffix . '">
            <thead>';
            foreach ($labels as $label) {
                $retval .= '<th class="' . $label . '">' . __d(strtolower($model), $model . "." . $label, true) . '</th>';
            }

            if ($nbAction != 0) {
                $retval .= '<th class="actions" colspan="' . $nbAction . '">Actions</th>';
            }
            $retval .= '</thead><tbody>';
            $tabactions = array();
            $tabupdateElements = array();

            foreach ($actions as $key => $val) {
                $tabactions[$key] = $val['url'];
                $tabupdateElements[$key] = $val['updateElement'];
            }

            $retval .= $this->drawTree($tab, $model, $labels, $tabactions, $tabupdateElements, $rights);
            $retval .= '</tbody></table>';
        } else {
            $retval .= '<div style="padding: 10px;">';
            $retval .= __d(strtolower($model), $model . '.void');
            $retval .= "</div>";
        }
        $retval .= '<script type="text/javascript">';

        if (isset($tab[0]['right_view']) && $tab[0]['right_view']) {
            $retval .= '

                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemView").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
            if (isset($actions['view']['formMessage']) && $actions['view']['formMessage']) {
                //popup avec formulaire
                $retval .= '
                gui.formMessage({
                    updateElement: ' . $actions['view']['updateElement'] . ',
                    ' . ((!empty($actions['view']['formMessageWidth'])) ? 'width: ' . $actions['view']['formMessageWidth'] . ',' : '') . '
                    ' . ((!empty($actions['view']['formMessageHeight'])) ? 'height: ' . $actions['view']['formMessageHeight'] . ',' : '') . '
                    loader: true,
                    loaderMessage: gui.loaderMessage,
                    title: "' . __d(strtolower($model), 'View ' . $model, true) . '",
                    url: "' . $actions['view']['url'] . '/" + itemId,
                    buttons: {
                        "OK": function(){
                            $(this).dialog("close");
                        }
                    }
                });
                ';
            } else {
                if (!empty($actions['view']['updateElement'])) {
                    $retval .= '
                    gui.request({
                        url: "' . $actions['view']['url'] . '/" + itemId,
                        updateElement: ' . $actions['view']['updateElement'] . ',
                        loader: true,
                        loaderMessage: gui.loaderMessage
                    });
                ';
                } else {
                    $retval .= '
                    gui.request({
                        url: "' . $actions['view']['url'] . '/" + itemId
                    });
                ';
                }
                $retval .= '$("td.action img.itemView").parent().parent().removeClass("ui-state-focus");
                    $(item).parent().parent().addClass("ui-state-focus");

            ';
            }
            $retval .= '});';
        }
        if (isset($tab[0]['right_edit']) && $tab[0]['right_edit']) {
            $retval .= '

                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemEdit").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
            if (isset($actions['edit']['formMessage']) && $actions['edit']['formMessage']) {
                //popup avec formulaire
                $retval .= '
                gui.formMessage({
                    updateElement: ' . $actions['edit']['updateElement'] . ',
                    ' . ((!empty($actions['edit']['formMessageWidth'])) ? 'width: ' . $actions['edit']['formMessageWidth'] . ',' : '') . '
                    ' . ((!empty($actions['edit']['formMessageHeight'])) ? 'height: ' . $actions['edit']['formMessageHeight'] . ',' : '') . '
                    loader: true,
                    loaderMessage: gui.loaderMessage,
                    title: "' . __d(strtolower($model), 'Edit ' . $model, true) . '",
                    url: "' . $actions['edit']['url'] . '/" + itemId,
                    buttons: {
                        "' . __d('default', 'Button.submit') . '": function(){
                          var errorFormTabs = [];
                          var areValid = true;
                          $("form", $(this).parents(".ui-dialog")).each(function(){
                              if ($(this).attr("action") !== undefined && $(this).attr("enctype") != "multipart/form-data"){
                                  if (!$(this).valid()){
                                      areValid = false;
                                      var panelId;
                                      panelId = $(this).parents(".ui-tabs-panel").attr("id");
                                      errorFormTabs.push(panelId.substring(panelId.lastIndexOf("-")));
                                  }
                              }
                          });

                          if (areValid){
                              var submits = {};
                              var index = 0;
                              $(".ui-tabs-panel").each(function(){
                                  if ($("form", this).attr("action") !== undefined){
                                      submits[index] = {"url": $("form", this).attr("action"), "data": $("form", this).serialize()};
                                      index++;
                                  }
                              });
                              submits[index] = "end";

                              gui.submitAll({
                                  submits: submits,
                                  index: 0,
                                  endFunction: "' . $actions['edit']['refreshAction'] . '",
                                  loader: true,
                                  loaderElement: ' . $actions['edit']['updateElement'] . ',
                                  loaderMessage: gui.loaderMessage
                              });
                              $(this).dialog("close");
                          } else {
                            $(".ui-dialog .ui-tabs").tabs("select", errorFormTabs[0]);
                          }
                        },
                        "' . __d('default', 'Button.cancel') . '": function(){
                            $(this).dialog("close");
                        }
                    }
                });
                ';
            } else {
                if (!empty($actions['edit']['updateElement'])) {
                    $retval .= '
                    gui.request({
                        url: "' . $actions['edit']['url'] . '/" + itemId,
                        updateElement: ' . $actions['edit']['updateElement'] . ',
                        loader: true,
                        loaderMessage: gui.loaderMessage
                    }, function(data){
                          ' . $actions['edit']['refreshAction'] . '
                    });
                ';
                } else {
                    $retval .= '
                    gui.request({
                        url: "' . $actions['edit']['url'] . '/" + itemId
                    });
                ';
                }
                $retval .= '$("td.action img.itemEdit").parent().parent().removeClass("ui-state-focus");
                    $(item).parent().parent().addClass("ui-state-focus");

            ';
            }
            $retval .= '});';
        }

        //TODO: Terminer d adapter les nouveaux parametres $actions, $options

        if (isset($tab[0]['right_delete']) && $tab[0]['right_delete']) {
            $retval .= '
                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemDelete").unbind("click");
                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemDelete").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
            if (!empty($actions["delete"]['updateElement'])) {
                $retval .= '
                    gui.message({
                        title: "' . __d('default', 'Confirmation de suppression') . '",
                        msg: "' . __d('default', 'Voulez-vous supprimer cet élément ?') . '",
                        buttons: {
                            "' . __d('default', 'Button.confirm') . '": function(){
                                $(this).dialog("close");
                                gui.request({
                                    url: "' . $actions['delete']['url'] . '/" + itemId,
                                    updateElement: ' . $actions["delete"]['updateElement'] . ',
                                    loader: true,
                                    loaderMessage: gui.loaderMessage
                                }, function(data){
                                    getJsonResponse(data);
                                     ' . $actions['delete']['refreshAction'] . '
                                });
                            },
                            "' . __d('default', 'Button.cancel') . '": function(){
                                $(this).dialog("close");
                            }
                        }
                    });
                ';
            } else {
                $retval .= '
                    gui.request({
                        url: "' . $actions['delete']['url'] . '/" + itemId
                    });
                ';
            }
            $retval .= '
                });
            ';
        }

        //TODO: deplacer le fonctionnemment de la liste hierarchisée dans gui.js

        $retval .= '
            $(".treeCollapseButton").unbind("click");
            $(".treeCollapseButton").click(function(){
                var element = $(this).parent().parent();
                var id = element.attr("id");

                if (element.attr("state") == "open"){
                    element.attr("state", "closed");

                    var str = $(this).attr("src");
                    var reg = new RegExp("(down.png)","g");
                    if (str.match(reg)) {
                        str = str.replace(reg, "right.png");
                    }
                    $(this).attr("src", str);

                    $("." + id).each(function(){
                        if ($(this).hasClass("treeTrParent")){
                            $(this).attr("state", "closed");

                            var str = $(".treeCollapseButton", this).attr("src");
                            var reg = new RegExp("(down.png)","g");
                            if (str.match(reg)) {
                                str = str.replace(reg, "right.png");
                            }
                            $(".treeCollapseButton", this).attr("src", str);
                        }
                        $($(this)).hide();
                    });
                } else if (element.attr("state") == "closed"){
                    element.attr("state", "open");

                    var str = $(this).attr("src");
                    var reg = new RegExp("(right.png)","g");
                    if (str.match(reg)) {
                        str = str.replace(reg, "down.png");
                    }
                    $(this).attr("src", str);

                    $("." + id).each(function(){
                        if ($(this).hasClass("treeTrParent")){
                            $(this).attr("state", "open");

                            var str = $(".treeCollapseButton", this).attr("src");
                            var reg = new RegExp("(right.png)","g");
                            if (str.match(reg)) {
                                str = str.replace(reg, "down.png");
                            }
                            $(".treeCollapseButton", this).attr("src", str);
                        }
                        $($(this)).show();
                    });
                }
            });
        </script>';
        if ($echo) {
            echo $retval;
            $retval = true;
        }
        return $retval;
    }

    /**
     *
     * @param type $tab
     * @param type $model
     * @param type $labels
     * @param type $actions
     * @param type $updateElements
     * @param type $rights
     * @param type $treeTrParentsId
     * @param type $lvl
     */
    public function drawTree($tab, $model, $labels, $actions, $updateElements, $rights, $treeTrParentsId = array(), $lvl = 0) {
        $retval = '';
        foreach ($tab as $item) {
            $hasChildren = isset($item['children'][0]);
            if ($hasChildren) {
                $treeTrParentsId[] = $this->treeTrId;
            }
            $retval .= $this->drawTreeLine($item, $model, $labels, count($actions), $updateElements, $rights, $lvl, $hasChildren, $treeTrParentsId);
            if ($hasChildren) {
                $lvl++;
                $retval .= $this->drawTree($item['children'], $model, $labels, $actions, $updateElements, $rights, $treeTrParentsId, $lvl);
                $lvl--;
                array_pop($treeTrParentsId);
            }
        }
        return $retval;
    }

    /**
     *
     * @param type $item
     * @param type $model
     * @param type $labels
     * @param type $nbAction
     * @param type $updateElements
     * @param type $rights
     * @param type $lvl
     * @param type $hasChildren
     * @param type $treeTrParentsId
     */
    public function drawTreeLine($item, $model, $labels, $nbAction, $updateElements, $rights, $lvl, $hasChildren, $treeTrParentsId) {
        $retval = "";
        $retval .= '<tr hasChildren="' . $hasChildren . '" state="open" id="treeTrId_' . $this->treeTrId . '" onmouseover="$(this).addClass(\'ui-state-highlight\');" onmouseout="$(this).removeClass(\'ui-state-highlight\');" itemId="' . $item[$model]['id'] . '" class="treeTr ';
        if (isset($item['retard']) && $item['retard']) {
            $retval .= 'ui-state-error ';
        }
        if ($hasChildren) {
            $retval .= 'treeTrParent ';
        }
        foreach ($treeTrParentsId as $trParentId) {
            if ($trParentId != $this->treeTrId) {
                $retval .= 'treeTrId_' . $trParentId;
                if ($trParentId != $treeTrParentsId[count($treeTrParentsId) - 1]) {
                    $retval .= " ";
                }
            }
        }

        foreach ($item as $linkedItemKey => $linkedItemVal) {
            if ($linkedItemKey != $model && $linkedItemKey != "right_view" && $linkedItemKey != "right_edit" && $linkedItemKey != "right_delete") {
                if (isset($linkedItemVal['id']) && !empty($linkedItemVal['id'])) {
                    $retval .= " " . $linkedItemKey . "_" . $linkedItemVal['id'];
                } else {
                    foreach ($linkedItemVal as $linkedItem2) {
                        if (!empty($linkedItem2['id'])) {
                            $retval .= " " . $linkedItemKey . "_" . $linkedItem2['id'];
                        }
                    }
                }
            }
        }

        $retval .= '">';

        $indentItem = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
        $indentation = '';
        for ($i = 0; $i < $lvl; $i++) {
            $indentation .= $indentItem;
        }
        if ($hasChildren) {
            $indentation .= $this->Html->image('/img/arrow_down.png', array('class' => 'treeCollapseButton'));
        }

        $retval .= "<td> " . $indentation . " " . $item[$model][$labels[0]] . "</td>";

        if ($nbAction != 0) {
            if (isset($rights['view'])) {
                $retval .= '<td class="action">';
                if ($rights['read']) {
                    $retval .= $this->Html->image('/img/view.png', array(
                        'title' => __d('default', 'Button.view'),
                        'alt' => __d('default', 'Button.view'),
                        'class' => "itemView",
                        'itemId' => $item[$model]['id']
                            )
                    );
                }
                $retval .= '</td>';
            }

            if (isset($rights['read'])) {
                $retval .= '<td class="action">';
                if ($rights['read']) {
                    $retval .= $this->Html->image('/img/edit.png', array(
                        'title' => __d('default', 'Button.edit'),
                        'alt' => __d('default', 'Button.edit'),
                        'class' => "itemEdit",
                        'itemId' => $item[$model]['id']
                            )
                    );
                }
                $retval .= '</td>';
            }

            if (isset($rights['delete'])) {
                $retval .= '<td class="action">';
                if ($rights['delete']) {
                    $retval .= $this->Html->image('/img/delete.png', array(
                        'title' => __d('default', 'Button.delete'),
                        'alt' => __d('default', 'Button.delete'),
                        'class' => "itemDelete",
                        'itemId' => $item[$model]['id']
                            )
                    );
                }
                $retval .= '</td>';
            }
        }
        $retval .= "</tr>";
        $this->treeTrId++;
        return $retval;
    }

    /**
     *
     * @param type $treeMix
     * @param type $treeMixOptions
     */
    public function treeMix($treeMix, $treeMixOptions) {
        $tab = $treeMix;
        $model = $treeMixOptions["parent"]["model"];
        $fields = $treeMixOptions["parent"]["fields"];
        $actions = $treeMixOptions["parent"]["actions"];
        $options = $treeMixOptions["parent"]["options"];

        $childrenModel = $treeMixOptions["children"]["model"];
        $childrenFields = $treeMixOptions["children"]["fields"];
        $childrenActions = $treeMixOptions["children"]["actions"];
        $childrenOptions = $treeMixOptions["children"]["options"];

        //parametrage par défaut
        $settings = array(
            'filter' => true,
            'title' => '',
            'paginator' => false
        );
        //application des parametre personnalisé
        foreach ($options as $key => $val) {
            $settings[$key] = $val;
        }
        //initilisation des variables
        $idSuffix = str_replace(array(' ', '.'), '', microtime());
        $nbItem = count($tab);
        $nbAction = count($actions);
        $applyEditButton = false;
        $applyViewButton = false;
        $applyDeleteButton = false;

        //preparation du titre
        if (!$settings['paginator'] && !$settings['noTitle']) {
            if ($settings['title'] != '') {
                $settings['title'] .= ' (total : ' . $nbItem . ')';
            } else {
                $settings['title'] = 'Total : ' . $nbItem;
            }
        }

        //affichage du titre
        echo '<h3 class="bannette">' . $settings['title'] . '</h3>
            <div class="bannette_panel">';
        //affichage de la table html
        if ($nbItem > 0) {
            //affichage du systeme de filtre
            if ($settings['filter']) {
                echo '<div class="controlsFilter" id="filters_' . $idSuffix . '">';
                echo '<span class="bttnFilter">';
                echo $this->Html->image('icons/filtre.png');
                echo $this->Html->image('/img/filter_down_16.png');
                echo '</span>';
                echo '<span class="statusFilter">';
                echo $this->Html->image('/img/filter_off_16.png');
                echo '</span>';
                echo '</div>';
            }

            echo '<table class="bannette_table ui-widget ui-corner-all ui-widget-content" id="filterTable_' . $idSuffix . '">
            <thead>';

            //affichage des entetes de la table html
            foreach ($fields as $field) {
                if (!$settings['paginator']) {
                    echo '<th class="' . $field . '">' . __d(strtolower($model), $model . "." . $field, true) . '</th>';
                } else {
                    echo '<th class="' . $field . '">' . $this->Paginator->sort($model . "." . $field, __d(strtolower($model), $model . "." . $field)) . '</th>';
                }
            }

            if ($nbAction != 0) {
                echo '<th class="actions" colspan="' . $nbAction . '">Actions</th>';
            }
            echo '</thead>
            <tbody>';
            //parcours des elements
            foreach ($tab as $item) {
                $idSuffixcreated = strtotime($item[$model]['created']);

                //debut de la ligne du tableau html parametrage de l identifiant et des classes de chaque ligne
                echo '<tr id="listTrId_' . $item[$model]['id'] . '" onmouseover="$(this).addClass(\'ui-state-highlight\');" onmouseout="$(this).removeClass(\'ui-state-highlight\');" itemId="' . $item[$model]['id'] . '" state="open" class="listTr';
                if (isset($item['retard']) && $item['retard']) {
                    echo 'ui-state-error ';
                }
                //elements enfants dans un tableau cakephp (permet d effectuer des liaisons javascript qui suivent la structure de la base de données)
                foreach ($item as $linkedItemKey => $linkedItemVal) {
                    if ($linkedItemKey != $model && $linkedItemKey != "right_view" && $linkedItemKey != "right_edit" && $linkedItemKey != "right_delete") {
                        if (isset($linkedItemVal['id']) && !empty($linkedItemVal['id'])) {
                            echo " " . $linkedItemKey . "_" . $linkedItemVal['id'];
                        } else {
                            foreach ($linkedItemVal as $linkedItem2) {
                                if (!empty($linkedItem2['id'])) {
                                    echo " " . $linkedItemKey . "_" . $linkedItem2['id'];
                                }
                            }
                        }
                    }
                }
                echo '">';
                //affichage du contenu de chaque champ
                foreach ($fields as $field) {
                    echo '<td>';
                    if ($field == $fields[0]) {
                        echo $this->Html->image('/img/arrow_down.png', array('class' => 'treeMixCollapseButton')) . " ";
                    }
                    echo $item[$model][$field];
                    if ($field == $fields[0]) {
                        if (count($item[$childrenModel]) > 1) {
                            echo ' (' . count($item[$childrenModel]) . ' ' . __d('soustype', 'soustypes') . ')';
                        } else {
                            echo ' (' . count($item[$childrenModel]) . ' ' . __d('soustype', 'soustype') . ')';
                        }
                    }

                    echo '</td>';
                }
                //definition des icones d action
                if ($nbAction != 0) {

                    if (isset($item['right_edit']) && $item['right_edit']) {
                        echo '<td class="action">';
                        $applyEditButton = true;
                        echo $this->Html->image('/img/edit.png', array(
                            'title' => __d('default', 'Button.edit'),
                            'alt' => __d('default', 'Button.edit'),
                            'class' => "itemEdit",
                            'itemId' => $item[$model]['id']
                                )
                        );
                        echo '</td>';
                    }
                    if (isset($item['right_view']) && $item['right_view']) {
                        echo '<td class="action">';
                        $applyViewButton = true;
                        echo $this->Html->image('/img/view.png', array(
                            'title' => __d('default', 'Button.view'),
                            'alt' => __d('default', 'Button.view'),
                            'class' => "itemView",
                            'itemId' => $item[$model]['id']
                                )
                        );
                        echo '</td>';
                    }
                    if (isset($item['right_delete']) && $item['right_delete']) {
                        echo '<td class="action">';
                        $applyDeleteButton = true;
                        echo $this->Html->image('/img/delete.png', array(
                            'title' => __d('default', 'Button.delete'),
                            'alt' => __d('default', 'Button.delete'),
                            'class' => "itemDelete",
                            'itemId' => $item[$model]['id']
                                )
                        );
                        echo '</td>';
                    }
                }
                echo '</tr>';
                echo '<tr><td colspan="' . (count($fields) + $nbAction) . '" style="padding-left: 100px;">';

                $newTab = array();
                foreach ($item[$childrenModel] as $childItem) {
                    $childItem['right_edit'] = true;
                    $childItem['right_delete'] = true;
                    $newTab[] = array($childrenModel => $childItem, 'right_edit' => true, 'right_delete' => true);
                }

                $this->draw($newTab, $childrenModel, $childrenFields, $childrenActions, $childrenOptions);

                echo'</td></tr>';
            }
            echo '</tbody>
            </table>';
        } else {
            //si aucun element dans le tableau passe en parametre
            echo '<div style="padding: 10px;">';
            echo __d(strtolower($model), $model . '.void');
            echo "</div>";
        }


        echo '<script type="text/javascript">';
        //utilisation du system de filtre
        if ($settings['filter']) {
            echo 'gui.filter({ element: $("#filters_' . $idSuffix . '"), table: $("#filterTable_' . $idSuffix . '") });';
        }
        //définition des actions des boutons d actions
        if ($applyViewButton) {
            echo '

                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemView").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
            if (isset($actions['view']['formMessage']) && $actions['view']['formMessage']) {
                //popup avec formulaire
                echo '
                    gui.formMessage({
                        updateElement: ' . $actions['view']['updateElement'] . ',
                        ' . ((!empty($actions['view']['formMessageWidth'])) ? 'width: ' . $actions['view']['formMessageWidth'] . ',' : '') . '
                        ' . ((!empty($actions['view']['formMessageHeight'])) ? 'height: ' . $actions['view']['formMessageHeight'] . ',' : '') . '
                        loader: true,
                        loaderMessage: gui.loaderMessage
                        title: "' . __d(strtolower($model), 'View ' . $model, true) . '",
                        url: "' . $actions['view']['url'] . '/" + itemId,
                        buttons: {
                            "' . __d('default', 'Button.close') . '": function(){
                                $(this).dialog("close");
                            }
                        }
                    });
                ';
            } else {
                if (!empty($actions['view']['updateElement'])) {
                    //requete ajax mettant a jour un element existant
                    echo '
                    gui.request({
                        url: "' . $actions['view']['url'] . '/" + itemId,
                        updateElement: ' . $actions['view']['updateElement'] . ',
                        loader: true,
                        loaderMessage: gui.loaderMessage
                    });
                ';
                } else {
                    //requete ajax basique
                    echo '
                    gui.request({
                        url: "' . $actions['view']['url'] . '/" + itemId
                    });
                ';
                }
                echo '$("td.action img.itemView").parent().parent().removeClass("ui-state-focus");
                     $(item).parent().parent().addClass("ui-state-focus");
            ';
            }
            echo '});';
        }
        if ($applyEditButton) {
            echo '

                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemEdit").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
            if (isset($actions['edit']['formMessage']) && $actions['edit']['formMessage']) {
                //popup avec formulaire
                echo '
                gui.formMessage({
                    updateElement: ' . $actions['edit']['updateElement'] . ',
                    ' . ((!empty($actions['edit']['formMessageWidth'])) ? 'width: ' . $actions['edit']['formMessageWidth'] . ',' : '') . '
                    ' . ((!empty($actions['edit']['formMessageHeight'])) ? 'height: ' . $actions['edit']['formMessageHeight'] . ',' : '') . '
                    loader: true,
                    loaderMessage: gui.loaderMessage,
                    title: "' . __d(strtolower($model), 'Edit ' . $model, true) . '",
                    url: "' . $actions['edit']['url'] . '/" + itemId,
                    buttons: {
                        "' . __d('default', 'Button.submit') . '": function(){
                            var form = $(".ui-dialog-content form");
                            gui.request({
                                url: form.attr("action"),
                                data: form.serialize(),
                                loaderElement: ' . $actions['edit']['updateElement'] . ',
                                loaderMessage: gui.loaderMessage
                            }, function(){
                                ' . $actions['edit']['refreshAction'] . '
                            });
                            $(this).dialog("close");
                        },
                        "' . __d('default', 'Button.cancel') . '": function(){
                            $(this).dialog("close");
                        }
                    }
                });
                ';
            } else {
                if (!empty($actions['edit']['updateElement'])) {
                    echo '
                    gui.request({
                        url: "' . $actions['edit']['url'] . '/" + itemId,
                        updateElement: ' . $actions['edit']['updateElement'] . ',
                        loader: true,
                        loaderMessage: gui.loaderMessage
                    });
                ';
                } else {
                    echo '
                    gui.request({
                        url: "' . $actions['edit']['url'] . '/" + itemId
                    });
                ';
                }
                echo '$("td.action img.itemEdit").parent().parent().removeClass("ui-state-focus");
                    $(item).parent().parent().addClass("ui-state-focus");

            ';
            }
            echo '});';
        }
        if ($applyDeleteButton) {
            echo '
                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemDelete").unbind("click");
                $("#filterTable_' . $idSuffix . ' > tbody > tr > td.action img.itemDelete").click(function(){
                    var item = $(this);
                    var itemId = $(this).attr("itemId");';
            if (!empty($actions['delete']['updateElement'])) {
                echo '
                    gui.message({
                        title: "' . __d('default', 'Confirmation de suppression') . '",
                        msg: "' . __d('default', 'Voulez-vous supprimer cet élément ?') . '",
                        buttons: {
                            "' . __d('default', 'Button.confirm') . '": function(){
                                $(this).dialog("close");
                                gui.request({
                                    url: "' . $actions['delete']['url'] . '/" + itemId,
                                    updateElement: ' . $actions['delete']['updateElement'] . ',
                                    loader: true,
                                    loaderMessage: gui.loaderMessage
                                }, function(){
                                    ' . $actions['delete']['refreshAction'] . '
                                });
                            },
                            "' . __d('default', 'Button.cancel') . '": function(){
                                $(this).dialog("close");
                            }
                        }
                    });
                ';
            } else {
                echo '
                    gui.request({
                        url: "' . $actions['delete']['url'] . '/" + itemId
                    });
                ';
            }
            echo '
                });
            ';
        }
        echo '

            $(".treeMixCollapseButton").click(function(){
                var tr = $(this).parent().parent();
                $($(this).parent().parent().next("tr")).toggle();
                if (tr.attr("state") == "open"){
                    var str = $(this).attr("src");
                    var reg = new RegExp("(down.png)","g");
                    if (str.match(reg)) {
                        str = str.replace(reg, "right.png");
                    }
                    $(this).attr("src", str);
                    tr.attr("state", "closed");
                } else if (tr.attr("state") == "closed"){
                    var str = $(this).attr("src");
                    var reg = new RegExp("(right.png)","g");
                    if (str.match(reg)) {
                        str = str.replace(reg, "down.png");
                    }
                    $(this).attr("src", str);
                    tr.attr("state", "open");
                }
            });

            </script>
            </div>';
    }

    /**
     *
     * @param type $items
     * @param type $options
     * @return type
     */
    public function jqueryListView($items = array(), $options) {
        $retval = "";
        if (!empty($items) && !empty($options)) {
            //application des choix par défaut
            $settings = array_merge(array('listId' => 'browser', 'listClass' => 'filetree'), $options);

            //recuperation des
            $listId = $settings['listId'];
            $listClass = $settings['listClass'];

            $parentNodeSettings = $settings['nodes'][0];
//      $childNodeSettings = $settings['nodes'][1];

            $nestedListItems = array();
            foreach ($items as $item) {
                $parentNodeSettings['actions']['id'] = $item[$parentNodeSettings['model']]['id'];
                $parentNodeSettings['actions']['model'] = $parentNodeSettings['model'];

                $noIdParent = $item[$parentNodeSettings['model']]['id'] == '' ? ' ui-state-focus' : '';
                $nestedListParent = $this->Html->tag('span', $item[$parentNodeSettings['model']][$parentNodeSettings['field']] . /* ' (' . (count($item[$childNodeSettings['model']])) . ')' . */ $this->_genActions($parentNodeSettings['actions']), array('id' => $parentNodeSettings['actions']['model'] . '_' . $parentNodeSettings['actions']['id'], 'class' => 'folder' . $noIdParent));

                $nestedListItems[] = $nestedListParent;
            }
            $retval = $this->Html->nestedList($nestedListItems, array('id' => $listId, 'class' => $listClass));
        }
        return $retval;
    }

    /**
     *
     * @param type $items
     * @param type $options
     * @return type
     */
    public function jqueryTreeView($items = array(), $options) {
        $retval = "";
        if (!empty($items) && !empty($options)) {
            //application des choix par défaut
            $settings = array_merge(array('listId' => 'browser', 'listClass' => 'filetree'), $options);

            //recuperation des
            $listId = $settings['listId'];
            $listClass = $settings['listClass'];

            $parentNodeSettings = $settings['nodes'][0];
            $childNodeSettings = $settings['nodes'][1];

            $nestedListItems = array();
            foreach ($items as $item) {
                $parentNodeSettings['actions']['id'] = $item[$parentNodeSettings['model']]['id'];
                $parentNodeSettings['actions']['model'] = $parentNodeSettings['model'];

                $noIdParent = $item[$parentNodeSettings['model']]['id'] == '' ? ' ui-state-focus' : '';

                $specialClass = '';
                if (!empty($parentNodeSettings['classField'])) {
                    $specialClassItemValue = $item[$parentNodeSettings['model']][key($parentNodeSettings['classField'])];
                    $specialClass = ' ' . $parentNodeSettings['classField'][key($parentNodeSettings['classField'])][$specialClassItemValue];
                }

                $nestedListParent = $this->Html->tag('span', $item[$parentNodeSettings['model']][$parentNodeSettings['field']] . ' (' . (count($item[$childNodeSettings['model']])) . ')' . $this->_genActions($parentNodeSettings['actions']), array('class' => 'folder' . $noIdParent . $specialClass));

                $nestedListItems[$nestedListParent] = array();
                foreach ($item[$childNodeSettings['model']] as $child) {
                    $selected = isset($settings['selectedChild']) && $settings['selectedChild'] == $child['id'];

                    $childNodeSettings['actions']['id'] = $child['id'];
                    $childNodeSettings['actions']['model'] = $childNodeSettings['model'];

                    $nestedListItems[$nestedListParent][] = $this->Html->tag('span', $child[$childNodeSettings['field']] . $this->_genActions($childNodeSettings['actions'], $selected), array('class' => 'file' . (($selected) ? ' ui-state-valid' : '')));
                }
            }
            $retval = $this->Html->nestedList($nestedListItems, array('id' => $listId, 'class' => $listClass));
        }
        return $retval;
    }

    public function jqueryThreadedView($items = array(), $options) {
        $retval = "";
        if (!empty($items) && !empty($options)) {
            //application des choix par défaut
            $settings = array_merge(array('listId' => 'browser', 'listClass' => 'filetree'), $options);

            //recuperation des
            $listId = $settings['listId'];
            $listClass = $settings['listClass'];

            $parentNodeSettings = $settings['nodes'][0];
            $nestedListItems = array();
            foreach ($items as $item) {
                $parentNodeSettings['actions']['id'] = $item[$parentNodeSettings['model']]['id'];
                $parentNodeSettings['actions']['model'] = $parentNodeSettings['model'];

                $noIdParent = $item[$parentNodeSettings['model']]['id'] == '' ? ' ui-state-focus' : '';
                $nestedListItem = $this->Html->tag('span', $item[$parentNodeSettings['model']][$parentNodeSettings['field']] . ' (' . (count($item['children'])) . ')' . $this->_genActions($parentNodeSettings['actions']), array('class' => 'folder' . $noIdParent));
                if (!empty($item['children'])) {
                    $nestedListItem .= $this->jqueryThreadedView($item['children'], $options);
                }
                $nestedListItems[] = $nestedListItem;
            }
            $retval = $this->Html->nestedList($nestedListItems, array('id' => $listId, 'class' => $listClass));
        }
        return $retval;
    }

    /**
     *
     * @param type $actions
     * @param type $selected
     * @return type
     */
    private function _genActions($actions = array(), $selected = false) {
        $retval = "";
        if (!empty($actions) && $actions['id'] != '') {
            $spans = '';
            foreach ($actions as $kAction => $action) {
                if ($kAction != 'id' && $kAction != 'model') {

                    $statusImg = null;
                    if (!empty($action['icoAlt']) && $selected) {
                        $statusImg = $this->Html->image($action['ico'], array('style' => 'max-width: 24px;max-height: 24px;margin-top:-5px', 'class' => 'statusImg'));
                    }

                    $img = $this->Html->image(!empty($action['icoAlt']) && $selected ? $action['icoAlt'] : $action['ico'], array('alt' => __d('default', 'Button.' . $kAction . ($selected ? '_alt' : '')), 'id' => $actions['model'] . '_' . $actions['id'], 'class' => 'actionImg ' . $kAction . 'Bttn ' . $actions['model'], 'style' => 'max-width: 24px;max-height: 24px;margin-top:-5px'));
                    $spans .= ($statusImg != null ? $statusImg : '') . $this->Html->link($img, $action['url'] . '/' . $actions['id'], array('escape' => false, 'class' => 'treeviewBttn'));
                }
            }
            $retval = $this->Html->tag('span', $spans, array('style' => 'float:right;'));
        }
        return $retval;
    }

}

?>
